/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
	is : 'maximo-asset-detail',
	behaviors : [ BaseComponent, CommonDetailBehavior ],
	listeners : {

		'maximo-tile-edit-apply' : '_applyEdits',
		'maximo-lookup-selected' : '_applyEditLookUpSelected',
		'maximo-table-tile-delete-action' : '_deleteChild',
		'assetdetail' : '_displayDetails',
		'maximo-lookup-closed' :'_handleLookupClose',
		'maximo-tile-edit-cancel': '_handleTileEditCancel',
		'maximo-checkbox-changed': '_handleCheckboxChange',
		'callDeleteAsset': '_handleDeleteAsset',
  		'deleteAssetRecord': '_deleteAssetRecord',
  		'showwodetail' : '_displayWODetail',
  		'wosupervisor' : '_displayWODetail',
  		'maximo-lookup-arrowicon' : '_moveAsset',
  		'image-add-success':'_imageAddSuccess',  		
  		'inner-app-saved' : '_recordChangedFromIframe',
  		'supervisorwo-hist' : '_showsupervisorWoHist',
  		'launch-classic':'_launchClassic',
  		'iframe-saved' : '_recordChangedFromIframe',
  		'managefailurecode':'_managefailurecode',
  		'image-upload-for-moved':'_imageUploadForMoved',
  		'isLoctionStoreroomType':'_isLoctionStoreroomType'
	},
	properties : {

		record : {
			type : Object,
			notify : true,
			observer: '_getLocationClause'
		},
		
		_assetdata : {
			type : Object,
			value : function() {
				return {};
			}
		},
		status : {
			type : String,
			notify : true
		},
		assetDescription : {
			type : String,
			notify : true
		},
		collection : {
			type : Object
		},
		/**
	     * property used to make the tiles read only
	     * This functionality is specifically used for Supervisor and Technician
	     */		
		readOnly: {
			type: Boolean,
		    observer: '_editableAssetFields',
			value: false,
			notify:true
		},
  		back:{
  			type: String
  		}, 
  		assetparent :{
			type : String,
		},
		parentClause :{
			type : String,
    	},
    	locationClause :{
    		type : String,
			notify: true
    	},	
    	siteid:{
    		type : String,
    	},
    	previewColumns : {
			type : Array,
			value : []
		},
    	woHistColumns : {
			type : Array,
			value : []
		},
		checkboxchecked: {
			type: Boolean,
			value: false
		},
		menuItems : {
        	type: Array,
        	//inline if at label was required for unit test propose
    		value: [{"label":$M?$M.localize('uitext','mxapiasset','deleteassetlabel'):"delete asset","event":"callDeleteAsset","icon":"Maximo:Trash"}],
    		notify:true    		
        },
		/**
	     * force the component fire an event when a record is clicked on location or asset  table
	     */	
		notifyDetail: {
			type: Boolean,
			value: false
		},
		/**
	     * Flag for children data loaded or not
	     */
		childAssetsLoaded : {
			type : Boolean,
			value : false,
			notify : true
		},
		/**
	     * Flag for related records data loaded or not
	     */
		relatedRecordsLoaded : {
			type : Boolean,
			value : false,
			notify : true
		},	
		contextSubscriptions: { 
            value: ['inner-app-saved'] 
		},
		
		orgClause :{
			type : String,
			computed:'_getCurrentOrg(siteid)'
    	},
	},
	
	ready: function() {
		var self = this;
		//cancel edit on click of side menu
		$j('body').on('click', 'div.sideMenu', function(e){
			$j(self).find('maximo-tile').each(function(index, item){
				self._cancelTileEdit(e, item);
			});
		});
		
		//cancel editing other tiles on focus of any tile
		$j(self).on('focusin', 'maximo-tile document.body', function(e){
			$j(self).find('maximo-tile').not(this).each(function(index, item){
				self._cancelTileEdit(e, item);
			});
			self._hideLookup();
		});
		
		//prevent edit record on Enter keypress by detaching keypress event on maximo-text inside maximo-badge-tile
		$j(self).off( 'keypress', 'maximo-badge-tile maximo-text');
		
		this._checkAttachmentAddButton();
		
		
	},
	
	attached: function() {
		//condition used for using name on botc and description on non-botc environment
		var descLabel = 'description';
				
		this.siteid= ($M.getMaxauth().whoami.defaultSite) ? $M.getMaxauth().whoami.defaultSite : '';
		
		this.previewColumns =  [ {
			'attribute' : 'assetnum',
			'title' : $M.localize('uitext', 'mxapiasset', 'id'),
			'sortable' : true,
			'hyperlink' : true,
			'listener' : 'assetdetail',
			'width':'20%',
			'flexGrow': '1'
		}, {
			'attribute' : 'description',
			'title' : $M.localize('uitext', 'mxapiasset', descLabel),
			'sortable' : true,
			'width':'40%',
			'flexGrow': '2'
		}, {
			'attribute' : 'status_description',
			'sortattribute' : 'status',
			'title' : $M.localize('uitext', 'mxapiasset', 'status'),
			'sortable' : true,
			'width':'20%',
			'flexGrow': '1',
			'conditionalformatting':'{"attribute":"status","values": {"ACTIVE": "color:'+this.getComputedStyleValue('--Primary-green50')+';font-weight:bold"}}'
			
		} ];
		
		this.woHistColumns =  [ {
			'attribute' : 'wonum',
			'title' : $M.localize('uitext', 'mxapiasset', 'id'),
			'sortable' : true,
			'hyperlink' : true,
			'listener' : 'showwodetail',
			'width':'20%',
			'flexGrow': '1'
		}, {
			'attribute' : 'description',
			'title' : $M.localize('uitext', 'mxapiasset', 'description'),
			'sortable' : true,
			'width':'40%',
			'flexGrow': '2'
		}, {
			'attribute' : 'actfinish_localized',
			'title' : $M.localize('uitext', 'mxapiasset', 'actualfinish'),
			'sortable' : true,
			'sortattribute':'actfinish',
			'sortType':'descending',
			'flexGrow': '1.5',
			
			
		} ];
		
		$j(this).find('maximo-tile').each(function(){
			var maximoTile = this;
			if(maximoTile.childTile && maximoTile.childTile.nodeName === 'MAXIMO-IMAGE-TILE'){
				$j(maximoTile).children().first().css('z-index', '4');
			}
		});
		if($M.isMobile() && !$M.workScape.hideFooter && $M.workScape.show){
			$j(this).closest('div.absolutePage').css({'height':'calc(100% - 80px)','margin-bottom':'50px'});
			 
		}
	},

	_showList : function(e) {
		$j(this.$.createasset).hide();
		$j(this.$.maindiv).show();
	},
	
	_addAsset : function(e) {
		if(!$M.getMaxauth().whoami.defaultSite){
			$M.showDialog(this, 'accessmessage', null, 'maximo-message-with-ok', false, null, { 'MsgText':$M.localize('messages', 'access', 'nodefaultsite') });
		} else {
			$j(this.parentElement).hide();
			$j('#asset_createassetmain').find('.mainPage').show();
			if(this.back){
				$j('#summary_createassetsum').find('.mainPage').show();						
			} else {
				$j('#summary_createassetsum').find('.mainPage').hide();	
			}
		}
	},

	/**
	 * apply any edits made to the record
	 */
	_applyEdits : function(e) {
		this.collection=$M.getGlobalResource('assetcollection');
		switch (e.target.dataset.scopeId) {

		case 'common_detail_asset_statusTile':
			this._saveupdateMethod(e.target, this.collection,'StatusUpdate');
			break;

		case 'common_detail_asset_leadimg':
			this._updatedetail(e.target, this.collection);
			break;
			
		case 'common_detail_asset_typeTile':
			this._saveupdateMethod(e.target, this.collection,'TypeUpdate');
			break;	
			
		case 'common_detail_asset_assetParentTile':
			this._saveupdateMethod(e.target, this.collection,'parentstatusUpdate');
			break;	
			
		case 'common_detail_asset_serialNumTile':
			this._saveupdateMethod(e.target, this.collection,'serialNumUpdate');
			break;
			
		case 'common_detail_asset_childAssetTableTile':
			this._setChildAsset(e);
			break;
		case 'common_detail_asset_assetPriorityTile':
			this._saveupdateMethod(e.target, this.collection,'priorityUpdate');
			break;
		case 'common_detail_asset_assetFailureClassTile':
			this._saveupdateMethod(e.target, this.collection,'failureUpdate');
			break;
			
		}
	},

    _applyEditLookUpSelected : function(e)
    {    	
    	var outerTile = $j(e.target).closest('maximo-tile');
    	if(outerTile && outerTile.length) {
    		var innerTile = outerTile.find('.inner-tile');
    		if(innerTile.length) {
    			this.async(function(){
	    			$j(innerTile).find('input').css('width', '120px');
	    		}, 100);
    			
    			if(innerTile[0].dataset.scopeId === 'common_detail_asset_statusTile') {
    				this._setStatusCheckbox(innerTile[0]);
    			 } else if(innerTile[0].nodeName === 'MAXIMO-TABLE-TILE') {
 	    				this._applyEdits(e);
 	    		 } else {
    				innerTile[0]._applyEdit(e);
    			 }
    		}
    	}
    },
    _setStatusCheckbox: function(targetElement){
    	var checkbox = $j(targetElement).find('maximo-checkbox');
      	if(this.record.status_maxvalue === 'DECOMMISSIONED'){
      		this.checkboxchecked = checkbox[0].checked = true;
	  		checkbox.find('.switch, .checkbox').hide();
	  		targetElement.checkboxlabel = $M.localize('uitext', 'mxapioperloc', 'decommissionedlabel');	 	     		
		} else {
			this.checkboxchecked = checkbox[0].checked = false;
			checkbox.find('.switch, .checkbox').show();  
			targetElement.checkboxlabel = $M.localize('uitext', 'mxapioperloc', 'statuscheckboxlabel');	 	     		
		}
    },
    
	_saveupdateMethod : function(e, collectionToRefresh, tileToUpdate) {

		var params = '';
		var responseProperties = collectionToRefresh.attributeNames;
		var siteId = this.record.siteid;
		this.$.common_detail_asset_assetresource.resourceUri=this.record.href;
		
		if(this.record.moved){
			this._recordMoved(e,this.record,responseProperties);
			return;
		}
		
	   else if (tileToUpdate === 'StatusUpdate') {
			params = {
				status : this.record.status,
				rollToAllChildren: this.checkboxchecked,
				removeFromActiveRoutes: false,
				removeFromActiveSP: false,
				changePMStatus: false
			};
		} else if (tileToUpdate === 'TypeUpdate') {
			params = {
				assettype : this.record.type,
				assettype_description : this.record.assettype_description,
				siteid : siteId
			};

		}
		else if (tileToUpdate === 'serialNumUpdate') {
			params = {
					serialnum : this.$.common_detail_asset_serialNumTile.value,
					siteid : siteId
			};
		}
		else if (tileToUpdate === 'priorityUpdate') {
			params = {
					priority : this.$.common_detail_asset_assetPriorityTile.value,
					siteid : siteId
			};
			this.$.common_detail_asset_assetPriorityTile.emptyRecord=false;
		}
		else if (tileToUpdate === 'failureUpdate') {
			params = {
					failurecode : this.record.failurecode,
					siteid : siteId
			};
		}

		else if (tileToUpdate === 'parentstatusUpdate') {
			
			
			var parent=$j(this.$.common_detail_asset_parentstatus).find('#backtext_input').val();
			
			var confirmResourceUri = this.record.href;
			confirmResourceUri +='?action=wsmethod:moveSingleAsset';
			this.$.common_detail_asset_assetresource.resourceUri=confirmResourceUri;
			

			var newLoc = this.record.asset_locations.location ;
			
			if (parent!=='' && this.record.parent.location) {
				newLoc = this.record.parent.location;
			}
			
			params = {
					'newLocation':newLoc,
					'newSite': siteId
			};
			
				if (parent !== '') {
				params.newParent = parent;
			}
				
		}
		var self=this;
		var componentId=e.dataset.scopeId;
		
		$M.toggleWait(true);
		
		if (tileToUpdate === 'StatusUpdate' && !this.record.moved) {
			this.$.common_detail_asset_assetresource.updateAction('wsmethod:changeStatus', params, responseProperties).then(function(result) {
				if(result.response){
					//self.record=result.response;
					self._refreshRecord();
				}
				$M.toggleWait(false);
			}, function(error) {
				self._getPreEditValues(self,componentId,e);
				$M.showResponseError(error);
			});
		} else {
				self.$.common_detail_asset_assetresource.updateRecord(params, responseProperties, true)
							.then(function(result) {
								if(result.response){
									self.record=result;									
								}
								self._refreshRecord();
								if($j(self.$.common_detail_asset_parentstatus).find('#backtext_input').val()===''){
									self.$.common_detail_asset_assetParentTile.label='';									
								}
								$M.toggleWait(false);
							}, function(error) {
								self._getPreEditValues(self,componentId,e);
								$M.toggleWait(false);
								$M.showResponseError(error);
							});
				
		}
	},

	_updatedetail : function(e) {
		var self=this;
		var location=$j(this.$.common_detail_asset_locationMT).find('#backtext_input').val();
		var newsiteId = this.record.siteid;
		var componentId=e.dataset.scopeId;	
		var responseProperties = this.$.childassetcollection.attributeNames;
		
	
		if(this.record.moved){
			this._recordMoved(e,this.record,responseProperties);
			return;
		}
		
		this.$.common_detail_asset_assetresource.resourceUri=this.record.href;
		
				var params = {			
							location : location,
			                siteid : newsiteId 
			                
					};
		
			
		this.$.common_detail_asset_assetresource.updateRecord(params, responseProperties, true).then(
				function(result) {
					$M.toggleWait(false);
					self.record=result;										
				}, function(error) {
					
					self._getPreEditValues(self,componentId,e);
					
					
					$M.toggleWait(false);
					$M.showResponseError(error);
					
				});
	},

	/**
	 * Edit the record
	 */
	_updateAsset : function(e) {
		if(this.readOnly){
			return;
		}
		var newsiteId = this.record.siteid;
		if (this.$.common_detail_asset_strAssetDescription.value.trim().length === 0) {
			this.$.common_detail_asset_strAssetDescription.value = this.assetDescription;
			return false;
		}
		var params = {
			'description' : this.$.common_detail_asset_strAssetDescription.value,
			'siteid' : newsiteId 
		};
		this._updateDescription(e.target,params, this.collection);
	},

	_updateDescription : function(e,dataToUpdate, collectionToUpdate) {
		
		var responseProperties = 'description';
		
		if(this.record.moved){
			this._recordMoved(e,this.record,responseProperties);
			return;
		}
		
		this.$.common_detail_asset_assetresource.resourceUri=this.record.href;
		var data = {
			'href' : this.record.href
		};
		Object.keys(dataToUpdate).forEach(function(key) {
			data[key] = dataToUpdate[key];
		});
		
		$M.toggleWait(true);
		this.assetDescription = data.description;
		this.$.common_detail_asset_assetresource.updateRecord(data, responseProperties, true).then(
				function(result) {
					$M.toggleWait(false);
				
				}, function(error) {
					$M.toggleWait(false);
					$M.showResponseError(error);
					
				});
	},		
	/**
	 * return icon for status
	 */
	_getStatusIcon : function(status) {
		if (status) {
			var data = this.getStyle(status);
			return data.icon;
		} else {
			return '';
		}
	},
	/**
	 * return color for status
	 */
	_getClassForIcon : function(status) {

		if (status) {
			var data = this.getStyle(status);
			return this.getComputedStyleValue(data.color);
		} else {
			return '';
		}
	},
	/**
	 * return json data
	 */
	getStyle : function(status) {
		var data = {};
		Object.keys(status).forEach(function(key) {
			var keyArray = key.split('^');
			var valueArry = status[key].split('^');
			keyArray.forEach(function(item, index) {
				data[item] = valueArry[index];
			});
		});
		return data;
	},
	/**
	 * Make the Asset details fields readonly for supervisor and technician
	 */
	_editableAssetFields: function(){
		var dataHost = this.dataHost._rootDataHost;
		if(dataHost && (dataHost.tagName==='WORKSCAPE-SUPERVISOR' || dataHost.tagName==='MAXIMO-TECHNICIAN-MAIN' )){
			 $j(this).find('maximo-image-editor').each(function(){ this.setAttribute('is-edit',false); });
			 $j(this).find('maximo-text').each(function(){ this.setAttribute('readonly',true); });
			 $j(this).find('maximo-table-tile').each(function(){this.enableDelete = false;});
		}
	},
		/**
	    * Hide/Show Attachment link.
	    */
	   _showAddAttachmentLink : function(attachments){
		   if(attachments && attachments.length>0){
			   return false;
		   }
		   return true;
	   },
	   
	   _setChildAsset : function(e) {
		var self = this;
		var parent = this.record.assetnum;
		this.assetparent=parent;
		
		var assetToUpdate = [];
		
		if (e.detail.identifier === 'common_detail_asset_childAssetTableTile') {		
			var items = e.detail.records;
			this._assetdata=items;
			items.forEach(function(item) {
				var params={
						
						newLocation : item.location,
						newParent : parent,
						newSite : item.siteid,
						href : item.href	
						};
				
				assetToUpdate.push(params);				
			});
				$M.toggleWait(true);
			if(!this.record.moved){
				self.$.childassetcollection.createRecordBulk(assetToUpdate,'wsmethod:moveSingleAsset',true);
			}
			else{
				var responseProperties = this.collection.attributeNames;
				this._recordMoved(e.target,this.record,responseProperties);
				}
		}
		
		
		
	},
		
	_handleBulkSuccess : function(e)
	{		
		if(e.target.id === 'childassetcollection'){
			this._refreshRecord();
		} 
	},
	
	_refreshRecord: function(){		
		var maximoQbe = new MaximoQbe();
		maximoQbe.setEquals('parent',this.record.assetnum);
		maximoQbe.setEquals('siteid',this.record.siteid);
		var qbeObject = maximoQbe.toJSON();
		this.$.childassetcollection.queryWhere = qbeObject;
		$M.toggleWait(true);
		this.childAssetsLoaded = false;
		var self = this;
		this.$.childassetcollection.refreshRecords().then(
				function(result) {
					self.childAssetsLoaded = true;
					$M.toggleWait(false);
				});
		
        var whereClause=['oslc.where=assetnum=%22'+encodeURIComponent(this.record.assetnum)+'%22 and istask=%220%22','internalvalues=1','ignorecollectionref=1'];
        
        this.$.common_detail_asset_woHist.additionalParams=whereClause;
        this.relatedRecordsLoaded = false;
        var self = this;
        this.$.common_detail_asset_woHist.refreshRecords().then(
				function() {
					self.relatedRecordsLoaded = true;
				});
        
        this.$.common_detail_asset_assetresource.resourceUri = this.record.href;
        this.$.common_detail_asset_assetresource.loadRecord().then(function(result){
        self.record = result;
        
         if(result.asset_locations && result.asset_locations.type_maxvalue!==undefined && (result.asset_locations.type_maxvalue==='STOREROOM' || result.asset_locations.type_maxvalue==='COURIER' || result.asset_locations.type_maxvalue==='LABOR')){
        	 self.$.common_detail_asset_leadimg.editable = false;
    		 self.$.common_detail_asset_tableTile.setAttribute('hidden',true); 
             self.$.common_detail_asset_parentstatus.setAttribute('hidden',true);
         }else{        	
             self.$.common_detail_asset_tableTile.removeAttribute('hidden');   
             self.$.common_detail_asset_parentstatus.removeAttribute('hidden');
        }
         
         if(result.parent && result.parent.location && result.parent.location.type_maxvalue!==undefined  && (result.parent.location.type_maxvalue==='STOREROOM' || result.parent.location.type_maxvalue==='COURIER' || result.parent.location.type_maxvalue==='LABOR')){        	 
        	 self.$.common_detail_asset_assetParentTile.editable = false;	
         }else{
        	 if(self.domHost && !( self.domHost.tagName==='WORKSCAPE-SUPERVISOR' || self.domHost.tagName==='MAXIMO-TECHNICIAN-MAIN')){
        		 self.$.common_detail_asset_assetParentTile.editable = true;	
        	 }
         }
         self._getParentClause(result.siteid,result.asset_locations.type_maxvalue);
        });
	},
	
	_handleError: function (e) {
		$M.toggleWait(false);
		var error = e.detail.Error;
		if(error) {
			$M.notify(error.message, $M.alerts.error);
		}
		if(e.target.id === 'childassetcollection'){
			this._refreshRecord();
		} 
	},
	
	_deleteChild : function(e) {
		var self = this;
		var confirmResourceUri =e.detail.record.href;
		confirmResourceUri +='?action=wsmethod:moveSingleAsset';
		this.$.common_detail_asset_assetresource.resourceUri=confirmResourceUri;

		var params = {
				'newLocation':e.detail.record.location,
				'newSite':e.detail.record.siteid
		};
			
		var responseProperties = 'assetnum,parent,siteid';

		$M.toggleWait(true);
		
		this.$.common_detail_asset_assetresource.updateRecord(params, responseProperties, true)
				.then(function() {
					var that = self;
					self.childAssetsLoaded = false;
					self.$.childassetcollection.refreshRecords().then(
									function(result) {
										that.childAssetsLoaded = true;
										$M.toggleWait(false);
									});
						}, function(error) {
							$M.showResponseError(error);
							self.childAssetsLoaded = true;
							$M.toggleWait(false);
							self.$.childassetcollection.refreshRecords();							
						});

	},
	
	_displayDetails : function(e) {
		if(!e.currentTarget._editMode && !e.target.matches('button')) {
		var self = this;
		var maximoQbe = new MaximoQbe();
		
		var asset = '';
		
		if(e.detail.model) {
			asset = e.detail.model.__data__.record.assetnum;			
		} else {			
			asset = e.currentTarget.value;
			
		}	
		
		if(asset){
		maximoQbe.setEquals('assetnum', asset);
		maximoQbe.setEquals('siteid',this.record.siteid);
		var qbeObject = maximoQbe.toJSON();
		this.$.childCollection.queryWhere = qbeObject;
		$M.toggleWait(true);
		this.$.childCollection.collectionUri = '';
		this.$.childCollection.refreshRecords().then(
			function(result) {	
				$M.toggleWait(false);
				if(result.response.member && result.response.member.length > 0) {
					
					document.body.scrollTop = 0;
					document.documentElement.scrollTop = 0; 
					var dataHost = self.dataHost._rootDataHost;
					if(self.notifyDetail){
						self.fire('show-common-asset-details', {
							'record' : result.response.member[0]
						});	
					}else if(dataHost && ( dataHost.tagName==='WORKSCAPE-SUPERVISOR' || dataHost.tagName==='MAXIMO-TECHNICIAN-MAIN' )){
						self.fire('showAssetDetail', {
							'record' : result.response.member[0]
						});
					} else {
						self.fire('display-asset-detail', {
							'record' : result.response.member[0],
							'collection' : self.collection
						});
					}
				}						
			});
		}
	}
	},
	 /**
	 * Get asset clause for asset table tile Lookup
	 */
	_getParentClause : function(locationsite,type){
		var parentClause = 'responseos=MXAPIASSET&setupdata={\'asset\':{}}';
		if(locationsite !== undefined && locationsite.length > 0){
			if(type==='COURIER' || type==='LABOR' || type==='STOREROOM'){
				parentClause += "&oslc.where=siteid=%22"+encodeURIComponent(locationsite)+"%22%20and%20itemnum!%3D%22*%22";
			}else{ 
				parentClause += "&savedQuery=lookupAsset&sqp:siteid="+encodeURIComponent(locationsite)+"&oslc.where=siteid=%22"+encodeURIComponent(locationsite)+"%22";
			}
		} 
		this.parentClause=parentClause;		
    },	
	_getPreEditValues :function(self,componentId,targetElement) { 
		if (self && self.$[componentId] !== undefined) {
			var previousvalue = self.$[componentId]._preEditValues.value;
			self.$[componentId].value = previousvalue;

			var previousImage = self.$[componentId]._preEditValues.image;
			self.$[componentId].image = previousImage;

			var previousLabel = self.$[componentId]._preEditValues.label;
			self.$[componentId].label = previousLabel;
			
			
			this.$.childCollection.collectionUri = this.record.href;
			$M.toggleWait(true);
			this.$.childCollection.refreshRecords().then(function(result){	
				$M.toggleWait(false);
				if(result.response) {
					self.record = result.response;
					if(targetElement.dataset.scopeId === 'common_detail_asset_statusTile') {
						$j(targetElement).closest('maximo-badge-tile').attr('color',self._getClassForIcon(self.record.status_domainmeta));
						$j(targetElement).closest('maximo-badge-tile').attr('icon', self._getStatusIcon(self.record.status_domainmeta));
						self._setStatusCheckbox(targetElement);
					}
				}
			});	
			
		}
	},
	
	_displayLocationDetails : function(e, myrecord) {	
		if((!e.currentTarget._editMode && !e.target.matches('button'))||(e.detail.id==='asset_woHistDetail_locationimg')) {
			var self = this;
			var maximoQbe = new MaximoQbe();	
			var location;
			if(e.currentTarget.value){
				location = e.currentTarget.value;
			}
			else if(this.record.asset_locations){
				location = this.record.asset_locations.location;			
			}			
			if(this.record.asset_locations && (this.record.asset_locations.type_maxvalue==='STOREROOM')){
				return false;
			}
			
			if(location){
				maximoQbe.setEquals('location', location);
				var qbeObject = maximoQbe.toJSON();
				this.$.common_detail_asset_curretlocation.queryWhere = qbeObject;
				$M.toggleWait(true);
				this.$.common_detail_asset_curretlocation.collectionUri = '';
				this.$.common_detail_asset_curretlocation.refreshRecords().then(
					function(result) {		
						$M.toggleWait(false);
						if(result.response.member && result.response.member.length) {	
							var dataHost = self.dataHost._rootDataHost;
							if(self.notifyDetail){
								self.fire('show-common-location-details', {
									'record' : result.response.member[0]
								});	
							}else if(dataHost && (dataHost.tagName==='WORKSCAPE-SUPERVISOR' || dataHost.tagName==='MAXIMO-TECHNICIAN-MAIN' )){
								self.fire('show-location-detail', {
									'record' : result.response.member[0]
								});
							} else {
								$M.setSubRoute('admPlaces/detail');
								if(e.detail.id==='asset_woHistDetail_locationimg'){
									self.fire('common-location-details', {
										'record' : result.response.member[0],
										'collection' : $M.getGlobalResource('common_detail_location_curretlocation'),
										'back' : 'wohistdetail',
										'worecord': myrecord
									});
								}
								else
								{
									if($j.find('#location').length>0){
										 $j.find('#location')[0].back='asset-detail';
									}
									self.fire('common-location-details', {
										'record' : result.response.member[0],
										'collection' : $M.getGlobalResource('common_detail_location_curretlocation'),
									});
								}
							}
						}						
					});
			}
		}			
	},
		
	_isAddAssetHidden: function(){
		return $M.currentWorkscape !== 'manageassets';
	},
	/**
	 * Handle Collection Refresh for table-tile
	 */
	_handleCollectionRefresh: function(){
		window.dispatchEvent(new Event('resize'));										
		var childAssetInAsset=$j(this.$.common_detail_asset_childAssetTableTile).find('#showassetdetail_common_detail_asset_childAssetTableTile_table');
		if(childAssetInAsset.length>0){
			childAssetInAsset[0].actionCallback=this.getLocalFunction('_actionCallback');
		 }
	},
	
	/**
	  * Perform actions after tile edit cancel
	  */
	_handleTileEditCancel: function(e){
		//set pre-edit values after tile edit cancel
		this._getPreEditValues(this,e.target.dataset.scopeId,e.target);
	},
	
	/**
	  * Perform actions after lookup is closed
	  */
	  _handleLookupClose : function(e) { 
			var maximoTile = $j(e.target).closest('maximo-tile');
			if(maximoTile.length){
				var innerTile = maximoTile.find('.inner-tile');
				if(innerTile.length){
					if(innerTile[0].dataset.scopeId === 'common_detail_asset_statusTile'){
						return false;
					}else {
						this._closeLookup(innerTile[0]);
					}
				} 
			}
		},
		
		/**
		 *set checkboxchecked value on toggle of maximo-checkbox
		 */
	  	_handleCheckboxChange: function(e){
	  		this.checkboxchecked = e.detail.checked;
	    },
	    /**
		 *clause to remove SYSTEMLOC location
		 */
	    _getLocationClause : function(){
	    	
	   		
			var locationClause = 'responseos=MXAPIOPERLOC&oslc.where=siteid=%22'+encodeURIComponent(this.record.siteid )+'%22'; 
				if(this.record && this.record.itemnum){
				locationClause += "&domaininternalwhere=type=HOLDING,OPERATING,SALVAGE,VENDOR,REPAIR";		
				}else{
				  locationClause += "&domaininternalwhere=type=HOLDING,OPERATING,SALVAGE,VENDOR,REPAIR,COURIER,LABOR";	
				}
				locationClause += '&setupdata={%22siteid%22:%22'+encodeURIComponent(this.record.siteid)+'%22}';
			
			this.locationClause = locationClause;
			if(this.record){
				this.$.common_detail_asset_showassetDetailImage.$.myRecord.resourceUri = this.record.href+"?responseos=MXAPIASSET";
			}
		},
				/**
		 * Hide/Show Attachment Add Button. 
		 */	
		_checkAttachmentAddButton : function(){			
			if($M.currentWorkscape === 'supervisor' || $M.currentWorkscape === 'technician'){
				 this.$.attachment_tile.disable();
			}
			else{
				 this.$.attachment_tile.enableActions();
			}
		},

		_handleDeleteAsset : function(e){
			var assetData = e.target.dataset;
			$M.showDialog(this, 'deleteAssetId', null, 'maximo-asset-deletedialog', false, null, {'datasetAssetRecord':assetData , 'MsgText':$M.localize('uitext','mxapiasset','deleteconfirmation') });
		},
		
		/**
	     * Call Web Service to delete User record .
	     */
		_deleteAssetRecord: function(){
			var _self = this;
			$M.toggleWait(true);
			this.fire('show-wait-spinner', true);
			
			var _complete = function(){
				
				_self.$.common_detail_asset_assetresource.deleteRecord(_self.record.href).then(function(response) {
					$M.toggleWait(false);
					$M.getGlobalResource('assetcollection').refreshRecords();
					_self.fire('asset-list', {'record':_self.record,'panelName':_self.panelName,'schema': _self.schema});	
					
				}, function(error) {
					$M.removeNotification(this);
					$M.toggleWait(false);
					$M.notify($M._getErrorResponse(error).message, $M.alerts.warn);
				});
				
			};
			
			var _undo = function(){
				$M.toggleWait(false);
				return;
			};

			$M.notify($M.localize('uitext','mxapiasset','deletedmsg') + ": " + this.record.description + ' ', $M.alerts.warninfo, _complete, _undo);
						

		},
		
		/**
	     * Listener used to filter and display data related to the selected WO
	     */
		_displayWODetail : function(e) {
			var _self = this;
			var currentworkcenter = $M.$.contextAppRouter.data.workcenter;
			if(currentworkcenter ==='supervisor' || currentworkcenter ==='technician'){
				this.fire('wo-detail', {'record': e.detail.model.record,  'collection':this.$.common_detail_asset_woHist, 'sourcePage': 'woHistoryAsset'});				
			}else{ 
			var wonum = '';
				if(e.detail.wonum){
					wonum = e.detail.wonum;
				} else {
					wonum = e.detail.model.__data__.record.wonum;
				}
				var whereClause=['oslc.where=assetnum=%22'+encodeURIComponent(this.record.assetnum)+'%22 and wonum=%22'+encodeURIComponent(wonum)+'%22 and istask=%220%22','internalvalues=1','ignorecollectionref=1'];
				this.$.common_detail_asset_woHist.additionalParams=whereClause;
				this.relatedRecordsLoaded = false;
				this.$.common_detail_asset_woHist.refreshRecords().then(function(result) {
					self.relatedRecordsLoaded = true;
					if(result.response.member && result.response.member.length) {
							$M.setSubRoute('admAsset/admWorkorder');
							document.body.scrollTop = 0;
							document.documentElement.scrollTop = 0;						
							_self.fire('wo-hist', {'record':result.response.member[0],'collection': _self.$.common_detail_asset_woHist,'panelName':'wohist-panel','schema': _self.$.common_detail_asset_woHist.schema});
							
							}
				});
			}
		},
		
		/**
	     * Set the list of work orders to be displayed on WO List
	     */
		_setWOHistList : function(e) {
			var whereClause=['oslc.where=assetnum=%22'+encodeURIComponent(this.record.assetnum)+'%22 and istask=%220%22','internalvalues=1','ignorecollectionref=1'];
			this.$.common_detail_asset_woHist.additionalParams=whereClause;
			this.relatedRecordsLoaded = false;
			var self = this;
			this.$.common_detail_asset_woHist.refreshRecords().then(function(result) {
				self.relatedRecordsLoaded = true;
			});
		},
		
		 /**
		  * Show MoveAsset flyout on click of lookup add icon  
		  */	
		_moveAsset : function(e){
				
			this.assetDialog = $M.showFlyout('','maximo-move-asset', null,{'hideFooter':true,'top':'150px','width':'100%','height':'300px','recordData':this.record});
			this.async(function(){
				$j('.underlay').css({'z-index':'1600'});
				$j('.flyout .internal').css({'justify-content':'center', 'padding': '10px 0px' });
				$j('.flyout .closeButton').css({'text-align':'right', 'padding-left': '0px'});
				$j('.flyout').css({'right':'0','margin':'0 auto','width': '450px', 'z-index':'1601'});
				$j('.flyout').find('input').css('padding', '3px 0px');
				}, 100);
			},

			
			/**
			 * Refresh collection record by assetnum and siteid
			 */
		_refreshCollectionData: function(){		
			var self=this;
			var whereClause=['oslc.where=assetnum=%22'+encodeURIComponent(this.record.assetnum)+'%22%20and%20siteid=%22'+encodeURIComponent(this.record.siteid)+'%22','internalvalues=1','ignorecollectionref=1'];
			this.childAssetsLoaded = false;
			this.relatedRecordsLoaded = false;
			this.dataHost.$.assetcollectionmain.additionalParams=whereClause;
			this.dataHost.$.assetcollectionmain.refreshRecords().then(function(result){
	        	self.record=result.response.member[0];
	        	self.childAssetsLoaded = true;
	        	self.relatedRecordsLoaded = true;
	        	if(result.response.member[0].moved){
	  			  self.$.common_detail_asset_serialNumTile.removeAttribute('editable');
	  			  self.$.common_detail_asset_assetPriorityTile.removeAttribute('editable');
	  			}
	        });
	},
		
		/**
		 * Function calls when record has already been moved
		 */
		_recordMoved :function(e,record,responseProperties){
			var self=this;
			
			var params = {
					siteid:record.siteid
				};
			
			var confirmResourceUri = record.href;
			
			this.$.common_detail_asset_assetresource.resourceUri = confirmResourceUri+'?action=wsmethod:moveSingleAsset';
			setTimeout(function() {
			self.$.common_detail_asset_assetresource.updateRecord(params, responseProperties, true)
						.then(function(result) {
							$M.toggleWait(false);
						}, function(error) {
							self._getPreEditValues(self,e.dataset.scopeId,e);
							$M.toggleWait(false);
							$M.showResponseError(error);
						});	
		}, 20);	
		},
		
		/**
		 * set imagelibref in current record
		 */
		_imageAddSuccess: function(e){
			if (e.currentTarget.record && e.currentTarget.record._imagelibref){
				e.currentTarget.record._imagelibref = e.detail._imagelibref;
			}
		 },
		 
			
		/**
	 	 * display a record
		 * 
	 	 */
		showDetail : function(asset){			
			this.record = asset;
			this._refreshRecord();
			this._editableAssetFields();
		},
		
		/* 
	     *  Record Refresh when comes back from iframe classic 
	     */ 
	    _recordChangedFromIframe: function(e){
	      var self=this;
	  	  this.$.common_detail_asset_assetresource.resourceUri=this.record.href;
	      if(e.type==='iframe-saved'){
	    	  this.domHost.$.assetDetailsdiv.style.display='block';
	      }
			  this.$.common_detail_asset_assetresource.loadRecord().then(function(result){
				  
				 self.$.common_detail_asset_assetPriorityTile.emptyRecord = (result.priority === undefined);
				 if(result.moved){
					 self.dataHost.$.showassetdetail.$.common_detail_asset_serialNumTile.removeAttribute('editable');
					 self.dataHost.$.showassetdetail.$.common_detail_asset_assetPriorityTile.removeAttribute('editable');
				}
				 e.detail.record= result;
				 setTimeout(function() {
					self.dataHost._displayAssetDetails(e);
				 }, 20);
			  });
		  },	    
	    
	    /**
		 * Display the WO History details page(maximo supervisor details)
		 */
		_showsupervisorWoHist : function(e) {
			$j(this.$.createasset).hide();
			$j(this.$.uploadasset).hide();
			$j(this.$.assetDetailsdiv).hide();
			$j(this.$.maindiv).hide();
			$j(this.$.assetDetails).hide();
			$j(this.$.woHisdiv).show();
			
			if(!e.detail.record.workorderownergroup)
				e.detail.record.workorderownergroup = '';
			this.$.woHisDetail.resetContentSelectorDefault();
			this.$.woHisDetail._showSubTab('section_details');
			$j(this).find('#assetDetails_woHisDetail')[0].record = e.detail.record;
			$j(this).find('#assetDetails_woHisDetail')[0].recordLight = e.detail.record;
			$j(this).find('#assetDetails_woHisDetail')[0].collection = e.detail.collection;
			$M.setSubRoute('woFlow/flowDetails');
		},
		
		/**
		 * Launch Classic to open user record
		 */
		_launchClassic : function(){			
			this.domHost.$.assetDetailsdiv.style.display='none';
		},
	    
	    _isItemnumExists: function(itemnum) {
            return !(itemnum);           
        },
        _isLoctionStoreroomType: function() {        	
            var self=this;
            var whereClause=['oslc.where=assetnum=%22'+encodeURIComponent(this.record.assetnum)+'%22%20and%20siteid=%22'+encodeURIComponent(this.record.siteid)+'%22','internalvalues=1','ignorecollectionref=1'];
                    
            this.dataHost.$.assetcollectionmain.additionalParams=whereClause;
            this.dataHost.$.assetcollectionmain.refreshRecords().then(function(result){
            self.record=result.response.member[0];
            if(self.record && self.record.asset_locations && self.record.asset_locations.type_maxvalue!==undefined && (self.record.asset_locations.type_maxvalue==='STOREROOM' || self.record.asset_locations.type_maxvalue==='COURIER' || self.record.asset_locations.type_maxvalue==='LABOR')){
            	self.$.common_detail_asset_tableTile.setAttribute('hidden',true); 
                self.$.common_detail_asset_parentstatus.setAttribute('hidden',true);
                self.$.common_detail_asset_leadimg.editable = false;
            }else{
                 self.$.common_detail_asset_tableTile.removeAttribute('hidden');   
                 self.$.common_detail_asset_parentstatus.removeAttribute('hidden');
                 self.$.common_detail_asset_leadimg.editable = true;	
              }
           });
       },        
        _managefailurecode : function(){
        	this.domHost.$.assetDetailsdiv.style.display='none';
        	$M.openMaximoRecord('FAILURE',null);
        },
        
        /**
         * Function call when try to upload image for moved asset record
         */
        _imageUploadForMoved : function(e){
        	var record=e.detail;
        	var params = {
					siteid:record.resourceData.siteid
				};
			
			var myRecordUri = record.resourceUri;
			
			record.resourceUri=myRecordUri+'&action=wsmethod:moveSingleAsset';
			$M.toggleWait(true);
			record.updateRecord(params).then(function() {
				$M.toggleWait(false);
			}, function(error) {
				$M.toggleWait(false);
				$M.showResponseError(error);
			});
        },
        /**
         * get login user organization
         */
        _getCurrentOrg : function(siteid){
    		var orgClause='';
    		if(siteid){
    			orgClause = 'oslc.where=orgid=%22'+encodeURIComponent($M.getMaxauth().whoami.defaultOrg)+'%22';
    		}
    		return orgClause;
    	},
    	/**
    	 *  Function to Hide/Show actions based on rotating asset 
    	 */
    	_actionCallback: function(record){    		
    		return !((record.asset_locations.type_maxvalue==='LABOR' || record.asset_locations.type_maxvalue==='COURIER') && record.itemnum);   		 
    	 }
        
});
