/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
	is : 'maximo-location-detail',

	behaviors : [ BaseComponent, CommonDetailBehavior, HTMLEncoder],

	listeners : {
		'maximo-tile-edit-apply' : '_editRcord',			
		'maximo-lookup-selected' : '_returnFromLookup',
		'maximo-table-tile-delete-action' : '_deleteTableRecord',
		'child-location-detail' : '_childLocationDetail',
		'child-asset-detail' : '_displayAssetDetail',
		'maximo-lookup-addicon' : '_showAddress',
		'setAddress':'_setAddress',
		'maximo-lookup-closed' :'_handleLookupClose',
	  	'maximo-checkbox-changed': '_handleCheckboxChange',
	  	'maximo-tile-edit-cancel': '_handleTileEditCancel',
	  	'assign-toplevel-loc':'_setAsTopLevel',
	  	'image-add-success':'_imageAddSuccess',
	  	'iframe-saved' : '_recordChangedFromIframe',
	  	'launch-classic':'_launchClassic',
	  	'maximo-serviceaddress-created': 'updateServiceAddress'
	},

	properties : {

		/**
		 * Location record
		 */
		record : {
			type : Object,
			notify : true,
			observer: '_getParentClause'
		},
		collection : {
			type : Object
		},
		schema : {
			type : Object
		},
		panelName : {
			type : String
		},
		locDescription : {
			type : String,
			notify : true
		},
		status : {
			type : String,
			notify : true
		},
		parentLocation : {
			type : String,
			notify : true
		},
		parentImage : {
			type : String,
			notify : true
		},
		parentDescription : {
			type : String,
			notify : true
		},
		lochierLocalref : {
			type : String,
			notify : true
		},
		lochierHref : {
			type : String,
			notify : true
		},
		//Used to keep the value for parent location tile if update parent fails
		originalParentLocation : {
			type : String
		},
		originalParentImage : {
			type : String
		},
		originalParentDescription : {
			type : String
		},
		originalLochierLocalref : {
			type : String
		},
		originalLochierHref : {
			type : String
		},
		originalParentRecord : {
			type : String
		},
		/**
		 * To get parent location clause
		 */
		parentClause :{
			type : String,
			
    	},
    	siteid: {
       		type: String
		},
		orgid: {
       		type: String
		}, 
    	locationsite: {
       		type: String
		},
		/**
		 * To get Asset Clause for asset table tile
		 */
		assetClause :{
			type : String			
    	},    	
    	previewAssets : {
			type : Array,
			value : []
		},		
		locsystemLocalref : {
			type : String,
			notify : true
		},    	
		childrenClause :{
			type : String,
			notify: true
    	},
    	childrenColumns: {
			type: Array,
			value : []
		},
		additionalparams : {
   			type: Array,
   			value: [],
   			notify: true
   		},
   		additionalassetparams : {
   			type: Array,
   			value: [],
   			notify: true
   		},
   		additionalParam :{
   			type : String,
			computed:'_setadditionalParam(orgid)'
   		},
   		/**
	     * property used to make the tiles read only
	     * This functionality is specifically used for Supervisor and Technician
	     */		
		readOnly: {
			type: Boolean,
		    observer: '_editableLocationFields',
			value: false,
			notify:true
		},
		checkboxchecked: {
			type: Boolean,
			value: false
		},
		
		selectedSystem: {
			type : String,
			notify : true
		},
		
		_systemToAssociate: {
			type : String
		},
		lochierarchyhref:{
			type:String
		},
		
		/**
	     * force the component fire an event when a record is clicked on location or asset  table
	     */	
		notifyDetail: {
			type: Boolean,
			value: false
		},

		/**
	     * Flag for children data loaded or not
	     */	
		childrenDataLoaded: {
			type: Object,
			value: false,
			notify: true
		},
		
		/**
	     * Flag for children assets data loaded or not
	     */	
		childAssetsDataLoaded: {
			type: Object,
			value: false,
			notify: true
		},
		contextSubscriptions: { 
            value: ['inner-app-saved'] 
		} ,
	},
	
	ready: function() {
		var self = this;
		//cancel edit on click of side menu
		$j('body').on('click', 'div.sideMenu', function(e){
			$j(self).find('maximo-tile').each(function(index, item){
				self._cancelTileEdit(e, item);
			});
		});
		
		//cancel editing other tiles on focus of any tile
		$j(self).on('focusin', 'maximo-tile document.body', function(e){
			$j(self).find('maximo-tile').not(this).each(function(index, item){
				self._cancelTileEdit(e, item);
			});
			self._hideLookup();
		});
		
		//prevent edit record on Enter keypress by detaching keypress event on maximo-text inside maximo-badge-tile
		$j(self).off( 'keypress', 'maximo-badge-tile maximo-text');
	},
	
	attached: function() {  		
		this.siteid = ($M.getMaxauth().whoami.defaultSite) ? $M.getMaxauth().whoami.defaultSite : '';
		var descLabel = 'description';
		
		this.previewAssets = [{
			'attribute' : 'assetnum',
			'title' : $M.localize('uitext', 'mxapioperloc', 'id'),
			'sortable' : true,
			'hyperlink' : true,
			'listener' : 'child-asset-detail',
			'flexGrow': '1'
		}, {
			'attribute' : 'description',
			'title' : $M.localize('uitext', 'mxapioperloc', descLabel),
			'sortable' : true,
			'flexGrow': '2'
		}, {
			'attribute' : 'status_description',
			'sortattribute' : 'status',
			'title' : $M.localize('uitext', 'mxapioperloc', 'status'),
			'sortable' : true,
			'flexGrow': '1',
			'conditionalformatting':'{"attribute":"status","values": {"ACTIVE": "color:'+this.getComputedStyleValue('--Primary-green50')+';font-weight:bold"}}'
		}];
		
		this.childrenColumns = [{
			'attribute' : 'location',
			'title' : $M.localize('uitext', 'mxapioperloc', 'id'),
			'sortable' : true,
			'hyperlink' : true,
			'listener' : 'child-location-detail',
			'flexGrow': '1'
		}, {
			'attribute' : 'description',
			'title' : $M.localize('uitext', 'mxapioperloc', descLabel),	
			'sortable' : true,
			'flexGrow': '2'
		}, {
			'attribute' : 'status_description',
			'sortattribute' : 'status',
			'title' : $M.localize('uitext', 'mxapioperloc', 'status'),	
			'sortable' : true,
			'flexGrow': '1',
			'conditionalformatting':'{"attribute":"status","values": {"ACTIVE": "color:'+this.getComputedStyleValue('--Primary-green50')+';font-weight:bold"}}'
		}];
		
		$j(this).find('maximo-tile').each(function(){
			var maximoTile = this;
			if(maximoTile.childTile && maximoTile.childTile.nodeName === 'MAXIMO-IMAGE-TILE'){
				$j(maximoTile).children().first().css('z-index', '4');
			}
		});
		if($M.isMobile() && !$M.workScape.hideFooter && $M.workScape.show){
			$j(this).closest('div.absolutePage').css({'height':'calc(100% - 80px)','margin-bottom':'50px'});
			 
		}
	},
	
	/**
	 * apply any edits made to the description
	 */
	_applyEdits : function(e) {
		if(!this.editable){
			return;
		}
		if (this.$.common_detail_location_textDescription.value.trim().length === 0) {
			this.$.common_detail_location_textDescription.value = this.locDescription;
			return false;
		}
		var params = {
			'description' : this.$.common_detail_location_textDescription.value
		};
		var locationDesc = this.$.common_detail_location_textDescription.value;
		//Update the description if there is a change
		if(locationDesc && this.locDescription && locationDesc.trim() !== this.locDescription.trim()) {
			this._updateLocation(params, this.collection);
		}
		
	},
	_updateLocation : function(dataToUpdate, collectionToUpdate) {
		var data = {};
		Object.keys(dataToUpdate).forEach(function(key) {
			data[key] = dataToUpdate[key];
		});
		var responseProperties = 'description,status';
		$M.toggleWait(true);
		this.locDescription = data.description;
		this.$.common_detail_location_locresource.resourceUri = this.record.href;
		this.$.common_detail_location_locresource.updateRecord(data, responseProperties, true).then(
				function(result) {
					collectionToUpdate.refreshRecords().then(function(){
						$M.toggleWait(false);
					});
				}, function(error) {
					$M.showResponseError(error);
					collectionToUpdate.refreshRecords().then(function(){
						$M.toggleWait(false);
					});
				});
	},

	_editRcord : function(e) {
		switch (e.detail.dataset.scopeId) {
	        case 'common_detail_location_statusTile':
	    		this._updateLocationStatus(e, this.collection);
	            break;
	            
	        case 'common_detail_location_locTypeTile':    	
	        	this._updateLocType(e, this.collection);
	            break;	
	            
	       case 'common_detail_location_parentLocationTile':
	            this._updateParent(e.target, this.$.common_detail_location_curretlocation);
	            break;
	      }		 
		
	},
	
	/**
		* Change location type and reflect the change to the user on the maximo-badge-tile
		*/
	  _updateLocType : function (e, collectionToRefresh) {
			var self = this;				
			var data = {};
			var targetElement = e.target;
			data = {
    			type : this.record.type,
    			siteid : this.record.siteid,
    			orgid:this.record.orgid
	    	};
				
			var responseProperties =collectionToRefresh.attributeNames;
			
			collectionToRefresh.resourceUri = self.record.href;
			
			$M.toggleWait(true);
			this.$.common_detail_location_locresource.updateRecord(data, responseProperties).then(function(result) {
			  	collectionToRefresh.refreshRecords().then(function(){
					if(result){						
						self._getAssetClause(result.siteid,result.type_maxvalue);
						self._refreshAssetTile();					 
					}
					$M.toggleWait(false);
				});  			 	
			}, function(error) {
				self._setPreEditValues(targetElement);
				$M.showResponseError(error);	
				$M.toggleWait(false);
			});
		},
		
  /**
	* Change the status on the location details and reflect the change to the user on the maximo-badge-tile
	*/
	_updateLocationStatus : function (e, collectionToRefresh) {
		var self = this;				
		var data = {};
		var targetElement = e.target;
		
    	data = {
			status : this.record.status,
			rolltoallchildren: this.checkboxchecked
		};

		var responseProperties = collectionToRefresh.attributeNames;
		
		collectionToRefresh.resourceUri = self.record.href;
		
		//call changestatus action
		$M.toggleWait(true);
		this.$.common_detail_location_locresource.updateAction('virtual:LOCCHANGESTATUS', data, responseProperties).then(function(result) {
			if(result.response){
				self.record=result.response;
				self._refreshChildrenTile();
				self._refreshAssetTile();
			}
				$M.toggleWait(false);
		}, function(error) {
			self._setPreEditValues(targetElement);
			$M.showResponseError(error);
		});
	},

	_getStatusIcon : function(status) {

		if (status) {
			var data = this.getStyle(status);
			return data.icon;
		} else {
			return '';
		}
	},
	_getClassForIcon : function(status) {

		if (status) {
			var data = this.getStyle(status);
			return this.getComputedStyleValue(data.color);
		} else {
			return '';
		}
	},
	
	getStyle : function(status) {
		var data = {};
		Object.keys(status).forEach(function(key) {
			var keyArray = key.split('^');
			var valueArry = status[key].split('^');
			keyArray.forEach(function(item, index) {
				data[item] = valueArry[index];
			});
		});
		return data;
	},
	
	/**
	* Change the parent of a location and display it on parent location tile
	*/
	_updateParent : function(e, collectionToRefresh)
	 {
		var _lochierarchy = [];
		var self = this;
		var newparent = '';
		var targetElement = e;
		
        var responseProperties = collectionToRefresh.attributeNames;
                
        $M.toggleWait(true);
        
        var object = {};
        	
        var systemid = '';
        var parent='';
        if(this._systemToAssociate){
        		systemid = this._systemToAssociate;    	
        		
        	} else {        	
        		systemid = this.selectedSystem;   
        	}
        		
        if(this._alreadyAtSystem(systemid)){
        	object = [{
    		    'newparent' : this.parentLocation,     			
     			'parent':this.record.parent,
     			'href' : self.lochierHref,
     			'_action' : 'Change'	 
        	}];
        	self.$.common_detail_location_locresource.resourceUri = this.record.href  + '/lochierloconly'; 
        	parent=this.parentLocation;
        } else {
        	
        /*	if(targetElement.parentSystemid){        		 
        		object.systemid = targetElement.parentSystemid;
        		object.parent = targetElement.value;
        		parent=targetElement.value;
        	} else {*/        		
        		object.systemid = systemid;
        		object.parent = targetElement.location;
        		parent=targetElement.location;
        	//}
        	self.$.common_detail_location_locresource.resourceUri = this.record.href;
        	
        }
         
        self.$.common_detail_location_locresource.updateRecord(object, responseProperties, true).then(
			function(result) { 			
				collectionToRefresh.refreshRecords().then(function(){
					/*if(result.lochierarchy && result.lochierarchy.length > 1 ){
						var curindex=0;
						var deleteindex;
						
					 	if(result.lochierarchy[0].parent==targetElement.parentLocation){
							curindex = 0;
							deleteindex = 1;
							}
						else {
							curindex = 1;
							deleteindex = 0;
						}
						self.parentLocation = '';
						self.parentDescription = '';
						self.parentImage = '';
						self.lochierLocalref = result.lochierarchy[curindex].localref;
						self.lochierHref = result.lochierarchy[curindex].href;
						self.record.parent = result.lochierarchy[curindex].parent;						
						self.parentLocation = result.lochierarchy[curindex].parent;
						self.parentDescription = targetElement.parentDescription;
						self.parentImage = targetElement.parentImage;
						self.originalLochierLocalref = self.lochierLocalref;
						self.originalLochierHref = self.lochierHref;
						self.originalParentRecord = self.record.parent;						
						self.originalParentLocation = self.parentLocation;
						self.originalParentDescription = self.parentDescription;
						self.originalParentImage = self.parentImage;
					
						$j(self).find('#location_createLocationDetail_systemiddiv')[0].hidden =  false;
						self._refreshChildrenTile();		
						self._populateSystem(result);
						self._getParentClause(false);
						self.$.systemSelect.hidden = false;
						 
						self.$.common_detail_location_locresource.deleteRecord(result.lochierarchy[deleteindex].localref).then(
								function(result) {
									collectionToRefresh.refreshRecords().then(function(){
										$M.toggleWait(false);
									});									
								}, function(error) {
									$M.toggleWait(false);
									$M.showResponseError(error);
								});	
						 
					}
					else*/ if(result.lochierarchy && result.lochierarchy.length > 0 ) {
						var hierarchiindex=0;
						self.parentLocation = '';
						//self.parentDescription = '';
						self.parentImage = '';
						hierarchiindex=self._getUpdatedHierarchi(result,systemid,parent);
						self.lochierLocalref = result.lochierarchy[hierarchiindex].localref;
						self.lochierHref = result.lochierarchy[hierarchiindex].href;
						self.record.parent = result.lochierarchy[hierarchiindex].parent;					
						self.record.systemid=result.lochierarchy[hierarchiindex].systemid;
						self.record.lochierarchy=result.lochierarchy;
						self.record.locsystem=result.locsystem;
						//self.selectedSystem=result.lochierarchy[hierarchiindex].systemid;
						self.parentLocation = result.lochierarchy[hierarchiindex].parent;
						self.parentDescription = result.lochierarchy[hierarchiindex].parent_location.description;
						self.parentImage = result.location_parent_sys._imagelibref;
						/*self.parentDescription = targetElement.parentDescription;
						self.parentImage = targetElement.parentImage;*/
						$j(self).find('#location_createLocationDetail_systemiddiv')[0].hidden =  false;
			
						self.originalLochierLocalref = self.lochierLocalref;
						self.originalLochierHref = self.lochierHref;
						self.originalParentRecord = self.record.parent;						
						self.originalParentLocation = self.parentLocation;
						self.originalParentDescription = self.parentDescription;
						self.originalParentImage = self.parentImage;
						
						self._populateSystem(result);
						self._refreshChildrenTile();
						self.$.systemSelect.hidden = false;
						self._getParentClause(false);
						self._systemToAssociate='';
					}
					$M.toggleWait(false);
				});					
			}, 
			function(error) {							
				self.lochierLocalref = self.originalLochierLocalref;
				self.lochierHref = self.originalLochierHref;
				self.record.parent = self.originalParentRecord;						
				self.parentLocation = self.originalParentLocation;
				self.parentDescription= self.originalParentDescription;
				self.parentImage = self.originalParentImage;
				$M.showResponseError(error);
			});  
    	    
	 },
		 	 
	 /**
	 * Get Location clause for parent Lookup
	 */
	 _getParentClause : function(populateSystem){
		 if(this.record){
			 this.$.common_detail_location_locationDetailImage.$.myRecord.resourceUri = this.record.href+"?responseos=MXAPIOPERLOC";
		}
	/*	 if(populateSystem.lochierarchy){
			 this._populateSystem(this.record); 
		}*/
		 if(this.record && this.record.lochierarchy){
		 this.lochierarchyhref 	=this.record.lochierarchy[0].localref;
		 }
		var locationClause = 'responseos=MXAPIOPERLOC';
		locationClause += "&oslc.where=location!=%22"+this.record.location+"%22 and lochierarchy{systemid=%22"+this.$.systemSelect.$.select.value+"%22} and%20siteid=%22"+this.record.siteid +"%22";		
		locationClause += " and status!=%22INACTIVE%22 and status!=%22SEALED%22 and status!=%22MISSING%22 and status!=%22BROKEN%22 and status!=%22DECOMMISSIONED%22";
		
		if(this.record.parent !== undefined && this.record.parent.length > 0){
			locationClause += "and lochierarchy{location!=%22"+this.record.parent+"%22}";
		} 
		this.parentClause = locationClause;	
		
		var childClause= 'responseos=MXAPIOPERLOC';
		childClause += "&oslc.where=location!=%22"+this.record.location+"%22";		
		childClause += " and status!=%22INACTIVE%22 and status!=%22SEALED%22 and status!=%22MISSING%22 and status!=%22BROKEN%22 and status!=%22DECOMMISSIONED%22";
		
		this.childrenClause = childClause;
		
		/*	
		if(this.record.locancestor && this.record.locancestor.length>0){
			var i;
			for(i=0;i<this.record.locancestor.length;i++){
				locationClause += "and lochierarchy{location!=%22"+this.record.locancestor[i].ancestor+"%22}";
			}
		}*/	
		
	
		
		
		
		
		if(this.record.systemid && ! this.record.parent){
			this.$.common_detail_location_locationtile_topLevel.hidden = false;
			this.$.common_detail_location_locationtile.hidden = true;
			$j(this.$.common_detail_location_locationtile_topLevel).find('p')[0].innerHTML = HTMLEncoder.decode($M.localize('uitext', 'mxapioperloc', 'toplevelparent'))
		} else{
			this.$.common_detail_location_locationtile_topLevel.hidden = true;
			this.$.common_detail_location_locationtile.hidden = false;
		}
			
		this._editableLocationFields();
			
	},
	
	 /**
	 * Get asset clause for asset table tile Lookup
	 */
	_getAssetClause : function(locationsite,type){
		var assetClause = '';
		if(locationsite !== undefined && locationsite.length > 0){
			if(type==='COURIER' || type==='LABOR'){
				assetClause = "oslc.where=siteid=%22"+locationsite+"%22%20and%20itemnum!%3D%22*%22&responseos=MXAPIASSET";
			}else{ 
				assetClause = "savedQuery=lookupAsset&sqp:siteid="+locationsite+"&oslc.where=siteid=%22"+locationsite+"%22&responseos=MXAPIASSET";
			}
		} 
		this.assetClause=assetClause;		
    },
	
	/**
	 * Set status checkbox attributes
	 */	
	_setStatusCheckbox: function(targetElement){
		var checkbox = $j(targetElement).find('maximo-checkbox');
		if(this.record.status_maxvalue === 'DECOMMISSIONED'){
     		this.checkboxchecked = checkbox[0].checked = true;
     		checkbox.find('.switch, .checkbox').hide();
     		targetElement.checkboxlabel = $M.localize('uitext', 'mxapioperloc', 'decommissionedlabel');	 	     		
 		} else {
 			this.checkboxchecked = checkbox[0].checked = false;
     		checkbox.find('.switch, .checkbox').show();  
     		targetElement.checkboxlabel = $M.localize('uitext', 'mxapioperloc', 'statuscheckboxlabel');	 	     		
 		}		
	},
	
    /**
	 * Update record for selected assets from asset table tile lookup
	 * fired on click of Done button
	 */
	_returnFromLookup: function(e){		 
		 var self=this;
		 self.$.common_detail_location_locresource.resourceUri = self.record.href; 		 
		 
		 var parent	= this.record.location;	
		 var childKey = '', localref='';	
		 var maximoTile = $j(e.target).closest('maximo-tile');
		 var params = {};
		 
		 if(e.target.dataset.scopeId === 'common_detail_location_assetLocation' ) {
			 self._updateAssetLocation(e);
			 
		 } else if(e.detail.identifier && e.detail.identifier ==='common_detail_location_childLocations') {			 
			 $M.toggleWait(true);			 
			 var items = e.detail.records;           			 
			 var dataToUpdate = [];
			 var self = this;		 
			 items.forEach(function(item) {				 
				 localref = item.href;	
				 var othersystem=false;
				   
				 if(item.locsystem && item.locsystem.length>0){
						item.locsystem.forEach(function(member){				
						 	if(member.systemid===self.selectedSystem){
						 		othersystem=true;
						 	}
						});	
					}
				 
	           	 if(item.lochierarchy && item.lochierarchy.length > 0 && othersystem) {
	           		 
	           		 childKey = item.lochierarchy[0].href;
	           		params = {
	   	                 'href':localref,
	   	                 'lochierarchy': [
	   	                    {
	   	                        'newparent': parent,
	   	                        'href': childKey,
	   	                        '_action': 'Change'
	   	                    }
	   	                 ]
	   	             };	
	           	 }	else { 
	           		params = {
	           				'href':localref,
	           				'systemid': self.selectedSystem,
	                		'parent': self.record.location
	   	             };	
	            	 }           	 
	        	 othersystem=false;   
	             
	         	 dataToUpdate.push(params);	           	
	           
			 });
			 
			 self.$.common_detail_location_childrenCollection.createRecordBulk(dataToUpdate, undefined, true);  
		   
	     } else if(e.detail.identifier ==='common_detail_location_locAddressTile') {
			  params = {			
			     saddresscode : e.detail.records[0].addresscode,
                 href : e.detail.records[0].href
			  };
			  $M.toggleWait(true);
			  self.$.common_detail_location_locresource.updateRecord(params,self.$.common_detail_location_curretlocation.attributeNames,true).then(
					function(result) {						
						self.$.common_detail_location_locAddressTile.value=self._setAddress(result);
						$M.toggleWait(false);
						self.$.common_detail_location_locAddressTile._applyEdit(e);						
					}, function(error) {
						$M.toggleWait(false);								
						$M.showResponseError(error);
					});			
	     } else if(e.detail.identifier && e.detail.identifier ==='common_detail_location_locationsystem') {
	    	 
	    	 var rec = e.detail.records[0];
	    	 
	    	 if(rec.lochierarchy){
	    		 this._systemToAssociate = rec.systemid;
	    		 var systemLocations = "responseos=MXAPIOPERLOC&oslc.where=lochierarchy{systemid=%22"+rec.systemid+"%22} and%20siteid=%22"+this.record.siteid+"%22";
	    		 	systemLocations += " and status!=%22INACTIVE%22 and status!=%22SEALED%22 and status!=%22MISSING%22 and status!=%22BROKEN%22 and status!=%22DECOMMISSIONED%22";
	    		 if(this.record.locancestor){
	    			 this.record.locancestor.forEach(function(item) {
		    			 if(item.systemid == rec.systemid){
		    				 systemLocations = systemLocations + " and location!=%22"+item.ancestor+"%22";
		    			 }
		 				
		 			});
	    		 }
	    		 
	    		 $M.showLookup('listSytemlocation', this,'common_detail_location_parentLocationTile_system', "/oslc/os/mxapioperloc/zombie", null, systemLocations);
	    	 }else{
	    		 this._systemToAssociate = rec.systemid;
	    		 $M.showDialog(this, 'assigTopLevel', null, 'maximo-location-tolevel-dialog', false, null, {'datasetLocationRecord':rec , 'MsgText':$M.localize('uitext','mxapioperloc','toplevellocation') });
	    		
	    	 }
	    	 
	    	
	     }else if(e.detail.identifier && e.detail.identifier ==='common_detail_location_parentLocationTile_system') {
	    	 	this.parentDescription=e.detail.records[0].description;
	            this._updateParent(e.detail.records[0], this.$.common_detail_location_curretlocation);
	     } else if(maximoTile.length){ 
	    	 var innerTile = ''; 
	    	 innerTile = maximoTile.find('.inner-tile');
	    	 if(innerTile.length) {
	    		this.async(function(){
	    			$j(innerTile).find('input').css('width', '120px');
	    		}, 100);
	    			    	 
	    		if(innerTile[0].dataset.scopeId === 'common_detail_location_statusTile'){
	    			this._setStatusCheckbox(innerTile[0]);
	    		} else if(typeof innerTile[0]._applyEdit === 'function') {
		    		innerTile[0]._applyEdit(e);
		    	}
	    	 }
		 } 
     
	},
	
	/**
	 * Add or change the assets of a location and display it on assets on location tile
	 */	
	_updateAssetLocation : function(e) {
		var self = this;
		var locationAsset = this.record.location;
		var dataToUpdate = [];
		if (e.target.dataset.scopeId === 'common_detail_location_assetLocation' ) {
			var items = e.detail.records;
			items.forEach(function(item) {
				var params = {
					newParent : item.parent,
					newLocation : locationAsset,
					newSite : item.siteid,
					href : item.href
				};				
				dataToUpdate.push(params);				
			});
			$M.toggleWait(true);
			self.$.common_detail_location_childassetcollection.createRecordBulk(dataToUpdate,'wsmethod:moveSingleAsset',true);	
		}
	},
	
	/**
	 * Handle Records after Bulk Update for child locations and asset location tile
	 */
	_handleCreateRecordBulk: function(e) {
		
		if(e.detail[0] && e.detail[0]._responsedata && e.detail[0]._responsedata.Error){
			$M.notify(e.detail[0]._responsedata.Error.message, $M.alerts.error);
			$M.toggleWait(false);
			return;
		}
		
		if(e.target.id === 'common_detail_location_childassetcollection'){
			this._refreshAssetTile();
		} else if(e.target.id === 'common_detail_location_childrenCollection') {
			this._refreshChildrenTile();
		}
		
	},
	
	/**
	 * Delete Record from table tile
	 */
	_deleteTableRecord : function(e) {
		var self = this;		
		var item = e.detail.record;
		var confirmResourceUri = item.href;	
		var params = {};
		var responseProperties = '';
		
		if(e.target.dataset.scopeId === 'common_detail_location_assetLocation') {		//delere from asset location table tile	
				
			self.$.common_detail_location_locresource.resourceUri = confirmResourceUri;			
			params = {
				newLocation: '',
				newSite : item.siteid
			};
			responseProperties = 'assetnum,description,status';		
					
			$M.toggleWait(true);
			self.$.common_detail_location_locresource.updateAction('wsmethod:moveSingleAsset',params, responseProperties).then(
					function(result) {
						self._refreshAssetTile();					
					}, function(error) {
						self._refreshAssetTile();
						$M.showResponseError(error);										
					});
		} else if(e.target.dataset.scopeId === 'common_detail_location_childLocations') {  //delere from child locations table tile
			
			var childKey = '';
			self.$.common_detail_location_locresource.resourceUri = confirmResourceUri+'/lochierloconly'; 
			
			if(item.lochierarchy && item.lochierarchy.length > 0) {	           		 
           		 childKey = item.lochierarchy[0].href;
           	 }	 
			
			$M.toggleWait(true);
			
			var recordToDelete = '';
			var self = this;
			
			item.lochierarchy.forEach(function(hier) {	
				if(hier.systemid == self.$.systemSelect.$.select.value && item.location == hier.location){
					recordToDelete = hier.localref;
				}
			});
			
			self.$.common_detail_location_locresource.deleteRecord(recordToDelete).then(
					function(result) {
						self.childrenDataLoaded = false;
						self.$.common_detail_location_childrenCollection.refreshRecords().then(function(){
							self.childrenDataLoaded = true;
							$M.toggleWait(false);
						});									
					}, function(error) {
						self.childrenDataLoaded = true;
						$M.toggleWait(false);					
						if (error && error.request && error.request.response && error.request.response.Error) {
							if(error.request.response.Error.reasonCode==='BMXAA2711E'){
								error.request.response.Error.message=error.request.response.Error.reasonCode+' - '+$M.localize('uitext','mxapioperloc','networkchilddelmsg');
							}
							$M.notify(error.request.response.Error.message, $M.alerts.error);
						}
					});											
									
				
		}
	},
	
	/**
	 * Refresh Asset Location Tile
	 */
	_refreshAssetTile: function(){	
		if(this.record && this.record.location){
			this.additionalassetparams=[];
			this.additionalassetparams.push('savedQuery=assetList&oslc.where=location=%22'+encodeURIComponent(this.record.location)+'%22&oslc.orderBy=-assetnum');
			$M.toggleWait(true);
			var self = this;
			this.childAssetsDataLoaded = false;
			this.$.common_detail_location_childassetcollection.refreshRecords().then(
					function(result) {
						self.childAssetsDataLoaded = true;
						$M.toggleWait(false);					
					});
		};
	},
			
	
/**
	 * Refresh Children Table Tile
	 */
	_refreshChildrenTile: function() {
	   	this.additionalparams=[];
	   	if(this.$.systemSelect.$.select.value){
	   		this.additionalparams.push('oslc.where=lochierarchy{parent=%22'+encodeURIComponent(this.record.location)+'%22} and lochierarchy{systemid=%22'+encodeURIComponent(this.$.systemSelect.$.select.value)+'%22}%20and%20siteid=%22'+this.record.siteid+'%22');
	   	} else {
	   		this.additionalparams.push('oslc.where=lochierarchy{parent=%22'+encodeURIComponent(this.record.location)+'%22}%20and%20siteid=%22'+encodeURIComponent(this.record.siteid) +'%22');
	   	}
			
		$M.toggleWait(true);
		var self = this;
		this.childrenDataLoaded = false;		
	   	this.$.common_detail_location_childrenCollection.refreshRecords().then(function(records){
	   		self.childrenDataLoaded = true;
	   		$M.toggleWait(false);
	   	});		
	},
	/**
	 * Show location detail on clicking of child locations 
	 */	
	_childLocationDetail : function(e) {
		if(!e.currentTarget._editMode && !e.target.matches('button')) {
			var self = this;
			var maximoQbe = new MaximoQbe();		
			var location = '';	
			
			if(e.detail.model) {
				location = e.detail.model.__data__.record.location;			
			} else {			
				location = e.currentTarget.value;
			}	
			
			if(location){
				maximoQbe.setEquals('location', location);
				maximoQbe.setEquals('siteid', this.record.siteid);
				var qbeObject = maximoQbe.toJSON();
				this.$.common_detail_location_curretlocation.collectionUri = '';
				this.$.common_detail_location_curretlocation.queryWhere = qbeObject;
				$M.toggleWait(true);
				this.$.common_detail_location_curretlocation.refreshRecords().then(
					function(result) {
						$M.toggleWait(false);
						var currentRecord = (result.response.member && result.response.member.length) ? result.response.member[0] : '';
						if(currentRecord) {							
							document.body.scrollTop = 0;
							document.documentElement.scrollTop = 0; 
							if(self.notifyDetail){
								self.fire('show-common-location-details', {
								'record' : result.response.member[0]
								});	
							}else if($M.currentWorkscape==='supervisor' || $M.currentWorkscape==='technician' ){
								self.fire('show-location-detail', {
									'record' : currentRecord
								});
							} else {
								self.fire('display-details', {
									'record' : currentRecord,
									'collection' : self.collection
								});
							}
						}						
					});
			}
		}
	},
	
	/**
	 * Collection error
	 */
	_handleError: function (e) {
		$M.toggleWait(false);
		var error = e.detail.Error;
		if(error){
			$M.notify(error.message, $M.alerts.error);
		}
	},
	
	/**
	 * Reset previous values on error on tile save
	 */
	_setPreEditValues: function(targetElement) {
		 $M.toggleWait(false);
		 this.collection.refreshRecords();
		 if(targetElement) {
			var previousvalue = targetElement._preEditValues.value;
			targetElement.value = previousvalue;
			
			var previousImage = targetElement._preEditValues.image;
			targetElement.image = previousImage;
			
			var previousLabel = targetElement._preEditValues.label;
			if(previousLabel!==undefined && previousLabel!==''){
			targetElement.label = previousLabel;
			}
			
			var self = this;
			this.$.common_detail_location_curretlocation.collectionUri = this.record.href;
			this.$.common_detail_location_curretlocation.refreshRecords().then(function(result){					
				if(result.response) {
					self.record = result.response;
					if(targetElement.dataset.scopeId === 'common_detail_location_statusTile') {
						$j(targetElement).closest('maximo-badge-tile').attr('color',self._getClassForIcon(self.record.status_domainmeta));
						$j(targetElement).closest('maximo-badge-tile').attr('icon', self._getStatusIcon(self.record.status_domainmeta)); 
						self._setStatusCheckbox(targetElement);
					}
				}
			});				
		 }		
	},
	
	/**
	 * Show Asset detail on clicking of child assets link 
	 */	
	_displayAssetDetail : function(e) {					
		var self = this;
		var maximoQbe = new MaximoQbe();
		maximoQbe.setEquals('assetnum', e.detail.model.__data__.record.assetnum);
		var qbeObject = maximoQbe.toJSON();
		this.$.common_detail_location_currentasset.queryWhere = qbeObject;
		$M.toggleWait(true);
		this.$.common_detail_location_currentasset.collectionUri = '';
		var dataHost = this.dataHost._rootDataHost; 
		this.$.common_detail_location_currentasset.refreshRecords().then(
			function(result) {	
				$M.toggleWait(false);
				if(result.response.member && result.response.member.length > 0) {		
					if(self.notifyDetail){
						self.fire('show-common-asset-details', {
							'record' : result.response.member[0]
						});	
					}else if(dataHost && (dataHost.tagName==='WORKSCAPE-SUPERVISOR' || dataHost.tagName==='MAXIMO-TECHNICIAN-MAIN')){
						self.fire('showAssetDetail', {
							'record' : result.response.member[0]
						});
					} else {
						$M.setSubRoute('admAsset/detail');
						$j(self.domHost.domHost.$.myAssetPages).find('#asset')[0].$.createassetmain.back = 'location-detail';
						self.fire('common-asset-details', {
							'record' : result.response.member[0],
							'collection' : e.target.__data__.collection
						});	
					}
				}						
			});
	},
	
	/**
	 * set additional paramater in lookup
	 */
	_setadditionalParam : function(orgid){		
		var additionalParam='responseos=MXAPISERADDRESS&oslc.where=orgid=%22'+orgid+'%22'; 
	
		return additionalParam;
	},
	/**
	 * Show Address in Address Tile  
	 */
	_setAddress :function(record) {
			var addressTile = $j(this.$.common_detail_location_locAddressTile).find('div.value, div.back.face');
			var addressval='';
			if(record.saddresscode){
				addressval= '<div class="address-line">'+HTMLEncoder.encode(record.serviceaddress.streetaddress) + '<br>';
				if(record.serviceaddress.addressline2){
				addressval += HTMLEncoder.encode(record.serviceaddress.addressline2) + '<br>'; 
				}
				addressval += HTMLEncoder.encode(record.serviceaddress.city) + ' '  + HTMLEncoder.encode(record.serviceaddress.stateprovince) + ' ' + HTMLEncoder.encode(record.serviceaddress.postalcode)+'</div>';
				
				addressTile.html(addressval);
				var strTitle = $j(addressval).html();
				$j(addressTile[0]).attr('title', strTitle.split('<br>').join('\n')); 
				$j('.address-line').css({'fontSize':'18px','textOverflow':'ellipsis', 'whiteSpace': 'nowrap', 'overflow':'hidden'});
				
			} else {
				$j(addressTile[1]).html($M.getNoDataString());
				addressTile = '';
			}			
			return addressTile;
		
		},
	 /**
	  * Show Address flyout on click of lookup add icon  
	  */	
		_showAddress : function(e){
			
			this.addressDialog = $M.showFlyout('','maximo-serviceaddress', null,{'hideFooter':true,'top':'150px','width':'100%','height':'300px','locationorgid':this.record.orgid,'sourceComponent':this});
			this.async(function(){
				$j('.underlay').css({'z-index':'1600'});
				$j('.flyout .internal').css({'justify-content':'left', 'padding-left': '30px'});
				$j('.flyout').css({'right':'0','margin':'0 auto','width': '480px', 'z-index':'1601'});
				$j('.flyout').find('input').css('padding', '3px 0px');
			}, 100);
		},
		
		/**
		 * Create a new location from a existing location
		 */
		_addLocation: function (e){
			this.dataHost._createLocationWizard();
		},
		
		_isAddLocationHidden: function(){
			return $M.currentWorkscape !== 'manageassets';
		},
		
		/**
		 * Handle Collection Refresh for asset and child location table-tile
		 */
		_handleCollectionRefresh: function(){
			window.dispatchEvent(new Event('resize'));
			var assetInLocation=$j(this.$.common_detail_location_assetLocation).find('#createLocationDetail_common_detail_location_assetLocation_table');
			if(assetInLocation.length>0){
				assetInLocation[0].actionCallback=this.getLocalFunction('_actionCallback');
			 }
		},
		/**
		 * Make the Location details fields readonly for supervisor and technician
		 */
		_editableLocationFields: function(){
			if($M.currentWorkscape==='supervisor' || $M.currentWorkscape==='technician' ){
				$j(this).find('maximo-image-editor').each(function(){ this.setAttribute('is-edit',false); });
				$j(this).find('maximo-text').each(function(){ this.setAttribute('readonly',true); });
				$j(this).find('maximo-table-tile').each(function(){this.enableDelete = false;});		
				$j(this).find('maximo-badge-tile').removeAttr('checkbox');
			}
	},
	/**
	 *set checkboxchecked value on toggle of maximo-checkbox
	 */
  	_handleCheckboxChange: function(e){
  		this.checkboxchecked = e.detail.checked;
		},
		
    /**
	  * Perform actions after lookup is closed
	  */
    _handleLookupClose : function(e) { 
		var maximoTile = $j(e.target).closest('maximo-tile');
		if(maximoTile.length){
			var innerTile = maximoTile.find('.inner-tile');
			if(innerTile.length){
				if(innerTile[0].dataset.scopeId === 'common_detail_location_statusTile'){
					return false;
				}else {
					this._closeLookup(innerTile[0]);
				}
			} 
		}
	},
	 /**
	  * Perform actions after tile edit cancel
	  */
	_handleTileEditCancel: function(e){
		//set pre-edit values after tile edit cancel
		this._setPreEditValues(e.target);
	},
	
	/**
	 * Show Systems lookup
	 */
	_showSystemLookup: function(){
		this._systemToAssociate = '';
		var systemClause = '';		
		if(this.record.locsystem && this.record.locsystem.length>0){
			this.record.locsystem.forEach(function(member){				
				systemClause += " and systemid!=%22"+encodeURIComponent(member.systemid)+"%22";
			});	
		}
		$M.showLookup('locationhierarchy', this,'common_detail_location_locationsystem', "/oslc/os/mxapilocsys?oslc.where=network=false%20and%20siteid=%22"+encodeURIComponent(this.record.siteid) +"%22"+systemClause, null, null);
	},
	
	/**
	 * Set top Level location
	 */
	_setAsTopLevel: function(e){
		this.$.common_detail_location_locationtile_topLevel.hidden = false;
		this.$.common_detail_location_locationtile.hidden = true;
		$j(this.$.common_detail_location_locationtile_topLevel).find('p')[0].innerHTML = HTMLEncoder.decode($M.localize('uitext', 'mxapioperloc', 'toplevelparent'));
		this._updateParent(e.detail, this.$.common_detail_location_curretlocation);
	},
	
	/**
	 * trigger the related system change
	 */
	_onSelectedSystemChange: function(e){
	
		var self = this;
		var newparent;
		var newlochierHref;
		var newlochierloclref;
		this.record.lochierarchy.forEach(function(item) {	
			if(item.systemid == e.target.value){
				if(item.parent){
					self.parentLocation = item.parent;
					//self.parentDescription = item.description;					
					self.parentDescription=item.parent_location.description;
					self.parentImage = item._imagelibref;
					self.$.common_detail_location_locationtile_topLevel.hidden = true;
					self.$.common_detail_location_locationtile.hidden = false;
					newparent=item.parent;
					newlochierHref=item.href;
					newlochierloclref=item.localref;
				}else{
					self.parentLocation = '';
					self.parentDescription = '';					
					self.parentImage = '';
					self.$.common_detail_location_locationtile_topLevel.hidden = false;
					self.$.common_detail_location_locationtile.hidden = true;
					$j(self.$.common_detail_location_locationtile_topLevel).find('p')[0].innerHTML = HTMLEncoder.decode($M.localize('uitext', 'mxapioperloc', 'toplevelparent'))
					
				}
			}
		});
		this.record.parent=newparent;
		this.lochierHref=newlochierHref;
		this.originalLochierLocalref = newlochierloclref;
		this._getParentClause(false);
		this._refreshChildrenTile();
	},
	
	/**
	 * Check if location is already at system
	 */
	_alreadyAtSystem: function(systemid){
		var isSystem=false;
		if(this.record.lochierarchy){
			this.record.lochierarchy.forEach(function(item) {	
				if(item.systemid === systemid){
					isSystem = true;
				}
			});
		}
		
		return isSystem;
	},
	
	/**
	 * Check if location is already at system
	 */
	_getUpdatedHierarchi: function(result,tgtsystem,tgtlocation){
		var updatedIndex=0;
		if(result.lochierarchy){		 			
		 	for(index=0; index<result.lochierarchy.length;index++){		 	
				if(result.lochierarchy[index].systemid === tgtsystem && result.lochierarchy[index].parent===tgtlocation){
					updatedIndex=index;
					break;
				} 
			} 
		}
		
		return updatedIndex;
	},
	/**
	 * Populate system selector
	 */
	_populateSystem: function(record){
		var externalValues=[];
		if(record.locsystem && record.locsystem.length>0){
			record.locsystem.forEach(function(member){
				externalValues.push(member.systemid + ':' + member.systemid);
			});
	
		}
		this.$.systemSelect.values = externalValues;
		//Populating data used on parent location tile. This is needed if a new parent is selected and an error is thrown when setting the new parent
		var i;
		this.originalParentRecord = this.record.parent;	
		this.selectedSystem=this.record.systemid;
		
		if(this.parentDescription === '' && this.record.location_parent_sys && this.record.location_parent_sys.description)
			this.parentDescription = this.record.location_parent_sys.description;
		else 
			this.originalParentDescription = this.parentDescription; 
		
		if(this.parentImage == '' && this.record.location_parent_sys && this.record.location_parent_sys._imagelibref)
			this.originalParentImage = this.record.location_parent_sys._imagelibref;
		else
			this.originalParentImage = this.parentImage; 
		
		if(this.lochierLocalref == ''){
			for(i=0;i<this.record.lochierarchy.length;i++){
				if(this.record.lochierarchy[i].parent == this.record.parent){
					this.originalLochierLocalref = this.record.lochierarchy[i].localref;
					this.originalLochierHref = this.record.lochierarchy[i].href;
					this.originalParentLocation = this.record.lochierarchy[i].parent;
					this.originalParentDescription = this.record.lochierarchy[i].parent_location.description;
				}
			}
		}
		else{
			this.originalLochierLocalref = this.lochierLocalref;
			this.originalLochierHref = this.lochierHref;
			this.originalParentLocation = this.parentLocation;
			this.originalParentDescription = this.parentDescription;
		}
	},
	/**
	 * set imagelibref in current record
	 */
	_imageAddSuccess: function(e){
		if (e.currentTarget.record && e.currentTarget.record._imagelibref){
			e.currentTarget.record._imagelibref = e.detail._imagelibref;
		}
	},
	
	/**
	 * display a record
	 * 
	 */
	showDetail : function(location){			
		this.record = location;
		this._refreshChildrenTile();
		this._refreshAssetTile();
		this._editableLocationFields();
	},
	
	/* 
     * Record Refresh when comes back from iframe classic  
     */ 
    _recordChangedFromIframe: function(e){
    	var self=this;
    	this.domHost.$.createlocationDetails.style.display ='block';
    	this.$.common_detail_location_locresource.resourceUri=this.record.href;
 		this.$.common_detail_location_locresource.loadRecord().then(function(result){
 			 e.detail.record= result;
 			 setTimeout(function() {
 				self.dataHost._displayDetails(e);
 			 }, 20);
    	});
     		
    },
	_isTypeNotOperating : function(type){
  	  	if(type!=='OPERATING'){					
  		   return true;
		}	    	
  	   return false;
     },
     
     
     /**
	 * Launch Classic to open user record
	 */
	_launchClassic : function(){
		this.domHost.$.createlocationDetails.style.display ='none';
	},
	
	/**
	 *  Function to Hide/Show actions based on rotating asset 
	 */
	_actionCallback: function(record){
		
		return !((record.asset_locations.type_maxvalue==='LABOR' || record.asset_locations.type_maxvalue==='COURIER') && record.itemnum);
		 
	 },
	 
     /**
	  * Update Service address 
	  */	 
	 updateServiceAddress: function(e){
		    var newServiceAddress = e.detail.serviceaddress;
		    var locationDetail=$M.workScape.querySelector('#location_createLocationDetail');
			var resourceUri=locationDetail.$.common_detail_location_locresource;
			var href=locationDetail.record.href;
			
			var params = {
					saddresscode : newServiceAddress.addresscode,
					href : href
			};
			resourceUri.resourceUri = href;
			resourceUri.updateRecord(params,locationDetail.collection.attributeNames,true).then(function(result) {
				locationDetail.$.common_detail_location_locAddressTile.value=locationDetail._setAddress(result);
				locationDetail.collection.refreshRecords();
				locationDetail.$.common_detail_location_locAddressTile.fire('flip-tile');

			}, function(error) {
				$M.toggleWait(false);
				$M.showResponseError(error);
			});	
	 }
});
