/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2017
 */

Polymer({

  is: 'maximo-inspector-execution',

  behaviors: [BaseComponent, Inspections, QuestionGroups, CascadeConditions, PaginateExecution],

  listeners: {
    'update-response-field'           : '_updateResponseField',
    'fetch-inspection-result'         : '_fetchInspectionResult',
    'status-changed'                  : '_inspFieldStatusChanged',
    'invalid-changed'                 : '_inspFieldInvalidChanged',
    'delete-attachment'               : '_deleteAttachment',
    'field-response-set'              : '_fieldResponseSet',
    'invalid-response'                : '_invalidResponse',
  },
  
  properties: {

    /**
     * set of inspection results
     */
    resultSet: {
      type: Array,
      value: [],
      observer: '_resultSetChanged'
    },

    /**
     * set of inspection forms
     */
    previewFormSet: {
      type: Array,
      value: [],
      observer: '_previewFormSetChanged'
    },

    /**
     * inspections extracted from resultset
     * used on template repeaters
     */
    inspections: {
      type: Array
    },
    
    /**
     * Indicates how to prepare inspections structure
     */
    flattenedStructure: {
    	type: Boolean,
    	readOnly: true,
    	value: false
    },

    /**
     * Flag to indicate preview mode
     */
    previewMode: {
      type: Boolean,
      value: false,
      readOnly: true
    },

    /**
     * available meters
     * fetched in container
     */
    availableMeters: {
      type: Object
    },

    /**
     * List of audios to execute for
     * audioguided inspections
     */
    inspectionPlaylist: {
      type: Array,
      value: function() {
        return [];
      }
    },

    /**
     * List of feedback audios
     * audioguided inspections
     */
    feedbackAudioList: {
      type: Array,
      value: function() {
        return [];
      }
    },

    /**
     * Store last audio loaded 
     * for audioguided inspection
     */
    lastAudioLoaded: {
      type: Number,
      value: 0
    },

    /**
     * Flag to mark if audio has started by user
     */
    audioGuidedStarted: {
      type: Boolean,
      value: false
    },

    /**
     * Boolean binding visibility of mx-audio controls
     */
    hideAudioControls: {
      type: Boolean,
      value: true
    },

    /**
     * Boolean to toggle visibility 
     * between form and question instructions
     */
    isFormInstruction: {
      type: Boolean,
      value: false
    },

    /**
     * Store mx-response-field to callback
     * when speech 2 text is done
     */
    callbackResponseField: {
      type: Object,
      value: null,
      readOnly: true
    },

    /**
     * Counter for replayed audios
     */
    replayedAudioCounter: {
      type: Number,
      value: function() {
        return 0;
      }
    },

    /**
     * times to repeat for invalid answer
     */
    repeatLimit: {
      type: Number,
      value: 2,
      readOnly: true
    },

    /**
     * Path to invalid responses audio
     */
    invalidResponseAudioSrc: {
      type: String,
      value: function() {
        return './audio/inspection/invalidresponse.webm';
      }
    },
    
    /**
     * Path to end of inspections audio
     */
    endOfInspectionAudioSrc: {
      type: String,
      value: function() {
        return './audio/inspection/endofinspection.webm';
      }
    },

    /**
     * DEPRECATED
     */
    stream:{
      type: Object    		
    },

  	/**
     * DEPRECATED
     */
    formattedMessages: {
      type: Array,
      value: function() {
            return [];
        }
    },

  	/**
  	 * DEPRECATED
  	 */
  	showMessages: {
  		type: Array,
  		value: function() {
  	        return [];
  	    }
  	},

    /**
     * Object received from parent
     * operated by CascadeConditions behavior
     */
    storedFormConditions: {
      type: Object,
      value: null
    },

    /**
     * Array received from parent
     * operated by CascadeConditions behavior
     */
    storedFieldDefaults: {
      type: Array,
      value: []
    },
    
    /**
     * Partial inspection fields
     */
    pagedInspections: {
    	type: Array,
    	value: []
    },

    /**
     * Request inprogress indicator
     */
    isPersistingData: {
      type: Boolean,
      value: false
    },

    /**
     * Processing data indicator
     */
    isProcessingData: {
      type: Boolean,
      value: false
    }

  },
  
  ready: function() { },

  attached: function() { },

  detached: function() {  },

  /**
   * update-response-field Listener
   */
  _updateResponseField: function(event) {

    this.set('isPersistingData', true);
    event.stopPropagation();
    var self = this;

    var detail = event.detail;
    var inspResult = {
        href: event.detail.inspresult.href
    };

    const formNum = detail.respfield.inspformnum;
    const srcFieldNum = detail.respfield.inspfieldnum;
    const resultNum = detail.respfield.resultnum
    const response = detail.respfield.txtresponse;

    var respField = detail.respfield;
    inspResult.inspfieldresult = [respField];

    var responseProperties = 'inspfieldresult';
    
    var mxResource = this.$.inspResultResource;
    mxResource.resourceUri = inspResult.href;
    mxResource.addid = true;

    var req = mxResource.updateRecord(inspResult, responseProperties, true);
	
    //Update response field in DB
    req.then(function(resp) {
      if(detail.datetimeField){
        resp.datetimeField = detail.datetimeField;
      }
      
      /* TODO
       * Find field being updated with response
       * Inject into field of questions as _outerinspfieldresult 
       */
      const outerResultField = resp.inspfieldresult.find(item => item.inspfieldnum === srcFieldNum);
      if (outerResultField) {
	        const fieldPath = self.discoverFieldPath(resultNum, srcFieldNum);
	        const objectReference = self.get(fieldPath);
	        objectReference._outerinspfieldresult = outerResultField;
      }

      if (response !== null && response !== undefined) {
        const payload = self.applyFieldConditions(formNum, srcFieldNum, resultNum, response);
        
        if (payload.length) {
        	self.notifyElementStatus(payload);
        	
        	const inspections = self.get('inspections');
        	const pageList = self.getPageList(inspections);
        	self.updatePageList(pageList);
        }
        
      }

      self.set('isProcessingData', true);
      self.set('isPersistingData', false);
      detail.successCallback(resp);

    }).catch( function(error){
    	console.error(error);
      var response = error;
      if (error && error.request) {
        if (error.request.response 
          && error.request.response.Error 
          && error.request.response.Error.message){

          response = error.request.response.Error.message;
        } else if (!error.request.isConnected) {
          
          response = $M.localize('messages', 'iface', 'CouldNotEstablishConnection');
        }
      }

      self.set('isProcessingData', true);
      self.set('isPersistingData', false);
      detail.errorCallback(response);
    });

  },

  /**
   * fetch-inspection-result listener
   * Get fresh result record to update in response field
   */
  _fetchInspectionResult: function(event) {

    event.stopPropagation();
    var self = this;

    var detail = event.detail;
    var inspResult = {
        href: event.detail.inspresult.href
    };

    var mxResource = this.$.inspResultResource;
    mxResource.resourceUri = inspResult.href;
    mxResource.addid = true;

    var req = mxResource.loadRecord();

    //Update response field in DB
    req.then(function(resp) {
      detail.successCallback(resp);
      
      const fieldResults = resp.inspfieldresult || [];
      
      fieldResults.forEach(fieldResult => {
    	  const fieldPath = self.discoverFieldPath(fieldResult.resultnum, fieldResult.inspfieldnum);
    	  const objectReference = self.get(fieldPath);
    	  objectReference._outerinspfieldresult = fieldResult;
      })
      
    }).catch( function(err){
      var response = err;
      if (typeof err === 'object') {
        var ironReqResp = err.request.response;
        response = ironReqResp.Error.message;
      }
      detail.errorCallback(response);
    });
  },
  
  /**
   * delete-attachment listener
   */
  _deleteAttachment: function(event) {

    var detail = event.detail;

    var mxResource = this.$.inspResultResource;
    mxResource.resourceUri = detail.inspresult.href;

    var req = mxResource.deleteAttachment(detail.attachment);

    //Update response field in DB
    req.then(function(resp) {
      detail.successCallback(resp);
    }).catch( function(err){
      var response = err;
      if (typeof err === 'object') {
        var ironReqResp = err.request.response;
        response = ironReqResp.Error.message;
      }
      detail.errorCallback(response);
    });
  },

  /**
   * field-response-set listener
   * give back audio control
   */
  _fieldResponseSet: function(event) {

    var detail = event.detail;

    // Play feedback audio
    if (detail.selectedOption) {
      var fbAudioIndex = this.findFeedbackAudio(detail.selectedOption);
      if (fbAudioIndex >= 0) {
        this.loadFeedbackAudio(fbAudioIndex);
      } else {
        // Position not found or audio doesnt exist
        this.playNextAudio();
      }
    } else {
      // Play next audio when feedback is complete
      this.playNextAudio();
    }
  },

  /**
   * invalid-response listener
   * Play field audio again
   */
  _invalidResponse: function(event) {

    var audioNode = this.$.mxInvalidRespAudio;
    var invalidResponseAudioSrc = this.invalidResponseAudioSrc;

    // Load audio
    if (invalidResponseAudioSrc) {
      audioNode.src = invalidResponseAudioSrc;
      audioNode.load();
    }

  },

  /**
   * status-change listener
   */
  _inspFieldStatusChanged: function(event) {
    var node = event.target;
    const resultNum = node.resultrecord.resultnum;
	const fieldNum = node.item.inspfieldnum;
    
    
	let payload = [];
	if (this.flattenedStructure) {
		payload = this.reflectChangesAncestor(resultNum, fieldNum);
	} else {
		payload = this.reflectChangesUpwards(resultNum, fieldNum);	
	}
    
    if (payload.length) {
    	
    	this.processPageChanges(payload);
    	this.notifyElementStatus(payload);
    }
    this.set('isProcessingData', false);
  },

  /**
   * invalid-change listener
   */
  _inspFieldInvalidChanged: function(event) {
    var node = event.target;
    const resultNum = node.resultrecord.resultnum;
	const fieldNum = node.item.inspfieldnum;
	
    var payload = this.reflectChangesUpwards(resultNum, fieldNum);
    if (payload.length) {
    	this.processPageChanges(payload);
    	this.notifyElementStatus(payload);
    }
  },

  /**
   * Inject property and value into item of nested structure
   * this way the change is observed by node in the template-repeater
   */
  changeItemPropertyValue: function(path, prop, value) {
	  let change = false;
	  const itemReference = this.get(path);
	  if (itemReference[prop] !== value) {
		  // SET
		  this.set(path.concat(prop), value);
		  this.updatePageItem(itemReference, prop, value);
		  change = true;
	  }
	  return change;
  },
  
  /**
   * Try applying attributes for matching fields
   */
  applyFieldConditions: function(formNum, fieldNum, resultNum, response) {
	  
	let changePayload = [];
    let fieldConditions = this.getFieldConditions(this.storedFormConditions, formNum, fieldNum, 'srcfield');

    // Set default attribute values
    for (let i = 0; i < fieldConditions.length; i++ ) {
      let fieldCondition = fieldConditions[i];
      
      const fieldPath = this.discoverFieldPath(resultNum, fieldCondition.tgtfield);
      const objectReference = this.get(fieldPath);
      
      if (objectReference) {
        // Find default attributes for field in storedFieldDefaults
        let fieldDefaults = this.fetchFieldDefaults(formNum, fieldCondition.tgtfield, this.storedFieldDefaults);
        // Re-Apply attributes values
        if (fieldDefaults) {
        	let visibilityChange = this.changeItemPropertyValue(fieldPath,'visible',fieldDefaults.visible);
        	let requiredChange = this.changeItemPropertyValue(fieldPath,'required',fieldDefaults.required);
          
        	if (visibilityChange || requiredChange) {
        		if (this.flattenedStructure) {
        			changePayload = changePayload.concat(this.reflectChangesAncestor(resultNum, fieldCondition.tgtfield));
        		} else {
        			changePayload = changePayload.concat(this.reflectChangesUpwards(resultNum, fieldCondition.tgtfield));
        		}
        		
        	}
        	changePayload = changePayload.concat(this.applyFieldConditions(formNum, fieldCondition.tgtfield, resultNum));
        }
      }
    }

    // Apply conditions
    for (let i = 0; i < fieldConditions.length; i++ ) {
      let fieldCondition = fieldConditions[i];
      
      const fieldPath = this.discoverFieldPath(resultNum, fieldCondition.tgtfield);
      const objectReference = this.get(fieldPath);
      
      if (objectReference) {
        if (this.checkConditionMatch(fieldCondition, response)) {

        	// Apply visibitily conditions
          let visibilityChange = this.changeItemPropertyValue(fieldPath,'visible',fieldCondition.visible);
          let requiredChange = this.changeItemPropertyValue(fieldPath,'required',fieldCondition.required);

          if (visibilityChange || requiredChange) {
            if (this.flattenedStructure) {
              changePayload = changePayload.concat(this.reflectChangesAncestor(resultNum, fieldCondition.tgtfield));
            } else {
              changePayload = changePayload.concat(this.reflectChangesUpwards(resultNum, fieldCondition.tgtfield));
            }
          }

          const inspfieldResult = this.getResponseObject(objectReference);
          let response = inspfieldResult ? this.extractAnswer(inspfieldResult, objectReference.fieldtype_maxvalue) : null;

          changePayload = changePayload.concat(this.applyFieldConditions(formNum, fieldCondition.tgtfield, resultNum, response));
        }
      }
    }
    return changePayload;
  },
  
  /**
   * Try to fetch response object
   * primarily
   * from field response attribute updated by `mx-response-field`
   * then from the inspfieldresult list
   */
  getResponseObject: function(fieldObject) {
    // 'response' is injected as answer updates
    if ('response' in fieldObject) {
      return fieldObject.response;
    } else if('inspfieldresult' in fieldObject) {
      return fieldObject.inspfieldresult.find(
        (i) => i.inspfieldnum === objectReference.inspfieldnum && i.resultnum === resultNum
      );
    }
    return null;
  },

  /**
   * Extract response according to type
   * to compare against conditions
   */
  extractAnswer: function(fieldResult, fieldType) {
    switch (fieldType) {
      case 'SO':
      case 'TR':
        return fieldResult.txtresponse;
      default:
        return null;
    }
  },

  /**
   * Return field full path as array
   */
  discoverFieldPath: function(resultNum, fieldNum) {
	  
	  let path;
	  
	  if (this.flattenedStructure) {
		  const fieldIndex = this.discoverFieldIndex(resultNum, fieldNum);
		  path = ['inspections', fieldIndex];
	  } else {
		  path = this.discoverFieldNestedPath(resultNum, fieldNum);
	  }
	  return path;
  },
  
  /**
   * Return field full path as array
   */
  discoverFieldIndex: function(resultNum, fieldNum) {
	  
	  const itemIndex = this.inspections.findIndex( (item) => {
		  return (this.isFieldObject(item) && item._lightresult.resultnum == resultNum && item.inspfieldnum === fieldNum);
	  });
	  
	  return itemIndex;
  },
  
  /**
   * Find field based on nested structure
   * Return path
   */
  discoverFieldNestedPath: function(resultNum, fieldNum) {
	  let path = [];
	  
	  const formIndex = this.inspections.findIndex( (resultElement) => {
		  return resultElement.inspresultnum == resultNum;
	  });
	  
	  if (formIndex >= 0) {
		  path = path.concat(['inspections', formIndex, 'questions']);		  
		  
		  const questions = this.get(path);
		  if (questions.length) {
		  
			  const fieldPath = this.discoverQuestionPath(questions, fieldNum);
			  if (fieldPath.length) {
				  path = path.concat(fieldPath);
			  }
		  }
	  }
	  
	  return path;  
  },
  
  /**
   * Find field path in nested question structure as array
   */
  discoverQuestionPath: function(itemSet, fieldNum) {
	  
	  let path = [];
	  let childPath;
	  let fieldIndex = -1;
	  for (let i = 0; i < itemSet.length; i++) {
		  
		  const item = itemSet[i];
		  
		  if (item.hasOwnProperty('children')) {
			  childPath = this.discoverQuestionPath(item.children, fieldNum);
			  if (childPath.length) {
				  path = path.concat(i, 'children', childPath);
			  }
		  } else if (item.hasOwnProperty('inspfield')) {
			  childPath = this.discoverQuestionPath(item.inspfield, fieldNum);
			  if (childPath.length) {
				  path = path.concat(i, 'inspfield', childPath);
			  }
		  } else if (item.hasOwnProperty('inspfieldnum') 
				  && item.inspfieldnum == fieldNum) {
			  path = path.concat(i);
			  break;
		  }
	  }
	  return path;
	  
  },
  
  /**
   * Carry changes to parents
   */
  reflectChangesAncestor: function(resultNum, fieldNum) {
	  
	  // Find item in list
	  let index = this.discoverFieldIndex(resultNum, fieldNum);
	  
	  let parentIndex = -1;
	  let gatheredChanges = [];
	  do {

		  // Find parent
		  parentIndex = this.findParent(index);
		  
		  // Get children
		  const children = this.findChildren(parentIndex);
		  
		  if (children.length) {
			  
			  // Evaluate according to all children
			  let change = this.processParentUpdate(parentIndex, children);
			  if (change) {
				  gatheredChanges = gatheredChanges.concat(change);
			  }
		  }
		  
		  index = parentIndex;
		  
	  } while (parentIndex >= 0);

	  // If changed call method again
	  
	  if (gatheredChanges.length) {
		  gatheredChanges = gatheredChanges.map( item => {
			  item['resultnum'] = resultNum;
			  return item;
		  });
	  }
	  
	  return gatheredChanges;
  },
  
  /**
   * Find children in linear structure
   */
  findChildren: function(parentIndex) {

	  const parent = this.inspections[parentIndex];
	  const children = [];
	  let validator, breaker;
	  
	  if (parent) {
		  if (this.isQuestionGroupObject(parent)) {
			  validator = this.isQuestionObject.bind(this)
			  breaker = function (item) {
				  this.isQuestion(item) && !this.isSameGroup(item, parent.groupid);
			  }.bind(this);
			  
		  } else if (this.isQuestionObject(parent)) {
			  validator = this.isFieldObject.bind(this);
			  breaker = function(item) {
				  return !this.isFieldObject(item);
			  }.bind(this);
			  
		  } else if (this.isFormObject(parent)) {
			  //  How to know what are the direct children of form in an array
			  validator = function (parent) {
				  return this.isQuestionGroupObject(parent) || (this.isQuestionObject(parent) && !this._isGroupChild(parent))
			  }.bind(this);
			  
			  breaker = function(item) {
				  return this.isFormObject(item);
			  }.bind(this);
		  }
	  }
	  
	  if (!validator) {
		  return children;
	  }
	  
	  const next = parentIndex + 1;
	  for (let i = next; i < this.inspections.length; i++) {
		  const test = this.inspections[i];
		  
		  if (breaker && breaker(test)){
			  break;
		  }
		  
		  if (validator(test)) {
			  children.push(test);
		  }
	  }
	  return children;
  },
  
  /**
   * Find parent of index in linear structure
   */
  findParent: function(index) {
	
	  let parentIndex = -1;
	  const child = this.inspections[index];
	  let func;
	  
	  if (child) {
	  	if (this.isFormObject(child)) {
			  return parentIndex;
			  
		  } else  if (this.isFieldObject(child)) {
			  func = this.isQuestionObject.bind(this);
			  
		  } else if (this.isQuestionGroupObject(child)) {
			  func = this.isFormObject.bind(this);
			  
		  } else if (this.isQuestionObject(child)) {
			  if (this._isGroupChild(child)) {
				  func = this.isQuestionGroupObject.bind(this);
			  } else {
				  func = this.isFormObject.bind(this);
			  }
		  }
	  }
	  
	  if (!func) {
		  return parentIndex;
	  }
	  
	  for (let i = index; i >= 0; i--) {
		  const test = this.inspections[i];
		  if (func(test)) {
			  parentIndex = i;
			  break;
		  }
	  }
	  
	  return parentIndex;
	  
  },
  
  
  /**
   * Initiate update process
   * 
   * Cascate 
   */
  reflectChangesUpwards: function(resultNum, fieldNum) {
	  
	  let gatheredChanges = [];
	  
	  const fieldPath = this.discoverFieldPath(resultNum, fieldNum);
	  const questionPath = fieldPath.splice(0, (fieldPath.length - 2));
	  
	  let change = this.processQuestionUpdate(questionPath);
	  if (change) {
		  gatheredChanges = gatheredChanges.concat(change);
	  }
	  
	  let path = questionPath.slice(0, (questionPath.length - 2));
	  
	  while (Boolean(change) && path.length) {

		  change = this.processUpperLevelUpdate(path);
		  if (change) {
			  gatheredChanges = gatheredChanges.concat(change);
		  }
		  
		  // remove from path 
		  path = path.slice(0, (path.length - 2));
	  }
	  
	  if (gatheredChanges.length) {
		  gatheredChanges = gatheredChanges.map( item => {
			  item['resultnum'] = resultNum;
			  return item;
		  });
	  }
	  
	  return gatheredChanges;
	  
  },
  
  /**
   * Update upper level attributes such as:
   * - visibility
   * - completion
   * Upper level could be question group or form
   * Notify about changes
   * 
   * @param levelPath
   * @returns Boolean 
   */
  processUpperLevelUpdate: function(levelPath) {
	
	  let changePayload = null;
	  const parent = this.get(levelPath);
	  
	  let children = parent.hasOwnProperty('children') ? parent.children : parent.questions;
	  // Filter visible children
	  children = children.filter( child => Boolean(child[this._visibleProperty]) === true );
	  
	  function includeChange(property, value) {
		  let obj = changePayload ? changePayload : {
			  num: (parent.hasOwnProperty('inspquestionnum') ? parent.inspquestionnum : parent.inspformnum),
			  type: parent.level
		  };
		  
		  obj[property] = value;
		  return obj;
	  }
	  
	  /**
       * Update level item visibility according to visible children
       * Check visible status
       */
      const levelVisibleStatus = Boolean(children.length);
      const currentVisibleStatus = parent[this._visibleProperty];
      if ( currentVisibleStatus !== levelVisibleStatus ) {
    	  // SET
		  this.set(levelPath.concat(this._visibleProperty), levelVisibleStatus);
		  this.updatePageItem(parent, this._visibleProperty, levelVisibleStatus);
		  changePayload = includeChange('visible',levelVisibleStatus);
      }
	  
      /**
       * Update required attribute of level item according to children
       * Check required status
       */
	  const requiredChildren = children.filter( child => Boolean(child[this._requiredProperty]) === true );
	  const levelRequiredStatus = requiredChildren.length === children.length;
	  const currentRequiredStatus = parent[this._requiredProperty];
	  if (currentRequiredStatus !== levelRequiredStatus) {
		  // SET
		  this.set(levelPath.concat(this._requiredProperty), levelRequiredStatus);
		  this.updatePageItem(parent, this._requiredProperty, levelRequiredStatus);
		  changePayload = includeChange('required',levelRequiredStatus);
	  }
	  
      /**
       * Update complete attribute of level item according to children
       * Check complete status
       */
	  const completedChildren = children.filter( child => Boolean(child[this._completeProperty]) === true );
	  const levelCompletionStatus = completedChildren.length === children.length;
	  const currentCompletionStatus = parent[this._completeProperty];
	  if (currentCompletionStatus !== levelCompletionStatus) {
		  // SET
		  this.set(levelPath.concat(this._completeProperty), levelCompletionStatus);
		  this.updatePageItem(parent, this._completeProperty, levelCompletionStatus);
		  changePayload = includeChange('completed',levelCompletionStatus);
	  }
	  
	  return changePayload;
  },
  
  /**
   * Process update in parent based on linear structure
   */
  processParentUpdate: function(parentIndex, children) {
	  
	  const parent = this.inspections[parentIndex];
	  var responses = [];
	  var changePayload = null;
	  var totalAnswered = 0;
	  var totalRequired = 0;
	  var totalCompleteRequired = 0;
	  var totalInvalid = 0;
	  
	  let visibiltyChildAttr, completeChildAttr,
	  	requiredChildAttr, invalidChildAttr;
	  
	  if (this.isQuestionObject(parent)) {
		  visibiltyChildAttr = 'visible';
		  completeChildAttr = 'answered';
		  requiredChildAttr = 'required';
		  invalidChildAttr = 'invalid';
	  } else {
		  visibiltyChildAttr = this._visibleProperty;
		  completeChildAttr = this._completeProperty;
		  requiredChildAttr = this._requiredProperty;
		  invalidChildAttr = this._invalidProperty;
	  }
	  
	  const visibiltyAttr = this._visibleProperty;
	  const completeAttr = this._completeProperty;
	  const requiredAttr = this._requiredProperty;
	  const invalidAttr = this._invalidProperty;
	  
	  const visibleChildren = children.filter( child => Boolean(child[visibiltyChildAttr]) === true );
	  
	  var self = this;
	  function includeChange(property, value) {
		  let obj = changePayload ? changePayload : {
			  num: (self.isQuestion(parent) ? parent.inspquestionnum : parent.inspformnum),
			  type: self.identifyObject(parent)
		  };
		  obj[property] = value;
		  return obj;
	  }
	  
	  // visible / required / complete
	  
	  for (var i = 0; i < visibleChildren.length; i++) {
		  const child = visibleChildren[i];

		  if (child[completeChildAttr]) {
			  totalAnswered++;
		  }

		  if (child[requiredChildAttr]) {
			  totalRequired++;
		  }

		  if (child[invalidChildAttr]) {
			  totalInvalid++;
		  }

		  if (child[requiredChildAttr] && child[completeChildAttr]) {
			  totalCompleteRequired++;
		  }
	  }
	  
	  
	  // TODO -----------------------------
	  // Test properties being reassigned
	  
	  /**
	   * Update question visibility according to visible fields
	   * Check visible status
	   */
	  let parentVisible = Boolean(visibleChildren.length);
	  let currentParentVisibleStatus = parent[visibiltyAttr];
	  if ( currentParentVisibleStatus !== parentVisible ) {
		  // SET
		  this.set(('inspections.' + parentIndex + '.' + visibiltyAttr), parentVisible);
		  changePayload = includeChange('visible', parentVisible);
	  }
	  
	  /**
       * Update complete attribute of level item according to children
       * Check complete status
       */
	  let parentCompletion = Boolean(totalAnswered + totalInvalid === visibleChildren.length
			  || totalRequired && totalCompleteRequired === totalRequired);
	  let currentParentCompletionStatus = parent[completeAttr];
	  if ( currentParentCompletionStatus !== parentCompletion ) {
		  // SET
		  this.set(('inspections.' + parentIndex + '.' + completeAttr), parentCompletion);
		  changePayload = includeChange('completed', parentCompletion);
	  }
	  
	  /**
       * Update required attribute of level item according to children
       * Check required status
       */
	  const parentRequiredChildren = visibleChildren.filter( child => Boolean(child[requiredChildAttr]) === true );
	  const levelRequiredStatus = parentRequiredChildren.length === visibleChildren.length;
	  const currentRequiredStatus = parent[requiredAttr];
	  if (currentRequiredStatus !== levelRequiredStatus) {
		  // SET
		  this.set(('inspections.' + parentIndex + '.' + requiredAttr), levelRequiredStatus);
		  changePayload = includeChange('required', levelRequiredStatus);
	  }

	  return changePayload;
  },
  
  /**
   * Update question attributes, such as:
   * - visibility
   * - required
   * - completion
   * Notify about changes
   * 
   * @param questionPath
   * @returns Boolean 
   */
  processQuestionUpdate: function(questionPath) {
	  
	const question = this.get(questionPath);
	var responses = [];
	var changePayload = null;
	var totalAnswered = 0;
	var totalRequired = 0;
	var totalCompleteRequired = 0;
	var totalInvalid = 0;
    
	var fields = question.inspfield.filter( field => Boolean(field.visible) === true );
	
	function includeChange(property, value) {
		let obj = changePayload ? changePayload : {
			num: question.inspquestionnum,
			type: 'question'
		};		
		obj[property] = value;
		return obj;
	}
	
	for (var i = 0; i < fields.length; i++) {
		const field = fields[i];

		if (field[this._completeProperty]) {
			totalAnswered++;
		}

		if (field.required) {
			totalRequired++;
		}

		if (field.invalid) {
			totalInvalid++;
		}

		if (field.required && field[this._completeProperty]) {
			totalCompleteRequired++;
		}
	}
	
	/**
     * Update question visibility according to visible fields
     * Check visible status
     */
    let questionVisible = Boolean(fields.length);
    let currentVisibleStatus = question[this._visibleProperty];
    if ( currentVisibleStatus !== questionVisible ) {
    	// SET
    	this.set(questionPath.concat(this._visibleProperty), questionVisible);
    	this.updatePageItem(question, this._visibleProperty, questionVisible);
    	changePayload = includeChange('visible', questionVisible);
    }
    
    /**
     * Update required attribute of question according to fields
     * Check require status
     */
    let questionRequired = (totalRequired > 0) ? true : false;
    let currentRequiredStatus = question[this._requiredProperty];
    if (currentRequiredStatus !== questionRequired) {
    	// SET
    	this.set(questionPath.concat(this._requiredProperty), questionRequired);
    	this.updatePageItem(question, this._requiredProperty, questionRequired);
    	changePayload = includeChange('required', questionRequired);
    }
	
    /**
     * Update complete attribute of question according to fields
     * Check complete status
     */
    let questionCompletion = false;
    if (totalAnswered + totalInvalid === fields.length
		|| totalRequired && totalCompleteRequired === totalRequired) {
    	questionCompletion = true;
    }
    let currentCompletionStatus = question[this._completeProperty];
    if (currentCompletionStatus !== questionCompletion) {
    	// SET
    	this.set(questionPath.concat(this._completeProperty), questionCompletion);
    	this.updatePageItem(question, this._completeProperty, questionCompletion);
    	changePayload = includeChange('completed', questionCompletion);
    }
    
    return changePayload;
  },
    
  
  /**
   * Fire event to update other panels
   */
  notifyElementStatus: function(payload) {
	  this.fire('element-status-change', payload);
  },
  
  /**
   * Replicate changes into page components
   */
  processPageChanges: function(payload) {
	  for (let change of payload) {
		  const itemIndex = this.pagedInspections.findIndex((item) => {
			  if (change.type === this.identifyObject(item)) {
				  if (change.type === 'question' || change.type === 'group') {
					  return change.num === item.inspquestionnum;
				  } else if (change.type === 'form') {
					  return change.num === item.inspformnum;
				  }
			  }
			  return false;
		  });

		  if (itemIndex >= 0) {
			  const path = ['pagedInspections', itemIndex];
			  if (change.hasOwnProperty('required')) {
				  this.set(path.concat(this._requiredProperty), change.required);
			  }
			  if (change.hasOwnProperty('completed')) {
				  this.set(path.concat(this._completeProperty), change.completed);
			  }	 
			  if (change.hasOwnProperty('visible')) {
				  this.set(path.concat(this._visibleProperty), change.visible);
			  }
		  }
	  }
  },
  
  /**
   * Converge structure creation based on flag
   */
  buildInspectionsStructure: function(originalSet) {

	  let inspections;
	  
	  if (this.flattenedStructure) {
		  inspections = this.createInspectionRootNode(originalSet);
		  this.stampAuxProps(inspections);
		  inspections = this.flatten(inspections);
		  
	  } else {
		  
		  inspections = this._extractForms(originalSet);
		  const pageList = this.getPageList(inspections);
		  this._setPageList(pageList);
		  
	  }
	  
	  return inspections;
  },

  /**
   * Observer resultSet
   */
  _resultSetChanged: function(newResultSet, oldResultSet) {

	  let inspections = this.buildInspectionsStructure(newResultSet);
	  
	  this.set('inspections', inspections);
	  
	  let flattenedQuestions = this.getFirstPage();
	  const pages = this.mountPage(flattenedQuestions);
	  this.set('pagedInspections', pages);
	  
	  
	  this.resetScroll();

    var inspInProg = !newResultSet ? [] 
    : newResultSet.filter(function(elem){  
      return elem.status_maxvalue === status_.INPROGRESS;
    });

    if (inspInProg.length) {
      this._buildAudioLists(newResultSet);
      this.loadInspectionAudio();
    }
  },

  
  /**
   * Build list of pages merging questions of all inspections
   * Filter by visibility
   */
  getPageList: function(inspections) {
	  
	  const pageList = inspections.reduce((accum, curr) => {
		  return accum.concat(curr.questions || []);
	  },[]).filter( item => item[this._visibleProperty]);
	  
	  return pageList;
  },

  /**
   * Promise to secure time for request response 
   * and further processing
   */
   waitRequest: async function() {
    let times = 0;
    const delay = 400;
    var self = this;
    return new Promise((resolve) => {
      var tid = setTimeout(repeat, delay);
      function repeat() {
        if (!self.isPersistingData && !self.isProcessingData) {
          abortTimer();
          resolve();
        } else if ( times > 10 ) {
          abortTimer();
          resolve();
        } else {
          tid = setTimeout(repeat, delay);
          times += 1;
        }
      }
      function abortTimer() {
        clearTimeout(tid);
      }
    });
  },

  /**
   * Move to next page
   */
  _goNextPage: function(event) {
	  
    this.waitRequest().then(() => {
      this.set('pagedInspections', []);
      this.$.repTemplateForms.render();
      
      const page = this.getNextPage();
      const mountedPage = this.mountPage(page);
      this.set('pagedInspections', mountedPage);
      
      this.resetScroll();
    }).catch(() => {});
  },
  
  /**
   * Move to previous page
   */
  _goPreviousPage: function(event) {
	  
    this.waitRequest().then(() => {
      this.set('pagedInspections', []);
      this.$.repTemplateForms.render();
      
      const page = this.getPreviousPage();
      const mountedPage = this.mountPage(page);
      this.set('pagedInspections', mountedPage);
      
      this.resetScroll();
    }).catch(() => {});
  },
  
  /**
   * Update object attribute from page 
   */
  updatePageItem: function(item, prop, value) {
	  
	  const objectRefType = this.identifyObject(item);
	  const objectRefAttr = this.getIdAttributeByType(objectRefType);
	  let pageItemIndex = this.pagedInspections.findIndex((i) => {
		  const curObjectType = this.identifyObject(i);
		  const curObjectAttr = this.getIdAttributeByType(curObjectType);
		  return (curObjectType === objectRefType && i[curObjectAttr] === item[objectRefAttr]);
	  });
	  
	  if (pageItemIndex >= 0) {
		  this.set(['pagedInspections',pageItemIndex,prop], value);
	  }

  },

  /**
   * Flat question structure 
   * Also include inspection information
   */
  mountPage: function(question) {
	  
	  let flattenedQuestions = []
	  
	  if (question) {

		  // Find inspection
		  let inspection = this.inspections.find(item => {
			  return item.inspresultnum === question.inspresultnum;
		  });
			  
		  flattenedQuestions = this.flatten(question);
		  
		  if (inspection) {
			  inspection = this.buildLightweightResult(inspection);
			  // Add as first node in pagedInspections
			  flattenedQuestions.unshift(inspection);
		  }
	  }
	  
	  return flattenedQuestions;
  },

  /**
   * Observer
   */
  _previewFormSetChanged: function(newSet, oldSet) {
	  
    const inspections = this.buildPreviewInspectionsStructure(newSet);
    this.set('inspections', inspections);
	  
    let flattenedQuestions = this.getFirstPage();
    const pages = this.mountPage(flattenedQuestions);
    this.set('pagedInspections', pages);
    
    if (newSet && newSet.length) {
      this._setPreviewMode(true);
    }
  },
  
  /**
   * Converge creation of inspection structure for preview
   * based on flattenedStructure flag
   */
  buildPreviewInspectionsStructure: function(originalSet) {

	  let inspections;
	  
	  if (this.flattenedStructure) {
		  
		  inspections = this.createPreviewInspectionRootNode(originalSet);
		  inspections = this.flatten(inspections);
		  
	  } else {
		  inspections = this._buildPreviewForms(originalSet);
		  const pageList = this.getPageList(inspections);
		  this._setPageList(pageList);
		  
	  }
	  
	  return inspections;
  },

  /**
   * Create root node for preview 
   */
  createPreviewInspectionRootNode: function(set) {
	  var inspSet = [];
	  // Check for null or undefined
	  if (set) {
		  // Fetch questions and group those with insp result
		  for (let form of set) {

			  var inspection = {
					  _id: form._id,
					  id: form.inspectionformid,
					  href: form.href,
					  name: form.name,
					  revision: form.revision,
					  inspformnum: form.inspformnum,
					  inspresultnum: 0,
					  instruction: form.description_longdescription,
					  location: null,
					  asset: null,
					  workorder: null,
					  meters: [],
					  lightresult: null,
					  readconfirmation: form.readconfirmation,
					  inspquestion: form.inspquestion
			  };

			  // Identify object with type
			  this.setLevel(inspection);

			  inspection[this._completeProperty] = form[this._completeProperty];
			  inspection[this._visibleProperty] = form[this._visibleProperty];
			  
			  // Create hierarchical structure of form questions
			  var questions = this._flatToHierarchy(form.inspquestion);

			  // Push questions as child of form
			  inspection.questions = questions;

			  inspSet.push(inspection);
		  }
	  }
	  return inspSet;
  },

  /**
   * Prepare inspection form set with null attributes due to preview mode
   */
  _buildPreviewForms: function(set) {
    var inspSet = [];

    // Check for null or undefined
    if (set) {

      // Fetch questions and group those with insp result
      set.forEach(function(inspForm) {

        // Build datalist first layer (insp form) 
        var form = inspForm;
        var inspection = {
            _id: form._id,
            inspectionformid: form.inspectionformid,
            href: form.href,
            name: form.name,
            revision: form.revision,
            inspformnum: form.inspformnum,
            inspresultnum: 0,
            instruction: form.description_longdescription,
            location: null,
            asset: null,
            workorder: null,
            meters: []
        };

        inspection[this._completeProperty] = false;
        inspection[this._visibleProperty] = form[this._visibleProperty];

        // Create hierarchical structure of form questions
        var questions = this._flatToHierarchy(form.inspquestion);

        // Push questions as child of form
        inspection.questions = questions;

        // Push form into main datalist array
        inspSet.push(inspection);

      }, this);

    }
    return inspSet;
  },
  
  /**
   * Label object based on data hierarchy
   * Used during form extraction (_extractForms)
   */
  setLevel: function(object) {
	  
	  const levelLabel = 'level';
	  let level;
	  
	  if (object.hasOwnProperty('workorder')) {
		  level = levels_.FORM;
	  } else if (this._isGroupHeader(object)) {
		  level = levels_.GROUP;
	  } else if (this._isGroupChild(object)) {
		  level = levels_.QUESTION;
	  } else if (object.hasOwnProperty('inspfieldnum')) {
		  level = levels_.FIELD
	  }
	  
	  if (level) {
		  object[levelLabel] = level;
	  }
  },
  
  /**
   * Create root of inspections structure for flattened structure
   */
  createInspectionRootNode: function(set) {
	  
	  var inspSet = [];

	  // Check for null or undefined
	  if (set && set.length) {
	    
		  // Fetch questions and group those with insp result
		  for (let inspResult of set) {

			  // Find available meters for this INSPECTION
			  var availableMeters = this.getAvailableMeters(inspResult, this.availableMeters);
	        
			  // Build datalist first layer (insp form) 
			  var form = inspResult.inspectionform[0];
			  
			  // Get lightweight object of inspResult
			  let lightInspection = this.buildLightweightResult(inspResult);
			  
			  var inspection = {
				_id: form._id,
				id: form.inspectionformid,
				href: form.href,
				name: form.name,
				revision: form.revision,
				inspformnum: form.inspformnum,
				inspresultnum: inspResult.resultnum,
				instruction: form.description_longdescription,
				location: Object.keys(inspResult.locations).length ? inspResult.locations : null,
				asset: Object.keys(inspResult.asset).length ? inspResult.asset : null,
				workorder: Object.keys(inspResult.workorder).length ? inspResult.workorder : null,
				meters: availableMeters,
				lightresult: lightInspection,
				inspquestion: form.inspquestion,
				inspfieldsresult: inspResult.inspfieldresult || [],
				readconfirmation: form.readconfirmation
			  };

			  // Identify object with type
			  this.setLevel(inspection);
			  
			  inspection[this._completeProperty] = form[this._completeProperty];
			  inspection[this._visibleProperty] = form[this._visibleProperty];
	        
			  inspSet.push(inspection);
		  }
	  }
	  
	  return inspSet;
  },
  
  
  /**
   * Stamps auxiliary attributes to every field
   * _lightresult
   * _outerinspfieldresult
   * readconfirmation
   */
  stampAuxProps: function(set) {
	  
	  for (let form of set) {
		  let lightResult = form.lightresult;
		  
		  if (form.inspquestion) {
			  for (let question of form.inspquestion) {
		  
			  
				  if (question.inspfield) {
					  for (let field of question.inspfield) {
				  
					  
						  field.readconfirmation = form.readconfirmation || false;
					      // Find meters on available meters for this field
					      field._lightresult = lightResult;
					      
					      // Array of inspfieldresult
					      let ownFieldResult = form.inspfieldsresult.find( fieldResult => {
					    	  return fieldResult.inspfieldnum === field.inspfieldnum; 
					      });
					      
					      if (ownFieldResult){
					    	  field._outerinspfieldresult = ownFieldResult;
					      }
					      
					  }
				  }
			  }
		  }
	  }
      
  },
  
  /**
   * IMPORTANT. Inside of inspfield is being added to extra properties
   * - _lightresult: lightweight version of inspection result
   * - _outerinspfieldresult: insp field result located in inspresult > inspfieldresult list
   */
  _extractForms: function(set) {
	  	
    var inspSet = [];

    // Check for null or undefined
    if (set) {
    
      // Fetch questions and group those with insp result
      set.forEach(function(inspResult) {

        // Find available meters for this INSPECTION
        var availableMeters = this.getAvailableMeters(inspResult, this.availableMeters);
        
        // Build datalist first layer (insp form) 
        var form = inspResult.inspectionform[0];
        var inspection = {
            _id: form._id,
            inspectionformid: form.inspectionformid,
            href: form.href,
            name: form.name,
            revision: form.revision,
            inspformnum: form.inspformnum,
            inspresultnum: inspResult.resultnum,
            instruction: form.description_longdescription,
            location: Object.keys(inspResult.locations).length ? inspResult.locations : null,
            asset: Object.keys(inspResult.asset).length ? inspResult.asset : null,
            workorder: Object.keys(inspResult.workorder).length > 1 ? inspResult.workorder : null,
            meters: availableMeters
        };

        inspection[this._completeProperty] = form[this._completeProperty];
        inspection[this._visibleProperty] = form[this._visibleProperty];
        
        this.setLevel(inspection);

        // Get lightweight object of inspResult
        var lightweightResult = this.buildLightweightResult(inspResult);

        var self = this;
        
        // Inject objects in every inspfield
        if (lightweightResult && form.inspquestion) {
        	form.inspquestion.forEach(function(question){
        		
        		question.inspresultnum = inspResult.resultnum;
        		self.setLevel(question);
        	
        		if (question.inspfield) {
        			question.inspfield.forEach(function(field){
        				
        				self.setLevel(field);

        				// Inject read confirmation from inspection
        				field.readconfirmation = form.readconfirmation || false;
                
        				// Find meters on available meters for this field
        				field._lightresult = lightweightResult;
        				
        				// Include meters at field level
        				field.meters = availableMeters;

        				if (field.inspfieldresult) {

        					if (field.inspfieldresult.length === 1){
        						var innerFieldResponse = field.inspfieldresult[0];
        						var outerFieldResult = self.findOuterRespField(inspResult, innerFieldResponse._id);
        						field._outerinspfieldresult = outerFieldResult;
        					}

        				}
        			}, self);
        		}
        	});
        }

        // Create hierarchical structure of form questions
        var questions = this._flatToHierarchy(form.inspquestion);
        // Push questions as child of form
        inspection.questions = questions;
        // Push form into main datalist array
        inspSet.push(inspection);

      }, this);
    }
    return inspSet;
  },

  /**
   * Creates ligthweight object 
   * from inspection result
   */
  buildLightweightResult: function(inspResult) {
    var lightObj = {};
    for (var key in inspResult) {
      if (typeof inspResult[key] !== 'object'){
        lightObj[key] = inspResult[key];
      }
    }
    return lightObj;
  },

  closeFormInstructions: function(event) {
    //TODO
  },

  /**
   * reset execution panel
   */
  reset: function() {
    // Close instructions panel/modal
    this.closeInstructions();
    // Reset audio guided loop
    this.audioGuidedStarted = false;
    // Reset response field node
    this._setCallbackResponseField(null);
    // Close speech synthesis case open
    this.$.speech2text.close();
  },

  /**
   * Reset icon
   * Clear checkmark whenever parameter changes
   */
  iconReset: function (element) {
    return 'Maximo:Circle';
  },
  
  /**
   * Reset status controller data attribute
   */
  statusReset: function(element) {
    return false;
  },

  /**
   * Fetch question details
   * Show info in dialog
   */
  _showQuestionInfo: function(event) {
    var question = event.model.inspectionItem;

    var mxResource = this.$.inspQuestionResource;
    mxResource.resourceUri = question.localref;
    mxResource.addid = true;
    var req = mxResource.loadRecord();
    $M.toggleWait(true);

    //Fetch long description
    req.then(function(resp) {

      this.$.groupInstructionsTitle.innerHTML = question.description;
      if (resp && resp.description_longdescription) {
        this.$.groupInstructionsText.innerHTML = resp.description_longdescription;
      }

      this.isFormInstruction = false;
      this.$.genericInstructionsDialog.classList.add('show');
      $M.toggleWait(false);

    }.bind(this)).catch( function(err){
      $M.toggleWait(false);
      var response = err;
      console.log({error: err});
    });

  },

  /**
   * Fetch question details
   * Show info in dialog
   */
  _showFormInstructions: function(event) {
    var inspection = event.model.inspectionItem;

    
    this.$.formInstructionsTitle.label = inspection.name;
    this.$.formInstructionRevision.value = inspection.revision;
    this.$.formInstructionResult.value = inspection.inspresultnum;

    this.$.formInstructionAsset.hidden = true;
    if (inspection.asset && inspection.asset.assetnum) {
      this.$.formInstructionAsset.hidden = false;
      this.$.formInstructionAssetDescription.label = inspection.asset.assetnum +' '+ inspection.asset.description;
    }

    this.$.formInstructionLocation.hidden = true;
    if (inspection.location && inspection.location.location) {
      this.$.formInstructionLocation.hidden = false;
      this.$.formInstructionLocationDescription.label = inspection.location.location +' '+ inspection.location.description;
    }

    this.$.formInstructionWO.hidden = true;
    if (inspection.workorder && inspection.workorder.wonum) {
      this.$.formInstructionWO.hidden = false;
      this.$.formInstructionWODescription.label = inspection.workorder.wonum +' '+ inspection.workorder.description;
    }

    this.$.formInstructionContent.hidden = (!inspection.instruction);
    this.$.formInstructionContentText.innerHTML = inspection.instruction || '';

    this.isFormInstruction = true;
    this.$.genericInstructionsDialog.classList.add('show');
  },
  
  /**
   * Listen to back button
   * fire event to show questions panel
   */
  _showQuestionPanel: function(event) {
    event.stopPropagation();
    this.fire('show-question-panel');
  },

  /**
   * 
   */
  _dropDownInstructions: function(e) {
    var self = this;
    var question = e.model.inspectionItem;
    var height = $j('#inspectormain_executionContainer_rightPanel').scrollTop();
    $j('#executionContainer_execution_groupInstructionsTitle')[0].innerHTML = question.description;
    $j('#executionContainer_execution_groupInstructionsText')[0].innerHTML = question.description_longdescription;
    $j('#executionContainer_execution_groupInstructionsText').css({
      'overflow': 'auto',
      'height': '75%',
      'width': '98%',
      'margin-bottom': '10px'
    });
    $j('#executionContainer_execution_groupInstructions')[0].classList
    .add('float-down');
    $j('#executionContainer_execution_groupInstructions').css({
      'display': 'block',
      'margin-top': height,
      'height': '40%',
      'overflow': 'hidden'
    });

    $j('#inspectormain_executionContainer_rightPanel').on('scroll', function() {
      self.closeInstructionsEvent(e);
      $j('#inspectormain_executionContainer_rightPanel').off('scroll');
    });

  },

  /**
   * Handle event for closing info dialog
   */
  closeInstructionsEvent(event) {
    this.closeInstructions();
  },

  /**
   * Hide dialog with info about question and form
   */
  closeInstructions: function() {
    // Remove class to hide panel again 
    this.$.genericInstructionsDialog.classList.remove('show');
  },

  /**
   * Set highlight class on group level
   */
  highlightHeader: function(question) {
	  return (this.isQuestionGroupObject(question)) ? 'highlight' : '';
  },

  /**
   * scroll in the right position of the element
   */
  showElement: function(element) {

    this.waitRequest().then(() => {
      if (element.hasOwnProperty('inspquestionnum')) {
        this.showQuestion(element);
      } else {
        this.showInspection(element);
      }
    }).catch(() => {});
  },

  /**
   * Scroll or show the inspection
   */
  showInspection: function(insp) {

	  // TODO
    if (insp) {
    	const resultNum = insp.inspresultnum;
    	// Finds first page of inspection result
    	const pageIndex = this.findPageByResult(resultNum);
    	if (pageIndex >= 0) {
    		
    		if (pageIndex !== this.getCurrentPageIndex()) {
	    		this.set('pagedInspections', []);
	    		this.$.repTemplateForms.render();
	    		
	    		const page = this.getPage(pageIndex);
	    		const mountedPage = this.mountPage(page);
	    		this.set('pagedInspections', mountedPage);
	    		
	    	} else {
				
	    		this.resetScroll();				
			}
    	}
    }
  },

  /**
   * Scroll or show the question
   */
  showQuestion: function(question) {
    if (question) {
    	
    	const resultNum = question.inspresultnum;
    	const questionNum = question.inspquestionnum;
    	
    	// Find page by questionnum
    	const pageIndex = this.findPageByQuestion(resultNum, questionNum);
    	if (pageIndex >= 0){
    		
    		if (pageIndex !== this.getCurrentPageIndex()) {
    		
	    		this.set('pagedInspections', []);
	    		this.$.repTemplateForms.render();
	    		  
	    		const page = this.getPage(pageIndex);
	    		const mountedPage = this.mountPage(page);
	    		this.set('pagedInspections', mountedPage); 
    		} else {
    			
    			const questionNode = this.querySelector('div[data-questionnum="'+question.inspquestionnum+'"]');

    			let height = 55;
    			if (this._isGroupChild(question)) {
    				height += 55;
    			}
    			
    			// now account for fixed header
    			const scrolledY = questionNode.offsetTop;
    			if(scrolledY){
    				this.$.inspectorExecutionDiv.scroll({
    					top: (scrolledY - height),
    					left: 0,
    					behavior: 'smooth'
    				});
    			}
    			
    		}
    		
    	}
      
    }
  },

  /**
   * Hide Info icon if group instructions are empty.
   */
  hideInfoIcon: function(question) {
    return !(this.isQuestion(question) && question.hasld);
  },
  
  
  /**
   * Show error in a specific inspection field
   */
  revealError: function(inspFieldResult, errorMsg) {

    // Locate the mx-response-field
    var resultNum = inspFieldResult.resultnum;
    var formNum = inspFieldResult.inspformnum;
    var questionNum = inspFieldResult.inspquestionnum;
    var fieldNum = inspFieldResult.inspfieldnum;
    var respFieldnode = this.findResponseFieldNode(resultNum, formNum, questionNum, fieldNum);

    // Call method to show error
    if (respFieldnode) {
      respFieldnode.showError(errorMsg);
    }

  },

  /**
   * Find mx-response-field node
   */
  findResponseFieldNode: function(resultNum, formNum, questionNum, fieldNum) {

    var root = this.$.inspectorExecutionDiv;

    var mxRespFieldNode = root.querySelector('maximo-response-field[data-inspresultnum="'+resultNum+'"][data-inspformnum="'+formNum+'"][data-questionnum="'+questionNum+'"][data-inspfieldnum="'+fieldNum+'"]');
    return mxRespFieldNode;

  },

  /************************ Audio Guided Section Begin ***********************/

  /**
   * Load audio given the index from playlist
   */
  loadInspectionAudio: function(index) {

    var playlist = this.inspectionPlaylist;
    var audioNode = this.$.mxAudio;

    if (!playlist.length) {
      this.hideAudioControls = true;
      console.log('No inspection found with audio guided enabled.');
      return false;
    }

    this.hideAudioControls = false;
    if (!index) {
      index = 0;
    }

    if (index > playlist.length-1) {
      console.log('End of audio list');
      this.audioGuidedStarted = false;
      index = 0;
    }

    // skip if audio is type OPTION and last audio type OPTION is same
    if (playlist[index] 
      && playlist[index].type === audioTypes_.OPTIONS 
      && this.isLastAudioOptionSame(index)) {
      // Store index as audio played
      this.lastAudioLoaded = index;
      // Consider audio played and skip audio play
      this.audioEnded();

      return;
    }

    // Does cache exist
    if (playlist[index]) {

      var src;
      if ('src' in playlist[index]) {
        src = playlist[index].src;
      } else {
        var audio = playlist[index].audio;
        src = this.getAudioSource(audio);  
      }

      audioNode.src = src;
      audioNode.load();

      this.lastAudioLoaded = index;
    }

  },

  /**
   * Load feedback audio from feedback audio list
   * Play next audio in inspection playlist 
   * if feedback audio not found
   */
  loadFeedbackAudio: function(index) {

    var audioList = this.feedbackAudioList;
    var audioNode = this.$.mxFeedbackAudio;

    if (!audioList.length) {
      console.log('No feedback audio.');
      this.playNextAudio();
      return false;
    }

    if (index === undefined || index === null) {
      console.log('No index provided.');
      this.playNextAudio();
      return false;
    }

    if (index > audioList.length-1) {
      console.log('Index out of range');
      this.playNextAudio();
      return false;
    }

    // Load audio
    if (audioList[index] && audioList[index].audio) {
      var audio = audioList[index].audio;
      var src = this.getAudioSource(audio);
      audioNode.src = src;
      audioNode.load();
    }
  },

  /**
   * Find mx-response-field node
   */
  findFeedbackAudio: function(optionFieldId) {

    var audioList = this.feedbackAudioList;
    var index = -1;
    optionFieldId = parseInt(optionFieldId);

    if (!isNaN(optionFieldId)) {
      index = audioList.findIndex(function(element){ return element.id === optionFieldId; });
    }

    return index;
  },

  /**
   * Run playlist backwards finding first OPTION
   * Return true if it audio match
   */
  isLastAudioOptionSame: function(index) {

    var playlist = this.inspectionPlaylist;
    var comparisonObj = playlist[index];
    var isEqual = false;

    if (!index || index <= 0) {
      return isEqual;
    }

    for (var i = (index-1); i >= 0; i--) {
      if (playlist[i].type === audioTypes_.OPTIONS){
        if (playlist[i].audio === comparisonObj.audio) {
          isEqual = true;
        }
        break;
      }
    }

    return isEqual;
  },

  /**
   * Generate Blob URL from audio instruction
   */
  getAudioSource: function(binaryAudio) {

    try {
      var data = 'data:audio/webm;base64,' + binaryAudio;
      var binary = this.convertDataURIToBinary(data);
      var blob = new Blob([binary], {type : 'audio/webm'});
      var blobUrl = URL.createObjectURL(blob);
      return blobUrl;

    } catch(error) {
      console.error(error);
      return;
    }

  },

  /**
   * mx-audio on-ended handler
   * Call audio ended method
   */
  audioEndedHandler: function(event) {
    this.audioEnded();
  },

  /**
   * Determine whether to play next audio
   */
  audioEnded: function() {

    var lastAudio = this.inspectionPlaylist[this.lastAudioLoaded];
    var nextAudioIndex = this.lastAudioLoaded + 1;
    var nextAudio = this.inspectionPlaylist[nextAudioIndex];

    if (lastAudio.type === audioTypes_.OPTIONS) {
      console.log('Waiting for user response');
      this.requestAudioResponse(lastAudio);
      return;
    }
    
    if (lastAudio.type === audioTypes_.FIELD &&  
        (!nextAudio || nextAudio.type !== audioTypes_.OPTIONS) ) {
      console.log('Waiting for user response');
      this.requestAudioResponse(lastAudio);
      return;
    }
    
    if (this.lastAudioLoaded === (this.inspectionPlaylist.length - 1)) {
      console.log('End of audio list');
      return;
    }

    // Grab next item to play
    this.playNextAudio();

  },

  /**
   * mx-audio on-canplay handler
   * Request play the loaded audio
   */
  audioLoaded: function(event) {
    // Check control variable
    if (this.audioGuidedStarted) {
      this.startAudio();
    } else {
      // First audio loaded
      this.audioGuidedStarted = true;
    }
  },

  /**
   * mx-feedback-audio on-ended handler
   * Play next audio from playlist
   */
  fbAudioEnded: function (event) {
    // Play next audio from inspection set
    this.playNextAudio();
  },

  /**
   * mx-feedback-audio on-canplay handler
   * Play feedback audio
   */
  fbAudioLoaded: function(event) {
    var audioNode = this.$.mxFeedbackAudio;
    audioNode.play();
  },

  /**
   * mx-invalidresponse-audio on-canplay handler
   * Play invalid audio
   */
  invalidRespAudioLoaded: function(event) {
    var audioNode = this.$.mxInvalidRespAudio;
    audioNode.play();
  },

  /**
   * mx-invalidresponse-audio on-ended handler
   * Play next audio from playlist
   */
  invalidRespAudioEnded: function(event) {

    if (this.replayedAudioCounter < this.repeatLimit) {
      this.replayAudio();
    } else {
      this.playNextAudio();
    }

  },

  /**
   * mx-speech-synthesis on-speech-done handler
   * Set response into proper response field
   */
  _grabResponse: function(event) {

    var resp = event.detail;
    console.log(resp);

    // Remove colon at the end
    resp = resp.replace(/\.$/g, '');

    // Place response according to field
    if (this.callbackResponseField) {
      this.callbackResponseField.setResponse(resp);

      // Clear response field
      this._setCallbackResponseField(null);

      // Play next audio only after notified by response field 
    }
  },

  /**
   * Play next audio in queue
   */
  playNextAudio: function() {
    var nextAudioIndex = this.lastAudioLoaded + 1;

    // Grab next item to play    
    this.loadInspectionAudio(nextAudioIndex);

    this.replayedAudioCounter = 0;
  },

  /**
   * Replay last audio
   */
  replayAudio: function() {
    var lastAudioIndex = this.lastAudioLoaded;
    // Grab last played item to replay    
    this.loadInspectionAudio(lastAudioIndex);

    this.replayedAudioCounter += 1;
  },

  /**
   * Start playing loaded audio
   */
  startAudio: function() {
    var audioNode = this.$.mxAudio;
    audioNode.play();
  },

  /**
   * Stop playing audio
   */
  stopAudioGuided: function() {
    var audioNode = this.$.mxAudio;
    audioNode.pause();
  },

  /**
   * Find the respective mx-response-field
   * Request audio response
   */
  requestAudioResponse: function(audioObject) {

    var resultNum = audioObject.resultnum;
    var formNum = audioObject.formnum;
    var questionNum = audioObject.questionnum;
    var fieldNum = audioObject.fieldnum;

    // Find mx-response-field
    var responseField = this.findResponseFieldNode(resultNum, formNum, questionNum, fieldNum);
    
    // Capture audio response
    if (responseField) {
      // Scroll into the component
      responseField.scrollIntoView({block: 'start', behavior: 'smooth'});

      var keywords = responseField.getFieldOptions();
      // Capture audio
      this.captureAudio(responseField, keywords);
    }

  },

  /**
   * Set node to callback
   * Open speech to text component
   */
  captureAudio: function(responseField, keywords) {
    if (!responseField) {
      console.warn('Response field should be provided');
      return;
    }

    // Store response field
    this._setCallbackResponseField(responseField);
    // Open speech
    this.$.speech2text.open(keywords);

  },

  /**
   * Build inspections audio playlist and 
   * Feedback audio list from 
   * inspection result set
   */
  _buildAudioLists: function(set) {
    var playlist = [];
    var feedbackList = [];
    var self = this;

    if (set) {
      // Fetch questions and group those with insp result
      set.forEach(function(inspResult) {
        // Build datalist first layer (insp form) 
        var form = inspResult.inspectionform[0];
        
        if (form && form.audioguided) {

          if (form.audiocacheform.length) {
            var inspAudio = {
                type: audioTypes_.FORM,
                id: form.inspectionformid,
                resultnum: inspResult.resultnum,
                formnum: form.inspformnum,
                audio: form.audiocacheform
            };
            playlist.push(inspAudio);
          }

          var hierarchalQuestions = self._flatToHierarchy(form.inspquestion);

          hierarchalQuestions.forEach(function(question){
            if (question.audiocachequestion.length) {
              var questionAudio = {
                  type: audioTypes_.QUESTION,
                  id: question.inspquestionid,
                  resultnum: inspResult.resultnum,
                  formnum: form.inspformnum,
                  questionnum: question.inspquestionnum,
                  audio: question.audiocachequestion
              };
              playlist.push(questionAudio);
            }

            if (question.hasOwnProperty('children')) {

              question.children.forEach(function(childQuestion){

                if (childQuestion.audiocachequestion.length) {
                  var questionAudio = {
                      type: audioTypes_.QUESTION,
                      id: childQuestion.inspquestionid,
                      resultnum: inspResult.resultnum,
                      formnum: form.inspformnum,
                      questionnum: childQuestion.inspquestionnum,
                      audio: childQuestion.audiocachequestion
                  };
                  playlist.push(questionAudio);
                }

                childQuestion.inspfield.forEach(function(field){
                  if (field.audiocachefield.length) {
                    var fieldAudio = {
                        type: audioTypes_.FIELD,
                        id: field.inspfieldid,
                        resultnum: inspResult.resultnum,
                        formnum: form.inspformnum,
                        questionnum: childQuestion.inspquestionnum,
                        fieldnum: field.inspfieldnum,
                        audio: field.audiocachefield,
                    };
                    playlist.push(fieldAudio);
                  }
                  if (field.audiocacheoption && field.audiocacheoption.length) {
                    var optionsAudio = {
                        type: audioTypes_.OPTIONS,
                        id: field.inspfieldid,
                        resultnum: inspResult.resultnum,
                        formnum: form.inspformnum,
                        questionnum: childQuestion.inspquestionnum,
                        fieldnum: field.inspfieldnum,
                        audio: field.audiocacheoption
                    };
                    playlist.push(optionsAudio);
                  }

                  // Extract feedback audios
                  if (field.inspfieldoption) {
                    field.inspfieldoption.forEach(function(option){
                      if (option.audiocachefback && option.audiocachefback.length) {
                        var optionFeedbackAudio = {
                            id: option.inspfieldoptionid,
                            resultnum: inspResult.resultnum,
                            formnum: form.inspformnum,
                            questionnum: childQuestion.inspquestionnum,
                            fieldnum: field.inspfieldnum,
                            audio: option.audiocachefback,
                        };
                        feedbackList.push(optionFeedbackAudio);
                      }
                    });
                  }
                });

              });

            } else {

              question.inspfield.forEach(function(field){
                if (field.audiocachefield.length) {
                  var fieldAudio = {
                      type: audioTypes_.FIELD,
                      id: field.inspfieldid,
                      resultnum: inspResult.resultnum,
                      formnum: form.inspformnum,
                      questionnum: question.inspquestionnum,
                      fieldnum: field.inspfieldnum,
                      audio: field.audiocachefield,
                  };
                  playlist.push(fieldAudio);
                }
                if (field.audiocacheoption && field.audiocacheoption.length) {
                  var optionsAudio = {
                      type: audioTypes_.OPTIONS,
                      id: field.inspfieldid,
                      resultnum: inspResult.resultnum,
                      formnum: form.inspformnum,
                      questionnum: question.inspquestionnum,
                      fieldnum: field.inspfieldnum,
                      audio: field.audiocacheoption
                  };
                  playlist.push(optionsAudio);
                }

                // Extract feedback audios
                if (field.inspfieldoption) {
                  field.inspfieldoption.forEach(function(option){
                    if (option.audiocachefback && option.audiocachefback.length) {
                      var optionFeedbackAudio = {
                          id: option.inspfieldoptionid,
                          resultnum: inspResult.resultnum,
                          formnum: form.inspformnum,
                          questionnum: question.inspquestionnum,
                          fieldnum: field.inspfieldnum,
                          audio: option.audiocachefback
                      };
                      feedbackList.push(optionFeedbackAudio);
                    }
                  });
                }
              });

            }

          });
        }
      });

    }

    // Add End of inspection only if there's any audio
    if (playlist.length) {
      var endInspections = {
          type: audioTypes_.STATIC,
          src: this.endOfInspectionAudioSrc
      };
      playlist.push(endInspections);
    }

    this.inspectionPlaylist = playlist;
    this.feedbackAudioList = feedbackList;

  },
  
  /**
   * Decode audio data 
   */
  convertDataURIToBinary: function (dataURI) {
    var BASE64_MARKER = ';base64,';
    var base64Index = dataURI.indexOf(BASE64_MARKER) + BASE64_MARKER.length;
    var base64 = dataURI.substring(base64Index);
    var raw = window.atob(base64);
    var rawLength = raw.length;
    var array = new Uint8Array(new ArrayBuffer(rawLength));
    for(var i = 0; i < rawLength; i++) {
      array[i] = raw.charCodeAt(i);
    }
    return array;
  },

  /**
   * REVIEW to be reused
   */
  handleStream(stream) {

    var self = this;
    console.log(stream);
    // cleanup old stream if appropriate
    if (this.stream) {
      this.stream.stop();
      this.stream.removeAllListeners();
      this.stream.recognizeStream.removeAllListeners();
    }
    this.stream = stream;
    // grab the formatted messages and also handle errors and such
    stream.on('data', self.handleFormattedMessage).on('end', this.handleTranscriptEnd).on('error', this.handleError);

    // when errors occur, the end event may not propagate through the helper streams.
    // However, the recognizeStream should always fire a end and close events
    stream.recognizeStream.on('end', () => {
      if (this.error) {
        this.handleTranscriptEnd();
      }
    });
  },

  /**
   * REVIEW to be reused
   */
  handleFormattedMessage(msg) {

    this.formattedMessages = this.formattedMessages.concat(msg);
    var lastMsg = this.formattedMessages.length - 1;
    var word = '';
    if (this.formattedMessages[lastMsg].results.length){
      word = this.formattedMessages[lastMsg].results[0].alternatives[0].transcript;            
      this.showMessages=word;
      console.log(new Date().toUTCString(), 'Maixmo field response =',word); 
    }else{
      console.log(new Date().toUTCString(), 'Words had not been detected.');
    }                    
    this.newMessage = word;                   
   
  },

  /**
   * TO BE MIGRATED TO MX-SPEECH-SYNTHESIS
   */
  handleError: function(err, extra) {
    console.error(err, extra);
    if (err.name === 'UNRECOGNIZED_FORMAT') {
      err = 'Unable to determine content type from file name or header; mp3, wav, flac, ogg, opus, and webm are supported. Please choose a different file.';
    } else if (err.name === 'NotSupportedError' /* && this.audioSource === 'mic'*/) {
      err = 'This browser does not support microphone input.';
    } else if (err.message === '(\'UpsamplingNotAllowed\', 8000, 16000)') {
      err = 'Please select a narrowband voice model to transcribe 8KHz audio files.';
    } else if (err.message === 'Invalid constraint') {
      // iPod Touch does this on iOS 11 - there is a microphone, but Safari claims there isn't
      err = 'Unable to access microphone';
    }
    this.error = (err.message || err);                
  },  

  /**
   * DEPRECATED
   */
  handleTranscriptEnd() {
	  console.info('Incorrect deprecated label');
    // note: this function will be called twice on a clean end,
    // but may only be called once in the event of an error
    //  this.setState({ audioSource: null });
  },

  /**
   * DEPRECATED
   */
  cancelRecording: function(event) {
	  console.info('Incorrect deprecated label');
    console.log (new Date().toUTCString(), 'Cancel Recording');
    if (this.stream) {
      this.stream.stop();
      this.stream.removeAllListeners();
      this.stream.recognizeStream.removeAllListeners();

      var lastMsg = this.formattedMessages.length - 1;
      var finalMesg = '';

      if (lastMsg > 0){
        finalMesg = this.formattedMessages[lastMsg].results[0].alternatives[0].transcript;
        console.log(new Date().toUTCString(),'Final Message to Send =',finalMesg); 
        this.formattedMessages= [];
      }else{
        console.log(new Date().toUTCString(),'No message to Send'); 
        //TODO: Send a message voice has not been detected. 
      }    
      return finalMesg;      
    } 
  },
  
  /************************ Audio Guided Section End ***********************/

  /**
   * Check if question is a group header
   */
  isGroup: function(question) {
    if ((question.groupid && question.sequence === 0) || (question.children)) {
      return true;
    } else {
    }
    return false;
  },

  /**
   * Fetch question Info details
   * Show and Hide  info in dialog
   */
  _showQuestionInstructions: function(event) {
    var qchild = event.model.inspectionItem;
    
    var mxResource = this.$.inspQuestionResource;
    mxResource.resourceUri = qchild.localref;
    mxResource.addid = true;
    var req = mxResource.loadRecord();
    $M.toggleWait(true);

    //Fetch long description
    req.then(function(resp) {

      this.$.groupInstructionsTitle.innerHTML = qchild.description;
      if (resp && resp.description_longdescription) {
        this.$.groupInstructionsText.innerHTML = resp.description_longdescription;
      }

      this.$.formInstructions.hidden = true;
      this.$.groupInstructions.hidden = false;
      this.$.genericInstructionsDialog.classList.add('show');
      $M.toggleWait(false);

    }.bind(this)).catch( function(err){
      $M.toggleWait(false);
      var response = err;
      console.log({error: err});
    });
  },
  
  /**
   * Compute question title
   */
  _buildQuestionTitle: function(question) {
	
	  if (this._isGroupChild(question)) {
		  return this._formatSequence(question.groupseq) + '. ' + question.description 
	  } else {
		  return question.groupseq + '. ' + question.description;
	  }
  },
  
  /**
   * Reset scrollable area
   */
  resetScroll: function() {
	  this.async(function() {
		  const firstNode = this.$.inspectorExecutionScrollerDiv.firstElementChild; 
		  this.scrollToNode(firstNode);
	  });
  },
  
  /**
   * Scroll viewport to the node
   */
  scrollToNode: function(node) {
//	  this.async(function() {
//		  this.$.inspectorExecutionDiv.scrollTop = 0;
//	  });
	  if (node) {
		  node.scrollIntoView({block: 'start', behavior: 'smooth'});
	  }
  },
  
  /**
   * Reset list from nested dom-repeater
   * From inside out
   */
  resetRepeaters: function() {

    this.querySelectorAll('template[is="dom-repeat"].fieldgroup').forEach( function(elem) {
      elem.items = [];
    });
    
    this.querySelectorAll('template[is="dom-repeat"].question').forEach( function(elem) {
      elem.items = [];
    });

    this.querySelectorAll('template[is="dom-repeat"].field').forEach( function(elem) {
      elem.items = [];
    });
    
    this.querySelectorAll('template[is="dom-repeat"].questiongroup').forEach( function(elem) {
      elem.items = [];
    });

    this.querySelectorAll('template[is="dom-repeat"].form').forEach( function(elem) {
      elem.items = [];
    });

  },

  /**
   * Return string representation of boolean
   * for element dataset attribute
   * @param {Boolean} bool 
   */
  getStr: function(bool) {
    if (bool === null || bool === undefined) {
      bool = false;
    }
    return bool.toString();
  },
  
  /**
   * Compute visibility of controller
   */
  _hideController: function(list) {
	  return list.length <= 1; 
  }

});
