/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2017
 */
Polymer({
	is: 'maximo-inspector-list',
	
  	behaviors: [BaseComponent, InfiniteScroll, ResizeBehavior],
  	
  	listeners: {
        'card-selected':    'cardSelected',
        'card-deselected':  'cardDeselected',
        'long-touch':       'longTouch',
        'inspections-screen-resize': '_screenResize',
    },
  	
    properties: {
    	
    	/**
    	 * List of inspection form
    	 */
    	dataSet: {
    		type: Array,
    		observer: 'dataSetReset'
		},
		
		totalPages: {
			type: Number
		},

		pageNum: {
			type: Number
		},
    	
    	/**
    	 * 
    	 */
    	selectedTile: {
    		type: Object
    	},
    	
    	/**
    	 * Select tab name
    	 */
    	selectedTabName: {
    		type: String,
    		observer: '_filterList' 
    	},
    	
    	/**
    	 * Indicator to show/hide message
    	 */
    	hasRecord: {
    		type: Boolean,
    		value: true,
    		readOnly: true
    	},
    	
    	/**
    	 * Hold message for each tab
    	 */
    	noInspections: {
    		type: String,
    		value: function() {
    			return $M.localize('uitext','mxapiinspresult','no_inspections');
    		}
    	},
    	
		additionalparams :{
			type : Array,
			notify : true
		},
		
		/**
		 * Array with selected inspections
		 */
		selectedInspections: {
		    type: Array,
		    value: []
		},
		
		/**
		 * Computed label for multiselect button
		 */
		openSelectBtnLabel: {
		    type: String,
		    value: ''
		},

		/**
		 * Computed label for Create Form button
		 */
		createFormBtnLabel: {
		    type: String,
		    computed: '_computeCreateFormButtonLabel()'
		},
		
		/**
		 * Collection reference to perform search
		 */
		collectionSearchRef: {
		  type: Object
		},
		/**
    	 * Indicator to show/hide loading bee icon
    	 */
    	loadingFlag: {
    		type: Boolean,
    		value: false
		},
		
		/**
    	 * Indicator if it´s a small device
    	 */
    	isSmallDevice: {
    		type: Boolean,
    		value: false, 
    		observer: '_screenResize'
		},
		
    	/**
    	 * Indicator there are selected cards
    	 */
    	hasMultiselectCard: {
    		type: Boolean,
    		value: false
    	},
    	
	},
	
	observers: [
		'_computeOpenSelectedButtonLabel(isSmallDevice, selectedInspections.splices)'
	  ],
	
	attached: function() {
		this.infiniteScrollingList(this.$.inspList, 'collectionSearchRef');
	},
	
	/**
	 * long touch handler
	 */
	longTouch: function(e) {
	    var toggleStatus = e.detail.toggle;
	    this.toggleMultiselect(toggleStatus);
	},
	
	/**
	 * Listener for card selection 
	 * Adds into controller array 
	 */
    cardSelected: function(ev) {
        //add to array
        var tile = ev.detail;
        //Object to be insert
        if (tile.record !== undefined){
	        var inspResultID = tile.record.inspectionresultid;
	        if (inspResultID) {
	            if (this.selectedInspections.indexOf(inspResultID) < 0) {
	                this.push('selectedInspections', inspResultID);
	          }
	            var allCards = this.$.inspectorListTileDiv.querySelectorAll('maximo-inspector-tile');
	            
	            for (var i = 0; i < allCards.length; i++) {
	            	if($j(allCards[i]).find('maximo-checkbox').prop('checked')===false|| $j(allCards[i]).find('maximo-checkbox').prop('checked')===undefined){         		
	            		this.singleValue=false;
	            		break;
	               }
	            	else if($j(tile).find('maximo-checkbox').prop('checked')===true){
	            		this.singleValue=true;
	            		
	            	}
	            }            	
	        }
        }
        if(this.singleValue===true){
        	this.$.selecAllCb.checked=true;
        	this.$.selecAllCbM.checked=true;
        }
               
        if (this.selectedInspections.length === 1) {
            this.toggleMultiselect(true);
        }
        
    },
    
    /**
     * Listener for card deselection
     * Removes from controller array 
     */
    cardDeselected: function(ev) {
        //remove from array
        var tile = ev.detail;
        var singleCheck=$j(tile).find('maximo-checkbox').prop('checked');
        //Object to be removed
        var inspResultID = tile.record.inspectionresultid;
        var index = this.selectedInspections.indexOf(inspResultID);
        if (index >= 0) {
            this.splice('selectedInspections', index, 1);
	   if(singleCheck !==undefined && !singleCheck){
		   this.single=true;
		   this.$.selecAllCb.checked=false;
		   this.$.selecAllCbM.checked=false;
         }
            if (this.selectedInspections.length === 0) {
                this.toggleMultiselect(false);
           }
      }
  },
    
    /**
     * Toggles selection ability of each card
     * Show/Hide buttons according to state
     */
    toggleMultiselect: function(status) {
        status = Boolean(status);
        
        //toggle cards multiselect
        var allCards = this.$.inspectorListTileDiv.querySelectorAll('maximo-inspector-tile');
        for (var i = 0; i < allCards.length; i++) {
            if (allCards[i].showSelector !== status) {
                allCards[i].showSelector = status;
            }
        }

        //toggle header buttons
        this.$.createform.hidden = status;
        this.$.openForms.hidden = !status;
        
        if (this.isSmallDevice){
        	this.$.selecAllCbM.hidden = !status;
	        this.$.selecAllCbM.dataset.name='';
	        this.$.selecAllCbM.checked =false;
	        this.$.selecAllCb.hidden = true;
        } else {
	        this.$.selecAllCb.hidden = !status;
	        this.$.selecAllCb.dataset.name='';
	        this.$.selecAllCb.checked=false;
	        this.$.selecAllCbM.hidden = true;
        }
        
        this.hasMultiselectCard = status;
    },
    
    /**
     * Select all checkbox change handler
     */
    selectAllChanged: function(event) {
        var generalState = event.detail;
        //toggle cards multiselect
        var allCards = this.$.inspectorListTileDiv.querySelectorAll('maximo-inspector-tile');
        if((!this.single || generalState===true) ||(this.singleValue===true)){
        for (var i = 0; i < allCards.length; i++) {
            allCards[i].updateSelectionState(generalState);
            this.single=false;
            this.singleValue=false;
       }
    }
 },
    
    /**
     * Card set observer
     * Clear selection whenever dataset is reset
     */
    dataSetReset: function(newv, oldv) {
        this.resetMultiSelection();
    },
    
    /**
     * Computed method for property openSelectBtnLabel
     */
    _computeOpenSelectedButtonLabel: function(isSmallDevice, array) {
    	if (this.isSmallDevice){
			this.openSelectBtnLabel = $M.localize('uitext','mxapiinspresult','create');
	    } else {
	    	this.openSelectBtnLabel = $M.localize('uitext','mxapiinspresult','inspector_unscheduled_inspection');  	
	    }
       	
       	var qtdSelected = this.selectedInspections.length;
		var total = this.dataSet.length;
		
	    if (qtdSelected == 0){
	    	this._resultCounter = $M.localize('uitext', 'mxapibase', 'countresults',[total]);
	    } else {
			if (!this.dataSet) {
				total = 0;
			}else {
				
				var params = [qtdSelected, total];
				
				this._resultCounter = $M.localize('uitext','mxapiinspresult','recordsselected', [qtdSelected, total]);
								
			}
	    	
	    }   
    },
    
    /**
     * Computed method for property createFormBtnLabel
     */
    _computeCreateFormButtonLabel: function() {
    	if (this.isSmallDevice){
	    	return $M.localize('uitext','mxapiinspresult','create');
	    } else {
	    	return $M.localize('uitext','mxapiinspresult','inspector_unscheduled_inspection');  	
	    }
    
    },
    
    /**
     * Multi select button tap handler
     * Send the list of selected records
     */
    openMultiSelected: function(e) {
        this._loadExecution(this.selectedInspections);
    },

	/**
	 * Opens Create Inspection View in order to start the inspection form creation.
	 */
	createUnscheduledInspection: function(e) {
		this.fire('createUnscheduledInspection');
	},
	
	/**
	 * DataSet Refresh - setHasRecord flag true/false if records exist
	 */
	dataSetRefresh: function () {
		var total;
		if (!this.dataSet) {
			total = 0;
		}else {
			total = this.dataSet.length;
		}

		var collectionTotal = !this.collectionSearchRef ? 0 : !this.collectionSearchRef.totalCount ? 0 : this.collectionSearchRef.totalCount;
		
		if (total && total < collectionTotal) {
		  this._resultCounter = $M.localize('uitext', 'mxapiinspresult', 'of_results',[total, collectionTotal]);
		} else {
		  this._resultCounter = $M.localize('uitext', 'mxapibase', 'countresults',[total]);
		}
		
		if (total === 0){
			this.hasRecord = false;
			this._setHasRecord(false);
		}else{
			this.hasRecord = true;
			this._setHasRecord(true);
		}
	},
	
	/**
	 * Listens to tab selection
	 */
	tabTap: function(e) {
		
		var name = e.currentTarget.getAttribute('name');
		//this.selectedTabName = name;
		this.selectTab(name);
	},
	
	/**
	 * Update selectedTabName property
	 */
	selectTab: function(tabName) {

	    if (this.selectedTabName !== tabName) {
	        this.selectedTabName = tabName;
	    } else {
	      this.fire('fetch-inspresult');
	    }

	},
	
	/**
	 * Used for collection initial load
	 */
	selectDefaultTab: function() {
	    //Get first child name of inspectorListContentSelector
	    var tabs = this.$.inspectorListContentSelector.getContentChildren();
	    var tab = tabs[0];
	    //Set as selected tab
	    this.selectTab(tab.getAttribute('name'));
	},
	
	/**
     * Refresh selected tab
     */
    refreshCurrentTab: function() {
        //Get first child name of inspectorListContentSelector
        var tabName = this.selectedTabName;
        //Set as selected tab
        this.selectTab(tabName);
    },
	
	/**
	 * Triggered by Status Filtering, Maximo Content Selector component.
	 * Change message according to selected tab
	 * 
	 * @param e
	 * @returns
	 */
	_filterList : function(tabName){

		//clear search when changing tabs
		var searchBar = this.$.inspectorListSearchBar;
		if(searchBar){
		  searchBar.clear();
		}	

		var queryName;

		//find external status value
		switch(tabName){
			case 'pending':
			  queryName = 'INSPRESULTPENDING';
			  break;
			case 'inprogress':
			  queryName = 'INSPRESULTINPROGRESS';
   			  break;
			case 'completed':
			  queryName = 'INSPRESULTCOMPLETE';
			  break;
			default:
			  queryName = 'INSPRESULTALL';
		}
		
		this.noInspections = $M.localize('uitext','mxapiinspresult','no_inspections');

		this.fire('fetch-inspresult', {queryname: queryName});

		this.resetMultiSelection();

	},

	resetMultiSelection: function() {
	  this.set('selectedInspections', []);
	  this.toggleMultiselect(false);
	},

	/**
	 * Highlight new record/card once its created
	 */
	selectRecord: function (formId) {
		if(formId){
			this.$.inspectorListTileTemplate.render();			
			var nodeArray = this.$.inspectorListDataScrollerDiv.querySelectorAll('maximo-inspector-tile');
			for (var i = 0; i < nodeArray.length; ++i) {
				if (nodeArray[i].__data__.record.inspectionresultid === formId){
					nodeArray[i].highlight();
				}
			}
		}
	},

	/**
	 * Event fired when a card is tapped
	 */
	cardTap: function(ev) {
		
		// Prevent load of inspection during tap
	    if (this.hasMultiselectCard && $M.isMobile()) {
	    	const tile = ev.target;
	    	if (tile && typeof tile.toggleSelection === 'function') {
	    		ev.target.toggleSelection();
	    	}
	    	return;
	    }
	    
	    //fetch record data
	    var inspectionRecord = ev.model.inspForm;

	    // Effective tap only when multiselect is not active 
	    if (ev.target.isStacked){
	      //Fetch children
	      this._loadChildren(inspectionRecord);
	    } else {
	      //Object to be insert
	      var inspResultID = inspectionRecord.inspectionresultid;
	      this._loadExecution([inspResultID]);
	    }
	},

	/**
	 * Request page change to execution
	 */
	_loadExecution: function(idSet) {
//		var self = this;
//		var availableVals = [];
//		var filterData = [{'filtertype': 'SIMPLE', 'field': 'inspectionresultid', 'availablevalues': availableVals}];
//		$M.toggleWait(true);
//		//create filter to fetch assignment record
//		for (var i = 0; i < idSet.length; i++) {
//		    var record = {
//		            value: idSet[i],
//		            selected: true
//		    };
//		    availableVals.push(record);
//		}
////		var inspectionRecord = e.model.inspForm;
////   		var filterByInspectionResultId = {};
////   		filterByInspectionResultId.value = inspectionRecord.inspectionresultid;
////   		filterByInspectionResultId.selected = true;
////   	   	filterData[0].availablevalues.push(filterByInspectionResultId);
//   	
//		var inspresultcollection = $M.getGlobalResource('inspresultcollection');
//		inspresultcollection.filterData = filterData;
//		
//		this.domHost.$.executionContainer.$.questionlist.resetUserChanges();
//		
//		inspresultcollection.refreshRecords().then(function(inspectionRecord){
//			//var inspectionRecord = e.model.inspForm;
//			//self.fire('loadInspection', inspectionRecord.response.member[0]);
//		});
	    
	    this.fire('load-inspection-set', {'inspectionIds': idSet});

	},

	/**
	 * Fetch children of WO parent
	 */
	_loadChildren: function(inspResult) {

	  this.fire('load-insp-children', {'inspection': inspResult});

	},

	/**
     * DEPRECATED
     * Controller array observer
     */
    selectedInspectionsAddedOrRemoved: function(changeRecord) {
        if (changeRecord) {
            var newLength = this.selectedInspections.length || 0;

            //TODO length maybe be going back to 1. 
            //TODO Array length=2, removes 1, array length=1
            //TODO Avoid calling method unnecessarily
            if (newLength === 1) {
                this.toggleMultiselect(true);
            }
            if (newLength === 0) {
                this.toggleMultiselect(false);
            }
        }
    },

	/**
	 * DEPRECATED
	 * Select pending tab
	 * Clear searchbar
	 */
	reset: function() {
		//console.log('Reset filter');
		this.$.inspectorListContentSelector.select(0);
		this.selectedTabName = 'pending';
	},
	
	/**
	 * window resize listener
	 */
	_screenResize: function() {

		if (this.deviceType == 'phone'){
			
			this.isSmallDevice = true;
			
			this.$.mobileButtonArea.appendChild(this.$.inspectorListCreateButton);
			
			this.$.createform.label = $M.localize('uitext','mxapiinspresult','create');
			this.$.inspectorListCreateButton.float = "right";
			
			this.$.titleArea.style.width = "70%";
			this.$.mobileButtonArea.width = "30%";

			this.$.selecAllCb.hidden = true;
			if (this.hasMultiselectCard){
				this.$.selecAllCbM.hidden = false;
			} else {
				this.$.selecAllCbM.hidden = true;
			}

            this.$.inspectorListSearchBarDiv.classList.add("search-bar-mobile");            
			
		} else {
			
			this.isSmallDevice = false;
			
			this.$.desktopButtonArea.appendChild(this.$.inspectorListCreateButton);
			
			this.$.createform.label = $M.localize('uitext','mxapiinspresult','inspector_unscheduled_inspection');
			
			this.$.titleArea.style.width = "100%";
			
			this.$.selecAllCbM.hidden = true;
			if (this.hasMultiselectCard){
				this.$.selecAllCb.hidden = false;
			} else {
				this.$.selecAllCb.hidden = true;
			};
			
			this.$.inspectorListSearchBarDiv.classList.remove("search-bar-mobile");
						
		};
	},
	
});
