/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2017
 */
Polymer({

  is: 'maximo-inspector-main',

  behaviors: [BaseComponent, Inspections],

  listeners: {
    'changePage'                : '_changePage',
    'createPendingResult'       : '_createPendingResult',
    'createUnscheduledInspection' : '_createUnscheduledInspection',
    'goback'                    : '_goBack',
    'processFormSelection'      : '_processFormSelection',
    'refreshFormCollection'     : '_refreshFormCollection',
    'startInspection'           : '_openInspections',
    'reloadInspection'          : '_reloadInspection',
    'loadInspection'            : '_loadInspectionExecution',
    'load-inspection-set'       : '_loadInspectionSetHandler',
    'updateResponse'            : '_updateResponse',
    'exitCompletion'            : '_exitCompletion',
    'startInspections'          : '_startInspections',
    'changeInspectionStatus'    : '_changeInspectionStatusEvent',
    'updateAssetLocMeters'      : '_updateAssetLocMeters',
    'fetchAvaialbleMeters'      : '_fetchAvailableMeters',
    'openWOCreation'            : '_openWOCreation',
    'complete-inspections'      : '_completeInspections',
    'load-insp-children'        : '_loadInspectionChildrenHandler',
    'fetch-inspresult'          : '_fetchInspectionResult',

    'completeInspection' : '_completeInspection',
      
     /*
    'validate-meters'           : '_validateInspectionsMeters',
    'record-updatewsmethod:updateAssetLocMeter' : '_handleMeterUpdatedSuccess',
    'record-updatewsmethod:updateAssetLocMeter-error' : '_handleMeterUpdatedFail',
      */
    
  },

  /**
   * DEPRECATED
   * 'start-batch'               : '_startBatch',
   * 'record-updatewsmethod:changeResultStatus' : '_handleRecordUpdatedSuccess',
   * 'record-updatewsmethod:changeResultStatus-error' : '_handleRecordUpdatedFail',
   */

  observers: ['meterCollectionsReady(assetReady, locationReady)'],

  properties: {

    /**
     * component selected in iron-pages 
     */
    selectedPage: {
      type: String,
      value: 'inspResultList'    	
    },

    totalPages:{
      type: Number
    },

    pageNum:{
      type: Number
    },

    formRecordData: {
      type: Object,
      notify: true
    },

    assetReady: {
      type: String
    },

    locationReady: {
      type: String
    },

    recordResult: {
      type: Object,
      notify: true
    },

    recordForm : {
      type: Object,
      value: null
    },

    /**
     * HREF used in resource component
     */
    inspResultHref : {
      type: String
    },

    /**
     * Synonym domain collection
     */
    formStatusSet: {
      type: Array
    },

    /**
     * Inspection Results status domain filter
     */
    domainFilter: {
      type: Array,
      value: function () {
        return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'INSPRESULTSTATUS', 'selected': true} ]}];
      }
    },

    /**
     * Synonym domain collection
     */
    woTypeSet: {
      type: Array
    },

    questionSort: {
      type: Array,
      value: function () {
        return [{collection: 'inspectionform.inspquestion', attributes: '%2Bgroupseq'}];
      }
    },
    startClicked : {
      type: Boolean,
      value: false
    },
    //Store navigation history
    historyStack: {
      type: Array,
      value: []
    },

    /**
     * Available meters for the inspection 
     */
    availableMeters: {
      type: Array
    },

    /**
     * Single Inspection Result
     */
    singleRecordData: {
      type: Array,
      observer: '_changeSingleRecord'
    },

    /**
     * inspection results pending creation
     */
    pendingCreationResultSet: {
      type: Array,
      value: []
    },

    /**
     * OSLC Query name used to list inspection results
     */
    queryNameList: {
      type: String
    },
    
    /**
     * boolean to check if the user 
 	 * come from classic direct to execution
     */
    straightExecution: {
    	type: Boolean,
        value: false   	
    },
    
  },

  ready: function() {
    $M.toggleWait(true);    
  },

  /**********  OBSERVERS Begin  *********/

  /**
   * singleRecordData observer
   */
  _changeSingleRecord : function(newVal){
    if (newVal && newVal.length) {
      //this.recordResult = newVal[0];
      this._loadInspectionExecution({'detail':newVal[0]});
    }
  },

  /**********  OBSERVERS End  *********/

  /**********  LISTENERS Begin  *********/

  /**
   * Listens to changePage event
   */
  _changePage: function (pageName, isGoingBack) {	

    if(pageName.detail !== undefined){
      pageName = pageName.detail;
    }

    //release memory
    if (pageName==='inspResultList'){
      this.$.executionContainer.questions = [];
      this.$.executionContainer.dataListQuestions = [];
    }
    
    if(isGoingBack){
        this.isGoingBack=isGoingBack;
        
        if (['executionContainer','start'].includes(this.selectedPage) 
        		&& this.straightExecution && !!window.opener){
        	window.close();
        }
    }
    
    if (this.selectedPage !== pageName) {
    	
      if (!isGoingBack || isGoingBack !== true) {
        this.historyStack.push(this.selectedPage);
      }
      this.selectedPage = pageName;
      document.body.scrollTop = 0;
      this._fixButtons(pageName);
    }
  },

  /**
   * Check if it's ever used
   * Handle create inspection event
   * Delegates to createInspction method
   */
  _createPendingResult: function(e) {

    var status = 'PENDING';
    var arrayProm = [];
    var elContext = this; 
    this.pendingCreationResultSet.forEach(function(newInspResult){
      var promise = elContext._createInspectionResult(newInspResult, status);
      arrayProm.push(promise);
    });

    Promise.all(arrayProm).then(function(values) {
      elContext._recordsSavedSuccessfully();
    }).catch(function (reason) {
      console.error('An error occurred', reason);
      $M.toggleWait(false);
    });

  },

  /**
   * Callback when all records are created
   * Clear pendingResults list
   * Refresh page list
   * Toggle off spinner
   * Change page
   */
  _recordsSavedSuccessfully: function(goToExecution) {

    //Clear list of records to be created
    this.pendingCreationResultSet = [];

    //Refresh records
    this.$.inspresultlistcollection.refreshRecords();

    var pageName;
    if (goToExecution){
      pageName = 'executionContainer';
    } else {
      pageName = 'inspResultList';
    }
    //Change page
    this._changePage(pageName);

    //Turn off spinner
    $M.toggleWait(false);
  },

  /**
   * Send request to create inspection result record
   */
  _createInspectionResult : function(inspResult, status){

    var referenceObject = this.$.createInspection.selectedReferenceObject;
    var referenceAsset = this.$.createInspection.selectedAsset;
    var referenceLocation = this.$.createInspection.selectedLocation;
    var referenceSiteid = this.$.createInspection.selectedSiteid;


    // Result to be created should be alone in recordData array
    // with flag pendingCreation as true
//  if (this.recordData.length !== 1 ) {
//  console.error('Fail to create record');
//  return false;
//  }

    //Grab the whole form record
    var record = inspResult.inspectionform[0];

    var container = {};

    //container.responseProps = '*,inspectionform.name,asset.assetnum,asset.description,locations.description,locations.location';
    container.responseProps = this.getDefaultInspResultResponseProperties();

    var currentUser = $M.userInfo.personid;
    var object = {};
    object.createdby = currentUser;
    object.orgid = record.orgid;
    object.referenceobject = referenceObject;

    if (referenceObject === 'ASSET') {
      object.asset = referenceAsset;
      object.location = null;
    } else if (referenceObject === 'LOCATION') {
      object.location = referenceLocation;
      object.asset = null;
    }

    object.siteid = referenceSiteid;

    if(typeof(status)==='object'){
      object.status = 'PENDING';
    } else {
      object.status = status;
    }

    //filter status object using internal value to get external value.
    var filterStatusObject = this.formStatusSet.filter(function(o){return o.maxvalue === object.status;} );
    var externalStatus = filterStatusObject[0].value;

    object.status = externalStatus;
    object.revision = record.revision;
    object.inspformnum = record.inspformnum;
    object.revision = record.revision;

    container.record = object;

    $M.toggleWait(true);

    //this.$.inspresultcollection.addInternalValues = true;
    //this.$.inspresultcollection.createRecord(container.record, container.responseProps);
    return this.$.inspresultlistcollection.createRecord(container.record, container.responseProps);
  },

  /**
   * New Inspection Result Record Created on button click
   */
  _createUnscheduledInspection: function (e) {
    //show form page 
    this.$.createInspection.clearForm();
    this.$.startInspection.reset();
    //this.$.inspresultcollection.getNewRecordData();
    this.$.inspresultlistcollection.getNewRecordData();
    $M.toggleWait(true);
  },

  /**
   * Create / Save record as In progress
   * Lead user to execution page
   */
  _startInspections: function(e) {

    var elContext = this;
    var newstatus = 'INPROG';

    // Reset execution panel
    this.$.executionContainer.resetPanels();

    // Create new inspection results IN PROG status
    if (this.pendingCreationResultSet.length) {
      var promiseArray = [];
      this.pendingCreationResultSet.forEach(function(result){
        var promise = elContext._createInspectionResult(result, newstatus);
        promiseArray.push(promise);
      });

      Promise.all(promiseArray).then(function(values) {
        // Set values in recordData
        elContext._setNewInspections(values);
        elContext._recordsSavedSuccessfully(true);
      }, this).catch(function (reason) {
        console.error('An error occurred', reason);
        $M.toggleWait(false);
      });
    }
    // OR Update inspection results status to IN PROG
    else {
      var toChangeStatus = [];
      if (this.recordData) {
        this.recordData.forEach(function(result) {
          if (result.status_maxvalue === 'PENDING') {
            toChangeStatus.push(result);
          }
        });
      }

      this.changeStatusInspectionList(toChangeStatus, newstatus).then( function (response) {
        elContext.$.executionContainer.processResultSetChange(elContext.recordData)

        elContext.$.inspresultlistcollection.refreshRecords();

        var pageName = 'executionContainer';
        //Change page
        elContext._changePage(pageName);

        //Turn off spinner
        $M.toggleWait(false);
        
      }, function(reason){
        
        $M.toggleWait(false);
        console.error('Failed to change inspections to ' + newstatus);
        console.error('Reason:' + reason);

      });
    }

  },
  
  /**
   * Filter request response list 
   * and set it main list recordData
   */
  _setNewInspections: function(requestList) {
    var recordData = [];
    requestList.forEach(function(request){
      if (request.response) {
        recordData.push(request.response);
      }
    });
    this.set('recordData', recordData);
  },

  /**
   * Change inspection status in bulk
   */
  changeStatusInspectionList: function(inspResultList, newStatus) {

    //filter status object using internal value to get external value.
    var filterStatusObject = this.formStatusSet.filter(function(o){return o.maxvalue === newStatus;} );
    newStatus = filterStatusObject[0].value;

    var bulkBody = [];
    for (var i = 0; i < inspResultList.length; i++) {
      var newRecord = {};
      newRecord.href = inspResultList[i].href;
      newRecord.status = newStatus;
      bulkBody.push(newRecord);
    }

    $M.toggleWait(true);

    //Call method to update record
    return this.$.changeStatusBulkCollection.createRecordBulk(bulkBody, 'wsmethod:changeResultStatus', true);
  },

  /**
   * DEPRECATED
   * Bulk operation success callback
   */
  _changeStatusBulkSuccessHandler: function(e) {
    //Refresh records
    this.$.inspresultlistcollection.refreshRecords();

    var pageName = 'executionContainer';
    //Change page
    this._changePage(pageName);

    //Turn off spinner
    $M.toggleWait(false);
  },

  /**
   * DEPRECATED
   * Bulk operation error callback
   */
  _changeStatusBulkErrorHandler: function() {
    $M.toggleWait(false);
    console.log('Failed to complete status change operation'); 
  },

  /**
   * Handles event 
   * Delegates to change inspection function
   */
  _changeInspectionStatusEvent: function (e) {
    var inspectionResult = e.detail.inspresult;
    var newStatus = e.detail.newstatus;
    this._changeInspectionStatus(inspectionResult, newStatus);
  },

  /**
   * Load set of inspections  
   * show inspection assessment page 
   */
  _openInspections: function(e) {

    console.log('heading to execution form');

    var inspFormData = e.detail;
    if(inspFormData){
      //this.$.executionContainer.recordData = inspFormData;              

      //this.$.executionContainer.record = recordData.inspectionform[0];

      //this.recordResult = inspFormData;
      this.recordForm = inspFormData;

      //this.$.executionContainer.record = recordData;
    } else {
      //this.$.executionContainer.record = recordData;
      this.recordForm = inspFormData;
    }

    //TODO maybe save the form before changing page

    this._changePage('executionContainer');
  },  

  /**
   * Go Back to Inspection Result List
   */
  _goBack: function (e) {		
    //this._changePage('inspResultList');

	//Clear pending creation list
    this.pendingCreationResultSet = [];
  
    if (this.historyStack.length < 1) {
      this._changePage('inspResultList');
      return;
    }

    //clear searchbar
    if($j('#inspectorCreateSearchBar')){
      if($j('#inspectorCreateSearchBar').length>0){
        $j('#inspectorCreateSearchBar')[0].clear();
      }
    }

    var pageName = this.historyStack.pop();
    this._changePage(pageName, true);
  },

  /**
   * Select form when creating Unscheduled Inspection
   */
  _processFormSelection: function(e){

    //Setting object reference
    var selectedObjectType = this.$.createInspection.selectedReferenceObject;
    var selectedObject = this.$.createInspection.selectedElement;

    var selectedType = (selectedObjectType==='LOCATION')?'location':'asset';
    var assetLocationLabel = $M.localize('uitext','mxapiinspresult',selectedType);
    e.detail.assetLocationLabel = assetLocationLabel;

    //Setting object description
    var description = this.$.createInspection.inputValue;
    e.detail.description = description;

    var inspResult = {};
    inspResult.referenceobject = selectedObjectType;
    if ( selectedObjectType === 'LOCATION' ) {
      inspResult.locations = selectedObject;
    }else if ( selectedObjectType === 'ASSET' ) {
      inspResult.asset = selectedObject;	
    }

    var recordForm = e.detail;
    inspResult.inspectionform = [recordForm];

    this.set('pendingCreationResultSet', [inspResult]);
    //this.recordData = [inspResult];

    var pageName = e.detail.page;
    this._changePage(pageName);
  },

  /**
   * Listens to refreshFormCollection event
   */
  _refreshFormCollection : function(e){
    this.$.inspformList.refreshRecords();
    this.$.createInspection.dataSet = this.formListRecordData;
  },

  /**
   * fetch single record based on href set
   */
  _reloadInspection: function(e) {
    var record = e.detail.record;
    if (record) {
      this.inspResultHref = record.href;
    }
    this.$.inspResultResource.loadRecord();
  },

  /**
   * 
   * Listens to load-inspection-set
   */
  _loadInspectionSetHandler: function(event) {

    //Get id set from event
    var idSet = event.detail.inspectionIds;

    this._loadInspectionSet(idSet);

  },
  
  /**
   * Load a set of inspections
   */
  _loadInspectionSet: function(idSet){
    
    //Activate spinner
    $M.toggleWait(true);

    //Set filter on inspresultcollection mx-collection
    var availableVals = [];
    var filterData = [{'filtertype': 'SIMPLE', 'field': 'inspectionresultid', 'availablevalues': availableVals}];

    //create filter to fetch assignment record
    for (var i = 0; i < idSet.length; i++) {
      var record = {
          value: idSet[i],
          selected: true
      };
      availableVals.push(record);
    }

    var inspresultcollection = this.$.inspresultcollection;
    inspresultcollection.filterData = filterData;

    //Fetch records
    inspresultcollection.refreshRecords();

    //Execute `resetUserChanges` in executionContainer.$.questionlist
    this.$.executionContainer.resetPanels();
  },

  /**
   * Listens to load-insp-children
   */
  _loadInspectionChildrenHandler: function(event) {

    // parent inspection result
    var inspResultParent = event.detail.inspection;

    this._loadInspectionChildren(inspResultParent);
  },

  /**
   * Load children of inspection
   */
  _loadInspectionChildren: function(inspResultParent) {

    //Activate spinner
    $M.toggleWait(true);

    //Set filter to match orgid, siteid and inspResultParent.referenceobjectid === parent
    var maximoQbe = new MaximoQbe();
    maximoQbe.setEquals('orgid', inspResultParent.orgid)
      .setEquals('siteid', inspResultParent.siteid)
      .setEquals('parent', inspResultParent.referenceobjectid);

    var collection = this.$.inspresultchildrencollection;
    collection.queryWhere = maximoQbe.toJSON();

    //Fetch records
    collection.refreshRecords();

    //Execute `resetUserChanges` in executionContainer.$.questionlist
    this.$.executionContainer.resetPanels();
  },

  /**
   * Load inspection
   * Listens to loadInspection event
   */
  _loadInspectionExecution: function (e) {

    var recordResult = e.detail;
    this.recordResult = recordResult;

    if (recordResult.status_maxvalue === 'PENDING') {
      this.recordForm = recordResult.inspectionform[0];
      this.$.startInspection.isResultCreated = true;
      this._changePage('start');
    } else if (recordResult.status_maxvalue === 'INPROG') {
      this._changePage('executionContainer');
    } else if (recordResult.status_maxvalue === 'COMPLETED') {
      this._changePage('executionContainer');
    }

  },

  /**
   * Based on a set of insp results
   * Suggest the most appropriate page for action
   * If any record is PENDING user should be taken to Start Page
   * Otherwise user goes to Execution Page
   * @return String nextPage
   */
  defineNextPage: function() {

    var nextPage;
    //var report = [];
    if (this.recordData && this.recordData.length) {
      for (var i = 0; i < this.recordData.length; i++) {
        var inspResultStatus = this.recordData[i].status_maxvalue;
        //report[inspResultStatus] ? report[inspResultStatus] += 1 : report[inspResultStatus] = 1;
        inspResultStatus = inspResultStatus ? inspResultStatus.toUpperCase() : inspResultStatus;
        if (inspResultStatus === 'PENDING') {
          nextPage = 'start';
          break;
        } else {
          nextPage = 'executionContainer';
        }
      }
    }

    return nextPage;
  },

  /**
   * Listens to updateResponse event
   */
  _updateResponse : function(e){
    var record = e.detail.recorddata;
    var field = e.detail.field;
    record.inspfieldresult = [];
    //field._action='Add';
    record.inspfieldresult.push(field);
    var properties = {'href':record.href,'object':'INSPFIELDRESULT'};

    //var responseProperties = '*,inspectionform.name,asset.description,locations.description';
    var responseProperties = this.getDefaultInspResultResponseProperties();
    this.$.inspresultcollection.createOrUpdateChildRecord(field, properties, responseProperties);
  },

  /**
   * Send user to list page
   * Reset tab filter 
   */
  _exitCompletion: function(e) {
    //this.$.inspResultList.selectDefaultTab();
    this.$.inspResultList.refreshCurrentTab();
    var pageName = 'inspResultList';
    this._changePage(pageName);
  },

  /**
   * Change insp result set to complete status 
   */
  _completeInspections: function(event) {

    // Request change status in bulk
    var inspResultSet = event.detail.resultset;
    var newstatus = 'COMPLETED';
    var request = this.changeStatusInspectionList(inspResultSet, newstatus);

    request.then( function (response) {

      // Flag indicate whether to show completion summary panel
      this.$.executionContainer.openCompleteSummaryPending = true;

      // Update execution page with updated inspections
      var idSet = this.recordData.map(function(ele){ return ele.inspectionresultid; });
      // Load completed inspections
      this._loadInspectionSet(idSet);

      // Refresh main list
      this.$.inspresultlistcollection.refreshRecords();

      //Turn off spinner
      //$M.toggleWait(false);

    }.bind(this), function(error){

      var response = error;
      if (error && error.request) {
        if (error.request.response 
          && error.request.response.Error 
          && error.request.response.Error.message){

          response = error.request.response.Error.message;
        } else if (!error.request.isConnected) {
          
          response = $M.localize('messages', 'iface', 'CouldNotEstablishConnection');
        }
      }

      $M.toggleWait(false);
      $M.notify(response, $M.alerts.error);

      console.error('Failed to change inspections to ' + newstatus);
      console.error('Reason:' + response);
    });
  },
  
  /**
   * DEPRECATED
   * Request validation of meter for each inspection result in set
   */
  _validateInspectionsMeters: function(event) {

    var inspResultSet = event.detail.resultset;
    //var responseProperties = this.getDefaultInspResultResponseProperties();
    var self = this;
    
    var promList = [];
    $M.toggleWait(true);

    if (inspResultSet && inspResultSet.length) {

      inspResultSet.forEach( function (inspResult) {

        var asset = null;
        var location = null;
        if (inspResult.asset.assetnum) {
          asset = inspResult.asset.assetnum;
        } else if (inspResult.locations.location) {
          location = inspResult.locations.location;
        }

        var params = {
            'referenceobject' : inspResult.referenceobject, 
            'referenceobjectid' : inspResult.referenceobjectid, 
            'asset' : asset,
            'location' : location,
            'siteid' : inspResult.siteid,
            'orgid' : inspResult.orgid, 
            'inspformnum' : inspResult.inspformnum, 
            'resultnum' : inspResult.resultnum 
        };

        this.$.resultResource.resourceUri = inspResult.href;
        //this.$.inspResultResource.updateAction('wsmethod:updateAssetLocMeter', params, responseProperties);
        var prom = this.$.resultResource.updateAction('wsmethod:updateAssetLocMeter', params);
        promList.push(prom);

      }, this);
    }

    Promise.all(promList).then(function(responses) {
      $M.toggleWait(false);
      
      if (this._checkMetersResponse(responses)) {
        
        // Request change status in bulk
        var newstatus = 'COMPLETED';
        var request = this.changeStatusInspectionList(inspResultSet, newstatus);

        request.then( function (response) {

          // Update execution page with updated inspections
          var idSet = this.recordData.map(function(ele){ return ele.inspectionresultid; });
          // Load completed inspections
          self._loadInspectionSet(idSet);
          // Show completion summary
          this.$.executionContainer.openCompletionSummaryModal();
          
          // Refresh main list
          self.$.inspresultlistcollection.refreshRecords();

          //Turn off spinner
          //$M.toggleWait(false);

        }.bind(this), function(reason){
          $M.toggleWait(false);
          console.error('Failed to change inspections to ' + newstatus);
          console.error('Reason:' + reason);
        });
      }
    }.bind(this)).catch(function (reason) {
      console.error('An error occurred', reason);
      $M.toggleWait(false);
    });

  },

  /**
   * Fire change status case there's no meter errors
   */
  _checkMetersResponse: function(responses) {

    var errors = [];
    var resultSet = [];
    var self = this;

    responses.forEach( function(request) {
      var inspResult = request.response;
      resultSet.push(inspResult);
      var innerErrors = self.findMeterErrors(inspResult);
      if (innerErrors.length) {
        errors = errors.concat(innerErrors);
      }
    });

    if (errors.length) {
      // TODO notify user invalid meter
      //$M.notify(errors, $M.alerts.error);
      //$M.notify(errors.join(), $M.alerts.warn);

      //There are errors that prevent the Inspection from being completed. Please review
      var errorMsg = $M.localize('uitext','mxapiinspresult','inspections_completion_error');
      $M.notify(errorMsg, $M.alerts.warn);

      return false;
    }
    return true;
  },

  /**
   * Look for any meter error against field responses
   */
  findMeterErrors: function(record) {
    var meterErrors = [];
    if (record && record.inspfieldresult) {
      for (var i = 0; i < record.inspfieldresult.length; i++) {
        var fieldResult = record.inspfieldresult[i];
        if (fieldResult.errorflag === 1) {
          this.$.executionContainer.revealErrorInPanels(fieldResult, fieldResult.errormessage);
          meterErrors.push(fieldResult.errormessage);
        }
      }
    }
    return meterErrors;
  },

  /**
   * DEPRECATED
   * replaced by _validateInspectionsMeters
   * Change status of inspection result
   * from pending to inprogress
   */
  _updateAssetLocMeters: function (e) {

    var detail = e.detail;
    //var inspectionResult = e.detail.inspresult;


    //var responseProperties = '*,inspectionform.name,asset.description,locations.description';
    //var responseProperties = this.getDefaultInspResultResponseProperties();
    this.inspResultHref = inspectionResult.href;

    var asset = null;
    var location = null;
    if (inspectionResult.asset.assetnum) {
      asset = inspectionResult.asset.assetnum;
    } else if (inspectionResult.locations.location) {
      location = inspectionResult.locations.location;
    }

    $M.toggleWait(true);

    //var responseProperties2 = '*,inspectionform.name,asset.description,locations.description';
    var responseProperties2 = this.getDefaultInspResultResponseProperties();
    var params2 = {'referenceobject' : inspectionResult.referenceobject, 'referenceobjectid' : inspectionResult.referenceobjectid, 'asset' : asset, 'location' : location, 'siteid' : inspectionResult.siteid, 'orgid' : inspectionResult.orgid, 'inspformnum' : inspectionResult.inspformnum, 'resultnum' : inspectionResult.resultnum };
    this.$.inspResultResource.updateAction('wsmethod:updateAssetLocMeter', params2, responseProperties2);


  },

  /**
   * Listener for fetch-inspresult
   */
  _fetchInspectionResult: function(event) {
    var queryName = event.detail.queryname ? 
        event.detail.queryname : null;

    if (queryName) {
      this.queryNameList = queryName;
    }
    var self= this;
    self.$.inspResultList.loadingFlag = true;
    
    this.$.inspresultlistcollection.refreshRecords();

  },

  /**
   * Listener for fetchAvailableMeters
   */
  _fetchAvailableMeters: function(event) {

    // clear loaded Meter data
    this.assetMeterData = null;
    this.locationMeterData = null;

    this.assetReady = undefined;
    this.locationReady = undefined;

    var recordResult = event.detail;
    //var objtype = recordResult.referenceobject;
    //var itemId = this.getElementIdentification(recordResult);

    //this.requestAvailableMetersForElement(objtype, itemId);
    if ((recordResult.asset.assetnum) && (recordResult.asset.assetnum.length > 0)) {
      var maximoQbe = new MaximoQbe();
      maximoQbe.setEquals('assetnum',recordResult.asset.assetnum).setEquals('active','true');
      var qbeObject = maximoQbe.toJSON();

      //this.$.assetMeterCollection.searchAttributeName = recordResult.asset.assetnum;
      this.$.assetMeterCollection.queryWhere = qbeObject;
      this.$.assetMeterCollection.refreshRecords();
    } else {
      this.assetReady = 'true';
    }

    if ((recordResult.location) && (recordResult.location.length > 0)) {
      var maximoQbe = new MaximoQbe();
      maximoQbe.setEquals('location',recordResult.location).setEquals('active','true');
      var qbeObject = maximoQbe.toJSON();

      //this.$.locationMeterCollection.searchAttributeName = recordResult.location;
      this.$.locationMeterCollection.queryWhere = qbeObject;
      this.$.locationMeterCollection.refreshRecords();
    } else {
      this.locationReady = 'true';
    }
  },

  /**
   * Listens to openWOCreation
   */
  _openWOCreation: function (event) {
    this._showModal();
  },

  /**
   * Listens to cancelWOCreation
   */
  _cancelWOCreation: function(e) {
    this._closeModal();
  },

  /**
   * Listens to save event from maximo-inspection-workorder-create element
   */
  _createWorkOrder: function(e) {

    var detail = e.detail;
    var currentResult = this.recordResult;

    if (!currentResult) {
      console.error('No record selected');
      return;
    }

    var properties = 'wonum, siteid, orgid';
    var recordData = {};

    if (detail.worktype) {
      recordData.worktype = detail.worktype;
    }

    if (detail.description) {
      recordData.description = detail.description;
    }

    recordData.siteid = currentResult.siteid;
    recordData.orgid = currentResult.orgid;

    var id = this.getElementIdentification(currentResult);
    var objtype = currentResult.referenceobject;
    if (objtype === 'ASSET') {
      recordData.assetnum = id;
    } else if (objtype === 'LOCATION') {
      recordData.location = id;
    } 

    $M.toggleWait(true);
    this.$.workordercollection.createRecord(recordData, properties);

  },

  /**
   * Listens to start-batch operation button click
   */
  _startBatch: function(ev) {
    $M.alert('Operation under development', $M.alerts.info);
  },

  /**********  LISTENERS End  *********/

  /***********  COLLECTION n RESOURCE HANDLERS Begin  **********/

  /**
   * mx-collection inspresultlistcollection 
   * refreshRecords success handler
   */
  _handleListRefreshed: function(e) {
    this.$.inspResultList.dataSetRefresh();
    $M.toggleWait(false);
    this.$.inspResultList.loadingFlag = false;
  },

  /**
   * mx-collection inspresultlistcollection
   * error handler
   */
  _handleListError: function(e) {
	  var error = e.detail.Error;
    console.error(error);
    if (error) {
		  $M.notify(error.message, $M.alerts.error);
	  }

	  this.$.inspResultList.dataSetRefresh();
	  $M.toggleWait(false);
  },

  /**
   * mx-collection inspresultcollection 
   * refreshRecords success handler
   */
  _collectionRefreshSuccessHandler: function(ev) {
    // Fetch records
    /// Records already fetch into recordData

    // Switch off spinner
    $M.toggleWait(false);

    //Switch to next page
    var nextPage = this.defineNextPage();
    this._changePage(nextPage);
  },

  /**
   * mx-collection inspresultcollection 
   * refreshRecords error handler
   */
  _collectionRefreshErrorHandler: function(ev) {
    // Notify error
    var error = ev.detail.Error;
    console.error(error);
    $M.notify(error.message, $M.alerts.error);
    // Switch off spinner
    $M.toggleWait(false);
  },

  /**
   * mx-collection inspresultchildrencollection 
   * refreshRecords success handler
   */
  _childrenCollectionRefreshSuccessHandler: function(ev) {

    // Switch off spinner
    $M.toggleWait(false);

    //Replace recordData with children result
    this.set('recordData', this.childrenSet);

    //Switch to next page
    var nextPage = this.defineNextPage();

    //Set new page
    this._changePage(nextPage);
  },

  /**
   * mx-collection inspresultchildrencollection 
   * refreshRecords error handler
   */
  _childrenCollectionRefreshErrorHandler: function(ev) {

    // Notify error
    var error = ev.detail.Error;
    console.error(error);
    $M.notify(error.message, $M.alerts.error);
    // Switch off spinner
    $M.toggleWait(false);

  },

  /**
   * mx-collection inspresultlistcollection/inspresultlistcollection/inspformList
   * error handler
   */
  _handleError: function (e) {
    console.warn('Listener deprecated');
    var error = e.detail.Error;
    console.error(error);
    $M.notify(error.message, $M.alerts.error);

    this.$.inspResultList.dataSetRefresh();
    $M.toggleWait(false);
  },

  /**
   * mx-collection inspresultlistcollection getNewRecordData success handler
   */
  _handleNewRecordCreated : function(e){

    var recordData = e.detail;
    this.$.createInspection.inspOrgid = e.detail.orgid;
    this.mbo = recordData;
    this._changePage('createInspection');
    $M.toggleWait(false);
  },

  /**
   * DEPRECATED
   * mx-collection inspresultlistcollection createRecord success handler
   * Handle response when inspection result record is created successfully. 
   */
  _handleSaveInspectionResultRecordCreated : function(e){

    debugger;
    return;
    var recordData = e.detail;
    this.$.inspresultlistcollection.refreshRecords();

    //reset
    this.$.createInspection.selectedReferenceObject = '';
    this.$.createInspection.selectedAsset = '';
    this.$.createInspection.selectedLocation = '';
    this.$.createInspection.clearFirstStep();
    this.$.createInspection._setHideInput(true);

    this.highlightedId = recordData.inspectionresultid;

    this.recordResult = recordData;

    var pageName;
    if (this.startClicked === false){
      pageName = 'inspResultList';
    } else {
      pageName = 'executionContainer';
    }

    this.startClicked = false;
    this._changePage(pageName);
    $M.toggleWait(false);
  },

  /**
   * mx-collection inspresultcollection saveNewRecord success handler
   */
  _handleRecordSaved: function (e) {
    //TODO
  },

  /**
   * mx-collection inspresultcollection createOrUpdateChildRecord success handler
   */
  _handleChildRecordCreated : function(e){
    var inspResultResponse = e.detail;

    this.$.executionContainer.inspfieldresult = inspResultResponse.inspfieldresult;

    var form = (inspResultResponse.inspectionform.length > 0) ? inspResultResponse.inspectionform[0] : inspResultResponse.inspectionform ;
    this.$.executionContainer.inspform = form;
    //this.$.executionContainer.setClonedQuestions(form.inspquestion);

    //this.$.inspresultcollection.refreshRecords();
  },

  /**
   * mx-collection inspresultstatusdomain refreshRecords success handler
   */
  _handleStatusDomainSet: function(e) {
    $M.toggleWait(true);
    if (this.domainStatusCollection.length > 0) {
      this.formStatusSet = this.domainStatusCollection[0].synonymdomain;
    }else {
      this.formStatusSet = [];
    }
    //this.$.inspresultcollection.refreshRecords();
    //this.$.inspresultlistcollection.refreshRecords();

    this.requestInspectionResultList();
  },

  /**
   * mx-collection inspresultstatusdomain error handler
   */
  _handleStatusDomainSetError: function(e) {
    $M.toggleWait(true);
    this.formStatusSet = [];
    //this.$.inspresultcollection.refreshRecords();
    //this.$.inspresultlistcollection.refreshRecords();
    this.requestInspectionResultList();
  },

  /**
   * mx-collection inspformlistcollection refreshRecords success handler
   * Handle Refresh for Form Record Data Refresh
   */
  _handleFormRecordDataRefreshed: function (objs) {
    this.$.createInspection.dataSet = this.formRecordData;
    this.$.createInspection.dataSetRefresh();
    $M.toggleWait(false);
  },

  /**
   * mx-collection inspformList refreshRecords success handler
   * Handle Refresh for Form List Refresh containing only top level records, no children
   */
  _handleFormRecordDataRefreshedList: function (objs) {
    this.$.createInspection.dataSet = this.formListRecordData;
    this.$.createInspection.dataSetRefresh();
    $M.toggleWait(false);
  },

  /**
   * mx-collection elementMeterCollection refreshRecords success handler
   * Handle success for fetching element meter collection
   */
  _handleElementMeterRefreshed: function(e) {
    var collection = this.$.elementMeterCollection;
    this.availableMeters = collection.collectionData;
  },

  /**
   * mx-collection elementMeterCollection error handler
   * Handle error for fetching element meter collection
   */
  _handleElementMeterError: function(e) {
    console.error('Fail to meters for that inspection.');
    console.error(e.detail.error);
    this.availableMeters = [];
  },

  /**
   * mx-resource inspResultResource load record success handler
   */
  _recordLoadSuccessHandler: function(e) {
    var inspResultResponse = e.detail;
    this.recordResult = inspResultResponse;

    this.$.executionContainer.inspfieldresult = inspResultResponse.inspfieldresult;

    var form = (inspResultResponse.inspectionform.length > 0) ? inspResultResponse.inspectionform[0] : inspResultResponse.inspectionform ;
    this.$.executionContainer.inspform = form;
  },

  /**
   * mx-resource inspResultResource updateAction changeResultStatus success handler
   * Listens to resource update success response
   */
  _handleRecordUpdatedSuccess : function (e) {
    //debugger;
    return;
    this.$.inspresultcollection.refreshRecords();

    var recordResult = e.detail;
    this.recordResult = recordResult;
    //Set recordResult with response
    var recordStatus = recordResult.status;
    var recordStatusMaxval = recordResult.status_maxvalue;
//  var filterStatusObject = this.formStatusSet.filter(function(o){return o.maxvalue === 'COMPLETED';} );
//  var completedStatus = filterStatusObject[0].value;
//  var filterStatusObject2 = this.formStatusSet.filter(function(o){return o.maxvalue === 'INPROG';} );
//  var inprogStatus = filterStatusObject2[0].value;
    //if (recordStatus === completedStatus) {
    if (recordStatusMaxval === 'COMPLETED') {
      this._changePage('completeInspection');
      //} else if(recordStatus === inprogStatus) {
    } else if(recordStatusMaxval === 'INPROG') {
      this.$.startInspection.isResultCreated = false;
      this._changePage('executionContainer');
    }
    $M.toggleWait(false);
  },

  /**
   * mx-resource inspResultResource updateAction changeResultStatus error handler
   * Listens to resource update fail response
   */
  _handleRecordUpdatedFail: function(e) {
    console.error('Fail to update status of inspection result record.');
    console.error(e.detail.error);
    $M.toggleWait(false);
  },

  /**
   * mx-resource inspResultResource updateAction updateAssetLocMeter success handler
   * Listens to resource update success response
   */
  _handleMeterUpdatedSuccess : function (e) {
    this.recordResult = e.detail;
    //this.recordForm = e.detail.inspectionform;
    this.$.executionContainer.changeStatus();

    $M.toggleWait(false);
  },

  /**
   * mx-resource inspResultResource updateAction updateAssetLocMeter error handler
   * Listens to resource update fail response
   */
  _handleMeterUpdatedFail: function(e) {
    this.$.inspresultcollection.refreshRecords();
    console.error('Fail to update status of inspection form meters record.');
    console.error(e.detail.error);
    $M.toggleWait(false);
  },

  /**
   * mx-resource workordercollection createRecord success handler
   * Listens to wo resource creation success response
   */
  _createRecordSuccessHandler: function(e) {
    var response = e.detail;
    this.$.woForm.clearForm();
    this._closeModal();
    $M.toggleWait(false);
    var newWoNum = response.wonum;
    var msg = $M.localize('uitext','mxapiinspresult','wo_created',[newWoNum]);
    $M.notify(msg, $M.alerts.info);
  },

  /**
   * mx-resource workordercollection createRecord error handler
   * Listens to wo resource creation fail response
   */
  _createRecordErrorHandler: function(e) {
    var response = e.detail;
    $M.toggleWait(false);
    $M.notify(response.Error.message, $M.alerts.error);
  },

  _handleAssetMeterRefreshed : function(e) {
    this.assetReady = 'true';
  },

  _handleAssetMeterError : function(e) {
    this.assetReady = 'true';
    console.error(e.detail.error);
  },

  _handleLocationMeterRefreshed : function(e) {
    this.locationReady = 'true';
  },

  _handleLocationMeterError : function(e) {
    this.locationReady = 'true';
    console.error(e.detail.error);
  },

  /***********  COLLECTION n RESOURCE HANDLERS End  **********/

  /**
   * Called from workscape when intercepted with details subroute
   * load minimal info to determine
   * load inspresult or children
   */
  preloadInspResult: function(inspResultId) {

    // User collection preLoadInspResultCollection to fetch record
    var collection = this.$.preLoadInspResultCollection;
    // Collection filter
    var maximoQbe = new MaximoQbe();
    maximoQbe.setEquals('inspectionresultid', inspResultId);
    var qbeObject = maximoQbe.toJSON();
    collection.queryWhere = qbeObject;
    // Fetch record
    var promise = collection.refreshRecords();

    promise.then(function(response){
      // Get record
      if (collection.collectionData && collection.collectionData.length === 1) {

        var record = collection.collectionData[0];

        if (record.referenceobject === types_.PARENTWO) {
          this._loadInspectionChildren(record);
        } else {
          this._loadInspectionSet([inspResultId]);
        }

      } else {
        // No response or too many records
        console.warn('Problems to load inspection result');
      }

    }.bind(this), function(error){

      console.error(error);
      $M.alert(error, $M.alerts.error);

    });

  },

  _getFooterNavButtons : function(e){
    //Screen navigation buttons 
    return [ {'id':'goback','action':'true','label':this.localize('uitext','mxapibase','Cancel'),'event':'goback','icon':'chevron-left'}];
  },

  /**
   * Footer buttons containing Cancel, Save Pending and Start
   * For Start/Instructions page
   */
  _getFooterStartButtons : function(){

    var startLabel, icon;
    if ( this.$.startInspection.isBatch() ) {
      startLabel = this.localize('uitext','mxapiinspresult','start_batch');
      icon = 'Maximo:Inspect';
    } else {
      startLabel = this.localize('uitext','mxapiinspresult','start');
      icon = 'Maximo:Confirm';
    }
    //Screen navigation buttons
    buttons = [];
    buttons.push({'id':'inspectorCancel','action':'true','label':this.localize('uitext','mxapibase','Cancel'),'event':'goback','icon':'chevron-left'});

    var setPendingBtn = {'id':'inspectorSave','action':'true','label':this.localize('uitext','mxapiinspresult','savePending'),'event':'createPendingResult','icon':'Maximo:Save'};
    if (this.pendingCreationResultSet.length) {
      //setPendingBtn.disabled = true;
      buttons.push(setPendingBtn);
    }

    buttons.push({'id':'inspectorStart','default':true,'label':startLabel,'event':'startInspections','icon': icon});;
    
    $j($M.workScape.$.footerToolbar).css({'max-height':'120px'});

    return buttons;
  },

  /**
   * Change footer buttons depending on the page
   */
  _fixButtons: function(page){
    var buttons = this._getFooterNavButtons();
    var context = this;
    switch(page){
      case 'executionContainer':
        buttons = null;
        break;
      case 'completeInspection':
        buttons = null;
        break;
      case 'inspResultList':
        buttons = null;
        break;
      case 'start':
        buttons = this._getFooterStartButtons();
        break;
      case undefined:
        buttons = null;
        break;
      case 'createInspection':
        context = this.$.createInspection;
        break;
      default:
        break;
    }
    $M.workScape.updateFooterToolbar(buttons, context);
  },

  /**
   * Fetch lightweigth list of inspection results
   */
  requestInspectionResultList: function () {

//    var defaultOrg = $M.getMaxauth().whoami.defaultOrg;
//    var defaultSite = ($M.getMaxauth().whoami.defaultSite) ? $M.getMaxauth().whoami.defaultSite : '';
//
//    var maximoQbe = new MaximoQbe();
//    maximoQbe.setEquals('orgid', defaultOrg).setEquals('siteid', defaultSite);
//    var qbeObject = maximoQbe.toJSON();
//    this.$.inspresultlistcollection.queryWhere = qbeObject;
//
//    this.$.inspresultlistcollection.filtered=true;

    this.$.inspResultList.selectDefaultTab();
    //this.$.inspresultlistcollection.refreshRecords();
  },
  
  /**
   * Open create wo modal
   */
  _showModal: function() {
      var modal = this.$.mainModal;
      modal.style.display = 'block';
  },
  
  /**
   * Close create wo modal
   */
  _closeModal: function(e) {
      var modal = this.$.mainModal;
      modal.style.display = 'none';
  },

  meterCollectionsReady: function(assetReady, locationReady) {

    if (this.assetReady !== 'true' || this.locationReady !== 'true') {
      return;
    }

    // fetch results of each collection
    if ((this.assetMeterData && (this.assetMeterData.length > 0)) 
        && (this.locationMeterData && (this.locationMeterData.length > 0))) {
      // Generate array with common meters
      var locationMetersName = this.locationMeterData.map(function(m){ return m.metername; });
      var commonItemsArray = this.assetMeterData.filter(function(aMeter) {

        return locationMetersName.indexOf(aMeter.metername) >= 0; 
      }); 
      // Combine arrays
      var availableMeters = this.assetMeterData.concat(this.locationMeterData);

      //Remove commonItens from combined array
      var commonMetersName = commonItemsArray.map( function(meter) { return meter.metername; });
      this.availableMeters = availableMeters.filter(function (meter) { return commonMetersName.indexOf(meter.metername) == -1; });
    } else if (this.assetMeterData && (this.assetMeterData.length > 0)) {
      this.availableMeters = this.assetMeterData;
    } else if (this.locationMeterData && (this.locationMeterData.length > 0)) {
      this.availableMeters = this.locationMeterData;
    } else {
      this.availableMeters = [];
    }

//  this.assetReady = undefined;
//  this.locationReady = undefined;

  },

  /**
   * DEPRECATED
   * mx-collection inspresultcollection 
   * refreshRecords success handler
   */
  _handleRecordDataRefreshed: function (objs) {
    console.warn('Listener deprecated');
    this.$.inspResultList.dataSetRefresh();
    this.$.inspResultList.selectRecord(this.highlightedId);
    this.highlightedId = null;
    //this._refreshCurrentRecord();
    $M.toggleWait(false);
  },

  /**
   * DEPRECATED
   * Change status of inspection result
   * from pending to in progress
   * return promise
   */
  _changeInspectionStatus: function (inspectionResult, newStatus) {

    //filter status object using internal value to get external value.
    var filterStatusObject = this.formStatusSet.filter(function(o){return o.maxvalue === newStatus;} );
    newStatus = filterStatusObject[0].value;

    //var responseProperties = '*,inspectionform.name,asset.description,locations.description';
    var responseProperties = this.getDefaultInspResultResponseProperties();
    this.inspResultHref = inspectionResult.href;

    var params = {'status' : newStatus};

    $M.toggleWait(true);

    //Call method to update record
    return this.$.inspResultResource.updateAction('wsmethod:changeResultStatus', params, responseProperties);
  },

  /**
   * Enable wait layer, initiated on refresh of collection
   */
  _loading : function(){
	  if(!this.$.inspResultList.loadingFlag){
         $M.toggleWait(true);
	  }
  }

});
