/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2017
 */
Polymer({

  is: 'maximo-inspector-questions-panel',

  behaviors: [ BaseComponent, QuestionGroups, Inspections ],

  listeners: {
    'chooseQuestion': '_questionSelectedinList',
    'toggleLoading' : '_toggleLoading'
  },

  properties: {

    /**
     * Entry point when change is observed
     */
    resultSet: {
      type: Array,
      value: [],
      observer: '_resultSetChanged'
    },

    /**
     * set of inspection forms
     */
    previewFormSet: {
      type: Array,
      value: [],
      observer: '_previewFormSetChanged'
    },

    /**
     * Flag to indicate preview mode
     */
    previewMode: {
      type: Boolean,
      value: false,
      readOnly: true
    },

    /**
     * DEPRECATED
     * inspection form template
     */
    inspform: {
      type: Object,
      observer: '_inspFormChanged'
    },

    /**
     * flag to show or hide instruction
     */
    hasInstruction: {
      type: Boolean,
      computed: '_checkInstruction(inspform)'
    },

    /**
     * inspection form response field
     */
    inspfieldresult: {
      type: Array,
      value: []
    },

    /**
     * questions array of inspection template
     */
    questions: {
      type: Array,
      value: [],
    },

    /**
     * list of question passed to datalist component
     */
    dataListQuestions: {
      type: Array,
      value: []
    },

    /**
     * original data list questions
     * to restore after filtering
     */
    originalDataListQuestions: {
      type: Array,
      value: [],
      readOnly: true
    },

    /**
     * Question progress object
     * read only
     */
    progress: {
      type: Object
    },
    
    /**
     * Get total questions
     * Bind to progress bar
     */
    progTotal: {
      type: Number,
      computed: '_getTotalProgBar(progress)'
    },

    /**
     * Get complete questions
     * Bind to progress bar
     */
    progComplete: {
      type: Number,
      computed: '_getCompleteProgBar(progress)'
    },
    
    /**
     * flag if total questions same as required total
     * read only
     */
    allRequired: {
      type: Boolean,
      value: false,
      computed: 'computeAllRequired(progress)'
    },

    /**
     * flag if has non required questions
     */
    nonRequired: {
      type: Boolean,
      value: false,
      computed: 'computeNonRequired(progress)'
    },

    /**
     * hide Divisor indicator
     * read only
     */
    hideDivisor: {
      type: Boolean
    },

    /**
     * Check box label
     */
    hideOptional: {
      type: String,
      value: function() {
        return $M.localize('uitext', 'mxapibase', 'No');
      }
    },

    /**
     * completion Filter
     */
    completionFilter: {
      type: String,
      value: ''
    },

    /**
     * hide optional filter
     */
    requiredFilter: {
      type: Boolean,
      value: false
    },

    /**
     * Flag to allow user complete inspection execution
     */
    isInspectionComplete: {
      type: Boolean,
      value: false,
      observer: '_allowInspectionCompletion'
    },

    /**
     * Object type
     */
    objectType: {
      type: String,
      computed: 'computeObjectType(recordResult)'
    },

    /**
     * flag to show or hide Done Button
     */
    isCompleted: {
      type: Boolean,
      value: false
    },

    /**
     * available meters
     */
    meters: {
      type: Array,
      observer: '_metersChanged'
    },

    /**
     * flag to show or hide work order details Button
     */
    hasWorkOrder: {
      type: Boolean,
      computed: '_hasWorkOrder(refList)'
    },

    /**
     * flag to show or hide asset details Button
     */
    hasAsset: {
      type: Boolean,
      computed: '_hasAsset(refList)'
    },

    /**
     * flag to show or hide location details Button
     */
    hasLocation: {
      type: Boolean,
      computed: '_hasLocation(refList)'
    },

    /**
     * compute asset info
     */
    assetInfo: {
      type: String,
      computed: '_computeAssetInfo(recordResult)'
    },

    /**
     * compute location info
     */
    locationInfo: {
      type: String,
      computed: '_computeLocationInfo(recordResult)'
    },

    tooltipObject: {
      type: Object,
      computed: '_computedTooltipObject(recordResult)'
    },

    /**
     * Flag to hide wo tooptip icon
     */
    hideWOInfo: {
      type: Boolean,
      computed: '_computedHideWOIcon(recordResult)'
    },

    /**
     * Number of Questions required
     */
    numRequired: {
      type: String,
      computed: '_computeNumRequired(progress)'
    },

    /**
     * Number of Questions Total
     */
    numTotal: {
      type: String,
      computed: '_computeNumTotal(progress)'
    },

    /**
     * Required message
     */
    requiredMessage: {
      type: String,
      computed: '_computeRequiredMessage(numRequired)'
    },

    /**
     * Total message
     */
    totalMessage: {
      type: String,
      computed: '_computeTotalMessage(numTotal)'
    },
    
    /**
     * references list
     */
    refList: {
      type: Object,
      computed: '_buildReferenceList(resultSet)'
    },

    /**
     * complete auxiliary attribute
     */
    _iconProperty: {
      type: String,
      value: '_icon',
      readOnly: true
    },

    /**
     * resultnum auxiliary attribute
     */
    _resultNumProperty: {
      type: String,
      value: 'inspresultnum',
      readOnly: true
    },
    
    /**
     * Define if panels can toggle
     * based on device type
     */
    isToggleablePanel: {
    	type: Boolean,
        value: true,
        observer: '_isToggleablePanelChanged'
    },
    
  },

  observers: [ 'hasDivisor(allRequired, nonRequired)',
    'filtersChanged(completionFilter, requiredFilter)' ],

  ready: function() {
    this.isInspectionComplete = false;
  },

  /**
   * Observer resultSet
   */
  _resultSetChanged: function(newResultSet, oldResultSet) {
    if (newResultSet) {  
      //Hierarchize and set icon attribute for each question
      var formsGroup = this._buildDataListQuestions(newResultSet);

	    if (formsGroup) {
	      // Store original questions (without filters)
	      this._setOriginalDataListQuestions(JSON.parse(JSON.stringify(formsGroup)));
	    }

	    // Load question set in mx-datalist
      this.set('dataListQuestions', formsGroup);

      // Compute isCompleted property
      this.isCompleted = this.checkSetCompletion(newResultSet);
    }

    // Compute progress property
    this.progress = this.computeQuestionProgress(this.originalDataListQuestions);

  },

  /**
   * Observer previewFormSet
   */
  _previewFormSetChanged: function(newSet, oldSet) {
    
    if (newSet) {  
      // Fetch questions from insp result set
      var formsGroup = this._buildPreviewDataListQuestions(newSet);
      if (formsGroup) {
        this._setOriginalDataListQuestions(JSON.parse(JSON.stringify(formsGroup)));
      }
      this.set('dataListQuestions', formsGroup);
      this.isCompleted = false;
      
      if (newSet.length) {
        this._setPreviewMode(true);
      }
    }    
    this.progress = this.computeQuestionProgress(this.originalDataListQuestions);

    
  },

  /**
   * Complex Observer for completionFilter and requiredFilter
   */
  filtersChanged: function(completionFilter, requiredFilter) {
    if (this.originalDataListQuestions) {
      // Filter data list
      let filteredDataList = this.filterDataList(this.originalDataListQuestions, completionFilter,requiredFilter);
      
      // Apply filtered question set and refresh mx-datalist
      this.dataListQuestions = filteredDataList;
      //this.$.questionlist.refresh();
    }
  },

  /**
   * Fetch questions from each inspection form
   * 
   * @params {Array} inspFormSet
   * @return {Array} hierarchical structure of questions 
   *  with theirs respective forms
   */
  _buildPreviewDataListQuestions: function(set) {
    var formsGroup = [];

    if (!set || typeof set[Symbol.iterator] !== 'function' 
      || !set.length) {
      return formsGroup;
    }

    // Fetch questions and group those with insp result
    set.forEach(function(inspForm){
      // Build datalist first layer (insp form) 
      var form = inspForm;
      var formParent = {
          _id: form._id,
          id: form.inspectionformid,
          href: form.href,
          listdesc: form.name,
          inspresultnum: 0
      };

      
      formParent._icon = 'Maximo:Circle';
      formParent._required = form[this._requiredProperty];
      formParent._visibleProperty = form[this._visibleProperty];
      form.inspquestion.forEach(function(inspQuestion) {
        inspQuestion._icon = inspQuestion[this._visibleProperty] === false ? 
        'Maximo:Hide' : 'Maximo:Circle';

        inspQuestion.inspresultnum = 0;
        if (inspQuestion.inspfield) {
            for (var i = 0; i <= inspQuestion.inspfield.length; i++) {
                var field = inspQuestion.inspfield[i];
                if (field && field.required && field.required === true) {
                    inspQuestion._required = true;
                    break;
                }
            }
        }
      }, this);
      
      // Create hierarchical structure of form questions
      var questions = this._flatToHierarchy(form.inspquestion);
      // Push questions as child of form
      formParent.children = questions;
      // Push form into main datalist array
      
      formsGroup.push(formParent);
    }, this);

    return formsGroup;
  },

  /**
   * Fetch questions from each inspection result
   * 
   * @params {Array} resultSet
   * @return {Array} hierarchical structure of questions 
   *  with theirs respective forms
   */
  _buildDataListQuestions: function(set) {
    
    var formsGroup = [];
    
    if (!set || typeof set[Symbol.iterator] !== 'function' 
      || !set.length) {
      return formsGroup;
    }
    
    // Fetch questions and group those with insp result
    set.forEach(function(inspResult){
      // Build datalist first layer (insp form)
      if (!inspResult.inspectionform || !inspResult.inspectionform.length) {
        return;
      }

      var form = inspResult.inspectionform[0];
      var formParent = {
          _id: form._id,
          id: form.inspectionformid,
          href: form.href,
          listdesc: form.name,
          inspresultnum: inspResult.resultnum,
      };
      
      formParent[this._completeProperty] = form.hasOwnProperty(this._completeProperty) ? form[this._completeProperty] : false;
      formParent[this._requiredProperty] = form.hasOwnProperty(this._requiredProperty) ? form[this._requiredProperty] : false;
      formParent[this._visibleProperty] = form.hasOwnProperty(this._visibleProperty) ? form[this._visibleProperty] : false;
      formParent[this._iconProperty] = form[this._completeProperty] === true ? 'Maximo:Confirm' : 'Maximo:Circle';

      // retrieve asset or location information for the left panel
      var record = inspResult;
      if (record.asset && 'assetnum' in record.asset) {
        formParent.subtitle = record.asset.assetnum + ' ' + record.asset.description;
      } else if (record.locations && 'location' in record.locations) {
        formParent.subtitle = record.locations.location + ' ' + record.locations.description;
      } else {
        formParent.subtitle = null;
      }
      
      var questions = [];
      if (form.inspquestion 
        && typeof form.inspquestion[Symbol.iterator] === 'function' 
        && form.inspquestion.length) {
        // Create hierarchical structure of form questions
        questions = this._flatToHierarchy(form.inspquestion);
        var self = this;

        questions.forEach(function(inspQuestion) {
          self.setQuestionAuxiliaryAttributes(inspQuestion, inspResult.resultnum);
        });

        // Filter required questions to apply required flag in form
  //      var filterRequiredQuestions = questions.filter(function(question) {
  //        return question.hasOwnProperty('_required') && question._required === true;
  //      });
  //      if (filterRequiredQuestions.length) {
  //        formParent._required = true;
  //      }
      }
      // Push questions as child of form
      formParent.children = questions;

      // Push form into main datalist array
      formsGroup.push(formParent);
    }, this);

    return formsGroup;
  },

  /**
   * Include auxiliary attributes
   * 
   * _icon
   * 
   * ** Applied to hierarchical structure **
   */
  setQuestionAuxiliaryAttributes: function(question, resultNum) {

    question[this._iconProperty] = question[this._visibleProperty] === false ? 
        'Maximo:Hide' : question[this._completeProperty] === true ? 
        'Maximo:Confirm' : 'Maximo:Circle';
    question[this._resultNumProperty] = resultNum;

    if (this._isGroupHeader(question)) { //Check if children questions are required

      var qChildren = (question.children) ? question.children : [];
      for (var j = 0; j < qChildren.length; j++) {
        var qChild = qChildren[j];
        //Set question as required if it finds at least one required question
        this.setQuestionAuxiliaryAttributes(qChild, resultNum);
      }
    }

  },

  /**
   * isInspectionComplete observer
   */
  _allowInspectionCompletion: function(isInspectionComplete) {
    this.$.completeButton.disabled = !isInspectionComplete;
    this.$.completeButton.classList.remove('hiddenIcon');
    if (!isInspectionComplete) {
      this.$.completeButton.classList.add('hiddenIcon');
    }
  },

  /**
   * DEPRECATED
   * Record / InspectionForm observer
   */
  _inspResultChange: function(newV, oldV) {

    var inspForm = newV.inspectionform;
    if (!inspForm) {
      return;
    }

    this.fire('fetchAvaialbleMeters', newV);

    if (inspForm && inspForm.length > 0) {
      inspForm = inspForm[0];
    }

    //persist filters. Due to attachments refresh
    if (newV && oldV && oldV.resultnum !== newV.resultnum) {
      this.resetFilters();
    }

    var inspFieldResult = (newV.inspfieldresult) ? newV.inspfieldresult : [];

    this.set('inspfieldresult', inspFieldResult);
    this.set('inspform', inspForm);

    this.isCompleted = (newV.status_maxvalue === 'COMPLETED') ? true : false;

  },

  /**
   * Inspection Form Observer
   */
  _inspFormChanged: function(newInspForm) {
    if (!newInspForm) {
      return;
    }
    var questions = newInspForm.inspquestion;
    if (this.meters) {
      questions = this.toggleRequiredFlagForUnmatchingMeter(questions,
          this.meters);
    }

    this.setClonedQuestions(questions);

    questions = this._removeChildrenForStructureChange(questions);
    questions = this._flatToHierarchy(questions);

    this.set('questions', questions);

  },

      /**
       * Clone question list and store in variables
       */
      setClonedQuestions: function(questions) {

        questions = this._removeChildrenForStructureChange(questions);
        questions = this._flatToHierarchy(questions);

        this.set('dataListQuestions', this._cloneQuestions(questions));
        this._setOriginalDataListQuestions(this._cloneQuestions(questions));

        this.filterQuestions(this.completionFilter, this.requiredFilter);

      },

      /**
       * Clone question set to apply filter
       */
      _cloneQuestions: function(questions) {

        var cloneSet = JSON.parse(JSON.stringify(questions));

        /*for (var i = 0; i < cloneSet.length; i++) {
          var question = cloneSet[i];

          var qChildren = (question.children) ? question.children : [];
          for (var j = 0; j < qChildren.length; j++) {
            this.setQuestionAuxiliaryAttributes(qChildren[j]);
          }

          this.setQuestionAuxiliaryAttributes(question);
        }*/

        return cloneSet;

      },

      /**
       * Finds if question is complete matching inspfield and inspresponsefield
       */
      _isQuestionComplete: function(question) {
        //How to identify if question is done ?
        //If question is not required check if any field has any response
        //If question is required check if required fields are answered

        var isComplete = false;
        var fields = (question.inspfield) ? question.inspfield : [];
        var answers = this._extractValidResponses();

        if (answers.length <= 0) {
          return isComplete;
        }

        var resultField;

        if (question.required === false) { //If question is not required check if any field has any response

          var sampleField = fields[0];
          for (var i = 0; i < answers.length; i++) {
            resultField = answers[i];

            if (sampleField.inspformnum === resultField.inspformnum
                && sampleField.inspquestionnum === resultField.inspquestionnum
                && sampleField.revision === resultField.revision) {

              isComplete = true;
            }
          }

        } else { //If question is required check if required fields are answered

          var reqFields = this._extractRequiredFields(fields);
          isComplete = true;

          outerl: for (var j = 0; j < reqFields.length; j++) {
            var qfield = reqFields[j];

            innerl: for (var k = 0; k < answers.length; k++) {
              resultField = answers[k];

              if (qfield.inspformnum === resultField.inspformnum
                  && qfield.inspquestionnum === resultField.inspquestionnum
                  && qfield.inspfieldnum === resultField.inspfieldnum
                  && qfield.revision === resultField.revision
                  && resultField.revision !== undefined
                  && resultField.revision !== '') {

                continue outerl;

              }
            }
            isComplete = false;
            break;
          }
        }

        return isComplete;
      },

      /**
       * Grab valid responses
       */
      _extractValidResponses: function() {

        var answers = (this.inspfieldresult) ? this.inspfieldresult : [];
        var validAnwsers = [];
        for (var i = 0; i < answers.length; i++) {

          if ((answers[i].txtresponse && answers[i].txtresponse !== '')
              || (answers[i].numresponse && answers[i].numresponse !== '')
              || (answers[i].doclinks && answers[i].doclinks.member && answers[i].doclinks.member.length > 0)) {
            validAnwsers.push(answers[i]);
          }
        }

        return validAnwsers;
      },

      /**
       * Grab required fields
       */
      _extractRequiredFields: function(fields) {

        var requriedFields = [];

        for (var i = 0; i < fields.length; i++) {
          if (fields[i].required === true) {
            requriedFields.push(fields[i]);
          }
        }

        return requriedFields;
      },

      /**
       * Find progress about all questions
       * Compute function
       */
      computeQuestionProgress: function(questions) {

        var progress = {};

        var qTotal = 0;
        var qCompleted = 0;
        var qReqTotal = 0;
        var qRecCompleted = 0;

        if (questions && questions.length > 0) {
          qTotal = this.countQuestions(questions);
          qReqTotal = this.countRequiredQuestions(questions);
          qCompleted = this.countCompletedQuestions(questions);
          qRecCompleted = this.countRequiredAndCompletedQuestions(questions);
        }

        if (this.previewMode === true) {
          this.isInspectionComplete = false;
        } else if (qTotal === qCompleted || qReqTotal === qRecCompleted
            || qReqTotal === 0) {
          this.isInspectionComplete = true;
        } else {
          this.isInspectionComplete = false;
        }

        progress.total = qTotal;
        progress.completed = qCompleted;

        progress.reqtotal = qReqTotal;
        progress.reqcompleted = qRecCompleted;

        return progress;

      },
      
      /**
       * Check if all inspections are complete
       */
      checkSetCompletion: function(resultSet) {
        var completion = true;
        for (var i = 0; i < resultSet.length; i++) {
          if (resultSet[i].status_maxvalue !== status_.COMPLETED ) {
            completion = false;
            break;
          }
        }
        return completion;
      },

      /**
       * Count total questions
       */
      countQuestions: function(questions) {

        var totalQuestions = 0;

        for (var i = 0; i < questions.length; i++) {
          var q = questions[i];

          if (q.children) {
            totalQuestions += this.countQuestions(q.children);
          } else {
            if (q[this._visibleProperty]) {
              totalQuestions++;
            }
          }

        }
        return totalQuestions;
      },

      /**
       * Count required questions
       */
      countRequiredQuestions: function(questions) {

        var totalReqQuestions = 0;
        outerl: for (var i = 0; i < questions.length; i++) {
          var q = questions[i];
          if (q.children) {
            totalReqQuestions += this.countRequiredQuestions(q.children);
          } else {
            if (q[this._requiredProperty] === true) {
              totalReqQuestions++;
            }
          }

        }
        return totalReqQuestions;

      },

      /**
       * Count completed questions
       */
      countCompletedQuestions: function(questions) {
        var totalCompletedQuestions = 0;
        for (var i = 0; i < questions.length; i++) {
          var q = questions[i];

          if (q.children) {
            totalCompletedQuestions += this.countCompletedQuestions(q.children);
          } else if (q[this._completeProperty] === true 
            && q[this._visibleProperty] === true) {
            totalCompletedQuestions++;
          }

        }
        return totalCompletedQuestions;
      },

      /**
       * countRequiredAndCompletedQuestions
       */
      countRequiredAndCompletedQuestions: function(questions) {

        var totalCompReq = 0;

        for (var i = 0; i < questions.length; i++) {
          var q = questions[i];

          if (q.children) {
            totalCompReq += this.countRequiredAndCompletedQuestions(q.children);
          } else if (q[this._completeProperty] === true 
              && q[this._requiredProperty] === true 
              && q[this._visibleProperty] === true) {
            totalCompReq++;
          } 
        }
        return totalCompReq;
      },

      /**
       * Listens to selected question
       */
      _questionSelectedinList: function(e) {
        if (e.detail._selectedRecords && (e.detail._selectedRecords.length > 0)) {
          var selectedRecord = e.detail._selectedRecords;
          selectedRecord = selectedRecord[0];
          if (selectedRecord[this._visibleProperty]) {
            this.fire('showQuestion', selectedRecord);
          }
          this.$.questionlist.clearSelected();
        }
      },

      /**
       * Open dialog
       */
      _showInfo: function(e) {
        e.stopPropagation();
        this.$.instructionDialog.classList.remove('hiddenContent');
      },

      /**
       * Closes dialog
       */
      closeDialog: function(e) {
        this.$.instructionDialog.classList.add('hiddenContent');
      },

      /**
       * Back one page
       */
      exitExecution: function(e) {
        this.fire('exit', {previewMode: this.previewMode});
      },

      /**
       * Reset filters
       */
      resetFilters: function() {
        //Reset filters
        this.set('requiredFilter', false);
        this.$.checkbox.checked = false;
        this.set('completionFilter', '');
        this.$.questionSelector.select(0);
      },

      /**
       * Completing inspection
       * Skipping meter validation
       */
      completeInspections: function(event) {
        if (this.previewMode) {
          window.close();
          return;
        }

        var detail = {resultset: this.resultSet};

        this.fire('complete-inspections', detail);

      },

      /**
       * REPLACED By completeInspections
       * Before completing inspection meters should be checked
       */
      validateMeters: function(event) {
        if (this.previewMode) {
          window.close();
          return;
        }

        var detail = {resultset: this.resultSet};

        this.fire('validate-meters', detail);

      },
      
      /**
			 * DEPRECATED
			 * Replaced by validateMeters
       * Complete inspection
       */
      done: function(e) {

        if (this.previewMode) {
          window.close();
          return;
        }

//        var detail = {
//            'inspresult': this.recordResult
//        };
        
        var detail = {resultset: this.resultSet};

        //this.fire('changeInspectionStatus', detail);
        //this.fire('updateAssetLocMeters', detail);
        this.fire('complete-inspections', detail);
      },

      /**
       * Take Action button click handler
       */
      takeAction: function(ev) {
        var tooltip = this.$.takeActionToolTip;
        var content = this.$.takeActionToolTipContainer;
        tooltip.show(content, ev.currentTarget);

        // For platforms different than mobile
        if (!this.isToggleablePanel) {
	        var visibleItems = content.querySelectorAll('maximo-button:not([hidden])').length;
	        var top = tooltip.elementOffset.top - $j(this).offset().top - (visibleItems * 15);
	        $j(tooltip.$.tooltip).css({'top':top+'px'});
        }
      },

      /**
       * Fire change status case there's no meter errors
       */
      changeStatus: function() {
        var hasMeterErrors = this.findMeterErrors(this.recordResult);
        if (!hasMeterErrors) {
          var detail = {
              'inspresult': this.recordResult,
              'newstatus': 'COMPLETED'
          };
          this.fire('changeInspectionStatus', detail);
        }
      },

      /**
       * Look for any meter error against field responses
       */
      findMeterErrors: function(record) {
        var hasMeterErrors = false;
        if (record && record.inspfieldresult) {
          for (var i = 0; i < record.inspfieldresult.length; i++) {
            var fieldResult = record.inspfieldresult[i];
            if (fieldResult.errorflag === 1) {
              hasMeterErrors = true;
              break;
            }
          }
        }
        return hasMeterErrors;
      },

      /**
       * Complex observer 
       */
      hasDivisor: function(allRequired, nonRequired) {
        this.set('hideDivisor', (allRequired || nonRequired));
      },

      /**
       * Computed property refList
       */
      _buildReferenceList: function(resultSet) {

        var structure = {
            asset:[],
            location:[],
            workorder: []
        };

        if (resultSet && resultSet.length) {
          for (var result of resultSet) {

            if (result.asset && Object.keys(result.asset).length){
              structure.asset.push(result.asset);
            }

            if (result.locations && Object.keys(result.locations).length) {
              structure.location.push(result.locations);
            }

            var workOrderAvailableProps = (!result.workorder) ? [] : Object.keys(result.workorder)
              .filter(function(prop){return prop!=='parent';});
            if (workOrderAvailableProps.length) {
              structure.workorder.push(result.workorder);
            }
            
          }
        }
        return structure;
      },

      /**
       * Compute object type based on object reference of record result
       */
      computeObjectType: function(recordResult) {

        var objtype = recordResult.referenceobject;
        return $M.localize('uitext', 'mxapiinspresult', objtype.toLowerCase());
      },

      /**
       * Computed function for all required flag
       * Compute
       */
      computeAllRequired: function(progress) {

        var allRequired = false;
        if (progress.reqtotal === progress.total) {
          allRequired = true;
        }

        return allRequired;
      },

      /**
       * Computed function for zero required flag
       * Compute
       */
      computeNonRequired: function(progress) {

        var nonReq = true;
        if (progress.reqtotal > 0) {
          nonReq = false;
        }

        return nonReq;
      },

      /**
       * Compute show hide flag for instructions
       */
      _checkInstruction: function(inspform) {
        var instruc = inspform.description_longdescription;
        if (instruc) {
          return true;
        }
        return false;
      },

      /**
       * Computed progTotal
       */
      _getTotalProgBar: function(progress) {
        return Number(progress.total);
      },


      /**
       * Computed progComplete
       */
      _getCompleteProgBar: function(progress) {
        return Number(progress.completed);
      },

      //------- Filtering functions -------\\
      
      /**
       * Filter question set
       * by complete and required
       */
      filterDataList: function(/*Array*/set,
          /*String*/completionFilter,
          /*Boolean*/requiredOnly) {

        // Copy original question set
        var originalQuestionSet = JSON.parse(JSON.stringify(set));

        if (completionFilter) {

          if (completionFilter === 'complete') {
            originalQuestionSet = this.filterCompleted(originalQuestionSet);
          } else if (completionFilter === 'incomplete') {
            originalQuestionSet = this.filterCompleted(originalQuestionSet, true);
          }

        }

        // Filter required
        if (requiredOnly) {
          originalQuestionSet = this.filterRequired(originalQuestionSet);
        }

        return originalQuestionSet;

      },

      /**
       * Listens to tab selection
       */
      selectTab: function(e) {

        var selectedTab = e.currentTarget.getAttribute('name');

        selectedTab = (selectedTab) ? selectedTab.toLowerCase() : selectedTab;

        //find external status value
        this.set('completionFilter', selectedTab);

      },

      /**
       * Listens to toggle changes
       */
      checkboxChanged: function(e) {

        var state = e.detail;

        if (state && state === true) {
          this.set('requiredFilter', true);
          this.set('hideOptional', $M.localize('uitext', 'mxapibase', 'Yes'));
        } else {
          this.set('requiredFilter', false);
          this.set('hideOptional', $M.localize('uitext', 'mxapibase', 'No'));
        }
      },

      /**
       * DEPRECATED
       * Filter question set by complete/incomplete
       */
      filterQuestions: function(/*String*/completionFilter, /*Boolean*/
          requiredOnly) {

        //Restore original set
        this.dataListQuestions = JSON.parse(JSON
            .stringify(this.originalDataListQuestions));

        if (requiredOnly && requiredOnly === true) {
          this.dataListQuestions = this.filterRequired(this.originalDataListQuestions);
        }

        if (completionFilter) {

          if (completionFilter === 'complete') {
            this.dataListQuestions = this.filterCompleted(this.originalDataListQuestions);
          } else if (completionFilter === 'incomplete') {
            this.dataListQuestions = this.filterCompleted(this.originalDataListQuestions, true);
          }

        }

        this.$.questionlist.refresh();

      },

      /**
       * Iterate over list of insp results and filter required question
       */
      filterRequired: function(itemSet) {
        for (var i = 0; i < itemSet.length; i++) {
          var item = itemSet[i];
          if ('children' in item) {
            item.children = this.filterRequiredQuestions(item.children);
          }
        }
        return itemSet;
      },

      /**
       * Filter required items in list
       * Return array with required items
       */
      filterRequiredQuestions: function(questionSet) {

        for (var i = 0; i < questionSet.length; i++) {
          var q = questionSet[i];
          if (this._isGroupHeader(q)) {
            q.children = this.filterRequiredQuestions(q.children);
          }
        }

        questionSet = questionSet.filter(function(val) {
          if ('children' in val) {
            if (!val.children.length){
              return false;
            } else {
              for (var i = 0; i < val.children.length; i++) {
                var child = val.children[i];
                if (child[this._requiredProperty] === true){
                  return true;
                }
              }
            }
          }
          return val[this._requiredProperty];
        }, this);

        return questionSet;
      },

      /**
       * Filter completed items in list
       */
      filterCompleted: function(questionSet, isIncomplete) {

        for (var i = 0; i < questionSet.length; i++) {
          var q = questionSet[i];
          if (this._isGroupHeader(q)) {
            q.children = this.filterCompleted(q.children, isIncomplete);
            if (q.children && q.children.length > 0) {
            	if (isIncomplete && isIncomplete === true) {
            		q[this._completeProperty] = false;
            	} else {
            		q[this._completeProperty] = true;
            	}
            } else {
            	if (isIncomplete && isIncomplete === true) {
            		q[this._completeProperty] = true;
            	} else {
            		q[this._completeProperty] = false;
            	}
            }
          } else if (q.children && q.hasOwnProperty('subtitle')) {
              q.children = this.filterCompleted(q.children, isIncomplete);
          }
        }

        if (isIncomplete && isIncomplete === true) {
          questionSet = questionSet.filter(function(val) {
            if (val.children && val.children.length <= 0) {
              return false;
            }
            return (!val[this._completeProperty]) && val[this._visibleProperty];
          }, this);
        } else {
          questionSet = questionSet.filter(function(val) {
            if (val.children && val.children.length > 0) {
              return true;
            }
            return val[this._completeProperty] && val[this._visibleProperty];
          }, this);
        }

        return questionSet;
      },

      /**
       * Observer for meters array
       */
      _metersChanged: function(newv) {

        //Restarting processing again
        this._inspFormChanged(this.inspform);

      },
      
      /**
       * Observer for isToggleablePanel flag
       */
      _isToggleablePanelChanged: function(newv, oldv) {

    	  this.toggleClass('mobile', newv, this.$.header);
    	  this.toggleClass('mobile', newv, this.$.footer);
    	  this.toggleClass('mobile', newv, this.$.optinalFilter);
    	  this.toggleClass('mobile', newv, this.$.content);

      },

      //**** DEPRECATED functions ****\\

      /**
       * DEPRECATED
       * Listen to question update from execution component
       */
      questionUpdate: function(e) {
        e.stopPropagation();
      },

      /**
       * DEPRECATED
       * 
       * Fetch questions from inspection form
       * Re structure questions
       */
      _fetchQuestions: function(inspform) {

        //		if (!inspform._originalinspquestion) {
        //			inspform._originalinspquestion = JSON.parse( JSON.stringify( inspform.inspquestion ) );
        //		}

        //Convert flat structure to hierarchical
        var questions = inspform.inspquestion;
        //		questions = JSON.parse(JSON.stringify(inspform._originalinspquestion));
        questions = this._removeChildrenForStructureChange(questions);
        questions = this._flatToHierarchy(questions);

        for (var i = 0; i < questions.length; i++) {
          var question = questions[i];

          var qChildren = (question.children) ? question.children : [];
          for (var j = 0; j < qChildren.length; j++) {
            //this.setQuestionAuxiliaryAttributes(qChildren[j]);
          }

          //this.setQuestionAuxiliaryAttributes(question);
        }
        this.dataListQuestions = this._cloneQuestions(questions);
        return questions;

      },

      /**
       * DEPRECATED
       * Update complete flag of question
       */
      _updateQuestionCompletion: function(question, doneFlag) {

        //Find question 
        var localQuestion = this.findQuestion(question, this.questions);

        //Update question
        if (localQuestion) {
          localQuestion.completed = doneFlag;
          localQuestion._icon = (doneFlag) ? 'Maximo:Confirm' : 'Maximo:Circle';
        }
      },

      /**
       * DEPRECATED
       * Find matching question
       */
      findQuestion: function(question, qSet) {

        var match = null;

        for (var i = 0; i < qSet.length; i++) {
          var q = qSet[i];
          if (q.children) {
            match = this.findQuestion(question, q.children);
            if (match !== null) {
              break;
            }
          } else {
            if (question.inspquestionid === q.inspquestionid) {
              match = q;
              break;
            }
          }
        }

        return match;
      },
      
      /**
       * Update element within left panel that has changed on the right panel (panel sync)
       */
      updateElements: function(elementsChanged) {
    	  
    	  if (elementsChanged != null) {
    		  
    		  // make sure it is iterable
    		  let array = [].concat(elementsChanged);
    		  let rSet = this.originalDataListQuestions;
    		  
	    	  for (let element of array) {
	    		  rSet = this.updateElement(element, rSet);
    		  }
	    	  
	    	  // Update already update it
	          this.updateDataList(rSet);
	          this.progress = this.computeQuestionProgress(rSet);
    	  }
    	  
      },
      
      /**
       * Update element within left panel that has changed on the right panel (panel sync)
       */
      updateElement: function(elementChanged, resultSet) {

    	  if (resultSet && resultSet.length) {

    		  let resultFound = resultSet.find( (result) => {
    			  return result.inspresultnum === elementChanged.resultnum 
    		  });
    		  
    		  var localElement = this.findElement(resultFound, elementChanged);
    		  
    		  //Update element
    		  if (localElement) {

    			  if (elementChanged.hasOwnProperty('visible')) {
    				  localElement[this._visibleProperty] = elementChanged.visible;
    			  }

    			  if (elementChanged.hasOwnProperty('completed')) {
    				  localElement[this._completeProperty] = elementChanged.completed;
    			  }

    			  if (elementChanged.type === 'question' && elementChanged.hasOwnProperty('required')) {
    				  localElement[this._requiredProperty] = elementChanged.required;
    			  }

    			  localElement[this._iconProperty] = (localElement[this._visibleProperty] === false) ? 
    					  'Maximo:Hide' : (localElement[this._completeProperty]) ? 
    							  'Maximo:Confirm' : 'Maximo:Circle';
    		  }
    		  
    		  return resultSet;
    	  }

      },

      /**
       * Update dataListQuestions within left panel that has changed on the right panel (panel sync)
       */
      updateDataList: function(newDataList) {
    	  
        let filteredDataList = this.filterDataList(newDataList, this.completionFilter, this.requiredFilter);  
        
        // Apply filtered question set and refresh mx-datalist
        this.dataListQuestions = filteredDataList;
      },

      /**
       * Fetch element based on type
       */
      findElement: function(inspResult, element) {
        if (element.type === 'form') {
          return inspResult;
        } else if ((element.type === 'question') || (element.type === 'group')){
          var qSet = inspResult.children;
          var match = null;
          for (var i = 0; i < qSet.length; i++) {
            var q = qSet[i];
            if (q.children) {
              match = this.findElement(q, element);
              if (match !== null) {
                break;
              } else if (element.num === q.inspquestionnum) {
                match = q;
                break;
              }
            } else {
              if (element.num === q.inspquestionnum) {
                match = q;
                break;
              }
            }
          }
          return match;
        }
      },

      /**
       * Check if inspection record has work order associated.
       */
      _hasWorkOrder: function(refList) {
        return (!refList) ? false : (!refList.workorder) ?
            false : (!refList.workorder.length) ? false : true;
      },

      /**
       * Check if inspection record has asset associated.
       */
      _hasAsset: function(refList) {
        return (!refList) ? false : (!refList.asset) ?
            false : (!refList.asset.length) ? false : true;
      },

      /**
       * Check if inspection record has location associated.
       */
      _hasLocation: function(refList) {
        return (!refList) ? false : (!refList.location) ?
            false : (!refList.location.length) ? false : true;
      },

      /****** Tooltip tap handlers ******/

      /**
       * Handle inspection summary button clicked
       * Show completion summary modal
       */
      openSummary: function(event) {
        this.fire('open-completion-summary');
        this.$.takeActionToolTip.hide();
      },

      /**
       * Launch classic WO Tracking Maximo
       */
      showWoDetails: function(ev) {

        var idList = this.refList.workorder.map( function(wo) {
          return wo.workorderid;
        });

        if (idList.length === 1) {
          $M.openMaximoRecord('wotrack', idList[0]);
        } else {
          var sqlWhere = 'workorderid in (' + idList.toString() + ')';
          $M.openMaximoRecord('wotrack', null, 'sqlwhere', sqlWhere);
        }

        this.$.takeActionToolTip.hide();

      },

      /**
       * Launch classic Asset App Maximo
       */
      showAssetDetails: function(ev) {

        var idList = this.refList.asset.map( function(asset) {
          return asset.assetuid;
        });

        if (idList.length === 1) {
          $M.openMaximoRecord('asset', idList[0]);
        } else {
          var sqlWhere = 'assetuid in ( ' + idList.toString() + ')';
          $M.openMaximoRecord('asset', null, 'sqlwhere', sqlWhere);  
        }

        this.$.takeActionToolTip.hide();

      },

      /**
       * Launch classic Location App Maximo
       */
      showLocationDetails: function(ev) {

        var idList = this.refList.location.map( function(loc) {
          return loc.locationsid;
        });

        if (idList.length === 1) {
          $M.openMaximoRecord('location', idList[0]);
        } else {
          var sqlWhere = 'locationsid in (' + idList.toString() + ')';
          $M.openMaximoRecord('location', null, 'sqlwhere', sqlWhere);
        }

        this.$.takeActionToolTip.hide();

      },

      /**
       * Hidden function for tooltip attributes
       */
      _hideField: function(field) {
        if (!field) {
          return true;
        }
        return false;
      },

      /**
       * Computed value for hideWO icon flag
       */
      _computedHideWOIcon: function(record) {
        var objtype = record.referenceobject;
        if (objtype === 'WORKORDER' || objtype === 'WOACTIVITY' || objtype === 'MULTIASSETLOCCI') {
          return false;
        }
        return true;
      },

      /**
       * Fetches number of questions required when inspection result changes
       */
      _computeNumRequired: function(progress) {

        return progress.reqcompleted + "/" + progress.reqtotal;
      },

      /**
       * Fetches number of questions total when inspection result changes
       */
      _computeNumTotal: function(progress) {

        return progress.completed + "/" + progress.total;
      },

      /**
       * Computes message of inspection required questions
       */
      _computeRequiredMessage: function(numRequired) {
        return $M
        .localize('uitext', 'mxapiinspection', 'required', [ numRequired ]);
      },

      /**
       * Computes message of inspection total questions
       */
      _computeTotalMessage: function(numTotal) {
        return $M.localize('uitext', 'mxapiinspresult', 'total', [ numTotal ]);
      },

      /**
       * Computed value for location info
       */
      _computeLocationInfo: function(record) {
        if (record.locations && 'location' in record.locations) {
          return record.locations.location + ' ' + record.locations.description;
        }
        return null;
      },

      /**
       * Computed value for asset info
       */
      _computeAssetInfo: function(record) {
        if (record.asset && 'assetnum' in record.asset) {
          return record.asset.assetnum + ' ' + record.asset.description;
        }
        return null;
      },

      /**
       * Reset mx-datalist reset user changes
       */
      reset: function() {
        this.resetFilters();
        this.$.questionlist.resetUserChanges();
      },

      /**
       * Computed binding label for Exit button
       */
      _computeExitLabel: function(isCompleted, isPreviewMode) {
        var label;
        if ( isCompleted || isPreviewMode ) {
          label = $M.localize('uitext','mxapiinspresult','exit');
        } else {
          label = $M.localize('uitext','mxapiinspresult','exit_inspection');
        }
        return label;
      },
      
      /**
       * DEPRECATED
       * Check if inspection record is in a COMPLETED status.
       */
      _isCompleted: function(record) {
        return (record.status_maxvalue === 'COMPLETED') ? true : false;
      },

      /**
	   * Loading be icon in the center of the datalist panel.
	   */
      _toggleLoading: function(){
    	  this.async(function(){
	    	  this.$.questionlist.toggleLoading(true);
    	  },200);
      }
});
