/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2018
 */
Polymer({

	is: 'maximo-inspection-condition-card',

	behaviors: [BaseComponent, QuestionGroups],

	listeners: {},

  /**
   * Fired when bin icon is clicked.
   *
   * @event delete-condition
   */

	properties : {

		/**
		 * Main object to populate card
		 */
		condition: {
			type: Object,
			value: null,
		},

		/**
		 * fieldOptions
		 */
		options: {
			type: Array,
			value: []
		},

		/**
		 * List of options formatted for mx-select
		 */
		optionsListFormatted: {
			type: Array,
			computed: '_computeAvailableOptions(options)'
		},

		/**
		 * available fields for target
		 */
		targets: {
			type: Array,
			value: []
		},

		/**
		 * List of targets formatted for mx-select
		 */
		targetListFormatted: {
			type: Array,
			computed: '_computedTargetList(targets)'
		},

		/**
		 * Stores initial state for visible 
		 * attribute for later comparisons
		 */
		initialVisibleState: {
			type: Boolean,
			computed: '_storeOriginalVisibleState(condition)'
		},

		/**
		 * Stores initial state for required 
		 * attribute for later comparisons
		 */
		initialRequiredState: {
			type: Boolean,
			computed: '_storeOriginalRequiredState(condition)'
		},
		
		/**
		 * Card index
		 */
		index: {
			type: Number,
			value: 0
		},

		/**
		 * Tile title
		 */
		title: {
			type: String,
			computed: '_computeConditionTitle(index)'
		},
		
		/**
		 * Sets the card as Read-only (for Saved Conditions)
		 */
		isReadOnlyCard: {
			type: Boolean,
			computed: '_isReadOnlyCard(condition)'
		},
		
		/**
		 * Sets the card as Read-only (for Saved Conditions)
		 */
		isAvailableToSaveCard: {
			type: Boolean,
			computed: '_computeAvailableToSaveCard()'
		},		

	},
	
	observers: [
		'_computeAvailableToSaveCard(optionsListFormatted, targetListFormatted)'
	],

	created : function(){
	},
	ready : function(){
	},
	attached: function(){
	},

	/********** OBSERVERS begin **********/

	/**
	 * Compute condition card title
	 * @param {Number} index 
	 */
	_computeConditionTitle: function(index) {
		const visualIndex = ++index;
		return $M.localize('uitext','mxapiinspection','condition_index', [visualIndex]);
	},

	/**
	 * Copy and parse field options for display
	 * @param {Object} condition 
	 */
	_computeAvailableOptions: function(options) {
		let availableOptions = [];
		if (options && options.length) {
			availableOptions = options.map(opt => {
				if (opt.description){
				    //Single Options 
					return opt.description;
				} else {
					//Meter Options
					return opt;
				}
			});
			
			var isAlpha = false;
			for (i=0; i<availableOptions.length; i++){
				if (isNaN(availableOptions[i])){
					isAlpha = true;
				}
			}

			if (isAlpha){
				availableOptions.sort();
			} else {
				//Sort numeric fields
				availableOptions.sort(function(a, b){return a-b});
			}			
		}
		const entry = ':' + $M.localize('uitext','mxapiinspection', 'select_option');
		availableOptions.unshift(entry);

		return availableOptions;
	},

	/**
	 * Copy and parse input field targets
	 * @param {Object} condition 
	 */
	_computedTargetList: function(targets) {
		let availableTargets = [];
		if (targets && targets.length) {
			availableTargets = targets.map(field => {
				return `${field.inspfieldnum}:${field.description}`;
			});
		}
		const entry = ':' + $M.localize('uitext','mxapiinspection','select_response_field');
		availableTargets.unshift(entry);
		return availableTargets;
	},
	
	
	/**
	 * Check available field targets
	 * 
	 */
	_checkAvailableTargets: function(targets){
	    var payload = {
	        question: this.condition.srcquestion,
	        successCallback: this.fetchResultSuccessHandler.bind(this),
	        errorCallback: this.fetchResultFailureHandler.bind(this)
	    };
	    this.condition.tgtfield = "";
	    this.fire('fetch-conditions-list', payload);

	},
	
	  /**
	   * Success load conditions handler
	   * Load record callback from maximo-inspection-form-conditions
	   */
	  fetchResultSuccessHandler: function(response) {

		var conditionsSameSrc = [];
	    var targets = this.targets;
	    var targetsList = [];
		var exists = false;

		/* The target can be used only once */
		for(i=0; i<targets.length; i++){
	    	exists = false;
	    	for (i2=0; i2<response.length; i2++){
	    		if (response[i2].tgtfield){
					if( (response[i2].tgtfield == targets[i].inspfieldnum)
						&& (response[i2].srcquestion == this.condition.srcquestion)
						&& (response[i2].srcfield == this.condition.srcfield)
						&& (response[i2].srctxtresponse == this.condition.srctxtresponse)
					){
		    			exists = true;
					}
				}
			}
	        if (!exists){
	        	targetsList.push(targets[i]);
	        }
	    }
				
	    var availableTargets = [];
		if (targetsList && targetsList.length) {
			
			availableTargets = targetsList.map(field => {
				return `${field.inspfieldnum}:${field.description}`;
			});
		}	
		const entry = ':' + $M.localize('uitext','mxapiinspection','select_response_field');
		availableTargets.unshift(entry);
		
		targetListFormatted = availableTargets.sort(this._sortTargets);

		this.$.targetField.values = targetListFormatted;
		
	  },
	  
	    /**
		 * Sort targets numerically
		 */
	    _sortTargets: function(a, b) {
	    	 if (a.trim() === "" || a === undefined){
	    	    a = "0";
	    	 }

	    	 if (b.trim() === "" || b === undefined){
	    	    b = "0";
	    	 }
	    	return parseFloat(a.substring(a.indexOf(":") + 1)) - parseFloat(b.substring(b.indexOf(":") + 1));
		},
		
		
	  /**
	   * Fail load conditions handler
	   * Load record callback from maximo-inspection-form-conditions
	   */

	  fetchResultFailureHandler: function(response) {
		  console.error(response);
	  },
	 
	    

	/**
	 * Get copy initial value of visible attribute
	 * @param {Object} condition 
	 */
	_storeOriginalVisibleState: function(condition) {
		return condition ? condition.visible : false;
	},

	/**
	 * Get copy initial value of required attribute
	 * @param {Object} condition 
	 */
	_storeOriginalRequiredState: function(condition)	{
		return condition ? condition.required : false;
	},

	/**
	 * Store visible state to condition object
	 * @param {*} event 
	 */
	_visibleStateChangeHandler: function(event) {
		let state = event.detail;
		if (this.condition){
		    this.condition.visible = state;
		
			if (state === false) {
				this.condition.required = false;
				this.querySelector("#requiredCheckbox").checked = false;
			}
			this._computeAvailableToSaveCard();
		}	
		
	},

	/**
	 * Store required state to condition object
	 * @param {*} event 
	 */
	_requiredStateChangeHandler: function(event) {
		let state = event.detail;
		
		if (this.condition){
			this.condition.required = state;
			
			if (state === true) {
				this.condition.visible = true;
				this.querySelector("#visibleCheckbox").checked = true;
			}
			this._computeAvailableToSaveCard();
		}
	},
	
	/**
	 * Check if the Condition already exists
	 * @param {Object} condition 
	 */
	_isReadOnlyCard: function(condition) {
		if (condition && condition.inspcascadeoptionid){
			this.$.saveButton.disabled = true;
			return true;
		}else{
			return false;
		}
		
	},


	/********** OBSERVERS end *** HANDLERS begin **********/

	/**
	 * Handles trash icon
	 * @param {Event} ev 
	 */
	_trashIconClick: function(ev) {
		// validate if condition was not saved, show warning message
		if (this.condition && (this.condition.inspcascadeoptionid === undefined)){
			$M.showDialog(this, this.condition, 'Warning', 'maximo-conditions-warning-dialog', false);
		} else {
			this.deleteMe();
		}
		
	},
	
	/**
	 * Handles trash icon
	 * @param {Event} ev 
	 */
	_saveClick: function(ev) {
	   this.condition.srctxtresponse = this.$.options.value;
	   this.condition.tgtfield = this.$.targetField.value;
	   this.fire('saveConditionsForm', this);
	},

	/********** HANDLERS end **********/

	/**
	 * Emits delete event
	 */
	deleteMe: function() {
		this.fire('delete-condition', this);
	},
	
	_computeAvailableToSaveCard: function(){
        // Enable Save button when Source and Target fields are selected, 
		// and the Visible/Required values are different from the original values from the Target Selected
		let temp = (!(this.isReadOnlyCard) &&
		        (this.$.options.value !== undefined) &&
		        (this.$.options.value !== "") &&
		        (this.$.targetField.value !== undefined ) &&
		        (this.$.targetField.value !== ""));
		
		if (temp === true){
			let isdefault = this._checkDefaults(this.condition);
			if (isdefault){
				this.$.saveButton.disabled = true;
				temp = false;
			} else {
				this.$.saveButton.disabled = false;
				temp = true;
			}
		} else {
			this.$.saveButton.disabled = true;
			temp = false;
		}
		return temp;
	},
	
	_checkDefaults: function(condition){
		let targets = this.targets;
		let selectedtarget = this.$.targetField.value;
		let oritargetvisible = '';
		let oritargetrequired = '';
		targets.forEach(element => {
			if (element.inspfieldnum == selectedtarget){
				oritargetvisible = element.visible;
				oritargetrequired = element.required;
			}
		});
		if ((this.condition.required == oritargetrequired) && (this.condition.visible == oritargetvisible)){
			return true;
		} else {
			return false;
		};
	},
	
	_setDefaultsVisibleRequired: function(condition){

		let selectedtarget = this.$.targetField.value;
		let targets = this.targets;
		targets.forEach(element => {
			if (element.inspfieldnum == selectedtarget){
				this.condition.required = element.required;
				this.querySelector("#requiredCheckbox").checked = element.required;
				
				this.condition.visible = element.visible;
				this.querySelector("#visibleCheckbox").checked = element.visible;
			}
		});
		this._computeAvailableToSaveCard();
	},
	

});
