/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2017
 */
Polymer({
	is: 'maximo-inspection-form-conditions',

	behaviors: [BaseComponent, QuestionGroups, Polymer.IronA11yKeysBehavior, CascadeConditions],

	listeners: {
		'saveConditionsForm': '_complete',
		'fetch-conditions-list': '_getConditionsList'
		
	},
	
	properties: {

		/**
		 * MX-Resource result
		 */
		formRecordRef: {
			type: Object,
			value: {},
			readOnly: true
		},

		/**
		 * processed question set
		 */
		questionSet: {
			type: Array,
			value:  []
		},

		/**
		 * All fields from inspection form
		 */
		inputFieldSet: {
			type: Array,
			value: []
		},

		/**
		 * Set of selected items in question datalist
		 */
		fieldSelection: {
			type: Array,
			value: []
		},

		/**
		 * Prop bind to disabled attribute 
		 * for add condition button
		 * Disabled when no field selection and fieldselection + unsaved condition
		 */
		disabledAddCondition: {
			type: Boolean,
			value: true,
			readOnly: true,
		},

		/**
		 * Prop bind to disabled attribute
		 * for save conditions form
		 */
		disabledSave: {
			type: Boolean,
			value: false
		},

		/**
		 * Inspection Form href
		 */
		resourceHref: {
			type: String,
			value: ''
		},

		/**
		 * Set event target for keysbehavior
		 */
		keyEventTarget: {
			type: Object,
			value: function() {
				return document.body;
			}
		},

		/**
		 * List of allowed fields to 
		 * create condition
		 */
		allowedFieldTypes: {
			type: Array,
			value: [fieldTypes_.SO, fieldTypes_.MM]
		},
		
		/**
		 * List of all conditions of the form
		 */
		conditionsList: {
			type: Array,
			value: []
		},

		/**
		 * Filtered condition list based on field selection
		 */
		filteredConditionList: {
			type: Array,
			value: []
		},
	
		/**
		 * Flag to show/hide instruction
		 */
		hideInstruction: {
			type: Boolean,
			computed: '_computeInstructionHiddenAttribute(filteredConditionList)'
		},

		/**
		 * available fields to set as target
		 * in condition
		 */
		availableFieldTargets: {
			type: Array,
			value: []
		},

		/**
		 * options of the field
		 */
		fieldOptions: {
			type: Array,
			value: []
		},

		/**
		 * conditions mark icon
		 */
		_iconProperty: {
			type: String,
			readOnly: true
		  },
		  
		/**
	     * confirm dialog (location)  
	     */
		confirmDialog: {
			type: Object 
		} 
		    
	},

	keyBindings: {
		'esc:keyup': '_escPressed'
	},

	observers: [
		'_filterListObserver(conditionsList, fieldSelection)',
		'_processAddButtonDisableAttribute(filteredConditionList, fieldSelection)'
	],

	ready : function() {},

	attached: function() {},

	/********** OBSERVERS begin **********/


	/**
	 * Filter condition list based on selection
	 * @param {Array} fieldSelection 
	 */
	_filterListObserver: function(conditionList, fieldSelection) {

		let filteredList = [];

		if (conditionList &&
			conditionList.length && 
			fieldSelection.length ) {

			let srcField = this.getSourceField();
	
			const availableTargets = this.fetchTargetOptions(srcField);
			this.set('availableFieldTargets', availableTargets);

            this.fetchFieldOptions(srcField);

			filteredList = conditionList.filter( condition => {
				if ( (condition.srcfield == fieldSelection[0].inspfieldnum) &&
				      (condition.srcquestion == fieldSelection[0].questionid)){
				    return condition;
				}
			});
			
			if(filteredList && filteredList.length){
				fieldSelection[0]._iconProperty = "Maximo:Flow";
			} else {
				fieldSelection[0]._iconProperty = "";
			}
		}

		this.set('filteredConditionList', filteredList);
	},

	/**
	 * Set disable attribute for
	 * add button based on fieldSelection
	 * @param {Array} fieldSelection 
	 */
	_processAddButtonDisableAttribute: function(conditionList, fieldSelection) {

		let disabaled = true;
		let allSaved = conditionList.every( cond => {
			return cond.href;
		});

		if (fieldSelection && fieldSelection.length &&
			(!conditionList.length || allSaved)) {

			disabaled = false;
		}

		this._setDisabledAddCondition(disabaled);
	},

	/**
	 * Compute hidden attribute for
	 * instruction based on conditionList prop
	 * @param {Array} conditionList 
	 */
	_computeInstructionHiddenAttribute: function(conditionList) {
		let hidden = false;
		if (conditionList && conditionList.length) {
			hidden = true;
		}
		return hidden;
	},

	/********** OBSERVERS end **** HANDLERS begin **********/

	/**
	 * esc key handler
	 * Close modal
	 * @param {Event} e 
	 */
	_escPressed: function(e) {
		this.closeMe();
	},

	/**
	 * Exit button clicked
	 * Close modal
	 * @param {Event} e 
	 */
	_close: function(e) {
		this.closeMe();
	},

	_complete: function(e) {
		this.saveConditions();
	},

	/**
	 * datalist select event handler
	 * @param {Event} e 
	 */
	_datalistItemSelected: function(e) {
		// Check if is field and allowed
		var selectedRecords = e.detail._selectedRecords;
		this.itemSelect(selectedRecords);
	},

	/**
	 * addCondition tap event handler
	 * @param {Event} e 
	 */
	_newCondition: function(e) {
		this.buildConditionCards();
	},

	/**
	 * Handles delete operation of
	 * individual conditional card
	 * @param {Event} ev 
	 */
	_deleteCondition: function(ev) {
		// Find item
		let item = ev.model.condition;
		// Call another method
		this.removeItem(item);
	},

	/**
	 * Handles success response for 
	 * record refreshing
	 * @param {Event} event 
	 */
	loadSuccessHandler: function(event) {
		// Process record question to display
		this.recordLoaded(this.$.inspectionFormResource.resourceData);
	},

	/**
	 * Handles error response for 
	 * record refreshing
	 * @param {Event} event 
	 */
	loadErrorHandler: function(event) {
		console.log({error: event});
		$M.toggleWait(false);
	},

	/**
	 * Handles success response for 
	 * record updating
	 */
	_updateSuccessHandler: function(event) {
		// Update condition based on response
		this.recordUpdated(this.$.inspectionFormResource.resourceData);
		$M.toggleWait(false);
	},

	/**
	 * Handles error response for 
	 * record updating
	 * @param {Event} event 
	 */
	_updateErrorHandler: function(event) {
		console.log({'error': event});
		$M.toggleWait(false);
	},

	/********** HANDLERS end **********/

	/**
	 * Build conditions based on selection and 
	 * set condition list
	 */
	buildConditionCards: function() {

		let srcField = this.getSourceField();
		const newCondition = this.buildNewConditionCard(srcField);
		this.conditionsList = this.conditionsList.concat([newCondition]);		
	},

	/**
	 * Build condition objects 
	 * based on field selection
	 */
	buildNewConditionCard: function(field) {

		return {
			fieldHref: field.href,
			required: field.required,
			visible: field.visible,
			srcquestion: field.questionid,
			srcfield: field.inspfieldnum
		}
	},

	/**
	 * Return available field inputs for that field
	 */
	fetchTargetOptions: function(sourceField) {

		if (sourceField && this.inputFieldSet) {
			let newarr = [];
			this.inputFieldSet.forEach(element => {
				newarr.push(element);
			});
			
			return newarr.filter(inputField => {
				return sourceField.inspfieldnum !== inputField.inspfieldnum;
			});
			
		} else {
			return []
		}
	},

	/**
	 * Return field options
	 */
	fetchFieldOptions: function(sourceField) {
		var values = [];
		if (sourceField.fieldtype_maxvalue == 'SO'){
			values = sourceField.inspfieldoption ? sourceField.inspfieldoption.map(opt => {
				         return Object.assign({}, opt);
				         }) : [];
		    this.set('fieldOptions', values);	         
		} else if (sourceField.fieldtype_maxvalue == 'MM') {
	        this.async(function(){
	            this._fetchAvailableOptionMeters(sourceField.metername).then(function(response){
                values = this._handleMeterDataResponse(response.response.member);
				this.set('fieldOptions', values);
	        	}.bind(this)).catch(function (reason) {
	              console.error('An error occurred', reason);
	            });
	        });
		}
	},
	
	  /**
	   * fetchAvailableMeters
	   */
	  _fetchAvailableOptionMeters: function(meterName) {
	  
		    var collection = this.$.meterOptionsCollection;
		    var maximoQbe = new MaximoQbe();
		    maximoQbe.setEquals('metername', meterName);
		    var qbeObject = maximoQbe.toJSON();
		    collection.queryWhere = qbeObject;
		    return collection.refreshRecords();
		    
	  },

	  /**
	   * Prepare response to load as value
	   */
	  _handleMeterDataResponse: function(e) {
	    var collection = this.$.meterOptionsCollection;
	    var domainArray = collection.collectionData;
	    var values = [];
	    if (domainArray.length > 0) {
	      var dmValues = domainArray[0].domainid;
	      if (dmValues.length > 0) {
	        var alnValues = dmValues[0].alndomainvalue;
	        values = alnValues.map(function(synonym) {
	          return synonym.value;
	        });	        
	      }
	    }
	    return values;
	  },

	/**
	 * Remove item from list
	 * @param {Object} item 
	 */
	 removeItem: function(item) {
		if (item.inspcascadeoptionid){
			item._action = 'Delete';
			list = [item];
		    this.updateRecordConditions(list);
		}else{
			this.conditionsList = this.conditionListSetUp(this.$.inspectionFormResource.resourceData);
			if ( !(this.conditionsList && this.conditionsList.length)){
				this.fieldSelection[0]._iconProperty = "";
				this.$.questionsList.refresh();
			}
		}
	},

	/**
	 * Stores allowed fields
	 * @param {Array} selectionList 
	 */
	itemSelect: function(selectionList) {

		if (selectionList
			&& selectionList.length
			&& selectionList.every(this.isAllowedField, this)) {
				this.fieldSelection = selectionList;

		} else {
			this.fieldSelection = [];
		}
	},

	/**
	 * Extract source field from selected fields 
	 * @returns {Object}
	 */
	getSourceField: function() {

		let srcField = {};
		if (this.fieldSelection && this.fieldSelection.length) {
			// Presume src field is always going to be the first item
			srcField = this.fieldSelection[0];
		}
		return srcField;
	},

	/**
	 * Immutably create question with field
	 * @param {Object} question 
	 */
	_fetchLowLevelQuestion: function(question, index) {
		
		const partialObject = {
			inspfield: [],
			children: question.inspfield
			.map((field, fieldIndex) => {
				let fieldTitle = field.description || $M.localize('uitext','mxapibase', 'NotEntered');
				let desc = field.description;
				field.description = question.groupseq + '.' + (fieldIndex+1+'').padStart(2,'0') + '. ' + desc;

				const partialFieldObject = {
	          questionid: question.inspquestionnum,
	          listdesc: question.groupseq + '.' + (fieldIndex+1+'').padStart(2,'0') + '. ' +  fieldTitle,
	          _id: field.inspfieldid,
	          inspfieldoption: !field.inspfieldoption ? null : field.inspfieldoption.map((option) => {
	            return Object.assign({}, option);
	          }),
	          _iconProperty: this.isFieldASource(field, this.conditionsList) ? 'Maximo:Flow' : null
				};

				return Object.assign({}, field, partialFieldObject);

			}).filter(this.isAllowedField, this)
		}

		return Object.assign({}, question, partialObject);
	},

	/**
	 * Return flag whether item 
	 * should be kept or removed
	 * @param {Object} fieldItem 
	 * @returns {Boolean}
	 */
	isAllowedField: function(fieldItem) {

		var isAllowed = this.allowedFieldTypes.includes(fieldItem.fieldtype_maxvalue);

		if (fieldItem.fieldtype_maxvalue === fieldTypes_.MM
			&& fieldItem.metertype_maxvalue !== meterTypes_.CHARACTERISTIC) {
			isAllowed = false;
		}
		return isAllowed;
	},

	/**
	 * Replicate condition list
	 * @param {Object} inspForm 
	 * @returns {Array}
	 */
	conditionListSetUp: function(inspForm) {
		let dataList = [];

		if (inspForm.inspcascadeoption) {
			dataList = inspForm.inspcascadeoption.map( item => {
				return Object.assign({}, item);
			});
		}

		return dataList;
	},

	/**
	 * Massage data to show in datalist
	 * @param {Object} inspForm 
	 * @returns {Array} 
	 */
	dataListSetUp: function(inspForm) {
		const dataList = [];
		let hierarchicalQuestions = [];

		if (inspForm.inspquestion && inspForm.inspquestion.length) {
		  hierarchicalQuestions = this._flatToHierarchy(inspForm.inspquestion);
		}

		dataListMember = Object.assign({}, inspForm, {
			listdesc: inspForm.name,
			inspectionformstatus: [],
			inspquestion: [],
			children: hierarchicalQuestions.map((question, index)=> {
				let obj = null;
				if (question.inspfield) {
					obj = this._fetchLowLevelQuestion(question, index);
				} else {
					const partialObject = {
						listdesc: question.description,
						children: question.children.map((innerQuestion, index) => {
							return this._fetchLowLevelQuestion(innerQuestion, index);
						})
					};
					obj = Object.assign({}, question, partialObject);
				}
				return obj;
			})
		});

		dataList.push(dataListMember);

		return dataList;
	},

	/**
	 * Fetch all fields for current
	 * inspection template
	 * @param {Object} record 
	 */
	fetchFields: function(record) {
		const fieldList = [];
		if (record.inspquestion) {
			for(let question of record.inspquestion) {
				var questionid = question.inspquestionnum;

				if (question.inspfield) {
                    let itemQuestion = question.inspfield;
                    itemQuestion.forEach((field, fieldIndex) =>{
						     let descwithindex = field.description;
						     descwithindex = question.groupseq + '.' + (fieldIndex+1+'').padStart(2,'0') + '. ' + descwithindex;

						const partialObject = {
						    descwithindex,
	              questionid
						};

						fieldList.push(Object.assign({}, field, partialObject));

					//}
					})
				}				
			}
		}
		return fieldList;
	},

	/**
	 * Set properties after 
	 * record gets loaded
	 * @param {Object} record 
	 */
	recordLoaded: function(record) {

		this._setFormRecordRef(record);

		let fieldSet = this.fetchFields(record);
		for(i=0; i<fieldSet.length; i++){
			fieldSet[i].description = fieldSet[i].descwithindex;
		}
		
		this.inputFieldSet = fieldSet;

		let conditionList = this.conditionListSetUp(record);
		this.conditionsList = conditionList;
		
		let dataListContent = this.dataListSetUp(record);
		this.questionSet = dataListContent;
		
		$M.toggleWait(false);
	},

	/**
	 * Update condition list
	 * after update
	 */
	recordUpdated: function(record) {
		
		let conditionList = this.conditionListSetUp(record);
		this.conditionsList = conditionList;
		if ( !(this.conditionsList && this.conditionsList.length)){
			this.fieldSelection[0]._iconProperty = "";
		}
		this.$.questionsList.refresh();
		
		$M.toggleWait(false);
	},

	/**
	 * Close the Conditions window and triggers "Cancel"
	 */
    _confirmClose: function(isTrue){

    	if (isTrue){
	    	this.resetDataListSelection();
			this.clearConditions();
			this.fire('cancel');
    	} 
    },	
	/**
	 * Show a warning dialog when tapping the Back button, and there are unsaved conditions
	 */
	closeMe: function() {
		let conditions = this.conditionsList;
		let isNotSaved = false;
		for (var i=0; i < conditions.length; i++){
			if (conditions[i].inspcascadeoptionid === undefined){
				isNotSaved = true;
			}
		}	

		if (isNotSaved){
			this.confirmDialog = $M.confirm($M.localize('uitext', 'inspectorsup', 'Your changes will not be saved. Do you want to continue?'),this._confirmClose,this, 1,[0,1],'','',false,null);

			this.async(function(){
				this.confirmDialog.querySelector('footer').style.textAlign='center';
				this.confirmDialog.querySelector('.dialogCloseButton').hidden = true;
				this.confirmDialog.querySelector('.messageIcon').hidden = true;
				$j(this.confirmDialog.$.footer).find('.buttonSpan').css('padding','0px 10px 0px 10px');
			}, 100); 
		} else {
			this._confirmClose(true);
		}
	},
    
	/**
	 * Return properties to initial state
	 */
	resetDataListSelection: function() {
		this.fieldSelection = [];
		this.$.questionsList.clearSelected();
	},

	/**
	 * Reset conditions list
	 */
	clearConditions: function() {
		this.conditionsList = [];
	},

	/**
	 * submit conditions list
	 */
	saveConditions: function() {

		let currentForm = this.formRecordRef;
		
		const inspcascadeoption = this.conditionsList.map( condition => {

		  const parcialObject = {
        orgid: currentForm.orgid,
        siteid: currentForm.siteid,
        inspformnum: currentForm.inspformnum,
        revision: currentForm.revision,
        tgtquestion: condition.tgtquestion 
      };

			let cascadeOption = Object.assign({}, condition, parcialObject);

			let newCascadeOption = Object.assign({}, cascadeOption);
			delete newCascadeOption.availableTargets;
			delete newCascadeOption.responseOptions;

			return newCascadeOption;
		});

		// Populating the tgtquestion value
		// For each inspcascadeoption, availableFieldTargets where inspfieldnum === inspcascadeoption tgtfield
		for (var i = 0; i < inspcascadeoption.length; i++) {
			for (var x = 0; x < this.availableFieldTargets.length; x++) {
				if (this.availableFieldTargets[x].inspfieldnum == inspcascadeoption[i].tgtfield) {
					inspcascadeoption[i].tgtquestion = this.availableFieldTargets[x].questionid;
					break;
				}
				
			}
		}

		this.updateRecordConditions(inspcascadeoption);
	},

	/**
	 * Dialog entry point
	 * Initial setup
	 */
	setUp: function(formRevision) {

		$M.toggleWait(true);

		if (formRevision && formRevision.href) {
			this.resourceHref = formRevision.href;
			// load resource
			this.$.inspectionFormResource.loadRecord(false);
		} else {
			$M.toggleWait(false);
		}

		$M.toggleWait(false);
	},

	/**
	 * Submit resource update
	 * @param {Array} conditionList
	 */
	updateRecordConditions: function(conditionList) {

		/**
		 * _action = 'Add|Update|Delete'
		 */
		const recordData = {
			inspcascadeoption: conditionList
		}

		const responseProperties = 'inspectionformid,name,inspformnum,orgid,inspcascadeoption{*}';

		$M.toggleWait(true);

		this.$.inspectionFormResource.updateRecord(recordData, responseProperties, true);
	},

	_openDialog: function(){
		
		$M.showDialog(this, '', 'Warning', 'maximo-conditions-warning-dialog', false);

	},
	
	_getConditionsList: function(event){
		
		try{
			event.stopPropagation();
			var detail = event.detail;
		    let conditionsList = this.conditionsList;
		    detail.successCallback(conditionsList);
		} catch (error){
			var response = error;
		    if (error && error.request) {
		        if (error.request.response 
		          && error.request.response.Error 
		          && error.request.response.Error.message){

		          response = error.request.response.Error.message;
		        } else if (!error.request.isConnected) {
		          
		          response = $M.localize('messages', 'iface', 'CouldNotEstablishConnection');
		        }
		      }
		      detail.errorCallback(response);
		};
	    
	},
	
});
