/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2018,2019
 */
Polymer({
	is: 'maximo-reservation-list',
  	behaviors: [BaseComponent,ItemMaster],
  	listeners: {
          'maximo-table-collection-sort' : 'collectionOrderByChanged',
          'table-action-clicked' : '_reservationActions',		  
          'value-changed': '_fromtoDateFilterUpdate',
          'delete-row':'_deleteReservation',
          'inventory-detail': '_displayItemDetail',
          'custom-conditional-formatting':'_handleConditionalFormatting',
          'collection-refresh-success': '_collectionRefreshHandler',
          'display-sitestore': '_displaySiteStore',
          'apply-sitestore-filter': '_applySiteStoreFilter',
          'maximo-table-rows-selected':'_selectReservationRow',
          'tableselectall-clicked':'_selectReservationRow',
          'display-next-reservation':'_displayNextReservation',
          'iron-overlay-opened': '_positionOverlay'
    },
  	properties : {
  		record: {
    		type:Object,
  			value:{}
		},
		reservationListColumns: {
			type: Array,
			value: []
		},
		menuItems : {
        	type: Array,
    		value: [],
    		notify:true    		
        },
		collectionOrderBy: {
			type: String,
			value: '%2Brequestnum'
		},
		additionalParams: {
			type: Array,
			value: []
		},
		
		defaultStoreroom: {
			type: String,
			notify:true 
		},
		
		defaultSite:{
			type: String,
			notify:true 
		},
		
		expendTarget:{
			type: String	
		}, 
		/**
		 * Interval in seconds to undo delete operation
		 */
    	undoIntervalSecs: {
    		Type: Number,
    		value: 3,
    		readOnly: false
    	},
    	
    	redirectedRecord : {
    		type: Object,
    		value: null
    	},
    	redirectedIssueRecord : {
    		type: Object,
    		value: null
    	},
    	/**
    	 * Flag for table initialization
    	 */
    	isTableInitialized : {
    		type: Boolean,
    		value: false
    	},
    	
    	batchReserveIssue : {
    		type: Boolean,
    		value: false
    	},
    	
    	currReservationIndex: {
    		type: Number,
    		value: 0
    	},
    	
    	buttonCount: {
    		type: Number,
    		value: 0
    	},
    	
    	actionFlag: {
    		type: String,
    		value: ''
    	},
		
		/**
		 * Display the actions button.
		 */
		actions:{
			type: Object,
			value: function(){
				return {
					header:{
						width: window.innerWidth/5+ 'px'
					},
					list:[
						{
							icon:'Maximo:Overflow',
							action: 'sractions',
							style: 'transform: rotate(90deg)',
							label: $M.localize('uitext', 'mxapiinvres', 'actions')
						}
					]
				};
			}
		},
		
		/**
		 * Select tab name
		 */
    	selectedTabName: {
    		type: String,
    		observer: '_loadSection'    		
    	},
    	
    	_showMultiSelect: {
            type: Boolean,
            value:false
        },
        overlayPosition: {
        	type: Object
        },
        isCollectionRefreshed : {
    		type: Boolean,
    		value: false
    	},
	},
	attached: function() {
  		var self = this;
  		
		$j(window).resize(function(){
			$j(self.$.reservationActionsOverlay).hide();
			self._setTableStyles();
		});
		$j('.maximo-table').on('scroll', function(){
			$j(self.$.reservationActionsOverlay).hide();
		});
	},
	
	_getSitesDefaultStore: function(){
		var self = this;
		var collection = this.$.invresstoreroom;
		var postObj = {};
		postObj.siteId = this.record.storelocsiteid;
		if(collection && !this.defaultStoreroom){
			collection.invokeAction('wsmethod:getDefaultStoreroomForSite', null, 'POST', postObj).then(function(results){
				if(results.response) {					
					self.defaultStoreroom = results.response.return;
					self.record.location = self.defaultStoreroom;
					if(self.selectedTabName === 'issueditem') {
						self.$.invissueditem._refreshIssuedCollection(self.record);
					}else {
						self._refreshCollection(self.record); 
					}
				}
			});
		}
	},
	
	_displaySiteStore: function(e){
		var record;
		if(e.detail && e.detail.record){
			record = e.detail.record;
		} else {
			record = e;
		}
		var sitestoreLabel = '';
		var sitelabel = '';
		var storelabel = '';
		if(record) {
			sitelabel = record.storelocsiteid;
			storelabel = record.location;
		} else {
			sitelabel = this.defaultSite;
			storelabel = this.defaultStoreroom;
		}
		if(!sitelabel) {
			sitelabel = 'allsites';
		}
		if(!storelabel) {
		    storelabel = 'allstores'; 
		}
		record.storelocsiteid = sitelabel;
		record.location = storelabel;
		sitelabel = (sitelabel === 'allsites')? $M.localize('uitext','mxapiinvres','allsites') : sitelabel;
		storelabel = (storelabel === 'allstores')? $M.localize('uitext','mxapiinvres','allstores') : storelabel;
		sitestoreLabel = sitelabel + ' - ' + storelabel;
		this.$.sitestoreedit.label = sitestoreLabel;
		if($M.getSubRoute() === 'reservations/issueditem' && this.selectedTabName === 'issueditem') {
			var issueRecord = {}; 
		    issueRecord = record;
			sessionStorage.setItem('issueRecord', JSON.stringify(issueRecord));
		} else {
			var reservationRecord = {};
			reservationRecord = record;
			sessionStorage.setItem('reservationRecord', JSON.stringify(reservationRecord));
		}
	 },
	 
	 _getDefaultStore: function(){
		 if($M.maxauth.whoami.defaultStoreroom) {
			 this.defaultStoreroom = $M.maxauth.whoami.defaultStoreroom;
			 this.record.location = this.defaultStoreroom;
		 } else if(!$M.maxauth.whoami.defaultStoreroom){
			 this._getSitesDefaultStore();
		 }
	 },
	 
	 _resetRecord: function() {
		 var siteStore = this.$.sitestoreedit.label.split(' - ');
		 if(siteStore.length > 0 && this.record.storelocsiteid !== 'allsites') {
			 this.record.storelocsiteid = siteStore[0];
		 }
 		 if(siteStore.length > 1 && this.record.location !== 'allstores') {
 			this.record.location = siteStore[1];
 		 }
	 },
	 _setRecordValues: function() {
		if(this.record && !this.record.storelocsiteid) {
			this.defaultSite = $M.maxauth.whoami.defaultStoreroomSite ? $M.maxauth.whoami.defaultStoreroomSite : $M.maxauth.whoami.defaultSite;
			this.record.storelocsiteid = this.defaultSite;
		}
		if(this.record && !this.record.location) {
			this._getDefaultStore();
		}
	 },
	 /**
  	 * Initialization of table with columns
  	 */
  	_initReservationTable: function(){
  		if(this.isTableInitialized) {
  			return;
  		}
  		var tableColumns = [{
				'attribute' : 'restype',
				'title' : $M.localize('uitext', 'mxapiinvres', 'type'),
				'flexGrow': '1.4',
				'sortable' : true,
				'conditionalformatting':'{"attribute":"restype","values": {"APHARD":"font-weight:bold"}}'
			},
			{
				'attribute' : 'requireddate_localized',
				'title' : $M.localize('uitext', 'mxapiinvres', 'date'),
				'flexGrow': '1.4',
				'sortable' : true,
				'type':'date',
				'format':$M.maxauth.whoami.dateformat.toUpperCase(),
				'filterattribute': 'requireddate',
				'sortattribute': 'requireddate',
				'conditionalformatting':'{"customcondition": "true","attribute":"requireddate_localized"}'
			},
			{
				'attribute' : 'wogroup',
				'title' : $M.localize('uitext', 'mxapiinvres', 'wo'),
				'flexGrow': '1.2',
				'sortable' : true
			},
			{
				'attribute' : 'itemnum',
				'title' : $M.localize('uitext', 'mxapiitem', 'item'),
				'flexGrow': '1',
				'sortable' : true,
				'hyperlink' : true,
   				'listener' : 'inventory-detail'
			},
			{
				'attribute' : 'description',
				'title' : $M.localize('uitext', 'mxapiitem', 'description'),
				'flexGrow': '3.5',
				'sortable' : true
			},
			{
				'attribute' : 'binnum',
				'title' : $M.localize('uitext', 'mxapiinventory', 'defaultbin'),
				'flexGrow': '1.2',
				'sortable' : true
			},
			{
				'attribute' : 'reservedqty',
				'title' : $M.localize('uitext', 'mxapiinvres', 'quantity'),
				'flexGrow': '1.5',
				'sortable' : true
			}
		];
		this.set('reservationListColumns', tableColumns);
		this._setActionValue();		
  		this.isTableInitialized = true;
  	},
	
	_loadReservationTable: function(){
		var self = this;
		if(self.record){
            self.redirectedRecord=self.record;
        }
        if(sessionStorage.getItem('reservationRecord')){
        	var rec = JSON.parse(sessionStorage.getItem('reservationRecord'));
            this.record.storelocsiteid = rec.storelocsiteid;
            this.record.location = rec.location;            
            if(this.$.fromdate.value !== rec.fromDateVal){
            	this.$.fromdate.value = rec.fromDateVal;
    		}
    		if(this.$.todate.value !== rec.toDateVal){
    			this.$.todate.value = rec.toDateVal;
    		}
		}
		this._initReservationTable();
		this._fromtoDateValueFormat();
		this._setRecordValues();
		this._refreshCollection(this.record);
		this._setTableStyles();
		this.$.contentSelector.select(0);
	},
	
	_setTableStyles() {
		this.$.reservation_list_table.maxHeight = ($j(window).height() - 220) +'px';
	},
	
	_refreshCollection: function(record) {
		var self = this;
		var reservationcollection = this.$.reservationcollection;
		this.additionalParams=[];
		this._showMultiSelect = false;
		var fromdaterange = 'oslc.where=requireddate>=%22'+record.fromDateVal+'%22';
		var todaterange = ' and requireddate<=%22'+record.toDateVal+'%22';
		var siteid = ' and storelocsiteid=%22'+record.storelocsiteid+'%22';
		var location = ' and location=%22'+record.location+'%22';
		var whereClause = '';
		whereClause = fromdaterange+todaterange;
		if(record.storelocsiteid && record.storelocsiteid !== 'allsites') {
			whereClause += siteid;
		} 
		if (record.location && record.location !== 'allstores') {
			whereClause += location;
		}
		whereClause += ' and reservedqty>0';
		if(whereClause) {
			this.additionalParams.push(whereClause);
		}
		reservationcollection.collectionUri='';
		this._displaySiteStore(record);
		if(reservationcollection && record.fromDateVal && record.toDateVal && record.storelocsiteid && record.location) {
			$M.toggleWait(true);
			reservationcollection.refreshRecords().then(function(){
				$M.toggleWait(false);
				self.isCollectionRefreshed = true;				
	  		});
		}
	},
	
	/**
	 * Hide the actions.
	 */
	_overFlowIcon : function(){
		this.$.reservationActionsOverlay.close();
	},
	
	/**
	 * Collection error
	 */
	_handleError: function (e) {
		var error = e.detail.Error;
		$M.notify(error.message, $M.alerts.error);
	},
	
	_reservationActions: function(e){
		if(this.selectedTabName === 'issueditem') {
			this.$.invissueditem._returnIssuedItem(e);
		} else {
			this.selectedRow = e.detail.source.closest('.tablerow');
			var pos = $j(e.detail.source).find('button').offset();
	        var left =  pos.left - 180.5;
	        var dockedPos = $j(this.$.reservation_list_table).find('#tableFilterRow').length? $j(this.$.reservation_list_table).find('#tableFilterRow')[0].getBoundingClientRect() : $j(this.$.reservation_list_table).find('.thead')[0].getBoundingClientRect();
	        var rowPos = $j(e.detail.source).closest('.tablerow')[0].getBoundingClientRect();
	        var top = pos.top - 19;
	        if(rowPos.top < dockedPos.bottom){
	        	top = pos.top;
	        }
	        this.overlayPosition = {'butonpos': pos,'left':left + 15,'top':top};
	        this.setRowOffset = top;
	        this.rowRecord = e.detail.record;
	        
			if(this.buttonCount > 0) {
				if(this.buttonCount === 1) {
					switch(this.actionFlag){
		        		case 'issue' :
		        			if(this.rowRecord.tostoreloc) {
		        				this._transferReservationRecord();
		        			} else {
		        				this._issueReservation(event);
		        			}
			        		break;
		        		case 'edit':
		        			this._editReservation(event);
		        			break;
		        		case 'delete':
		        			this._deleteReservationRecord(event);
		        			break;
		        	}
				} else {
			        this.$.reservationActionsOverlay.open();
				}
			}
			$j(this.$.transferActionDiv).css('display', 'none');
			$j(this.$.issueActionDiv).css('display', 'block');
			if(this.rowRecord.tostoreloc) {
				$j(this.$.transferActionDiv).css('display', 'block');
				$j(this.$.issueActionDiv).css('display', 'none');
				if(this.$.transferActionDiv.children.length === 0) {
					this.inventory_transfer=Polymer.Base.create('maximo-button',{'id':'inventory_transfer'});
					this.inventory_transfer.label=$M.localize('uitext', 'mxapistoreloc', 'transfer');
					this.inventory_transfer.setAttribute('action',true);
					this.inventory_transfer.setAttribute('icon', 'Maximo:Arrow2-Forward');
					$j(this).find('.transferActionDiv').append(this.inventory_transfer);
				}
			}
		}
	},	
	_siteStoreFlyout: function() {
		var filterRecord = this.record;
		var pos = $j(this.$.maximoAppHeader.$.main).position();
		this.siteStore = $M.showFlyout('site-store','maximo-reservation-sitestore-filter', null,{'hideFooter':true,'sitestoreschema':this.sitestoreschema,'invresData': filterRecord,'top': (pos.top + $j(window).scrollTop()),'width':'100%','height':'169px', 'flyoutowner':this});
		var siteStoreDialog = this.siteStore.$.flyout;
		var flyoutWidth = $j(window).width() > 667 ? '506px': '100%';
		
		this.async(function(){
			$j(siteStoreDialog).find('.internal').css({'justify-content':'left', 'padding-left': '15px', 'padding-top': '0px', 'height':'169px'});				
			$j(siteStoreDialog).css({'right':'0','margin':'0 auto','width': flyoutWidth, 'left': 'unset','box-shadow': '0 2px 4px 0 rgba(0,0,0,0.5)'});			    			
			$j(siteStoreDialog).find('input').css('padding', '3px 0px');
			$j(siteStoreDialog).find('#internal').css({'padding': '1px 0 0'});
		}, 100);
	},
	
	_fromtoDateFilterUpdate: function(e){
		this._fromtoDateValueFormat();
		
		if(this.$.fromdate.value !== this.fromDateVal){
			this.$.fromdate.value = this.fromDateVal;
		}
		if(this.$.todate.value !== this.toDateVal){
			this.$.todate.value = this.toDateVal;
		}
		if(this.isCollectionRefreshed) {			
			if(this.selectedTabName === 'issueditem') {
				this.$.invissueditem._refreshIssuedCollection(this.record);
			} else {
				this._refreshCollection(this.record);
			}
		}
	},
	
	_fromtoDateValueFormat: function(){
		var fromDateRangeFilter = '', toDateRangeFilter = '';
		var fromDateRangeFilterDate, toDateRangeFilterDate, fromDateVal, toDateVal;
		var dateFormat = $M.getMaxauth().whoami.dateformat.toUpperCase();
		
		if($M.isMobile()){
			dateFormat = 'YYYY-MM-DD';
		} 
		this.dateformat = dateFormat;
		if(this.$.fromdate.value){
			fromDateRangeFilterDate = new Date(new Date(moment(this.$.fromdate.value, dateFormat).format()).toISOString());
	        fromDateRangeFilter = Date.parse(fromDateRangeFilterDate);			
	    } 
		else {
          	if(this.selectedTabName === 'issueditem') {
          		fromDateRangeFilter = this._issuedItemfromDateRange(dateFormat);
    		} else{
    			fromDateRangeFilter = this._reservationfromDateRange(dateFormat);
    		}
	    }
		if(this.$.todate.value){
			toDateRangeFilterDate = new Date(new Date(moment(this.$.todate.value, dateFormat).format()).toISOString());
	        toDateRangeFilter = Date.parse(toDateRangeFilterDate);			
	    } else {
	    	toDateRangeFilterDate = new Date(new Date(moment().format(dateFormat)).toISOString());
	      	toDateRangeFilter = Date.parse(toDateRangeFilterDate);
	    }
		
		if(toDateRangeFilter){
		    toDateVal = moment(toDateRangeFilter).format(dateFormat);
		}
		if(fromDateRangeFilter){
		    fromDateVal = moment(fromDateRangeFilter).format(dateFormat);
		}
		
		this.fromDateVal = fromDateVal;
		this.toDateVal = toDateVal;
		this.record.fromDateVal =  fromDateVal;
		this.record.toDateVal = toDateVal;		
	},
	
	_reservationfromDateRange: function(dateFormat) {
		var fromDateReservation = new Date(new Date(moment().format(dateFormat)).toISOString());
      	return Date.parse(fromDateReservation);
	},
	
	_issuedItemfromDateRange: function(dateFormat) {
    	var fromDateIssuedItem = new Date(new Date(moment().subtract(1, 'days').format(dateFormat)).toISOString());
      	return Date.parse(fromDateIssuedItem);
	},
	
	collectionOrderByChanged: function(e) {
		e.target.collection.refreshRecords();
	},
       
     /**
	 * Displays overlay on card to undo/CLOSED action.
	 */		
	_close: function (e) {                       
		this.$.reservationActionsOverlay.close();
	},
	
	_editReservation: function(e){
		var self = this;
		$j(this.$.reservationActionsOverlay).hide();
		var tableOffset;
		var updatedBalanceSlider = $j(this.$.reservation_list_table).find('.table');
		var tableOffsetLeft = updatedBalanceSlider.offset().left;
		var tableRowWidth = updatedBalanceSlider.innerWidth() - 5;
		tableOffset =  this.setRowOffset;
		tableOffset = (parseFloat(tableOffset)+11)+ 'px';
		
		this.updateStoreDialog = $M.showFlyout('editReservationDetail', 'maximo-reservationedit-dialog',  null, {'hideFooter':true, 'updateResvRowData': this.rowRecord ,'constrainWidth':true, 'top':tableOffset});
        var flyoutDiv = $j(this.updateStoreDialog.$.flyout);
        this.async(function(){
        	flyoutDiv.css({'left': tableOffsetLeft + 'px', 'width': tableRowWidth, 'box-sizing': 'border-box'});
        	flyoutDiv.find('#editStoreDialog').css({'width': tableRowWidth});
            flyoutDiv.find('#internal').css({'padding': '1px 0 0','justify-content': 'left','border':'2px solid #4b8400'});
        },100);
        var tableRow =  $j('#reservation_list_reservation_list_table_tablerow_' + this.rowRecord._id)[0];
        
        if(!this.rowRecord['_expanded_']){
              this.expendTarget='actions-button';
              var twisty = tableRow.querySelector('maximo-button[data-twisty]');
              twisty.fire('tap');
  	    }else{
              this.expendTarget='data-twisty';
        }
        this.async(function(){
              var clone = $j(tableRow).next().find('.tabledetail').clone();
              $j(clone).css({'border-top':'1px solid #d0cac8fa'});
              $j(clone).find('.detailWrapper').css({'margin':'12px 40px','display':'inline-block','max-width':'200px'});
              $j(clone).find('.detaillabel').css({'width':'150px','font-size':'14px'});
              $j(clone).find('.notEntered').css({'font-style': 'italic'}); 
              flyoutDiv.find('.detailsrecord').append(clone);
        },100);
        $M.toggleDisabled([$M.workScape.querySelector('#workscape_headerbar'),$M.workScape.querySelector('#navbar_hamburger')], true);
	},
	
	_closedetails: function(e){
		var tableRow =  $j('#reservation_list_reservation_list_table_tablerow_' + this.rowRecord._id)[0];
		if(this.rowRecord['_expanded_'] && this.expendTarget=='actions-button'){
			 var twisty = tableRow.querySelector('maximo-button[data-twisty]');
             twisty.fire('tap'); 
		}
		$M.toggleDisabled([$M.workScape.querySelector('#workscape_headerbar'),$M.workScape.querySelector('#navbar_hamburger')], false);
	},
	
	_deleteReservationRecord: function (e) {  
	    var self = this;
	    self._overFlowIcon();
	    var message = $M.localize('uitext','mxapiinvres','reservationdeleted',[self.rowRecord.requestnum]);
	    var performDeleteOperation = function(overlayNode) {
	      console.log('Perform delete');
	      self.fire('delete-row', self.rowRecord);
	    };
	    var _revertDeleteOperation= function () {
		  };
	    var tableRow =  $j('#reservation_list_reservation_list_table_tablerow_' + this.rowRecord._id)[0];			    
	    $M.createOverlay(tableRow, message, performDeleteOperation,_revertDeleteOperation,this.undoIntervalSecs, this.parentElement.parentElement, null, true);
	},
		
	_deleteReservation: function(e){
		var self = this;
		var reservationcollection = this.$.reservationcollection;
		this.$.reservationResource.resourceUri=this.rowRecord.href;
	
	    $M.toggleWait(true);
	    this.$.reservationResource.deleteRecord(this.rowRecord.href).then(
          function(result) {
        	  reservationcollection.refreshRecords().then(function(){
                	 $M.toggleWait(false);
                   }, function(error) {
                         $M.toggleWait(false);
                         $M.showResponseError(error);
                  });
          }, function(error) {
                 $M.toggleWait(false);
                 $M.showResponseError(error);
          });
	},
		
	/**
	 * Issue the item to storeroom for which reservation was made
	 */
	_issueReservation: function(e){
		var self = this;
		var issueInventory={};
		$j(this.$.reservationActionsOverlay).hide();
		var tableOffset;
		var reservationTable = $j(this.$.reservation_list_table).find('.table');
		var tableOffsetLeft = reservationTable.offset().left;
		var tableOffsetWidth = $j(reservationTable).innerWidth() - 8;
		if(window.navigator.userAgent.toLowerCase().indexOf('firefox') > -1){
			tableOffsetWidth = $j(reservationTable).innerWidth() - 16;
		}
		var pos = reservationTable.find('#tableFilterRow').length ? reservationTable.find('#tableFilterRow')[0].getBoundingClientRect() : reservationTable.find('.thead')[0].getBoundingClientRect();
		tableOffset =  pos.bottom + $j(window).scrollTop();
		var reservationresource = this.$.reservationResource;
		reservationresource.resourceUri = this.rowRecord.inventory[0].matusetrans_collectionref;
		reservationresource.updateAction('new', null, '*', 'GET').then(
    		function(result){
    			if(result.response) {
    				issueInventory = result.response;
    				self.reservationIssueFlyout = $M.showFlyout('reservationIssue', 'maximo-reservation-issue-flyout',  null, {'hideFooter':true ,'issueItemRowRecord': self.rowRecord, 'issueInventory': issueInventory ,'constrainWidth':true, 'top':tableOffset, 'flyoutOwner':self, 'batchIssue':self.batchReserveIssue, 'reservationIndex':self.currReservationIndex});
    				self.async(function(){
    					var flyoutDiv = $j(self.reservationIssueFlyout.$.flyout);
        				flyoutDiv.css({'left': tableOffsetLeft + 'px', 'width': tableOffsetWidth, 'box-sizing': 'border-box'});
        				flyoutDiv.find('#internal').css({'padding': '1px 0 0','justify-content': 'left'});
        				flyoutDiv.find('.reservationIssue').css('width', tableOffsetWidth+'px');
    				}, 100);
    			}
    		},
    		function(error){
    			$M.toggleWait(false);
	            $M.showResponseError(error);
	        });
	},
		
	_displayDetailAttributes: function(){		
		var	detailobj = [
			{
				'attribute' : 'assetnum',			
				'label' : [$M.localize('uitext','mxapiinvres', 'assetnum')],
				'optional': false
				
			},
			{
				'attribute' : 'displaytaskid',				
				'label' : [$M.localize('uitext','mxapiinvres', 'displaytaskid')],
				'optional': false
				
			},
			{
				'attribute' : 'storelocsiteid',				
				'label' : [$M.localize('uitext','mxapiinvres', 'storelocsiteid')],
				'optional': false
				
			},
			{
				'attribute' : 'location',				
				'label' : [$M.localize('uitext','mxapiinvres', 'location')],
				'optional': false
				
			},
			{
				'attribute' : 'itemnum',				
				'label' : [$M.localize('uitext','mxapiinvres', 'itemnum')],
				'optional': false
				
			},
			{
				'attribute' : 'oplocation',				
				'label' : [$M.localize('uitext','mxapiinvres', 'oplocation')],
				'optional': false
				
			},
			{
				'attribute' : 'conditioncode',				
				'label' : [$M.localize('uitext','mxapiinvres', 'conditioncode')],
				'optional': false
				
			},
			{
				'attribute' : 'newsite',				
				'label' : [$M.localize('uitext','mxapiinvres', 'newsite')],
				'optional': false
				
			},
			{
				'attribute' : 'tostoreloc',				
				'label' : [$M.localize('uitext','mxapiinvres', 'tostoreloc')],
				'optional': false
				
			},
			{
				'attribute' : 'ponum',				
				'label' : [$M.localize('uitext','mxapiinvres', 'ponum')],
				'optional': true
				
			},
			{
				'attribute' : 'mrnum',				
				'label' : [$M.localize('uitext','mxapiinvres', 'mrnum')],
				'optional': true
				
			},
			{
				'attribute' : 'wonum',
				'label' : [$M.localize('uitext', 'mxapiinvres', 'refworkorder')],
				'optional': true
			}
		]
		return detailobj;
	},

	_displayItemDetail: function(e){
		$M.toggleWait(true);  	
    	var self=this;
    	this.domHost.addContent('invItemList');
    	var itemcollection = $M.getGlobalResource('itemcollection');
    	var itemnum;
    	
    	if(e.detail.model.record){
    		itemnum=e.detail.model.record.itemnum;
    	}
    	
    	var additionalparams=[];
			additionalparams.push('oslc.where=itemnum=%22'+itemnum+'%22');
			itemcollection.additionalParams=additionalparams;
			itemcollection.searchTermValue='';
			itemcollection.collectionUri='';
			itemcollection.refreshRecords().then(function(result){
			if(result.response){
				$M.setSubRoute('inventoryItems/itemDetail');
				self.fire('display-itempage',{'pagename':'itemDetail','record':result.response.member[0],'collection':itemcollection,'isRedirected': 'Reservation','redirectedRecord':self.redirectedRecord,'selectedTabName':self.selectedTabName,'schema':itemcollection.schema});
				$M.toggleWait(false);  	
			}
		});		 
	},
	
	_handleConditionalFormatting: function(e) {
        var record = e.detail.record;
        var datacolumn = e.detail.datacolumn;
        var dateformat = 'YYYY-MM-DD';
        var currentDate = new Date(moment().format(dateformat));
        var styleObject = JSON.parse(datacolumn.conditionalformatting);
        var columnData;
        var attribute = styleObject.attribute? styleObject.attribute : datacolumn.attribute;
        if(attribute){
            columnData = $M.getRelatedData(record, attribute);
        }           
        if(datacolumn.type && datacolumn.type==='date' && columnData){
            columnData = new Date(moment(columnData).format(dateformat));
            if(columnData < currentDate) {
                this.async(function(){
                	$j('#tabledata_'+attribute+record._id.split('-').join('_')).css({'color':this.getComputedStyleValue('--Primary-orange50'), 'font-weight':'bold'});   
                },100);
            }
        }
    },
    
    _collectionRefreshErrorHandler: function (ev){
    	// Notify error
        var error = ev.detail.Error;
        if(error && error.message) {
        	$M.notify(error.message, $M.alerts.error);
        }            
        $M.toggleWait(false);
    },
    
    _handlePageNavigation: function(e) {
    	var name = e.currentTarget.getAttribute('name');
    	this.selectTab(name);
	},
	/**
	 * Update selectedTabName property
	 */
	selectTab: function(tabName) {
	    if (this.selectedTabName !== tabName) {
	        this.selectedTabName = tabName;
	    }
	},
	
	_loadSection: function(tabName){
        var reservation = this;
        var sectionid = tabName + '_section';
        var myRecord;
        myRecord = this.record;
        $j(this.$[sectionid]).siblings().hide();
        $j(this.$[sectionid]).show();
        switch(tabName){
            case 'issueditem':
                $M.setSubRoute('reservations/issueditem');
                this.$.invissueditem._loadIssuedItemTable(reservation);
                break;
            case 'reservation':
                $M.setSubRoute('reservations');
                this._loadReservationTable();
                break;
            default: 
            	$M.setSubRoute('reservations');
            	this._loadReservationTable();
        }
        if($j(window).scrollTop()){
            window.scrollTo(0,0);
        }
    },
	
	_collectionRefreshHandler: function(e) {
		var collectionCount;
		if(e.detail.totalCount !== undefined){
			collectionCount = e.detail.totalCount;
		}else {
			collectionCount = e.target.totalCount;
		}
		$j(this.$.total_reservation_count).html(this._getUiTextValue('mxapiinvres','result',collectionCount));
	},
	
	_applySiteStoreFilter: function(e) {
		var record;
		if(e.detail && e.detail.record) {
			record = e.detail.record;
		}		
		if(this.selectedTabName === 'issueditem') {
			 var issueRecord = {}; 
		     issueRecord = record;
			 sessionStorage.setItem('issueRecord', JSON.stringify(issueRecord));
			 this.$.invissueditem._refreshIssuedCollection(record);
		}else {
			 var reservationRecord = {};
			 reservationRecord = record;
			 sessionStorage.setItem('reservationRecord', JSON.stringify(reservationRecord));
			 this._refreshCollection(record); 
		}
	},
	
	_selectReservationRow: function() {
		this.$.selected_record_count.value = this.$.reservation_list_table.selectedRecords.length;
		this._showMultiSelect = (this.$.reservation_list_table.selectedRecords.length > 0 );
		var table = this.$.reservation_list_table;
		var filterRow = table.$$('#tableFilterRow');
		var tableHeader = $j(table).find('.tableheader');
		var self = this;
		if(this._showMultiSelect) {
			$j(this.$.contentSelector).attr('add-class','disableLink');
			$j(table).find('.tableactions').find('div').hide();
			$j(table).find('#filter_toggle').attr('disabled',true);
			if(filterRow) {
				$j(filterRow).find('maximo-text').attr('readonly', true);
				$j(filterRow).find('maximo-button').attr('disabled', true);
			}			
		} else {
			$j(this.$.contentSelector).removeAttr('add-class');
			$j(table).find('.tableactions').find('div').show();
			$j(table).find('#filter_toggle').attr('disabled',false);
			if(filterRow) {
				$j(filterRow).find('maximo-text').attr('readonly', false);
				$j(filterRow).find('maximo-button').attr('disabled', false);
			}
		}
	},
	
	_exitMultiSelect: function(){
        this._showMultiSelect = false;
        this.$.reservation_list_table.selectAllRecords(false);
        this.batchReserveIssue = false;
        this.$.reservationcollection.refreshRecords().then(function(){
        	$M.toggleWait(false);
        });
    },
    
    _deleteSelected: function(){
    	var bulkReservation = [];
    	var reservationRecord = {};
    	var requestHeaders = {};
    	var self = this;
    	var reservationcollection = this.$.reservationcollection;
    	var selectedReservations = this.$.reservation_list_table.selectedRecords;
    	var deletionMsg = $M.localize('uitext', 'mxapiinvres', 'deletemsg', [selectedReservations.length]);
    	var deleteAction = true;
    	
    	if(selectedReservations.length > 0){
    		selectedReservations.forEach(function(reservation, index){
    			reservationRecord = {};
    			reservationRecord.href = reservation.href;
    			reservationRecord._action = 'Delete';
    			bulkReservation.push(reservationRecord);
    		});
    	}
    	
    	requestHeaders.allornothing = true;
		requestHeaders.method = 'BULK';
		var responseProperties = reservationcollection.attributeNames;
		this.$.reservationResource.resourceUri = this.$.reservation_list_table.selectedRecords[0].href.slice(0, this.$.reservation_list_table.selectedRecords[0].href.indexOf('mxapiinvres')+11);
		
		var undoCallback = function(){
    		$j(deletenotification).remove();
    		self._showMultiSelect = true;
    		deleteAction = false;
    	};
    	
    	this._showMultiSelect = false;
		var deletenotification = $M.notify(deletionMsg, $M.alerts.info, null, undoCallback, false);
		
		this.async(function(){
			$j(deletenotification).remove();
	    	if(deleteAction){
	    		this.deleteBatch(bulkReservation, this);
	    	}
		}, (this.undoIntervalSecs*1000));
    },
    
    deleteBatch: function(bulkReservation, self){
    	selfobj = self;
    	$M.toggleWait(true);
    	self.$.reservationResource.updateAction(null, bulkReservation, null, 'BULK').then(
			function(result) {
				if(result.response && result.response.length 
						&& result.response[0]._responsedata && result.response[0]._responsedata.Error) {
					$M.alert(result.response[0]._responsedata.Error.message, $M.alerts.warn);
				} else {
					selfobj._refreshCollection(self.record);
				}
				$M.toggleWait(false);
				selfobj._exitMultiSelect();
			 }, function(error) {
		         $M.showResponseError(error);
		         $M.toggleWait(false);
		         selfobj._exitMultiSelect();
	     		});
    },
    
    _issueSelected: function(){
    	var selectedReservations = this.$.reservation_list_table.selectedRecords;
    	this.selectedRow = $j(this.$.reservation_list_table).find('.tablerow')[0];
    	if(selectedReservations.length){
    		this.currReservationIndex = 0;
    		this.rowRecord = selectedReservations[this.currReservationIndex];
    		this.batchReserveIssue = true;
    		if(this.rowRecord.tostoreloc){
    			this._transferReservationRecord();
    		} else {
    			this._issueReservation();
    		}
    	}
    },
    
    _displayNextReservation: function(e){
    	var selectedReservations = this.$.reservation_list_table.selectedRecords;
    	var currentIndex = e.detail.reservationIndex;
    	if(currentIndex <= selectedReservations.length){
    		this.currReservationIndex = currentIndex;
    		this.rowRecord = selectedReservations[this.currReservationIndex];
    		this.batchReserveIssue = true;
    		if(this.rowRecord.tostoreloc){
    			this._transferReservationRecord();
    		} else {
    			this._issueReservation();
    		}
    	}
    },
    
    _setActionValue: function(){
    	var actionsvalue = {};
    	actionsvalue.header = {};
    	actionsvalue.header.width = window.innerWidth/5+ 'px';
    	actionsvalue.list = [];
    	actionsvalue.list[0] = {};
    	if(this._isSigOptionGranted('MXAPIINVRES', 'ISSUERESERVATION')) {
    		this.buttonCount++;
    		this.actionFlag = 'issue';
    	}
		if(this._isSigOptionGranted('MXAPIINVRES', 'EDITRESERVATION')) {
			this.buttonCount++;
			this.actionFlag = 'edit';
		}
		if(this._isSigOptionGranted('MXAPIINVRES', 'DELRESERVATION')) {
			this.buttonCount++;
			this.actionFlag = 'delete';
		}
		
		if(this.buttonCount > 0) {
			if(this.buttonCount === 1) {
				switch(this.actionFlag) {
				case 'issue':
					actionsvalue.list[0].icon = 'Maximo:Arrow2-Forward';
					actionsvalue.list[0].label = $M.localize('uitext', 'mxapiinvres', 'issue');
					actionsvalue.list[0].identifier = 'issueAction';
					break;
				case 'edit':
					actionsvalue.list[0].icon = 'Maximo:Edit';
					actionsvalue.list[0].label = $M.localize('uitext', 'mxapibase', 'edit');
					actionsvalue.list[0].identifier = 'updateAction';
					break;
				case 'delete':
					actionsvalue.list[0].icon = 'Maximo:Trash';
					actionsvalue.list[0].label = $M.localize('uitext', 'mxapibase', 'Delete');
					actionsvalue.list[0].identifier = 'deleteAction';
					break;
				}
				this.set('actions', actionsvalue);
			} else {
				this.set('actions', this.actions);
			}
		} else {
			this.set('actions', '');
		}
    },
    
    _transferReservationRecord: function() {
        var self = this;
        var schema = '';
        var transferInventory={};
        $j(this.$.reservationActionsOverlay).hide();
        var tableOffset;
        var reservationTable = $j(this.$.reservation_list_table).find('.table');
        var tableOffsetLeft = reservationTable.offset().left;
        var tableOffsetWidth = $j(reservationTable).innerWidth() - 8;
        var pos = reservationTable.find('#tableFilterRow').length ? reservationTable.find('#tableFilterRow')[0].getBoundingClientRect() : reservationTable.find('.thead')[0].getBoundingClientRect();
        tableOffset =  pos.bottom + $j(window).scrollTop();
         
        if(this.reservationschema && this.reservationschema.properties){
            schema = this.reservationschema.properties.inventory.items.properties.matrectrans.items;
        }
        var reservationresource = this.$.reservationResource;
        reservationresource.resourceUri = this.rowRecord.inventory[0].matrectrans_collectionref;
        reservationresource.updateAction('new', null, '*', 'GET').then(
           function(result){
               if(result.response) {
                   transferInventory = result.response;
                   self.reservationTransferFlyout = $M.showFlyout('reservationItemTransfer', 'maximo-reservation-itemtransfer',  null, {'hideFooter':true ,'transferItemRowRecord': self.rowRecord, 'transferInventory': transferInventory, 'constrainWidth':true, 'top':tableOffset, 'flyoutOwner':self, 'batchIssue':self.batchReserveIssue, 'reservationIndex':self.currReservationIndex, 'schema':schema});
                   self.async(function(){
                       var flyoutDiv = $j(self.reservationTransferFlyout.$.flyout);
                       flyoutDiv.css({'left': tableOffsetLeft + 'px', 'width': tableOffsetWidth, 'box-sizing': 'border-box'});
                       flyoutDiv.find('#internal').css({'padding': '1px 0 0','justify-content': 'left'});
                       flyoutDiv.find('.updatestoreinput').css('width', tableOffsetWidth+'px');
                   }, 100);
               }
           },
           function(error){
               $M.toggleWait(false);
                $M.showResponseError(error);
           });
    },
    _positionOverlay: function(e){
		var windowHeight = $j(window).height() - 150;
		var pos = this.overlayPosition.butonpos;
        var topLastItems = pos.top + 50;
        if((windowHeight+$j(window).scrollTop()) < topLastItems){
            $j('.reservationActionsBottomDiv').addClass('add-pos-top');
            $j('.storeroomActionsTopDiv').addClass('add-remove-border');
            if(this.buttonCount === 2) {
            	$j(this.$.reservationActionsBottomDiv).addClass('adjust-top');
            }
        } else {
        	$j('.reservationActionsBottomDiv').removeClass('add-pos-top');
        	$j(this.$.reservationActionsBottomDiv).removeClass('adjust-top');
            $j('.storeroomActionsTopDiv').removeClass('add-remove-border');
        }
        $j(this.$.reservationActionsOverlay).css({'left':this.overlayPosition.left,'top':this.overlayPosition.top});
	}
});