/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2018
 */
Polymer( {

	is : 'maximo-map',
	behaviors : [
			MxMap, MxLayer
	],
	properties : {
		maximoMap : Object,
		currentView : ol.View,
		mapReady : Boolean,
		layers : {
			type : Array
		},
		securityMapManager : {
			type : Object
		},
		mapName : {
			type : String
		},
		tempLayers : {
			type : Array
		},
		securityTokens : {
			type : Object
		},
		mapManager : {
			type : Object,
			value : null
		},
		toolsOpened : {
			type : Array
		},
		show: {
			type: Boolean,
			observer: '_onShowChanged'
		}
	},

	_onShowChanged: function() {
		if (this.maximoMap && this.maximoMap.map) {
			setTimeout(()=> {
				this.maximoMap.map.updateSize();	
			}, 150);
			
		}
		
	},
	
	ready : function () {

		// Check if the user has spatial license
		let licenses = $M.getLicenses();
		if ( !licenses.SPATIAL ) {
			$M.notify( $M.localize( 'messages', 'map', 'nospatialmap', null ), $M.alerts.error );
			return;
		}

		// Hide the footer to maximize the map.
		var footer = $M.workScape.querySelector( '#workscape_footer' );
		footer.style.display = 'none';

		this.addEventListener( '_tokensHandle', this._tokensHandle );

		let proxy = this.$.myauthenticator.getBaseUri() + "/webclient/pluss/proxy.jsp?";
		MxMap.setProxyURL( proxy );
		var mapManFilterData = [
				{
					'filtertype' : 'SIMPLE',
					'field' : 'mapsites.siteid',
					'availablevalues' : []
				}, {
					'filtertype' : 'SIMPLE',
					'field' : 'active',
					'availablevalues' : []
				}, {
					'filtertype' : 'SIMPLE',
					'field' : 'ismobile',
					'availablevalues' : []
				}
		];
		var siteFilterCondition = {};
		if ( !$M.getMaxauth().whoami.defaultSite ) {
			this.noActiveMap = this.localize( 'messages', 'mapcontrol', 'nullusersite', null );
			this.set( 'isNoActiveMap', true );
			this.set( 'selected', 3 );
			return;
		}
		// Filter Map Manager records considering the user site
		siteFilterCondition.value = $M.getMaxauth().whoami.defaultSite;
		siteFilterCondition.selected = true;
		mapManFilterData[ 0 ].availablevalues.push( siteFilterCondition );
		// Filter active Map Manager records
		var activeFilterCondition = {};
		activeFilterCondition.value = 1;
		activeFilterCondition.selected = true;
		mapManFilterData[ 1 ].availablevalues.push( activeFilterCondition );

		// Filter non-mobile Map Manager records
		var mobileFilterCondition = {};
		mobileFilterCondition.value = '0';
		mobileFilterCondition.selected = true;
		mapManFilterData[ 2 ].availablevalues.push( mobileFilterCondition );

		this.set( 'mapManagerFilterData', mapManFilterData );
		this.$.mapmanagercollection.refreshRecords();
	},

	_tokensHandle : function ( tokens ) {
		this.securityTokens = tokens.detail;
		if ( this.maximoMap == null ) {
			this.initialConfig();
		} else {
			// Refresh token for each layer
			var layers = this.maximoMap.map.getLayers();
			layers.forEach( function ( layer ) {
				var source = layer.getSource();
				var urls = [];
				if ( typeof source.getUrls !== "undefined" ) {
					urls = source.getUrls();
				}
				if ( urls.length > 0 ) {
					var url = urls[ 0 ];
					var indexProxy = url.indexOf( MxMap.getProxyURL() );
					// Remove proxy to get the new token
					if ( indexProxy > -1 ) {
						url = url.replace( MxMap.getProxyURL(), "" );
					}
					var tokenJson = this.securityTokens[ url ];
					if (tokenJson) {
						var tokenValue = tokenJson.token
						if ( tokenValue != null && tokenValue != "" ) {
							params = {
								"token" : tokenValue
							};
							console.log( 'layer to refresh token ', url );
							console.log( 'new token generated ', tokenValue );
							source.updateParams( params );
						}
					}
					

				}

			}, this )
		}
	},

	/**
	 * Method to create an OpenLayer's layer using the MapService record from MapManager
	 */
	_createArcgisLayer : function ( mapService, projection ) {

		var mapServiceUrl = mapService.url;
		var tokenJson = null;
		var tokenValue = null;
		var tokenSecurityEnabled = false;
		if ( this.securityTokens && this.securityTokens[ mapServiceUrl ] ) {
			tokenJson = this.securityTokens[ mapServiceUrl ];
			tokenValue = tokenJson.token;
			tokenSecurityEnabled = this.securityTokens[ mapServiceUrl ].token != "";
		}

		console.log( '[maximo-map] using proxy? ', usingProxy );

		var url = mapServiceUrl + '?f=json';
		if ( tokenValue != null && tokenValue != "" ) {
			console.log( "Token generated for URL: ", mapServiceUrl );
			console.log( "Token value: ", tokenValue );

			url = url + '&token=' + tokenValue;
		}
		this.$.xhr.url = url;

		var usingProxy = MxMap.getMapManager() && MxMap.getMapManager().useproxy == true;
		if ( usingProxy ) {
			mapServiceUrl = MxMap.getProxyURL() + mapServiceUrl;
		}

		return MxLayer.createArcgisLayer( mapService, mapServiceUrl, projection, tokenSecurityEnabled, tokenValue );

	},

	handleRequestResponse : function ( response ) {
		var responseJson = response.detail.response;
		if ( responseJson && responseJson.error ) {
			var url = response.detail.url;
			url = url.replace( '?f=json', '' );
			var servicename;
			if ( this.tempLayers && this.tempLayers[ url ] && this.tempLayers[ url ].servicename ) {
				servicename = this.tempLayers[ url ].servicename;
			}
			if ( servicename == null ) {
				servicename = url;
			}
			var errorMessage = responseJson.error.code + ' - ' + responseJson.error.message;
			$M.notify( $M.localize( 'messages', 'map', 'errorLoadingService', [
					servicename, errorMessage
			] ), $M.alerts.error );
		}
	},
	_loadLayersForTokenSecurity : function () {
		console.log( '[maximo-map] _loadLayersForTokenSecurity ' );
		this.set( 'securityMapManager', MxMap.getMapManager() );
	},

	/**
	 * Method to initialize the openlayer's map
	 */
	initialConfig : function () {
		this.tempLayers = {};
		var mapManager = MxMap.getMapManager();
		console.log('initial SPATIAL config for the map.');
		console.log('Map manager used: ', mapManager);
		var ol_layers = []; 
		var layers = mapManager.plussmapservice || [];
		if (layers && layers.length > 0) {
			layers.sort(function (a, b) {
				return b.serviceorder - a.serviceorder;
			});
		}
		
		let projection;

		/*
		 * Looks for the basemap spatial reference to use it on the view definition.
		 */
		layers.forEach( function ( layer ) {
			var plussSpatialReference = layer.plussSpatialReference;
			if (plussSpatialReference) {
				MxMap.loadSpatialReference(plussSpatialReference);
			}
			if ( layer.spatialreference && layer.istiledlayer == true )
				projection = 'EPSG:' + layer.spatialreference;
		}, this );
		
		// If projection is null, get it from the basemap created on webmap
		if (projection == null && mapManager.webmapenabled) {
			var webmapContent = mapManager.webMapContent;
			var baseMapLayers = webmapContent.baseMap.baseMapLayers;
			if (baseMapLayers.length > 0) {
				projection = 'EPSG:' + baseMapLayers[0]._spatialReference;
			}
		}
		
		

		var siteId = $M.getMaxauth().whoami.defaultSite;
		console.log( 'Searching for initial/full location for site, if not found the zoom 14 is the default: ', siteId );
		var centerX;
		var centerY;
		var initialZoom = 14;
		var mapSites = MxMap.getMapManager().mapsites;
		mapSites.forEach( function ( mapSite ) {
			if ( mapSite.siteid == siteId ) {
				if ( mapSite.zoomlevel ) {
					initialZoom = mapSite.zoomlevel;
				}
				if ( mapSite.initialx && mapSite.initialy ) {
					centerX = mapSite.initialx;
					centerY = mapSite.initialy;
				} else if ( mapSite.plussfullextx && mapSite.plussfullexty ) {
					centerX = mapSite.plussfullextx;
					centerY = mapSite.plussfullexty;
				}

			}
		}, this );

		/*
		 * The projection is then used to create the view. The view's projection will be reused to create all the arcgis
		 * layers.
		 */
		let currentView = MxMap.createInitialView( projection, initialZoom, layers );

		console.log('layers to load : ', layers);
		layers.forEach(function (layer) {
			if (!layer.webmapdefined) {
				this.tempLayers[layer.url] = layer;
				var newLayer = this._createArcgisLayer(layer, currentView.getProjection());
				ol_layers.push(newLayer);
			}
		}, this);
		
		

		/**
		 * If no initial/full extent location was found, the map will use the GPS location. If the user does not allow
		 * the GPS tracking, the worldwide map will be shown
		 */
		if ( !centerX && !centerY ) {
			var geolocation = new ol.Geolocation( {
				projection : currentView.getProjection()
			} );
			geolocation.setTracking( true );

			geolocation.on( 'change',  ()=> {
				var position = geolocation.getPosition();
				console.log( 'GPS position found ', position );
				this.maximoMap.setCenter( position[ 0 ], position[ 1 ] );
				geolocation.setTracking( false );
			}, this );

			geolocation.on( 'error', ( error )=> {
				console.error( error );
				geolocation.setTracking( false );
				this.maximoMap.map.getView().setCenter( ol.proj.transform( [
						0, 0
				], 'EPSG:4326', currentView.getProjection() ) );
				this.maximoMap.map.getView().setZoom( 2 );
			}, this );

			centerX = 0;
			centerY = 0;
		}
		console.log( 'Params: ' );
		console.log( 'zoom: ', initialZoom );
		console.log( 'centerX: ', centerX );
		console.log( 'centerY: ', centerY );
		
		var mapLocal = MxMap.createMap(
			ol_layers,
			this.$.map
		);
		
		if (mapManager.webmapenabled) {
			MxLayer.loadWebMapArcgisLayers(mapManager, mapLocal, currentView.getProjection());
			
			if (mapManager.mapServicesOrder && mapManager.mapServicesOrder.length > 0) {
				MxMap.reorderLayers();
			}
		}
		
		
		mapLocal.setCenter(centerX, centerY);
		
		var zoomControl = $M.workScape.querySelector( '.ol-zoom' );
		zoomControl.style.top = '60px';
		zoomControl.style.backgroundColor = '#133947';
		
		this.maximoMap = mapLocal;

	},

	/**
	 * Method used to handle the results from the Map Manager rest service
	 */
	_handleMapManagerDataRefreshed : function () {
		console.log( 'Loading map manager data ', this.mapManagerData );

		if ( this.mapManagerData.length == 0 ) {
			var siteId = $M.getMaxauth().whoami.defaultSite;
			console.error( 'No MapManager found for mapsite: ' + siteId );
			this.set( 'showDataError', true );
			$M.notify( $M.localize( 'messages', 'map', 'mapManagerNotFound', [
				siteId
			] ), $M.alerts.error );
			this.mapReady = false;
		} else {
			// Search for a SPATIAL map manager
			let mapManager = null;
			this.mapManagerData.forEach( function ( mapManager ) {
				if ( mapManager.mapprovider == 'SPATIAL' ) {
					MxMap.setMapManager( mapManager );
				}
			}, this );

			if ( MxMap.getMapManager() == null ) {
				$M.notify( $M.localize( 'messages', 'map', 'mapManagerNotFoundProvider', [] ), $M.alerts.error );
			} else {
				var tokenEnabled = MxMap.getMapManager().spatialtokensecurity;
				if ( tokenEnabled ) {
					/**
					 * The function _tokenHandle will be fired by maximo-security component when the tokens are ready to
					 * use, be careful, the same function will be fired every time the token expired
					 */
					console.log( 'Token Security enabled' );
					this._loadLayersForTokenSecurity();
				} else {
					this.set( 'showDataError', false );
					this.mapReady = true;
					this.async( function () {
						this.initialConfig();
					} );
				}
			}
		}
	},

} );
