/*
* Licensed Materials - Property of IBM
* 5724-U18
* Copyright IBM Corporation. 2016,2019
*/
Polymer({
	is: 'maximo-sr-card',
  	behaviors: [BaseComponent, HTMLEncoder],
  	
    properties: {
      	sr: {
      		type: Object,
      		notify: true,
      		observer : '_srChanged'
      	},
       
      	attachmentCounter: {
      		type: Object
      	},
      	
      	srURI: {
      		type: Number,
      		notify: true
      	},
      	
      	commentCounter: {
      		type: Number,
      		notify: true
      	},
      	isHistory: {
      		type: Boolean,
      		notify: true,
      	},
      	isDesktop: {
			type: Boolean,
			value: false,
			notify: true
		},
		
		_cssCard: {
			type: String,
			notify: true,
			computed: '_getCardCss(isDesktop)'
		},
		
		/**
		 * List of attributes must be loaded
	    */
		attributeList: {
			type: String,
			value: 'status,status_description,status_maxvalue,person._imagelibref,reportedbyname,reportdate_localized,reportdate,ticketid,description,description_longdescription,reportedpriority,reportedpriority_description,targetfinish,ownerperson.displayname,ownerperson._imagelibref,assetnum,asset.assetnum,asset.description,asset._imagelibref,location.description,location.location,location._imagelibref,classstructure.description_class,classstructure._imagelibref,tickect_template.instractions_longdescription,tickect_template.instractions'
		},	
	},
	

	
	/**
	 * srResource mx-resource 
	 * loadRecord success callback
	 */
	_recordLoadSuccessHandler: function(event) {
	    $M.toggleWait(false); 
	},
	
	/**
     * srResource mx-resource 
     * loadRecord error callback
     */
	_recordLoadErrorHandler: function(event) {
	    $M.toggleWait(false);
    },
    
    
    /**
	 * srResource mx-resource 
	 * updateRecord success callback
	 */
	_recordUpdatedSuccessHandler: function(event) {
	   $M.toggleWait(false);
	},
	
	
    /**
     * srResource mx-resource 
     * updateRecord error callback
     */
	_recordUpdatedErrorHandler: function(event) {
	    $M.toggleWait(false);
    },
	
    _loadDetailedRecord: function(sr) {
    	$M.toggleWait(true);
    	this.srURI=sr.href
    	this.$.srResource.loadRecord();
 			 $M.toggleWait(false);
    	return '';
	},
    
    
	attached: function () {
		var self = this;
		$j(window).on('resize', function(){
			 self._windowResize();
	    });
		
		this.async(function(){
			self._windowResize();
		}, 100);
	},
	
	_srChanged: function(newValue) {
		$j(this.$.viewSRCard).toggleClass('song2', false);
		 
		this._showPriority(newValue);
		this.updateCounters();
	},
	
	
	/**
	 * Hide priority section if no priority is defined on the record.
	 */
	_showPriority: function(sr){
		var priorityNode = this.$.priorityLabel;
		priorityNode.classList.remove('invisible');
		if(sr.reportedpriority){
			if(sr.reportedpriority === 1){
				priorityNode.classList.remove('invisible');
			}
			else{
				priorityNode.classList.add('invisible');
			}
		} else {
			priorityNode.classList.add('invisible');
		}
	},
	setPriorityLabel : function(){
		return $M.localize('uitext', 'mxapisr', 'UrgentRequest').slice(0, -1);
	},
	_showLocation : function(sr) {
		var locationLabel = this.setLocation(sr);
		if (!locationLabel || locationLabel.length === 0) {
			return 'display:none;';
		}
		return '';
	},
	getLocaleDate : function(date){
		var defaultlocale = ($M.getMaxauth().whoami.defaultLcale).split('_').join('-');
		var language = defaultlocale?defaultlocale:navigator.language;
		if(date){
		    var options = { weekday: 'short', year: 'numeric', month: 'long', day: 'numeric' };
			return new Date(date).toLocaleDateString(language, options);
		}
	},
	getCompletionDate: function(sr) {
		if (sr) {
			if (sr.actualfinish) {
				return this.getLocaleDate(sr.actualfinish);
			}
			else {
				return this.getLocaleDate(sr.statusdate);
			}
		}
	},
	_getDescription : function(sr){
		var description = sr.description?sr.description:'';
		return description;
	}, 
	_getReportedInfo: function(sr){
		var user = '';
		if(sr.reportedby !== $M.getMaxauth().whoami.userId){
			user = sr.reportedby;
		}
		else {
			user = $M.localize('uitext','maxpisr','you');
		}
		return $M.localize('uitext','mxapisr','CreatedByOn',[user, this.getLocaleDate(sr.reportdate)]);
	},
	setLocation : function(sr){
		var locationLabel;
		if(sr.location.location){
			locationLabel = sr.location.location + ' ';
		}
		else{
			locationLabel = '';
		}
		if(sr.location.description){
			locationLabel = locationLabel + sr.location.description;
		}
		else{
			locationLabel+='';
		}
		if(sr.serviceaddress.description){
			if(locationLabel !==''){
				locationLabel = locationLabel + ' - ' + sr.serviceaddress.description;
			}
			else{
				locationLabel = sr.serviceaddress.description;
			}
		}
		return locationLabel;
	},
	getCancelButton : function(sr){
		if(sr.reportedby === $M.userInfo.personid || sr.affectedperson === $M.userInfo.personid) {
			return false;
		}
		else{
			return true;
		}
	},
	getUnsubButton : function(sr){
		if(sr.reportedby === $M.userInfo.personid || sr.affectedperson === $M.userInfo.personid) {
			return true;
		}
		else{
			return false;
		}
	},
	_checkCancellable : function(sr){ 
		if(((sr.status === 'QUEUED') || (sr.status === 'NEW')) && (Object.getOwnPropertyNames(sr.relatedwo).length===0) && (Object.getOwnPropertyNames(sr.relatedticket).length ===0) && (!sr.worklog)){
			return true;
		}
		else{
			return false;
		}
	},
	onTapCancel : function(e){
		//var index = e.model.index;
		//var sr = this.recordData[index];
		e.stopPropagation();
	 
		this.fire('cancel-request',{sr:this.sr, comments:this.commentCounter});
	},
	_mapButtonClass : function(sr){ // temporary function. need to clarify the condition of showing showmap button.
		var locationLabel = this.setLocation(sr);
		if (!locationLabel || locationLabel.length === 0) {
			return 'noLocation';
		}
		return '';
	},
	updateCommentCount : function(e) {
		var eventData = e.detail;
		var index = eventData.index;
		var recordCount = eventData.recordCount;
		var comment = '';
		
		if (index === undefined) {
			return;
		}
		
		if(recordCount === 0 || recordCount === 1){
			comment = recordCount +' '+ $M.localize('uitext', 'mxapisr', 'comment');
		}
		else{
			comment =  recordCount +' '+ $M.localize('uitext', 'mxapisr', 'comments');
		}
		
		this.querySelectorAll('.commentButton')[index].label = comment;
	},
	
	showDetails: function(e) {
		if(!e.currentTarget.overlay || !e.currentTarget.overlay.contains(e.target)){
			this.fire('showdetails', "details");
		}
	},
	
	showComments: function(e) {
		e.stopPropagation();
		if(!e.currentTarget.overlay || !e.currentTarget.overlay.contains(e.target)){
			this.fire('showdetails', "comments");
		}
	},
	
	_setCommentCounter : function(sr){
		try {
			var counter = 0;
			if(sr.worklog){
				sr.worklog.forEach(function(comment){
					if(comment.clientviewable === true){
						counter = counter +1;
					}
				});
				this.commentCounter = counter;
			} else {
				this.commentCounter = 0;
			} 
		} catch(err) {
			this._displayUnexpectedError(err);
		}			
	},
	
	_getAttachmentCount : function(sr){
		return sr.doclinks.member.length;
	},
		
	_setAttachmentCounter : function(){
		var docs = this.sr.doclinks.member;
		if (docs && docs.length > 0) {
			this.attachmentCounter = docs.length;
		} else {
			this.attachmentCounter = 0;	
		}
	},
	
	_overdueClass: function(sr){
		if(!sr.targetfinish){
			return '';
		}
		return this.getLocaleDate(sr.targetfinish)<this.getLocaleDate(new Date())?'overdue':'';
	},
	_formatStatus: function(sr) {
		if (sr) {
			if (sr.status_maxvalue) {
				return sr.status_description + ' ';	
			}
		}
	},
	_updateAttachmentCount : function(e){
		var index = e.detail;
		this.querySelectorAll('.attachmentButton')[index].label = this.getAttachmentButton(this.recordData[index]);
	},
	/**
	 * triggered from ongoing card
	 */
	updateCounters: function() {
//		console.info('update counter');
//		console.log(this.sr);

		this._setCommentCounter(this.sr);
		this._setAttachmentCounter();
	},
	
	refreshCounters: function(card) {
		//commentCounter
		this.attachmentCounter = card.attachments;
		if (card.comments !== undefined) {
			this.commentCounter = card.comments;
			if(this.commentCounter>0){
				this.$.comments_active_btn.hidden=false;
				this.$.comments_btn.hidden=true;
			}
		}
	},
	
	_hideCancelInHistory: function (sr){
		if(this._isHistory(sr)){
			return true;
		}
	},
	
	_hideCommentButton: function (sr){
		this._setCommentCounter(this.sr);
		if(this.commentCounter === 0){
		return true;
		} else {
			return false;
		}
	},

	_isHistory: function(sr) {
		if (sr.status && (sr.status_maxvalue === 'RESOLVED' || sr.status_maxvalue === 'CLOSED')) {
			return true;
		}
		return false;
	},
	
	_windowResize : function(){
		this.isDesktop = !$M.isMobile();
	},
	
	_getCardCss: function(isDesktop) {
		var classCss= 'dataSetCard dataSetInternalCard';
		 if(!isDesktop){
			 classCss +=  ' mobile';
		 }
		 return classCss;
	},
	
	_getImage: function(record) {
	 
		 if(record && record.classstructure && record.classstructure._imagelibref){
			 return record.classstructure._imagelibref;
		 }
		 return undefined;
	},
	
	
});
