/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */ 
Polymer({
	is: 'maximo-sr-list-card',
	behaviors: [BaseComponent,InfiniteScroll],
	properties : {

    	/**
		 * List of attributes must be loaded
	    */
    	_attributeList: {
			type: String,
			value: 'reportedpriority,description,description_longdescription,targetfinish,actualfinish,statusdate,ticketid,status,status_maxvalue,status_description,reportedby,reportdate,location.location,serviceaddress.description,doclinks{title},worklog,relatedwo.relatedrecwonum,relatedticket.relatedreckey'	,
			notify: true
		},
		

		/**
      	 * List of attribute names used to search record  in the collection 
      	 */
		searchAttributeNames: {
			type: String,
			value: '',
			notify: true
		},
		
		recordData: {
			type: Object,
			notify: true
		},
		
		/**
      	 * total records count  
      	 */
		recordCount: {
			type: Number,
			value: -1,
			notify: true,
			reflectToAttribute:true
		},
		
		/**
      	 * Indicates the name of the query selected to be applied in the collection
      	 */
		selectedQueryName: {
			type: String,
			value: '',
			notify: true
		},
		
     	// interval in seconds
      	interval : {
      		type: Number,
      		value: 5
      	},
      	
      	/**
      	 * ticket Id used to apply highlight on Cards
      	 */
		newTicketId: {
			type: String
		},
		
		/**
      	 * ticket selected
      	 */
		selectedRecord: {
			type: Object
		},
		
		/**
      	 * Indicates if the screen is on desktop mode
      	 */
		isDesktop: {
			type: Boolean,
			value: false,
			notify: true
		},
		
		/**
      	 * css class used to display the cards
      	 */
		_cssCard: {
			type: String,
			notify: true,
			computed: '_getCardCss(isDesktop)'
		},
		
		/**
      	 * css class used to display the panel
      	 */
		_cssPanel: {
			type: String,
			notify: true,
			computed: '_getPanelCss(isDesktop)'
		},
		
		srStatusNew: {
			type: String
		},
		
		srStatusQueued: {
			type: String
		},
		 
		/**
    	 * sr Status  domain 
    	 */
    	statusDomain: {
    		type: Object ,
    		observer:'_handleStatusDomainSet'
    	},
    	
    	
    	/**
      	 * Indicates the kind of SR must  be displayed 
      	 */
		displayStatus:{
    		type: String,
			value: ''
    	},
    	
    	/**
      	 * Indicates the kind of SR must not be displayed 
      	 */
    	hiddenStatus:{
    		type: String,
			value: ''
    	},
    	
    	// list for on-going cancelling request
      	_requestsForCancelling : {
      		type: Object 
      	},
      	// list for on-going unsubscribing request
      	_requestsForUnsubscribing : {
      		type: Object 
      	},
      	
      	/**
      	 * Indicates if the search bar will be displayed or not.
      	 */
      	showSearch: {
			type: Object,
			value: false,
			observer:'_showSearchBar'
		},	
		
		/**
    	 * indicate if searchBar component was set on right place
       	 */
		_isSearchBarReady:{
			type:Array,
			value: false
		},
		
		/**
      	 * List of attribute names used to sort  the collection  
      	 */
		orderByAttributeNames:{
    		type: String,
			value: ''
    	},
    	
    	
    	/**
      	 * text displayed if there is no record 
      	 */
		noRecordLabel:{
    		type: String,
			value: ''
    	},

      	/**
      	 * Flag for Collection
      	 */
      	collectionRefreshed: {
			type: Boolean,
			value: false
		},	
		
		
		/**
    	 * Attribute Domain filter
    	 */
		srDomainFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'SRSTATUS', 'selected': true} ]}];
    		} 	
    	},
    	
    	
    	/**
      	 *   Collection SET
      	 */
    	srDomainSet: {
			type: Object ,
			observer:'_configureDomain'
		},	
		
		
	 
		RESOLVED_DOMAIN: {
			type: String 
		},
		
    	
	},
	
	/**
	 * set the  SearchBar in the right place
   	 */
	_showSearchBar : function(){
		if(this.showSearch){
			this.addContent("search");
			$j(this.$.srPanel.$.panelHeader).append( $j(this).find('#searchdiv')[0]);
		}
  	},
  	
	listeners: {
  		'cancel-request' : '_onCancel',
  		'showdetails' : '_showDetails',
  		'dom-change' : '_domChanged',
  		 
  	},
  	
  	created : function(){
  	},
  	
  	attached: function()
	{
		var self = this;
		this.async(function(){
			self.$.srDataList.scrollTarget=self.$.srDataListDiv; 
			self.infiniteScrollingList(this.$.srDataList, 'servicerequestcollection');
			self._windowResize();
		}, 100);
		
		if(!this._isSearchBarReady){
			this._showSearchBar();
			this._isSearchBarReady = true;
		}
		
		this.fire('maximo-sr-list-collection', this.$.servicerequestcollection);
	},
	
	ready: function()
	{
		var self = this;
		$j(window).on('resize', function(){
			 self._windowResize();
	    });
	},
	
	
	_configureDomain : function(){
		var self = this;
		var srDomainList = this.srDomainSet[0].synonymdomain;
		if(srDomainList.length>0){
			srDomainList.forEach(function(domain){
				if(domain.maxvalue=='RESOLVED'){
					self.RESOLVED_DOMAIN= domain.value;
				} 
			});
		}
	},
	
	/**
	 * notify the parent component that the collection size was updated
   	 */
	_updateRecordCount : function(count) {
		this.recordCount = count;
		this.fire('maximo-sr-list-update-count', {count: this.recordCount});
	},
	
	
	/**
	 * remove a record from list page
   	 */
	_removeRecord : function (ticketid) {
		var current = this.querySelector('.cardborder[data-ticketid="' + ticketid + '"]').parentNode;					
		current.remove();
		this._updateRecordCount(this.recordCount-1);
	},
	
	/**
	 * check if the SR can be Canceled
   	 */
	_checkCancellable : function(sr,comments){ 
		/*
		 * You should be able to cancel even if it has comments/worklog entries.
		 * you just can not cancel if the status is not NEW & not QUEUED or there is a related workorder or related ticket
		 */
		if(((sr.status !== 	this.srStatusQueued) && (sr.status !== 	this.srStatusNew)) || 
			Object.getOwnPropertyNames(sr.relatedwo).length || 
			Object.getOwnPropertyNames(sr.relatedticket).length) {
			return false;
		}
		else{
			return true;
		}
	},
	
	/**
	 * cancel an SR
   	 */
	_onCancel : function(e) {
		var sr;
		if (e.detail) {
			sr = e.detail.sr;
			this.cancelRequest(sr, e.detail.comments);
		}
	},
	
	/**
	 * cancel an SR
   	 */
	cancelRequest : function(sr,comments){
		var message;
		var requestCard = this;
		var currentItem = this.querySelector('.cardborder[data-ticketid="'+sr.ticketid+'"]');
		if(currentItem && requestCard.alreadyInvokedHref !== sr.href){
			requestCard.alreadyInvokedHref = sr.href;
			if (this._checkCancellable(sr,comments)) {
				message = $M.localize('messages','mxapisr','srCancelled',[sr.ticketid]);
				var completeCallback = function(toOverlay) {
					$j(toOverlay).toggleClass('song2',true);
					requestCard._cancelRequest(sr);
				};
				var undoCallback = function() {
					requestCard.alreadyInvokedHref = '';
					delete requestCard._requestsForCancelling[sr.ticketid];
				};
				this._requestsForCancelling[sr.ticketid] = {href:sr.href, status:'ready'};
				$M.createOverlay(currentItem, message, completeCallback, undoCallback, this.interval, this.$.srPanel.$.panelInternal, null);
			}
			else {
				requestCard.alreadyInvokedHref = '';
				var overlayButtonDiv = Polymer.Base.create('div', {'id':'overlaybuttondiv'+sr.ticketid, 'align': 'center'});
				var overlayButton = Polymer.Base.create('MAXIMO-BUTTON', {'label':$M.localize('uitext','mxapibase','Ok'),'action': 'action'});
				overlayButtonDiv.appendChild(overlayButton);
				overlayButton.setAttribute('index',sr.ticketid);
				overlayButton.setAttribute('class','buttonInsideOverlay maximo-sr-list-card');
				overlayButtonDiv.setAttribute('class', 'buttonDivInsideOverlay maximo-sr-list-card');
				message = $M.localize('messages', 'mxapisr', 'srWorkInProgress', [sr.ticketid]);
				$M.createOverlay(currentItem, message, null, null, 3, this.$.srPanel.$.panelInternal, overlayButtonDiv);
			}
		}

	},
	
	
	
	
	/**
	 * Brings up details panel
	 */
	_showDetails : function(e) {
		this.fire('open-sr-details', {'sr': e.target.sr, 'target': e.target, 'page':e.detail});
	},
	
	 
	
	
	/**
	 * set a card on  highlight mode
   	 */
	_domChanged : function(){
		if(this.newTicketId){
			this.highlightCard(this.newTicketId);
		}
	},
	
	/**
	 * set a card on  highlight mode
   	 */
	highlightCard : function(id){
		if(id){
			var targetCard = $j('.cardborder[data-ticketid="'+id+'"]');
			if(targetCard.length > 0){
				targetCard.addClass('highlight');
				setTimeout(function() {
					targetCard.removeClass('highlight');
				}, 3000);
				this.newTicketId = null;
			}
			else{
				this.newTicketId = id;
			}
		}
	},
	
	/**
	 * set a card on  highlight mode
   	 */
	updateCardCounters : function(card){
		
		if(card){
			var targetCard = $j('.cardborder[data-ticketid="'+card.record.ticketid+'"]');
			if(targetCard.length > 0){
				this.selectedRecord = null;
				targetCard = targetCard[0].parentNode;
				targetCard.refreshCounters(card);			}   
			else{
				this.selectedRecord = card;
			}
		}
	},
	
	
	/**
	 * apply filters and refresh the collection
   	 */
	refreshCollection : function(type){
		if (this.statusDomain  && this.statusDomain.length > 0){
			return this._refreshCollection(type);
		}else{
			var self = this;
			return new Promise(function (resolve, reject) {
				self.async(function(){
					self.refreshCollection(type);
	 			},100);
			}.bind(this));
		}
		
	},
	
	/**
	 * 
	 * apply filters and refresh the collection
	 * (internal use)
	 * 
   	 */
	_refreshCollection : function(type){
		this.collectionRefreshed = false;
		$M.toggleWait(true);
		
		var maximoQbe;
		var status = this.getStatus();
		
	    maximoQbe = new MaximoQbe();
	    maximoQbe.setIn('status',status);
		qbeObject = maximoQbe.toJSON();
		
		this.$.servicerequestcollection.queryWhere = qbeObject;
		this.recordCount = -1;
		var self = this;
		return 	self.$.servicerequestcollection.refreshRecords().then(function() {
				   if('recent'==type){
					   self.resetScroll(); 
				   }
				   self.collectionRefreshed = true;
				});
		
	},
	
	/**
	 * hide /show no records section
   	 */
	hideSectionNoRecords: function(data){
		if(data.length>0){
			return false;
		} else{
			return true;
		}
	},
	
	
	/**
	 * cancel a service request
   	 */
	_cancelRequest : function(sr) {
		var self = this;
		self._requestsForCancelling[sr.ticketid].status = 'requested';
		var actionData = {'status':  self.RESOLVED_DOMAIN};
		var responseProperties = 'status';
		this.$.srResource.resourceUri = sr.href;
		this.$.srResource.updateAction('wsmethod:changeStatus',actionData,responseProperties,null).then(function() {
			self.refreshCollection();
		});
		
	},
	
	/**
	 * check if there is any record
   	 */
	_noRecordsFound : function (count, collectionRefreshed){
		if(collectionRefreshed && count===0){
			return false;
		}else {
			return true;
		}
	},
	
	/**
	 * set the scree to desktop / mobile mode
   	 */
	_windowResize : function(){
		this.isDesktop = !$M.isMobile();
	},
	
	
	/**
	 * configure the status list
   	 */
  	_handleStatusDomainSet : function(e) {
  		if (this.statusDomain.length > 0) {
  			var synonymdomain = this.statusDomain[0].synonymdomain;
  			if (synonymdomain && synonymdomain.length > 0) {
  				for(var idx in synonymdomain) {
  					if (synonymdomain[idx].maxvalue === "QUEUED") {
  						this.srStatusQueued = synonymdomain[idx].value;
  					}
  					
  					if (synonymdomain[idx].maxvalue === "NEW") {
  						this.srStatusNew = synonymdomain[idx].value;
  					}
  				}
  			}
  		}else {
  			if ($M.checkSession(e)){
  				this.fire('value-data-error', e.detail.request.xhr.response);
  			}
  		}
    },
    
	/** 
	 * retrieve a status list excluding  hiddenStatus
	 */
	
	getStatus: function() {
		var result=''
		var separator=''
		var hiddenStatusList = this.hiddenStatus.split(',');
		var displayStatusList = this.displayStatus.split(',');
		
		if(hiddenStatusList[0]==''){
			hiddenStatusList= null;
		}
		if(displayStatusList[0]==''){
			displayStatusList= null;
		}
		this.statusDomain[0].synonymdomain.forEach(function(domain){
			if((displayStatusList && hiddenStatusList ) && (this.containsElement(displayStatusList,domain.maxvalue) || !this.containsElement(hiddenStatusList,domain.maxvalue))){
				result += separator + domain.value ;
				separator=',';
			}else  if(displayStatusList && this.containsElement(displayStatusList,domain.maxvalue) ){
				result += separator + domain.value ;
				separator=',';
			}else  if(hiddenStatusList && !this.containsElement(hiddenStatusList,domain.maxvalue) ){
				result += separator + domain.value ;
				separator=',';
			}
		}, this);
		return   result;
	},
	
	/**
	 * check if a list contains a element
   	 */
	containsElement: function(list , element) {
		var result = false;
		  list.some(function(value){
			  if(value == element){
				  result = true;
				}
			}, this);
		
		return result;
	},
	
	/**
	 * return the css class that must be used on card
   	 */
	_getCardCss: function(isDesktop) {
		 if(isDesktop){
			 return 'dataSetCard cardWrapper  ';
		 }else{
			 return 'dataSetCard cardWrapper  mobile';
		 }
	},
	
	/**
	 * return the css class that must be used on panel
   	 */
	_getPanelCss: function(isDesktop) {
		 if(isDesktop){
			 return 'srListContainer desktop  ';
		 }else{
			 return 'srListContainer   mobile';
		 }
	},
	
	/**
	 * reset the page Scroll
   	 */
	resetScroll : function(){
		var element  = this.$.srPanel.querySelector('.panelInternal');
		element.scrollTop = 0;
	},
	
	
	/**
	 * handle the collection refresh event
   	 */
	handleRecordDataRefreshed: function(){		
	
		this._updateRecordCount(this.$.servicerequestcollection.totalCount);
		this._requestsForCancelling = {};
  		this._requestsForUnsubscribing = {};
  		
  		var self = this; 
  		this.async(function(){
  			$j(self.$.srDataList).toggleClass('srListInitial',false);
  			this.async(function(){
  	  			self.$.srDataList.fire('iron-resize');
  	  		    $M.toggleWait(false);
  			}, 50);
		}, 350);
  		
  		 
    },
    
    /**
	 * handle the collection error event
   	 */
    _handleDataError : function(e){
		if (e.detail && e.detail.Error) {
			$M.alert(e.detail.Error.message);
		}
		$M.toggleWait(false);
	},
	
	/**
     * Enable wait layer, initiated on refresh of collection
     */
	_loading : function(){
		$M.toggleWait(true);
	}
	
});
