/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017
 */
Polymer({
	is: 'maximo-failure-detail',
	behaviors: [BaseComponent, Workorder],
	listeners : {
		'cancel-failure-long-desc': '_cancelFailureLongDescription',
		'maximo-lookup-selected' : '_returnFromLookup',
		'maximo-typeahead-selected' : '_returnFromLookup',
		'failure-lookup-data' : '_failureLookupProblemData',
		'failureChildLookup' :  '_fetchFailurecodeChildren',
		'store-long-desc': '_storeLongDesc'
	},
	properties : {
		
		/**
		 * To record object
		 */
		record : {
			type : Object,
			notify : true
		},
		/**
		 * To set the description in cause field
		 */
		causeDesc :{
			type : String,
			value : ''
		},
		/**
		 * To set the description in cause field
		 */
		remedyDesc : {
			type : String,
			value : ''
		},
		/**
		 * To set the problem href for cause lookup
		 */
		problemhref : {
			type : String,
			notify : true
		},
		/**
		 * To set the cause href for remedy lookup
		 */
		causehref : {
			type : String,
			notify : true
		},
		/**
		 * To set the remedy href
		 */
		remedyhref : {
			type : String
		},
		/**
		 * To set the problemcode object
		 */
		problemObj : {
			type : Object,
			value : ''
		},
		/**
		 * To set the cause object
		 */
		causeObj : {
			type : Object,
			value : ''
		},
		/**
		 * To set the remedy object
		 */
		remedyObj : {
			type : Object,
			value : ''
		},
		/**
		 * To set failure check in parent
		 */
		failureFlag : {
			type : Boolean,
			value : false
		},
		/**
		 * To set problem check in parent
		 */
		problemFlag : {
			type : Boolean,
			value : false
		},
		/**
		 * To set cause check in parent
		 */
		flagCause : {
			type : Boolean,
			value : false
		},
		/**
		 * To set remedy check in parent
		 */
		flagRemedy : {
			type : Boolean,
			value : false
		},
		/**
		 * To check failure class available or not
		 */
		failureclass : {
			type : Boolean,
			value : false
		},
		/**
		 * To set fail type domain
		 */
		failType : {
			type : Array
		},
	       /**
         * failurecode record href.
  	    */
	    failureCodeRecordHref :{
	      	type : String,
	       	value : ''
	    } 
	},
	
	/**
	 * Flyout for Long description with rich text editor 
	 */	
	_showLongdesc: function(e){
      	e.stopPropagation();
      	var flyoutOwner = $j(this.domHost).find('[data-scope-id="outerdiv"]')[0];
      	var flyout = $M.showFlyout('ld_flyout', '<div></div>',  e.currentTarget, {'owner':flyoutOwner, 'constrainWidth':true});
      	flyout.addButtons([{'label':this.localize('uitext','mxapibase','Cancel'),'action':true,'event':'cancel-failure-long-desc','eventTarget':this},{'label':this.localize('uitext','mxapibase','done'),'event':'store-long-desc','eventTarget':this}]);
		var value = '';
		var id = 'longdesc_';
       	if(this.record){
			var detailRecord = ($M.currentWorkscape === 'technician')?$M.workScape.querySelector('#techWODetail').record:$M.workScape.querySelector('#details').record;
			if(detailRecord && detailRecord.remarkdesc_longdescription){
				value = detailRecord.remarkdesc_longdescription;
				id += this.record._id;
			}
			var failureCode = this.record.failurecode?this.record.failurecode:detailRecord.failurecode;
			if(failureCode){
	 		  this._getFailureCodeUrl(failureCode);
	 	    }
		}
       	var maximoRTE = $M.createElement('maximo-richtexteditor');
       	maximoRTE.props = {
       		'id':id,
       		'placeholder':this.localize('uitext','mxapiwodetail','LongDescription'),
       		'value':value
       	};
       	this.async(function(){
	       	var div = $j(flyout.$.internal).find('div');
	      	div.css({'width':'100%'});
	      	div.append(maximoRTE);
       	},100);
       	this.LongDescRTE = maximoRTE;
      },

  	_cancelFailureLongDescription : function(e){
  		this.LongDescRTE.remove();
  		this.LongDescRTE = null;
    },
    
    
	/**
	 * Update the Remarks for the failure
	 */
    _updateRemarks : function(e){		
		var description=e.target.value.trim();
		if (!description && this.record.remarkdesc === undefined || this.record.remarkdesc===description) {
		    $M.toggleWait(false);
		    return;
	    }
		if(description.length >= 0){
			var self=this;
			setTimeout(function() {
				self._updateWO({'remarkdesc':description}, self.collection);
			}, 100); 
		}else{
			this.$.remarks.value=this.record.remarkdesc;
		}
	}, 
	
	/**
	 * To set the values from Item lookup
	 */
	_returnFromLookup : function(e) {
		var list = [];
		$j('[data-scope-id="fail_long_desc"]').prop('disabled',false);
		if (e.detail.records[0] && e.detail.records[0].failurecode !== undefined) {			
			if (e.target.dataset.scopeId == 'failureclass') {
				if(this.record.failurecode === e.detail.records[0].failurecode){
					this.$.failureclass.value = e.detail.records[0].description;
					return;
				} else {
					this.$.failureclass.value =  e.detail.records[0].description;
					this.record.failure.failurecode = e.detail.records[0].failurecode;
					this.problemFlag = false;
					this.flagCause = false;
					this.flagRemedy = false;
					this._updateWO({'failurecode':this.record.failure.failurecode}, this.collection);
					this.$.collapseIconProblemConfirm.hidden = true;
					this.$.collapseIconProblemWarning.hidden =  false;
					this.$.collapseIconCauseConfirm.hidden = true;
					this.$.collapseIconCauseDebug.hidden =  false;
					this.$.collapseIconRemedyConfirm.hidden = true;
					this.$.collapseIconRemedyRecommendation.hidden =  false;
					$j(this.$.cause).find('*').prop('disabled',true);
					$j(this.$.remedy).find('*').prop('disabled',true);
					this.$.problem.value = '';
					this.$.cause.value = '';
					this.$.remedy.value = '';
					this.problemhref = '';
					this.causehref = '';
					this.remedyhref = '';
					this.problemObj = {};
				}
			} else if(e.target.dataset.scopeId === 'problem'){
				this.flagCause = false;
				this.flagRemedy = false;
				this.problemFlag = true;
				if(this.$.problem.value === e.detail.records[0].failurecode.description){
					this._changeLookupData();
					this.flagCause = true;
				}
				this.record.problemcode = e.detail.records[0].failurecode.failurecode;
				this.record.problem.description = e.detail.records[0].failurecode.description;
				this.problem = e.detail.records[0].failurecode.failurecode;
				this.problemhref = e.detail.records[0].href+'/children';
				var self = this;
				this._updateWO({'problemcode':this.record.problemcode}, this.collection).then(function(result){
					self.$.problem.value = e.detail.records[0].failurecode.description;				
					self.$.collapseIconProblemConfirm.hidden = false;
					self.$.collapseIconProblemWarning.hidden =  true;
					self.$.collapseIconCauseConfirm.hidden = true;
					self.$.collapseIconCauseDebug.hidden =  false;
					self.$.collapseIconRemedyConfirm.hidden = true;
					self.$.collapseIconRemedyRecommendation.hidden =  false;
					$j(self.$.cause).find('*').prop('disabled',false);
					self.causeDesc = '';
					self.remedyDesc = '';
					var failCode = (self.problemObj.failurecode && self.problemObj.failurecode.failurecode)? self.problemObj.failurecode.failurecode : self.problemObj.failurecode;
					if(failCode !== e.detail.records[0].failurecode.failurecode){
						self.causeObj = {};
						self.remedyObj = {};
						self.causehref = '';
						self.cause = '';
						self.remedy = '';
					}
					self._fetchFailurecodeChildren(e);					
				});
			} else {
				// Used scope id for typeAheadSearch, identifier for resulting coming from lookup popup"
				if(e.target.dataset.scopeId === 'cause' || e.detail.identifier === 'cause'){
					this.flagRemedy = false;
					if(this.cause && this.causeObj && this.causeObj.failurecode){
						this.cause = e.detail.records[0].failurecode.failurecode;
						this.causeDesc = e.detail.records[0].failurecode.description;
						this.$.cause.value = e.detail.records[0].failurecode.description;
						this.causeObj.failurecode=e.detail.records[0].failurecode.failurecode;
						this.causeObj.linenum = e.detail.records[0].failurelist;
						this.causeObj._action = 'Update';
						list = [];
						list.push(this.causeObj);
						this._updateFailureReport(list, this.collection,e);
					} else {
						this.cause = e.detail.records[0].failurecode.failurecode;
						this.causeDesc = e.detail.records[0].failurecode.description;
						this.$.cause.value = e.detail.records[0].failurecode.description;
						var type = e.detail.records[0].type;
						this._updateFailureReport([{'_action':'Add','failurecode':e.detail.records[0].failurecode.failurecode, 'type' : type, 'linenum':e.detail.records[0].failurelist}], this.collection,e);
					}
					this.causehref = e.detail.records[0].href+'/children';
					this.$.collapseIconCauseConfirm.hidden = false;
					this.$.collapseIconCauseDebug.hidden =  true;
					this.$.collapseIconRemedyConfirm.hidden = true;
					this.$.collapseIconRemedyRecommendation.hidden =  false;
					$j(this.$.remedy).find('*').prop('disabled',false);
					this.remedyDesc = '';
					this.remedyObj = {};
					this.remedy = '';
				}
				if(e.target.dataset.scopeId === 'remedy' || e.detail.identifier === 'remedy'){
					if(this.remedyObj && this.remedyObj.failurecode){
						this.remedyDesc = e.detail.records[0].failurecode.description;
						this.remedyObj.failurecode = e.detail.records[0].failurecode.failurecode;
						this.$.remedy.value = e.detail.records[0].failurecode.description;
						this.remedyObj.linenum = e.detail.records[0].failurelist;
						this.remedyObj._action = 'Update';
						list = [];
						list.push(this.remedyObj);
						this._updateFailureReport(list, this.collection,e);
					} else {
						this.remedy = e.detail.records[0].failurecode.failurecode;
						this.remedyDesc = e.detail.records[0].failurecode.description;
						this.$.remedy.value = e.detail.records[0].failurecode.description;
						var type = e.detail.records[0].type;
						this._updateFailureReport([{'_action':'Add','failurecode':e.detail.records[0].failurecode.failurecode, 'type' : type,'linenum':e.detail.records[0].failurelist}], this.collection,e);
					}
					this.$.collapseIconRemedyConfirm.hidden = false;
					this.$.collapseIconRemedyRecommendation.hidden =  true;
				}
			}
		}
	},

	/**
	 * To show cause and remedy lookup
	 * Used scope id for typeAheadSearch, identifier for resulting coming from lookup popup
	 */
	_fetchFailurecodeChildren : function(e) {
		var self = this;
		if(e.target.dataset.scopeId === 'problem'){
			$M.showLookup('cause', self, 'cause',this.problemhref, null, null);
		} else if(e.target.dataset.scopeId === 'cause' || (e.detail && e.detail.identifier === 'cause' || e.detail.detail && e.detail.detail.identifier === 'cause' || e.detail.srcElement && e.detail.srcElement.dataset && e.detail.srcElement.dataset.scopeId ==='cause')){
			$M.showLookup('remedy', self, 'remedy',this.causehref, null, null);
		}
	},

	/**
	 * To get and set Failurecode for cause and remedy fields
	 */
	_getFailureData : function(failureType,failurecode){
		if(failureType === 'CAUSE'){
			failurecode = this.causeDesc;
		} else {
			failurecode = this.remedyDesc;
		}
		return failurecode;		
	},

	/**
	 * To update Cause and Remedy when select lookup data.
	 */
	_updateFailureReport: function(dataToUpdate, collectionToUpdate, e){
		$M.toggleWait(true);
		var self = this;
		var responseProperties = this.$.failureResource.attributeNames;
		this.$.failureResource.resourceUri = this.record.href+'/uxshowfailurereport';
		this.$.failureResource.updateRecord(dataToUpdate, responseProperties, true).then(function(result) {
			var failureReportArr = result.failurereport;
			if(failureReportArr){
				failureReportArr.forEach(function(failReport){ 
					if(self.domHost && self.domHost.domHost && self.domHost.domHost.getInternalDomainValue(self.failType,failReport.type) === 'PROBLEM'){ 
						self.problemObj = failReport;
						self.problemFlag = true;
						self.failureFlag = true;
					}
					if(self.domHost && self.domHost.domHost && self.domHost.domHost.getInternalDomainValue(self.failType,failReport.type) === 'CAUSE') {
						self.causeObj = failReport;
						self.flagCause = true;
					}
					if(self.domHost && self.domHost.domHost && self.domHost.domHost.getInternalDomainValue(self.failType,failReport.type) === 'REMEDY') {
						self.remedyObj = failReport;
						self.flagRemedy = true;
					}
				});
			}
			$M.toggleWait(false);
			self.fire('failureChildLookup', e);
		}, function(error) {
			if(self.domHost && self.domHost.domHost && self.domHost.domHost.getInternalDomainValue(self.failType,dataToUpdate[0].type) === 'CAUSE'){
				self.causeObj = {};
				self.cause = '';
				self.causeDesc = '';
				self.$.collapseIconCauseConfirm.hidden = true;
				self.$.collapseIconCauseDebug.hidden =  false;
			}
			if(self.domHost && self.domHost.domHost && self.domHost.domHost.getInternalDomainValue(self.failType,dataToUpdate[0].type) === 'REMEDY'){
				self.remedy = '';
				self.remedyDesc = '';
				self.remedyObj = {};
				self.$.collapseIconRemedyConfirm.hidden = true;
				self.$.collapseIconRemedyRecommendation.hidden =  false;
			}
			$M.toggleWait(false);
			$M.showResponseError(error);
		});
    },
    
    /**
	 * Loading of failure look up problem data.
	 */
    _failureLookupProblemData : function(){
    	if(this.$.problem.value === '' || this.$.problem.value === undefined){
    		$j(this.$.cause).find('*').prop('disabled',false)
    		$j(this.$.remedy).find('*').prop('disabled', false);
    		return false;
    	}
    	$j(this.$.cause).find('*').prop('disabled', true);
    	$j(this.$.remedy).find('*').prop('disabled', true);
    	this.problemFlag = true;
    	return true;
    },
    /**
	 * Changing the the look up information.
	 */
    _changeLookupData : function(e){
    	if((this.$.problem.value === undefined || this.$.problem.value === '') && this.problemFlag){     		
			this.$.collapseIconProblemConfirm.hidden = true;
			this.$.collapseIconProblemWarning.hidden =  false;
			this.$.collapseIconCauseConfirm.hidden = true;
			this.$.collapseIconCauseDebug.hidden =  false;
			this.$.collapseIconRemedyConfirm.hidden = true;
			this.$.collapseIconRemedyRecommendation.hidden =  false;
			$j(this.$.cause).find('*').prop('disabled', true);
			$j(this.$.remedy).find('*').prop('disabled', true)
			this.record.problemcode = '';
			this.record.problem.description = '';
			if(this.failureFlag && this.problemObj){
				this.record.problemcode = '';
				this.async(function(){
					this._updateWO({'problemcode':this.record.problemcode}, this.collection);
				},300); 
				this.problemObj = {};
				this.causeObj = {};
				this.remedyObj = {};
				this.problemFlag = false;
				this.causeDesc = '';
				this.remedyDesc = '';
				this.flagCause =true;
				this.problemhref = '';
				this.causehref = '';
			}
    	} else if((this.$.cause.value === '' && this.$.problem.value) && this.flagCause){    		
			this.$.collapseIconCauseConfirm.hidden = true;
			this.$.collapseIconCauseDebug.hidden =  false;
			this.$.collapseIconRemedyConfirm.hidden = true;
			this.$.collapseIconRemedyRecommendation.hidden =  false;
			$j(this.$.remedy).find('*').prop('disabled', true);
			if(this.failureFlag){
				if(this.causeObj && this.causeObj.failurecode){
					var list = [];				
					if(this.remedyObj && this.remedyObj.failurecode){
						this.remedyObj._action = 'Delete';
	    				list.push(this.remedyObj);
						this._deleteFailureReport(list);
						this.remedyObj = {};
						this.remedyDesc = '';
						this.flagRemedy = false;
					}
					list = [];
	    			this.causeObj._action = 'Delete';
	    			list.push(this.causeObj);
					this._deleteFailureReport(list);
					this.causeObj = {}; 
					this.flagCause = false;
					this.causehref = '';
				}
			}
    	} else if((this.$.remedy.value === '' && this.$.cause.value) && this.flagRemedy){
    		this.$.collapseIconRemedyConfirm.hidden = true;
    		this.$.collapseIconRemedyRecommendation.hidden =  false;
    		if(this.failureFlag){
	    		if(this.remedyObj && this.remedyObj.failurecode){
	    			var list = [];
	    			this.remedyObj._action = 'Delete';
	    			list.push(this.remedyObj);
					this._deleteFailureReport(list);
					this.remedyObj = {};
				}
    		}
    	} 
    },
        
    /**
     * To delete a given failure code.
     */
    _deleteFailureReport : function(data){
    	$M.toggleWait(true);
    	var self = this;
		var responseProperties = this.$.failureResource.attributeNames;
		this.$.failureResource.resourceUri = this.record.href+'/uxshowfailurereport';
		this.$.failureResource.updateRecord(data, responseProperties, true).then(function(result) {
        	$M.toggleWait(false);
        }.bind(this), function(error) {
        	$M.toggleWait(false);
            $M.showResponseError(error);
        });    	
    },
    
    /**
     * Show/Hide Problem,Cause,Remedy div on change og Failure class
     */
    _showFailureReport : function(){
    	this.failureclass = false;
    	$j('[data-scope-id="fail_long_desc"]').prop('disabled',false);
    	if(this.$.failureclass.value === undefined || this.$.failureclass.value === ''){  
    		this.failureclass = true;
    		$j('[data-scope-id="fail_long_desc"]').prop('disabled',true);
    		if(this.$.problem.value){
    			this.problemFlag = false;
    		}
    		if(this.record && this.record.failure){
    			this.record.failurecode = '';    			
	    		this.record.failure.failurecode = '';
	    		this.record.failure = {};
	    		this._updateWO({'failurecode':this.record.failure.failurecode}, this.collection);
	    		this.failureclass = true;
	    		$j('[data-scope-id="fail_long_desc"]').prop('disabled',true);
    		}
    	} else if(this.$.failureclass.value){
    		this.failureclass = false;
    		$j('[data-scope-id="fail_long_desc"]').prop('disabled',false);
    		if(this.$.problem.value === undefined && this.record.problemcode){
    			this.problemFlag = false;
    		} 	
    		if(this.record.failure && this.record.failure.failurecode && this.record.failure.description){
				return;
			} else if(this.record.failurecode){
				this.failureclass = false;
    	    	$j('[data-scope-id="fail_long_desc"]').prop('disabled',false);
    	    	return;
			} else {
				this.failureclass = true;
    	    	$j('[data-scope-id="fail_long_desc"]').prop('disabled',true);
    	    	return;
			}    
    	}
    }, 
    
    /**
     * To check the status for editable or readonly.
     */
    _getReadonlyFlag : function(status_maxvalue){
    	if(status_maxvalue === 'CLOSE' || status_maxvalue === 'CAN'){
    		return true;
    	} else { 
    		return false;
    	}
    },
	   
    /**
	    * Store the failure long description.
	    */
	   _storeLongDesc: function(e){
	 	   var desc;
	 	   if(this.record){
	 		   var failure = this.record.failure;
	 		   if(this.record && this.record.remarkdesc_longdescription){
	 			   desc = this.record.remarkdesc_longdescription;
	 		   }
	 		   var failureCode = this.record.failurecode?this.record.failurecode:failure.failurecode;
			   if(failureCode){
		 		  this._getFailureCodeUrl(failureCode);
		 	   }
		   }
	 	   var updatedDesc;
	 	   var tinyEditor = $j(tinyMCE.activeEditor);
	 	   if($M.isMobile() && tinyEditor){
	 		  updatedDesc = tinyEditor[0].getBody().innerHTML;
	 	   }else{
	 		  updatedDesc = this.LongDescRTE.value; 
	 	   }
	 	   if(desc !== updatedDesc){
	 		  var data = {};
		  	  data._action = 'Update';
		  	  data.description_longdescription= updatedDesc;
		  	  var self=this;
		  	  setTimeout(function() {
				self._updateWO({'remarkdesc_longdescription':data.description_longdescription}, self.collection);
					}, 100);
	 	   }
	 	   this._cancelFailureLongDescription();
	   },
	   /**
	    * Get failurecode href using failurecode.
	    */
	   _getFailureCodeUrl(failureCode){
        var self=this;
        var failureCollection=Polymer.Base.create('maximo-collection',{
           'id':'failureCodeCollection',
           'attributeNames':'href'
        });
        var maximoQbe = new MaximoQbe();
        maximoQbe.setEquals('failurecode',failureCode);
        var qbeObject = maximoQbe.toJSON();
        failureCollection.queryWhere = qbeObject; 
        failureCollection.collectionUri=$M.getMaxauth().baseUri + '/oslc/os/mxapifailurecode';
        failureCollection.refreshRecords().then(function(result){
            if(result){
                 self.failureCodeRecordHref=result.response.member[0].href;
            }
        });
     }
    
});