/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
	is: 'maximo-report-item-card',
  	behaviors: [BaseComponent, DirtyBehavior],
  	listeners: {
		'maximo-lookup-selected' : '_returnFromLookup',
  		'maximo-typeahead-selected' : '_returnFromLookup',
  		'is-dirty-changed': '_updateBorder',
  		'maximo-reset-element':'_resetCard'
  	},
	properties : {
		/**
    	 * the work order record
    	 */
		record: {
			type : Object,
			observer: '_clearDirty'
		},
		/**
    	 * Interval in seconds to 
    	 * undo delete operation
    	 */
    	undoIntervalSecs: {
    		Type: Number,
    		value: 3
    	},
    	/**
		 * To get customized href for lookup
		 */
    	customizeditemref :{
			type : String,
			computed: '_getCustomizeditemref(recordhref)'
		},
    	/**
		 * To get customized href for binnum lookup
		 */
    	customizedbinnumref :{
			type : String,
			computed: '_getCustomizedbinnumref(recordhref)'
		},
		
		/**
		 * To get workorder href
		 */
		recordhref : {
			type : String
		},
		/**
		 * Check for item card editable
		 */
		editable :{
            type: Boolean,
            value : false,
            observer: '_editableCardFields'
		},
		/**
		 * To identify new labor card.
		 */
		newCard : {
			type : Boolean
		},
		/**
		 * To get customized store room href for Item
		 */
    	customizedstorerroomref :{
			type : String,
			computed: '_getCustomizedStoreRoomRef(recordhref)'
		},
		/**
		 * To get customized task href for Item
		 */
		customizedtaskref :{
            type : String,
            computed: '_getCustomizedtaskref(recordhref)'
		},
		/**
    	 * Line Type filter
    	 */
		lineTypeDomainFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'LINETYPE', 'selected': true} ]}];
    		} 	
    	},
    	/**
    	 * Line Type result
    	 */
		lineTypeDomainSet: {
    		type: Array,
    		value: []
    	},
    	    	
    	/**
       	 * Default itemnum
       	 */
		itemnum: {
       		type: String
		},
		
		
		itemsetid: {
       		type: String
		},
		
		/**
       	 * Default location
       	 */
		location: {
       		type: String
		},
		
		/**
       	 * Default siteid
       	 */
		siteid: {
       		type: String
		},
		
		/**
		 * To get location clause
		 */
		locationClause :{
			type : String,
			computed: '_getLocationClause(siteid,location)'
    	},

		/**
		 * To get itemnum clause
		 */
    	itemnumClause :{
			type : String,
			computed: '_getItemnumClause(siteid,itemnum)'
		},
		
		/**
		 * To get binnum clause
		 */
		binnumClause :{
			type : String,
			computed: '_getBinnumClause(location,itemnum,siteid)'
		},
		
		/**
		 * Requestnum for reserved items
		 */		
        requestnum: {
			type: Number
		},

		/**
		 * To get clause for Site
		 */
		siteClause :{
			type : String,
			computed: '_defaultOrgClause(siteid)'
		},
		/**
		 * To get clause for rotate asset
		 */
		rotateItemClause : {
			type : String,
			computed: '_getRotateItemClause(siteid,itemnum,location)'
		},
		/**
		 * To check Item rotating or not
		 */
		rotating : {
			type : Boolean,
			value : false
		},
		/**
		 * To get customized URL for rotating asset
		 */
		customizedrotatingassetref : {
			computed: '_getCustomizedRotateAssetRef(recordhref)'
		},
		/**
		 * To assign the RotateAsset num
		 */
		assetnum : {
			type : String
		},
		/**
		 * To sort LineType
		 */
		lineTypeSort: {
		     type: Array,
		     value: function () {
		        return [{collection: 'maxdomain.synonymdomain', attributes: '%2Bmaxvalue'}];
		     }
	    }
	}, 
	
	/**
	 * To reset the div values.
	 */
	attached: function(){
  		this.parentResetButton = this.$.divReset.id; 
		this.showResetButton= true;
		$j(this.$.report_item_issued).find('label.label').css('font-size','18px');
  	},
     
	/**
      * Delete Item.
      * Displays overlay on card to undo Delete action.
      */
  	_deleteItem: function (e) {
        e.stopPropagation();
        this.fire('disableItemMaterialHeader', false);
        var self = this;
		this.showResetButton=false;
        var performDeleteItemOperation = function(overlayNode) {
        	self.fire('deleteItemCard', self);
        };
        var revertDeleteItemOperation = function(overlayNode){
		    self.showResetButton=true;
		   	if (self.newCard) {
		   		$j(self.$.report_item_innerDiv).toggleClass('dirtyBorder',true);
		   		self.fire('disableItemMaterialHeader', true);
		   	}
	    };
        if(this.$.report_item_type.$.select.value === this.getExternalValueForLineType('ITEM')){
			var itemDesc = self.$.report_item_description.value.trim();
        	if(itemDesc === undefined || itemDesc === ''){
        		itemDesc = $M.localize('uitext', 'mxapiwodetail', 'Item');
        	}
        	var itemDelMessage = $M.localize('uitext','mxapiwodetail','reportedmaterialdelete',[itemDesc]);
	        $M.createOverlay(this.$.report_item_card, itemDelMessage, performDeleteItemOperation, revertDeleteItemOperation, 
	       		 this.undoIntervalSecs, this.parentElement.parentElement, null);
        }
        else
        {
        	var materialDes = this.$.report_material_description.value.trim();
        	if(materialDes === undefined || materialDes === ''){
        		materialDes = $M.localize('uitext', 'mxapiwodetail', 'Material');
        	}        	
	      	var messageDelMaterial = $M.localize('uitext', 'mxapiwodetail', 'reportedmaterialdelete', [materialDes]);
	      	$M.createOverlay(this.$.report_item_card, messageDelMaterial, performDeleteItemOperation, revertDeleteItemOperation, 
    	    	 this.undoIntervalSecs, this.parentElement.parentElement, null);
        }
     },     
    
    /**
     * To check for card fields editable
     */
    _editableCardFields : function () {
    	if(this.editable){
            $j(this.$.report_item_issued).attr('readonly',false);
            $j(this.$.report_item_description).attr('readonly',false);
            $j(this.$.report_material_description).attr('readonly',false);            
            $j(this.$.report_item_card).find('maximo-select').each(function(){ this.removeAttribute('readonly') });
            $j(this.$.report_item_card).removeClass('non-editable-cont');
         } else {
           $j(this.$.report_item_card).addClass('non-editable-cont');
           $j(this.$.report_item_card).addClass('readonly-material-cont');
           $j(this.$.report_item_issued).attr('readonly',true);
           $j(this.$.report_item_description).attr('readonly',true);
           $j(this.$.report_material_description).attr('readonly',true);
           $j(this.$.report_item_card).find('maximo-select').each(function(){ this.setAttribute('readonly',true) });           
         }
      },
		  
	  /**
	   * To set the values from Item lookup
	   */
	  _returnFromLookup: function(e){
			if(e.detail.records[0] && e.detail.records[0].itemnum !== undefined){
				this.itemnum = e.detail.records[0].itemnum;
				this.$.report_item_description.value = e.detail.records[0].description?e.detail.records[0].description:e.detail.records[0].itemnum;
				this.$.report_item_itemnum.value = this.itemnum;
				this.rotating = e.detail.records[0].rotating;
				this.assetnum = '';
				this.$.report_item_rotating_asset.value = '';
				if(this.rotating){
					$j(this.$.report_item_issued).attr('readonly',true);
					this.$.report_item_rotating_asset.hidden = false;
				} else {
					$j(this.$.report_item_issued).attr('readonly',false);
					this.$.report_item_rotating_asset.hidden = true;
				}
				var self = this;
				if(this.$.report_item_storeroom.value === ''){
					this.location = '';
					this.async(function(){
						self._getLookupCount();
					},200);
				} else {
					 this.async(function(){					 	
					 	self._getBinLookupCount();
					 },250);
				}
			}
			if(e.detail.records[0] && e.detail.records[0].location && !e.detail.records[0].assetnum ){
				this.location = e.detail.records[0].location;
				this.$.report_item_storeroom.value = e.detail.records[0].description ? e.detail.records[0].description : e.detail.records[0].location;
				this.siteid = e.detail.records[0].siteid;
				this.$.report_item_site.value = e.detail.records[0].site.description;
				
				if(this.location){
		            this.checkStoreroomAccess(this.location, e.detail.records[0].description, this.siteid, false);
		        }
				
				var self = this;
				this.async(function(){
					 self._getBinLookupCount();
					 if(this.rotating){
						 var that = self;
						 self.async(function(){
						 	that._getAssetLookupCount();
						 },250);
					 }
				},200);
				
			}
			if(e.detail.records[0] && e.detail.records[0].binnum){
				this.$.report_item_bin.value = e.detail.records[0].binnum;
			}else if(e.detail.records[0] && (e.detail.records[0].binnum === undefined || e.detail.records[0].curbal)){
				this.$.report_item_bin.value = $M.localize('uitext', 'mxapibase', 'NotEntered');
			}
			if(e.detail.records[0] && e.detail.records[0].taskid){
				this.$.report_item_task.value = e.detail.records[0].taskid;
			}
			if(e.detail.records[0] && e.detail.records[0].taskid){
				this.$.report_material_task.value = e.detail.records[0].taskid;
			}
			
			if(e.detail.records[0] && !e.detail.records[0].location && e.detail.records[0].siteid && !e.detail.records[0].binnum ){
				// Getting value from MaxValue
				var report_item_type = this.$.report_item_type.$.select.value;
		        if(report_item_type === this.getExternalValueForLineType('ITEM')){
		        	this.siteid = e.detail.records[0].siteid;
		        	if(e.detail.records[0].site){
		        		this.$.report_item_site.value = e.detail.records[0].site.description;
		        	} else {
		        	    this.resetItemFields(e);
		        	}
		        }else{
		        	this.siteid = e.detail.records[0].siteid;
					this.$.report_material_site.value = e.detail.records[0].description;
		        }
			}
			
			if(e.detail.records[0] && e.detail.records[0].assetnum !== undefined && !e.detail.records[0].binnum){
				this.assetnum = e.detail.records[0].assetnum;
				this.$.report_item_rotating_asset.value = e.detail.records[0].description;
			}
			
		},
     
		/**
		  * Enable/disable save button for required item card fields.
		  */
		 _enableSave : function(e){
			 var self = this;
	          this.async(function(){
	          var report_item_type = this.$.report_item_type.$.select.value;
	          if(report_item_type === this.getExternalValueForLineType('MATERIAL')){
	        	  var materialDescription= this.$.report_material_description.value;
	   	  	 	  var unitCost = this.$.report_material_unit.value;
	   	  	      var siteId = this.$.report_material_site.value;
		          if(materialDescription && unitCost>=0 && siteId){
		        	 $j(this.$.saveitemcardDiv).find('*').prop('disabled',false);
		          } 
		          else{
		        	 $j(this.$.saveitemcardDiv).find('*').prop('disabled',true);
		          }
		          var taskInput = $j(this.$.report_material_task).find('input.maximo-text');
				  var taskButton = $j(this.$.report_material_task).find('maximo-button.lookup.maximo-text');
			  } else {
				  var itemDescription = this.$.report_item_description.value ? this.$.report_item_description.value : this.$.report_item_description_readonly.label;
		          var quantity = this.$.report_item_quantity.value;
		          var rotatingAsset = this.$.report_item_rotating_asset.value;
		          var storeroom = this.$.report_item_storeroom.value; 
		          var site = this.$.report_item_site.value;
		          if(e && e.target && e.target.id && e.target.id.includes('report_item_site') && !site) {
		        	  this.resetItemFields(e);
		          }
		          if(storeroom === undefined || storeroom === ''){
			          this.location = '';
			          this.$.report_item_bin.value = '';
			      }
		         
		          if(itemDescription === undefined || itemDescription === ''){
		        	  this.$.report_item_itemnum.value = '';
		        	  this.itemnum = '';
		        	  this.$.report_item_bin.value = '';
		        	  this.$.report_item_rotating_asset.value = '';
		        	  this.$.report_item_rotating_asset.hidden = true;
		          }
		          if(this.rotating){
		        	  if(itemDescription && quantity > 0 && quantity !== NaN && rotatingAsset && storeroom){
		        		  this.$.saveitemcardDiv.firstElementChild.disabled = false;
		        	  } else {
		        		  this.$.saveitemcardDiv.firstElementChild.disabled = true;
		        	  } 
	        	  } else {
			          if(itemDescription && quantity > 0 && quantity !== NaN && storeroom){
		        		  this.$.saveitemcardDiv.firstElementChild.disabled = false;
		        	  } else {
		        		  this.$.saveitemcardDiv.firstElementChild.disabled = true;
		        	  }
	        	  }
	        	  var taskInput = $j(this.$.report_item_task).find('input.maximo-text');
				  var taskButton = $j(this.$.report_item_task).find('maximo-button.lookup.maximo-text');
			  }
			  //Trigger lookup on focus event of the task field's input
	          taskInput.focus(function(event){
	        	  taskInput.attr('readonly',true);
	        	  taskInput.css('cursor','pointer');
	        	  taskButton.click();
			  });
		      //Execute only when report_item_sire is modified
		      if(e.target && (e.target.id==='report_item_site___text' || e.target.id==='report_item_card_report_item_site' || e.target.id==='report_item_site')) {
			          var currentSiteId = self.siteid;		          
			          if(!currentSiteId || currentSiteId===''){
			        	  self.siteid='';
			          }else {
			        	  self.siteid=currentSiteId;
			          }
			  }
		      //Add style for lookup icon to aligned with text input field
		      $j(self.$.report_item_wrapper).find('maximo-button.lookup iron-icon.icon.west').attr('style','margin-top:0px !important');
	         },100);
	         var storeLocation = this.$.report_item_storeroom.value;
	    	 if(!storeLocation){
	        	  this.location = '';
	    	 }
		     this._hideItem(false);
	      },

	      /**
	       * To Save/Update item card
	       */
	      _saveItem: function (e) {
	    	  var item = {};
	    	  var material = {};
		      var self = this;
		      var message;
		      this.async(function(){
		    	  var saveObject;
		    	  if(self.$.report_item_type.$.select.value === this.getExternalValueForLineType('ITEM')){
			          item._action='Add';
			          item.description = self.$.report_item_description.value ? self.$.report_item_description.value : self.$.report_item_description_readonly.label;
			          item.itemnum = self.$.report_item_itemnum.value;
			          item.linetype = self.$.report_item_type.$.select.value;
			          item.storeloc = self.location;
			          if(self.$.report_item_bin.value !== $M.localize('uitext', 'mxapibase', 'NotEntered')){
			        	  item.binnum = self.$.report_item_bin.value;
			          }
			          item.positivequantity = parseFloat(self.$.report_item_quantity.value);
			          item.taskid = self.$.report_item_task.value;
			          if(self.domHost){
						item.issuetype = self.$.report_item_issued.checked ? self.domHost.getExternalValueForIssuetype('ISSUE') : self.domHost.getExternalValueForIssuetype('RETURN');  
					}
			          item.siteid = self.siteid;
			          item.itemsetid = self.itemsetid;
			          if(self.rotating){
			        	  item.rotassetnum = self.assetnum;
			          }
			          if(self.requestnum){
			        	  item.requestnum = self.requestnum;	
			          }
			          saveObject = item;
			          message = $M.localize('uitext', 'mxapiwodetail', 'reportedItemsave');
		    	  }
		    	  else{
		              material._action='Add';
		    	  	  material.description = self.$.report_material_description.value;
		    	  	  material.linetype = self.$.report_item_type.$.select.value;
			    	  material.unitcost = parseFloat(self.$.report_material_unit.value);
			    	  material.positivequantity = parseFloat(self.$.report_item_quantity.value);
			    	  material.taskid = parseInt(self.$.report_material_task.value);
			    	  if(self.domHost){
						material.issuetype = self.$.report_item_issued.checked ? self.domHost.getExternalValueForIssuetype('ISSUE') : self.domHost.getExternalValueForIssuetype('RETURN');  
					}
			    	  material.siteid = self.siteid;
			    	  material.itemsetid = self.itemsetid;
			    	  var materialDes = self.$.report_material_description.value;
		  	      	  message = $M.localize('uitext', 'mxapiwodetail', 'wpnewmaterialsave', [materialDes]);
		  	      	  saveObject = material;
		    	  }
		          var performNewItemSaveOperation = function(overlayNode) {
		              self.fire('save-item-card', saveObject);
		      	  };
		      	  var revertSaveOperation = function(){
		    	  	  self.fire('disable-item-list', {'card':null, 'disabled':false,'revertOperation':true});
			      };
		    	  $M.createOverlay(self.$.report_item_card, message, performNewItemSaveOperation, revertSaveOperation, self.undoIntervalSecs, self.parentElement.parentElement, null);	    	  
		      },100);
	    	  this.fire('disable-item-list', {'card':self, 'disabled':true});	      
		  },
       
       /**
	  	 * This method will toggle border when card is in edit mode
	  	 */
	   _updateBorder: function(){
			$j(this.$.report_item_innerDiv).toggleClass('dirtyBorder',this.isDirty);
		},
		
		/**
	  	 * Reset the values.
	  	 */
		_clearDirty: function(e){
			this.clearDirtyStatus();
		},
		
		/**
         * To fetch the item lookup.
         */
		_getCustomizeditemref : function(recordhref){
			if(this.record && this.record.localref !== undefined){
	  			return this.record.localref;
	  		} else {
	  			return this.recordhref+'/uxshowactualmaterial/zombie';
	  		}
		},
		
		/**
         * To fetch the binnum lookup.
         */
		_getCustomizedbinnumref : function(recordhref){		
			if(this.record && this.record.localref !== undefined){
	  			return this.record.localref;
	  		} else {
	  			return this.recordhref+'/uxshowactualmaterial/zombie';
	  		}
		},
		
		/**
		 * Parse linetype to create key:description string array for Drop Down
		 */
		_typeList : function(record){
			var linetypes = record;
			var externalValues=[];
			if(linetypes.length>0){
				linetypes.forEach(function(member){
					externalValues.push(member.value+':'+(member.description?member.description:member.value));
				});
			}
			return externalValues.toString();
		},
		
		_handlelineTypeDomainSet : function(e){
			var linetypeFilteredSet = this.lineTypeDomainCollection[0].synonymdomain;
			this.lineTypeDomainSet = linetypeFilteredSet.filter(function(o){return (o.maxvalue === 'MATERIAL' || o.maxvalue === 'ITEM');} );
		},
		
		_handlelineTypeDomainSetError : function(e){
			this.lineTypeDomainSet = [];
		},
		/**
         * To fetch the task lookup.
         */
		_getCustomizedtaskref : function(recordhref){
        	return recordhref+'/showplanmaterial/zombie';                  
        },
		
        /**
	  	 * To fetch the StoreRoom for lookup.
	  	 */
		_getCustomizedStoreRoomRef : function(recordhref){
			if(this.record && this.record.localref !== undefined){
				return this.record.localref;
			}
	  		return recordhref+'/uxshowactualmaterial/zombie';			
		},
		
		/**
		 * For new card it is taking value from select box
		 * and existing card it is from maxvalue
		 * hide or show material/item field on the basis of item type
		 */	
		_hideItem : function(isLoaded){
			var item_type=this.isNew?this.$.report_item_type.$.select.value:this.$.report_item_type.maxValue;
			item_type=this.id==='newItemCard'?this.$.report_item_type.$.select.value:item_type;
			if(!item_type){
				return;
			}
			if(item_type !== this.getExternalValueForLineType('MATERIAL')){
				$j(this.$.report_item_innerDiv).find('.material_toggle > *').css('display','none');
				$j(this.$.report_item_innerDiv).find('.item_toggle > *').css('display','block');
				$j('.report_item_rotating_asset.item_toggle').show();
				if(isLoaded === true){
					this.$.report_material_description.value = '';
	                this.$.report_material_unit.value = 0;
	                this.$.report_item_task.value = '';
	                this.$.report_item_quantity.value = '';
	                this.$.report_item_issued.checked = true;
	                this.$.report_item_site.value = '';
	                this.siteid = '';
				} 
				$j(this.$.report_item_card).removeClass('editable-material-cont');
			}
			else{
				$j(this.$.report_item_innerDiv).find('.material_toggle').prop('hidden',false);
				$j(this.$.report_item_innerDiv).find('.material_toggle > *').css('display','block');
				if(isLoaded === true){
					this.$.report_item_description.value = '';
	                this.$.report_item_storeroom.value = '';
	                this.location = '';
	                this.$.report_item_bin.value = '';
	                this.$.report_item_quantity.value = '';
	                this.$.report_material_task.value = '';
	                this.$.report_item_issued.checked = true;
	                this.$.report_material_site.value = $M.getMaxauth().whoami.defaultSiteDescription;
	                this.$.report_item_itemnum.value='';
				}
				$j(this.$.report_item_card).addClass('editable-material-cont');
				$j(this.$.report_item_innerDiv).find('.item_toggle > *').css('display','none');
				$j('.report_item_rotating_asset.item_toggle').hide();
				if(this.editable === false){
					this.$.report_material_description.setAttribute('hidden',true);
				}
			}
			 if (this.$.report_item_type.$.select.value) {
				 this.selectedType = this.$.report_item_type.$.select.value;
			}
	},
	/**
	 * _showInfo will show the Tooltip.
	*/
	_showInfo: function(e) {		
		e.stopPropagation();
		var tooltip = this.$.info;
		$M.showTooltip(tooltip, e.currentTarget);
		
	},
	/**
     * To set issuetype
     */
	_getIssueType: function(record){
		return this.record.issuetype_maxvalue==='ISSUE'?true:false;
	},
	
	/*
	 * Get Location clause for Item Lookup
	 */
	 _getLocationClause : function(siteid,locationselected){
		var locClause = '';
        if(siteid){
			if(locationselected && locationselected !== ''){
        		locClause = "setupdata={\'matusetrans\':{\'siteid\':\'"+siteid+"\',\'storeloc\':\'"+locationselected+"\'}}";
        	} else {
        		locClause = "setupdata={\'matusetrans\':{\'siteid\':\'"+siteid+"\'}}";
        	}
        }
        return locClause;
	 },
	
	/*
	 * Get Itemnum clause for location Lookup
	 */
	_getItemnumClause : function(siteid,itemnumselected){
	    var itemClause = '';
        if(itemnumselected && itemnumselected!== ''){
        	if(siteid){
        		itemClause = "setupdata={\'matusetrans\':{\'siteid\':\'"+siteid+"\',\'itemnum\':\'"+itemnumselected+"\'}}";
        	} else {
        		itemClause = "setupdata={\'matusetrans\':{\'itemnum\':\'"+itemnumselected+"\'}}";	
        	}
        	
        } else{
        	if(siteid){
        		itemClause = "setupdata={\'matusetrans\':{\'siteid\':\'"+siteid+"\'}}";
        	}
        }
        return itemClause;
    },
	/*
	 * Get Itemnum, location and siteid clause for BinNum Lookup
	 */
	_getBinnumClause : function(locationselected,itemnumselected,siteidselected){
		if(itemnumselected && itemnumselected!== '' && locationselected && locationselected!==''){
			//return 'setupdata={%22matusetrans%22:{%22itemnum%22:%22'+itemnumselected+'%22,%22storeloc%22:%22'+locationselected+'%22,%22siteid%22:%22'+siteidselected+'%22}}';
			return 'setupdata={%22matusetrans%22:{%22siteid%22:%22'+siteidselected+'%22,%22storeloc%22:%22'+locationselected+'%22,%22itemnum%22:%22'+itemnumselected+'%22}}';
		}
		return "setupdata={%22matusetrans%22:{%22siteid%22:%22"+this.siteid+"%22}}";
	},
	
	/**
	 * Get default Organization clause for Site lookup
	 */
	_defaultOrgClause : function(siteid){
		if($M.getMaxauth() && $M.getMaxauth().whoami) {
			return "oslc.where=orgid=%22"+$M.getMaxauth().whoami.defaultOrg+"%22";	
		}
		return '';
	},
	
	/**
     * Reset new cards.
     */
     _resetCard: function (e) {    
   	  	if(this.newCard) {
        	this._deleteItem(e);
        }
   	  	else {
   	  		this.fire('reset-dirty-element');
   	  	}
     },
     
     /*
 	 * Get Item clause for Rotate Asset Lookup
 	 */
 	_getRotateItemClause : function(siteid,itemnumselected,locationselected){
	    var itemClause = '';
        if(itemnumselected && itemnumselected!== '' && siteid){
        	if(locationselected && locationselected !=''){
        		itemClause = 'setupdata={%22matusetrans%22:{%22siteid%22:%22'+siteid+'%22,%22storeloc%22:%22'+locationselected+'%22,%22itemnum%22:%22'+itemnumselected+'%22}}';        		
        	} else {
        		itemClause = 'setupdata={%22matusetrans%22:{%22siteid%22:%22'+siteid+'%22,%22itemnum%22:%22'+itemnumselected+'%22}}';
        	}       
        }
        return itemClause;
    },
    
    /**
  	 * To fetch the Rotate Asset for lookup.
  	 */
    _getCustomizedRotateAssetRef : function(recordhref){
		if(this.record && this.record.localref !== undefined){
			return this.record.localref;
		}
  		return recordhref+'/uxshowactualmaterial/zombie';			
	},
	
	/**
	 * To get the Storeroom lookup count when Item is selected
	 */ 
     _getLookupCount : function(){    	 
    	 var textcomponent = $j(this.$.report_item_storeroom).find('maximo-text')[0];
    	 var lookup = $M._lookups[textcomponent._lookupObject.name];
    	 var lookupURI =  textcomponent.lookup.href+'/getlist~'+lookup.getList +'?'+textcomponent.lookup.additionalLookupParams;

    	 //Find the record count
    	 this.$.reportitem.collectionUri = lookupURI;
    	 this.$.reportitem.attributeNames = lookup.attributeNames;
    	 var self = this;
         this.$.reportitem.refreshRecords().then(function(lookupData) {
             var storeroomCount = lookupData.response.responseInfo.totalCount;
             if(storeroomCount && storeroomCount === 1){
            	 var currentRecord = lookupData.response.member[0];
            	 if(currentRecord && currentRecord.location){
            		 self.checkStoreroomAccess(currentRecord.location, currentRecord.description, self.siteid, true);
    				 var that = self;
    				 self.async(function(){
    					 that._getBinLookupCount();
    				 },200);    					 
            	 }
             }
         });    	 
     }, 
      
     /**
      * To get the bin lookup count when Item and storeroom are selected.
      */
    _getBinLookupCount : function(){    	
		 var bincomponent = $j(this.$.report_item_bin).find('maximo-text')[0];
    	 var binLookup = $M._lookups[bincomponent._lookupObject.name];
    	 var binURI =  bincomponent.lookup.href+'/getlist~'+binLookup.getList +'?'+bincomponent.lookup.additionalLookupParams;
    	 
    	 this.$.reportitem.collectionUri = binURI;
    	 this.$.reportitem.attributeNames = binLookup.attributeNames;
    	 var self = this;
    	 this.$.reportitem.refreshRecords().then(function(lookupData) {
             var totalCount = lookupData.response.responseInfo.totalCount;
             if(totalCount && totalCount === 1){
            	 var binRec = lookupData.response.member[0];
            	 if(binRec && binRec.binnum || binRec.curbal !== '') {
      			 	self.$.report_item_bin.value = binRec.binnum;			 
      		 	}
             } else {
            	 self.$.report_item_bin.value = '';
             }
         }); 
     },
     
   /**
    * To get Rotating Asset lookup count
    */
     _getAssetLookupCount : function(){
     	 if(!this.rotating || !this.itemnum){
     	 	return;
     	 }
		 var assetcomponent = $j(this.$.report_item_rotating_asset).find('maximo-text');
		 var assetLookup;
		 var assetURI;
		 if(assetcomponent){
			 assetLookup = $M._lookups[assetcomponent.context._lookupObject.name];
			 assetURI =  assetcomponent.context._lookupObject.href+'/getlist~'+assetLookup.getList +'?'+assetcomponent.context._lookupObject.additionalLookupParams;
			 
	    	 this.$.reportitem.collectionUri = assetURI;
	    	 this.$.reportitem.attributeNames = assetLookup.attributeNames;
	    	 var self = this;
	    	 this.$.reportitem.refreshRecords().then(function(lookupData) {
	             var totalCount = lookupData.response.responseInfo.totalCount;
	             if(totalCount && totalCount === 1){
	                 var assetRec = lookupData.response.member[0];
	            	 if(assetRec && assetRec.assetnum) {
	            		self.assetnum = assetRec.assetnum;
	      			 	self.$.report_item_rotating_asset.value = assetRec.description;
	      		 	}
	             }
	         });
		 } 	 
     },
     /**
      * Handle the display of location description and storelocation.
      */
     _getStoreroomLocation : function(storeLoc,locations){
    	 return locations.description ? locations.description : storeLoc;
     },
     
     /**
      * To reset the item fields
      */
     resetItemFields : function(e){
	     this.$.report_item_itemnum.value='';
		 this.$.report_item_description.value='';
		 this.$.report_item_storeroom.value='';
		 this.$.report_item_bin.value = '';
		 this.$.report_item_rotating_asset.hidden=true;
		 this.rotating = false;
		 this.location = '';
		 this.assetnum = '';
		 if(e.detail.records){
		    this.$.report_item_site.value = e.detail.records[0].description;
		 }
	},
	/**
     * Handle the display of item description and item id.
     */
	_getItemValue : function(description, itemnum){
    	return description?description:itemnum;
    },
	
    /**
     * To check whether user have access to storeroom, if have access will set the storeroom fields 
     * if throws error, displaying the error message as notification and clearing the storeroom fields
     */
	checkStoreroomAccess : function(location, locationDesc, siteid, flag){
		var responseprops = 'storeloc,itemnum,siteid';
		var items = [];
		var itemData = {};
		itemData._action = 'Add';
		itemData.storeloc = location;
		itemData.siteid = siteid;
		itemData.positivequantity = 1;
		itemData.itemnum = this.itemnum;
		itemData.linetype ='ITEM';
		items.push(itemData);
		
		var self = this;
		this.domHost.$.timeresource.resourceUri = this.recordhref+ '/uxshowactualmaterial';
		$M.toggleWait(true);
		this.domHost.$.timeresource.updateRecord(items, responseprops, true, false, {'setvaluemode':true}).then( function(result) {
		   $M.toggleWait(false);
		   self.location = location;
		   if(locationDesc){
			   self.$.report_item_storeroom.value = locationDesc;
		   }else{
			   self.$.report_item_storeroom.value = location;
		   }
		}, function(error){
			$M.toggleWait(false);
			if(!flag || self.domHost.sendNotification){
				if(error.request.__data__.response && error.request.__data__.response.Error){
					$M.notify(error.request.__data__.response.Error.message, $M.alerts.warn);
					self.domHost.sendNotification = false;
				}
			}
			self.location = '';
			self.$.report_item_storeroom.value = '';
		});
	},
  
 	
	/**
	   * Fetch Internal Value based on External Value for Linetype
	   */
	   getInternalValueForLineType: function(externalValue){
		   var internalValue; 
		  if(this.lineTypeDomainSet){
			 this.lineTypeDomainSet.forEach(function(domainrecord){
					if(domainrecord.value===externalValue){
						internalValue = domainrecord.maxvalue;
					}	
			  });
		  }
		   return internalValue;
	   },
	   
	  /**
	   * Fetch External Value based on Internal Value for Linetype
	   */
	   getExternalValueForLineType: function(internalValue){
		   var externalValue; 
		  if(this.lineTypeDomainSet){
			 this.lineTypeDomainSet.forEach(function(domainrecord){
					if(domainrecord.defaults && domainrecord.maxvalue===internalValue){
						externalValue = domainrecord.value;
					}	
			  });
		  }
		  this.fire('get-external-value', externalValue);
		  return externalValue;
	   }
});
