/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
	is: 'maximo-technician-card',
	
  	behaviors: [BaseComponent,Workorder,HandlerWorkOrder],
  	
  	listeners: {
  		'designer-set-collections' : '_setDesignerCollections'
  	},
  	
    properties: {
    	
    	/**
    	 * technician record
    	 */
    	record : {
			type: Object,
			notify: true
		},
		
		labtransData : {
			type: Object,
			notify: true
		},
		   	
		/**
    	 * Interval in seconds to 
    	 * undo delete operation
    	 */
    	undoIntervalSecs: {
    		Type: Number,
    		value: 5,
    		readOnly: false
    	},	
    	
    	/**
    	 * schema for mxapiwodetail collection
    	 */
    	schema: {
    		type: Object
    	},
    	
    	/**
    	 * LT Type domain filter
    	 */
    	ltTypeSet: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'LTTYPE', 'selected': true} ]}];
    		} 	
    	},
    	startTime : {
			type: String,
		},
		startDate : {
			type: String,
		},
		stopTime : {
			type: String,
		},
		stopDate : {
			type: String,
		},
    	/**
    	 * The Element to highlight when hovering over in configuration mode.
    	 */
    	highlightElement: {
    		type: String,
    		value: 'tile'
    	},
    	_designerCollections: {
    		type: Array,
    		value: function(){
    			return [];
    		}
    	}
	},
	attached: function(){

		if($M.inDesignMode()){
			this.fire('designer-get-collections',this);
		}
		
		var styleObj = {
				paddingTop: function(){ return this.id.includes('stopdatetime1') ? '5px' : '0px';} ,
				textAlign: 'left'
		};
		
		if($M.isMobile()){
			$j(this.$.stopdatetime1.$.time_mobile_template_label)
				.add(this.$.startdatetime1.$.time_mobile_template_label)
				.css(styleObj);
		}
		else{
			$j(this.$.stopdatetime1.$.time_nonmobile_template_label)
			.add(this.$.startdatetime1.$.time_nonmobile_template_label)
			.css(styleObj);
		}
		
	},
		
	/**
	 * Is user in design mode
	 */
	_inDesignMode: function(){
		return $M.inDesignMode();
	},
	
	/**
	 * Hide priority section if no priority is defined on the record.
	 */
	showPriority: function(wo){
		if(wo.wopriority){
			return false;
		} else if(!$M.inDesignMode()) {
			return true;
		}
	},

	/**
	 * Hide worktype section if no worktype is defined on the record.
	 * The field will always appear if worktype attribute has changed by using drag/drop
	 */
	showWorktype: function(wo,id){
		
		if(id){ //check if worktype attribute has been replaced with drag/drop
			var component = $j(this).find('[cid='+id+']')[0];
			if(component){
				var maxComponent = component.getMyComponent();
				//get name of the attribute before it is bound and value exists
				var designerAttributes = JSON.parse(maxComponent.getAttribute('unbound-designer-attributes'));
				if(designerAttributes){
					//define the property where the attribute exists based on the type of component
					var valueAttribute = 'value';
					if(maxComponent.tagName === 'MAXIMO-LABEL'){
						valueAttribute = 'label';
					}
					var attribute = designerAttributes[valueAttribute];
					if(attribute !== 'worktype'){//if worktype attribute has changed with drag/drop, always show the field
						return false;
					}	
				}
			}
		}
		if(wo.worktype && !wo.istask){
			return false;
		} else if(!$M.inDesignMode()){
			return true;
		}
	},
	
	/**
	 * Show/Hide the Start link.
	 */
	_showStart : function(wo){
		if (!wo) {
			wo = this.record;
		}
		var userInfo = $M.getUserInfo();
		if(!userInfo || !userInfo.labor){
			var technicianCard = this;
			this.async(function(){
				technicianCard._showStart();
			}, 200);
			return;
		}
		if(userInfo || userInfo.labor || userInfo.labor.laborcode){
			var userLaborCode = userInfo.labor.laborcode;
			var craft = userInfo.labor.laborcraftrate.laborcode;		
			var userLabtrans=[];
			var labtransStatus='';
			if(wo.labtranstimerstatus){
				if(wo.labtranstimerstatus.length>0){
					wo.labtranstimerstatus.forEach(function(member){
						if(member.laborcode===userLaborCode){
							labtransStatus='ACTIVE';
						}
					});
				}
				else{
					labtransStatus=wo.labtranstimerstatus.timerstatus_maxvalue;
				}
			}
		}
			
		if(  (wo.status_maxvalue==='INPRG' || wo.status_maxvalue !== 'WAPPR' && wo.status_maxvalue !== 'COMP' && wo.status_maxvalue !== 'CAN') && (labtransStatus !== 'ACTIVE') && (userLaborCode !== undefined) && (craft !== undefined)){
			return false;
		
		} else {
			return true;
		}						
	},
	
	/**
	 * Show/Hide the Start link.
	 */
	_canShowStart : function(wo){
		var showStart = this.isAuthorized('MXAPIWODETAIL:STARTTIMER');
		if(!showStart){
			console.log('access denied for Start Work');	
			return true;
		}
		
	  return  this._showStart(wo); 
	},
	 /* Show/Hide the Report Actuals link.
	 */
	_showReportWork : function(wo){
		if(!(this.isAuthorized('REPORT_TIME') || this.isAuthorized('REPORT_FAILURES')|| this.isAuthorized('REPORT_MATERIAL'))){
			console.log('access denied for Report Work');	
			return true
		}
		
		if (!wo) {
			wo = this.record;
		}	
		var userInfo = $M.getUserInfo();
		if(!userInfo || !userInfo.labor){
			var technicianCard = this;
			this.async(function(){
				technicianCard._showStart();
			}, 200);
			return;
		}
		var userLaborCode = userInfo.labor.laborcode;
		var userLabtrans=[];
		var labtransStatus='';
		if(wo.labtranstimerstatus){
			if(wo.labtranstimerstatus.length>0){
				wo.labtranstimerstatus.forEach(function(member){
					if(member.laborcode===userLaborCode){
						labtransStatus='ACTIVE';
					}
				});
			}
			else{
				labtransStatus=wo.labtranstimerstatus.timerstatus_maxvalue;
			}
		}
				
		
		if(  (wo.status_maxvalue==='INPRG' || wo.status_maxvalue !== 'WAPPR' && wo.status_maxvalue !== 'CAN') && (labtransStatus === 'ACTIVE')){
			return false;			
		} else {
			return true;
		}	
		
	},
	
	/**
	 * Show/Hide the Complete link.
	 */
	_showStop : function(wo){
		var showStop = this.isAuthorized('MXAPIWODETAIL:STOPTIMER');
		if(!showStop){
			console.log('access denied for Stop Work');	
			return true;
		}
		if (!wo) {
			wo = this.record;
		}
		var userInfo = $M.getUserInfo();
		if(!userInfo || !userInfo.labor){
			var technicianCard = this;
			this.async(function(){
				technicianCard._showStop();
			}, 200);
			return;
		}
		var userLaborCode = userInfo.labor.laborcode;
		var userLabtrans=[];
		var labtransStatus='';
		if(wo.labtranstimerstatus){
			if(wo.labtranstimerstatus.length>0){
				wo.labtranstimerstatus.forEach(function(member){
					if(member.laborcode===userLaborCode){
						labtransStatus='ACTIVE';
					}
				});
			}
			else{
				labtransStatus=wo.labtranstimerstatus.timerstatus_maxvalue;
			}
		}
						
		if(  (wo.status_maxvalue==='INPRG' || wo.status_maxvalue !== 'WAPPR' && wo.status_maxvalue !== 'CAN') && (labtransStatus === 'ACTIVE')){
			return false;			
		} else {
			return true;
		}
			
	},
	
	/**
	 * Show/Hide the Line.
	 */
	_showLine : function(wo){
		if (!wo) {
			wo = this.record;
		}
		var userInfo = $M.getUserInfo();
		if(!userInfo || !userInfo.labor){
			var technicianCard = this;
			this.async(function(){
				technicianCard._showStart();
			}, 200);
			return;
		}
		var userLaborCode = userInfo.labor.laborcode;
		var userLabtrans=[];
		var labtransStatus='';
		if(wo.labtranstimerstatus){
			if(wo.labtranstimerstatus.length>0){
				wo.labtranstimerstatus.forEach(function(member){
					if(member.laborcode===userLaborCode){
						labtransStatus='ACTIVE';
					}
				});
			}
			else{
				labtransStatus=wo.labtranstimerstatus.timerstatus_maxvalue;
			}
		}
		
		if(  (wo.status_maxvalue==='COMP' || wo.status_maxvalue === 'WAPPR' && wo.status_maxvalue !== 'CAN') && (labtransStatus !== 'ACTIVE') ){	
			return true;			
		} else {
			return false;
		}		
	},
	
	/**
	 * Hide/Show image if it exists
	 */
	showImage : function(image){
		if (image){
			return false;
		} else {
			return true;
		}
	},

	/**
	 * Triggers page transition to detail.
	 */
	_workOrderDetail: function(e){
		this.fire('showWODetail', this.record);
	},

	/**
	 * Triggers page transition to detail.
	 */
	_reportWork: function(e){
		this.record.ltTypeSet=this.ltTypeSet;
		this.record.isWorkOrderComp=this.isWorkOrderComp;
		this.fire('reportWork',this.record);
	},
	
	
	/**
	 * Start Assignment
	 */
	_startAssignment : function() {
		
		var self = this;
		this.isWorkOrderComp=false;
		var labor = $M.userInfo.labor.laborcode;
		var card = this.$.undoOverlayDiv;
		var wonum = this.record.wonum;
		var worktype = this.record.worktype;
		var message = $M.localize('uitext','mxapiwodetail','woStatusStarted',[(worktype && !this.record.istask?worktype:'') ,wonum]);
		var _timerExpired = function(overlayNode) {
			self.$.worklinks1.hidden = true;
			self.fire('startAssignment', {'record': self.record, 'workType': self.$.select.value, 'laborcode':labor});
		};
		self.$.selectWorkType.hidden = true;
		$M.createOverlay(card, message, _timerExpired, null, self.undoIntervalSecs, null, null);	
	},
	
	
	_onSelectWorkOverlay: function() {
		var oriRecord = this.record;
	  	var self = this;
	  	var externalValues=[];
	  	this.$.confirmResource.resourceUri = this.record.href+'/uxshowactuallabor/newmbo/getlist~transtype?oslc.select=synonymdomainid,valueid,maxvalue,href,defaults,description,value,domainid&lean=1&querylocalized=1';
	  	 this.$.confirmResource.loadRecord().then(function(result){
	  					 
	  					if(result && result.member && result.member.length > 0){
	  						
	  						result.member.forEach(function(domainrecord){
				 	   	    		externalValues.push(domainrecord);
					  	   });
	  						self.record = oriRecord;
	  						if(externalValues.length > 0){
	  							self.record.ltTypeSet= externalValues;	
	  							self.ltTypeSet = externalValues;
	  						}
	  						
				 		}
	  					self.$.selectWorkType.hidden = false;	  					
	  					self.$.select.values = self._domainList(self.record.ltTypeSet);
	  					self._fixWorkTypeLabel();
	  					self._closeOverlay();
	  				 });
		
	},

	_offSelectWorkOverlay: function() {
	    this.$.selectWorkType.hidden = true;
	},
	
	
	/**
	 * Finish Assignment
	 */
	_finishAssignment : function() {	
			
		var self = this;
		var card = this.$.undoOverlayDiv;
		var wonum = this.record.wonum;
		var worktype = this.record.worktype;
		var workorderid = this.record.workorderid;
		var message = $M.localize('uitext','mxapiwodetail','woStatusComplete',[(worktype && !this.record.istask?worktype:''), wonum]);
		var question = $M.localize('uitext','mxapiwodetail','changeWoToComplete');
		var _noButtonText = $M.localize('uitext','mxapiwodetail','noStopOnly');
		var _yesButtonText = $M.localize('uitext','mxapiwodetail','Yes');
		var userInfo = $M.getUserInfo();
		if(!userInfo || !userInfo.labor){
				var technicianCard = this;
				this.async(function(){
					technicianCard._showStart();
				}, 200);
				return;
			}
			var userLaborCode = userInfo.labor.laborcode;		
			var labtranstimerstatus = self.record.labtranstimerstatus;
			var starttime = labtranstimerstatus.starttimeentered;
			var startdate = labtranstimerstatus.startdateentered;
				if(labtranstimerstatus.length>0){
					labtranstimerstatus.forEach(function(member){
						if(member.laborcode===userLaborCode){					
							starttime = member.starttimeentered_localized;
							startdate = member.startdateentered_localized;
						}
					});
				}	
		    var _yesButton = function(e){
				self.record.ltTypeSet=self.ltTypeSet;
				self.record.isWorkOrderComp=true;
				self.fire('reportWorkFromCard',self.record);
			};
					
			var confirmResourceUri = $M.getMaxauth().baseUri + '/oslc/os/mxapiwodetail?lean=1&oslc.where=workorderid%3D%22' + workorderid + '%22&oslc.select=maxvar.confirmlabtrans,maxvar.labtranstolerance'; 
			self.$.confirmResource.resourceUri = confirmResourceUri;
			self.$.confirmResource.loadRecord();
			var _noButton = function(overlayNode){		
					
				if (self.labtransData && self.labtransData.member[0].confirmlabtrans==='1') {
					
					self.$.setDateTime.hidden = false;
					self._closeOverlay();
					var timeformat = $M.getMaxauth().whoami.timeformat;
					var dateFormat = $M.getMaxauth().whoami.dateformat.toUpperCase();
					$M.getServerDateTime().then(function(response){
						if($M.isMobile()){
							self.startTime = moment(starttime,timeformat).format('HH:mm');
							self.startDate = moment(startdate, dateFormat).format('YYYY-MM-DD');
							self.stopTime = moment(response.time,timeformat).format('HH:mm');
							self.stopDate = moment(response.datetime, dateFormat).format('YYYY-MM-DD');
						} else {
							self.startTime = starttime;
							self.startDate = startdate;
							self.stopTime = response.time.toUpperCase();
							self.stopDate = moment(response.datetime, dateFormat).format(dateFormat);						
						}
					});			
				} else { 
					self.$.setDateTime.hidden = true;				
					self._stopAssignment();
				}
			};	
		var _undo = function(overlayNode) {	
			self.$.worklinks1.hidden = false;
		};
				
		$M.questionOverlay(card, question, _noButton, _yesButton, _noButtonText, _yesButtonText, null, null, null, null, true);
	 },
	
	_confirmHandler : function() {
		$M.toggleWait(false);
	},
	
	_confirmErrorHandler : function(e) {
		var error = e.detail.error;
		$M.notify(error.message, $M.alerts.error);
		$M.showResponseError(error);
		$M.toggleWait(false);
	},
	
	/**
	 * Stop Timer
	 */
	_stopAssignment : function() {
		var self = this;
		var card = this.$.undoOverlayDiv;
		var wonum = this.record.wonum;
		var worktype = this.record.worktype;
		var stopMessage = $M.localize('uitext','mxapiwodetail','woStatusStop',[(worktype && !this.record.istask?worktype:'') ,wonum]);
		var _timerExpired = function(overlayNode) {
			self.fire('stopAssignment', self.record, self.startTime, self.startDate, self.stopTime, self.stopDate);
		};
		
		$M.createOverlay(card, stopMessage, _timerExpired, null, self.undoIntervalSecs, null, null);
			self.$.setDateTime.hidden = true;
	},
	
	_offDateTimeOverlay: function() {
	    this.$.setDateTime.hidden = true;
	},
	
	/**
	 * Parse domains to create key:description string array for Drop Down
	 */
	_domainList : function(record){
		var self = this;
		var worktypes = record;
		var externalValues=[];
		if(worktypes && worktypes.length>0){
				worktypes.forEach(function(member){
					if(member.maxvalue==='WORK'){
						externalValues.push(member.value+':'+member.description);
						if(member.valueid==='LTTYPE|WORK'){
							self.$.select.value = 'WORK';
						}
					}
				});
			}
			return externalValues.toString();
	},
	/**
	 * @param status:- record status_maxvalue.
	 * @return:-color code as per the status_maxvalue.
	 */
	_getStatusColorCode: function(status){
  		var color = '#5596e6';
			switch (status) {
			case 'WMATL':
			case 'WSCH':
			case 'WPCOND':
			case 'WAPPR':
				color = '#fdd600';
				break;
			case 'INPRG':
				color = '#8cd211';
				break;
			default:
				color = '#5aaafa';
			}
        return color;
	},
	/**
	 * add the style in border of tile on hover
	 */
	 _addHoverStyle : function(event) {
			event.currentTarget.style = 'outline : 3px solid '+ this._getStatusColorCode(this.record.status_maxvalue)	+ ' !important; border-bottom: 10px solid '	+ this._getStatusColorCode(this.record.status_maxvalue)+ ' !important';
	},
	/**
	 * clear the style in border of tile on hover 
	 */
	_clearHoverStyle : function(event) {
			event.currentTarget.style = 'border-bottom : 10px solid '+ this._getStatusColorCode(this.record.status_maxvalue)+ ' !important;';
	},
	
	/**
	 * Close the overlay everytime we click outside of it(specified div in var container)
	 */
	_closeOverlay : function() {
		var self = this;
		var container = self.$.tile;
		var outsideClick = function(e) { 			
			if (!container.contains(e.target) && $j('maximo-tooltip').find(e.target).length === 0){
				self.$.selectWorkType.hidden = true;
				self.$.setDateTime.hidden = true;
				window.removeEventListener('tap',outsideClick);
			}
		}
		window.addEventListener('tap',outsideClick);
	},
	/**
	 * Set the collections on the card in order to retreive it in design mode
	 */
	_setDesignerCollections : function(e){
		this._designerCollections = e.detail;
	},
	/**
	  * fix for the worktype label on cards and 
	  * on wo details page.
	  */
	_fixWorkTypeLabel:function(){
		var leftAlignWorktypeLabel = {
				display    : "flex",
				padding    : "0px 0px 5px 50%",
				marginLeft : -(this.$.select.$.selectdiv.getBoundingClientRect().width/2) + "px"
		};
		$j(this.$.select.$.label).css(leftAlignWorktypeLabel);
	}
});
