/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
	is: 'maximo-technician-list',
	
  	behaviors: [BaseComponent,InfiniteScroll],
  	
  	listeners: {
  		'load-next-list' : '_loadNextList'
  	},
  	
    properties: {
    	
    	/**
    	 * List of inspection form
    	 */
    	dataSet: {
    		type: Array
    	},
    	
    	/**
    	 * collection of work order
    	 */
    	collection: {
			type: Object,
			value : null ,
			observer: 'dataSetRefresh'				 
		},
		
    	schema: {
    		type: Object
    	},
    	
    	/**
    	 * Indicator to show/hide message
    	 */
    	hasRecord: {
    		type: Boolean,
    		value: true,
    		readOnly: true
    	},
    	
		/**
		 * Filter Counts
		 */
		overdueCount: {
			type: Object,
			notify: true
		},
		todayCount: {
			type: Object,
			notify: true
		},
		tomorrowCount: {
			type: Object,
			notify: true
		},
		nextdaysCount: {
			type: Object,
			notify: true
		},
		completedCount: {
			type: Object,
			notify: true
		},
		allRecordCount: {
			type: Object,
			notify: true
		},  
		
    	/**
	     * classifiers map
	     */
    	sortSet: {
    		type: Object,
    		value:  []     
    	}   
	},   
	
	ready : function(){
		this.fire('domaincollection');
	},

	/**
	 * add listener  on search bar
	 */
	attached: function() {
		var that = this;
		this.sortSet = [{ key:'wopriority', displayName: $M.localize('uitext', 'mxapiwodetail', 'Priority') },
		  	      	{ key:'schedstart', displayName: $M.localize('uitext', 'mxapiwodetail', 'Scheduledstart') },
		  	      	{ key:'status', displayName: $M.localize('uitext', 'mxapiwodetail', 'Status')},
		  	      	{ key:'wonum', displayName: $M.localize('uitext', 'mxapiwodetail', 'Workorder') } 
		      	      ];
		 
		//evaluate which content selector should be displayed
		this.calculateFilterMenuStyle(this);
		this.infiniteScrollingList(this.$.technicianList, 'collection');
		$j('.maximo-searchbar >  .searchIcon').css({'margin-top':'9px'});
		//listen to screen resize to show correct filter menu style
		$j(window).on('resize', function () {
			that.calculateFilterMenuStyle(that);
		});
		
	},
	
	/**
	 * Calculate filter menu style
	 * Small device - 4.5" show blue content selector
	 * 
	 */
	calculateFilterMenuStyle : function(self){
		var screensize = $M.getPhysicalSize();
	  	if(screensize.width <4.5){
	  	  $j(self.$.contentSelector).removeAttr('light');  
	  	} else {
	  	  $j(self.$.contentSelector).attr('light','0');
	  	}
	},
		
	/**
	 * Set hasRecord flag on collection refresh
	 */
	dataSetRefresh: function ( ) {
		var total;
		
		if (this.collection) {
			this.dataSet = this.collection.collectionData;
		} else{
			this.dataSet = null;
		}

		if (!this.dataSet) {
			total = 0;
		}else {
			total = this.dataSet.length;
		}
		if (total === 0){
			this._setHasRecord(false);
		}else{
			this._setHasRecord(true);
		}
		if($M.isMobile() && $M.screenInfo.orientation === 'portrait' && window.screen.availWidth < '670' 
			&& this.$.technicianList._itemsPerRow != undefined  && this.$.technicianList._itemsPerRow === 0){
			this.$.technicianList._itemsPerRow = 1;
		}
		
	},
	
	
	/**
	 * Fires event to main to pass selected content selector filter.
	 */
	filterCollection: function(e){
		this.fire('filtercollection',e.currentTarget.getAttribute('name'));
	},
	/**
	 * Load next page data on scroll event in the iron-list.
	 */
	_loadNextList: function(e){
		this._listScrolling = false;
		var self = this;
		this.async(function(){
			self.infiniteScrollingList(this.$.technicianList);
		},200);
	},
	
	/**
	 * To fire a createworkorder event
	 */
	_createWorkorder : function(){
		this.fire('create-work-order');
	},
	
	/**
	 * Show create work button based on device type. 
	 */
	_woCreateButton : function(){
		if($M._getSessionDesignMode()){
			return true;
		}
		var show = false;
		if(!$M.checkSigoptions('MXAPIWODETAIL:INSERT')) {
			show = true;
		} else {
			if($M.isMobile()){
				this.$.createWork.label = '';
			} else{
				this.$.createWork.label = $M.localize('uitext','mxapiwodetail','createWork');
			}
		}
		return show;
	}
});
