/*
* @license
* Licensed Materials - Property of IBM
* 5724-U18
* Copyright IBM Corporation. 2017,2018
*/
Polymer({
	is: 'maximo-technician-plans',
	listeners: {
        'cancel-long-desc' : '_cancelLongDescription',
        'maximo-page-header-fixed' : '_toggleWSFixed'
	},
  	behaviors: [BaseComponent,Workorder],
	properties : {
		totalTaskCount: {
			type: Number,
			notify: true
		},
		totalMaterialCount: {
			type: Number,
			notify: true
		},
		tasksLoaded : {
			type: Object,
			value: false,
			notify: true
		},
		materialsLoaded : {
			type: Object,
			value: false,
			notify: true
		},
    	/**
    	 * Interval in seconds to 
    	 * undo approve operation
    	 */
    	undoIntervalSecs: {
    		type: Number,
    		value: 7,
    		readOnly: true
    	},
		contextSubscriptions: {
			value: ['maximo-page-header-fixed']
		},
		/**
    	 * To check whether the tools are loaded or not.
    	 */
		toolsLoaded : {
			type: Object,
			value: false,
			notify: true
		},
		/**
    	 * To get total number of tools.
    	 */
		totalToolsCount: {
			type: Number,
			notify: true
		},
		
		/**
		 * current tab name.
		 */
		currentTab: {
			type: String,
			notify: true
		}
	},
  	created : function(){
  		
  	},
  	ready : function(){
  		
  	},
  	attached: function(){

  	},
  	/*
  	 * Get sequence label
  	 */
  	_getSeqLabel: function(seqnum){
        return ( seqnum !== undefined ? seqnum + '. ' : '') ;
    },
  	/*
  	 * Check if status is completed
  	 */
  	_isCompleted: function(status){
        return ( status === 'COMP' ? true : false) ;
    },
  	/*
  	 * Owner label
  	 */
  	_getOwnerLabel: function(){
        return this.taskwoschema.properties.owner.title;
    },
  	/*
  	 * Owner Group label
  	 */
  	_getGroupLabel: function(){
        return this.taskwoschema.properties.ownergroup.title;
    },
    /*
     * Add(1) to index
     */
  	_getIndex: function(index){
        return index+1;
    },
    /*
     * Switch tab for detail, plan, comments and attachment
     */
	switchContent: function(event) {
		var curIdArray = event.currentTarget.id.split('_');
		var sectionId = 'section_' + curIdArray[curIdArray.length-1];
		this._showHideSection(sectionId);
	},
	/*
	 * Show and hide tab for detail, plan, comments and attachment
	 */
	_showHideSection: function(sectionName) {
		this.currentTab = sectionName;
		var currentList;
		$j(this.$.subcontainer).children('div').each(function(index, elem) {
			if (elem.id.indexOf(sectionName) > -1) {
				currentList = this;
			}
			$j(this).hide();
		});
		
		$j(currentList).show();
		this.loadPlanData();
	},
	/*
	 * Check if count is zero
	 */
	_showNoData: function(count,collectionRefreshed)
	{
	  if(collectionRefreshed && count===0){
    		return false;
    	}
      return true;
	},
	/*
	 * Load material and task data through refresh records and set taskCount and materialCount 
	 */
	loadPlanData : function(){
		//enable wait layer
		$M.toggleWait(true);
		
		var self = this;
		this.tasksLoaded = false;
		this.materialsLoaded = false;
		this.toolsLoaded = false;

		if(this._firstLoad){
			this.$.plansSelector.select(0);
		}
		//To load spinner making count as -1 , this means collections is being refreshed
		this.totalTaskCount= -1;
		this.totalMaterialCount = -1;
		this.totalToolsCount = -1;
		
		//tasks
		if(this.currentTab==='section_tasks'){
	  		this.$.workordertasks.collectionUri = this.record.href+'/wo_tasks.mxapiwodetail';
	  		this.$.workordertasks.refreshRecords().then(function(task){
	  			self.totalTaskCount = task.response.responseInfo.totalCount;
	  			this.taskPagenum = task.response.responseInfo.pagenum;
	  			this.totalTaskPages = task.response.responseInfo.totalPages;
	  			self.tasksLoaded = true;
	  			self.refreshCounters();
	  			$M.toggleWait(false);
	  		});
		}
  		
  		//material
		if(this.currentTab==='section_materials'){
	  		this.$.wpmaterial.collectionUri = this.record.href+'/wpmaterialtask';
	  		this.$.wpmaterial.refreshRecords().then(function(material){
	  			self.totalMaterialCount = material.response.responseInfo.totalCount;
	  			this.materialPagenum = material.response.responseInfo.pagenum;
	  			this.totalMaterialPages = material.response.responseInfo.totalPages;
	  			self.materialsLoaded = true;
	  			self.refreshCounters();
	  			$M.toggleWait(false);
	  		});
		}
  		
  		//tools
		if(this.currentTab==='section_tools'){
	  		this.$.wptools.collectionUri = this.record.href+'/showplantool';
	  		this.$.wptools.refreshRecords().then(function(tools){
	  			self.totalToolsCount = tools.response.responseInfo.totalCount;
	  			self.toolsLoaded = true;
	  			self.refreshCounters();
	  			$M.toggleWait(false);
	  		});
		}
		this._firstLoad = false;
		
	},
	
	/**
	 * Change Status of the current record to "Complete"
	 */
	completeTask : function (e) {
		
		e.stopPropagation();
		var self = this;
		
		this.tasksLoaded = false;
		this.materialsLoaded = false;
		//To load spinner making count as -1 , this means collections is being refreshed
		this.totalTaskCount = -1;
		
		var message = $M.localize('uitext','mxapiwodetail','CompleteTask');

		var performChangeStatusApproveOperation = function(overlayNode) {
			var record = self.$.plans_task_template.modelForElement(e.target).tasks;
			
			var isStatusAllowed = self._isValueInArray(record.canchangetomaxstatus,'COMP');

			if(isStatusAllowed){
				var externalStatus = self._getExternalValueForWOstatus('COMP');
				var params = {status: externalStatus};

				var responseProperties = 'status';
				
				self.$.taskresource.resourceUri=record.href;
				
				//call changestatus action
				self.$.taskresource.updateAction('wsmethod:changeStatus', params, responseProperties).then(function() {
					
					//tasks
			  		self.$.workordertasks.collectionUri = self.record.href+'/wo_tasks.mxapiwodetail';
			  		self.$.workordertasks.refreshRecords().then(function(task){
			  			$M.toggleWait(false);
			  			
			  			self.totalTaskCount = task.response.responseInfo.totalCount;
			  			self.taskPagenum = task.response.responseInfo.pagenum;
			  			self.totalTaskPages = task.response.responseInfo.totalPages;
			  			self.tasksLoaded = true;
			  		});
					
				});
			}
		};
		
		var card = $M.findCard(e.target);
		$M.createOverlay(card, message, performChangeStatusApproveOperation, '', this.undoIntervalSecs, this.parentElement.parentElement, null);

	},
	
    flipCard: function(e){
    	e.stopPropagation();
    	e.preventDefault();
    	var flipper = $M.findElementParent(e.target,'MAXIMO-FLIPPER');
    	if(!flipper) {
    		flipper = e.currentTarget;
    	}
    	flipper.showBack();
    },
    
    flipToFront: function(e){
    	e.stopPropagation();
    	e.preventDefault();
    	var flipper = $M.findElementParent(e.target,'MAXIMO-FLIPPER');
    	if(!flipper) {
    		flipper = e.currentTarget;
    	}
    	flipper.showFront();
    },
    _hasLD: function(task){
    	return task.description_longdescription !== undefined;
    },
    /**
     * Show the tasks readonly long description
     */
    _showLongdesc: function(e){
     	e.stopPropagation();
     	var flyout = $M.showFlyout('ld_flyout', '<div></div>',  e.currentTarget, {'owner':$j('.pageHeader')[1], 'constrainWidth':true});
       	flyout.addButtons([{'label':this.localize('uitext','mxapibase','done'),'event':'cancel-long-desc','eventTarget':this}]);
       	var record = this.$.plans_task_template.itemForElement(e.currentTarget);
       	var value = '';
       	var id = 'longdesc_';
		if(record && record.description_longdescription){
			value = record.description_longdescription;
			id += record._id;
		}
       	var maximoRTE = $M.createElement('maximo-richtexteditor');
       	maximoRTE.props = {
       		'id':id,
       		'placeholder':this.localize('uitext','mxapiwodetail','task_long_desc'),
       		'readonly':true, 
       		'value':value
       	};
       	this.async(function(){
	       	var div = $j(flyout.$.internal).find('div');
	      	div.css({'width':'100%'});
	      	div.append(maximoRTE);
       	},100);
       	this.LongDescRTE = maximoRTE;
	},
	_cancelLongDescription : function(e){
		this.LongDescRTE.remove();
		this.LongDescRTE = null;
  	},
  	/**
  	 * Enable the Card Hover for Task based on the length of task description
  	 * Length decided based on the  count of capital O's fit typically the widest character in Latin alphabet
  	 */
  	_enableCardHoverForTask: function(e){
  		var taskDesc=$j(e.currentTarget).find('#plans_task_description_label').attr('title');
  		if(taskDesc && taskDesc.length>=45){
  			$j(e.currentTarget).parent().parent().toggleClass('cardLayout_task_hover',true);
  		}else{
  			$j(e.currentTarget).parent().parent().toggleClass('cardLayout_task_hover',false);
  		}
  		
  	},
  	/**
  	 * Flip the Card for Task based on the length of task description
  	 * Length decided based on the  count of capital O's fit typically the widest character in Latin alphabet
  	 */
  	_flipCardForTask: function(e){
  		var taskDesc=$j(e.currentTarget).find('#plans_task_description_label').attr('title');
    	if(taskDesc && taskDesc.length>=45){
    		this.flipCard(e);
    	}
  	},
  	
  	_toggleWSFixed: function(e){
  		$j(this.$.plansSelectorDiv).toggleClass('fixedSelector', e.detail === true);
  		if(this.domHost && this.domHost.$.maximoAppHeader){
		  var pageHeaderHeight = $j(this.domHost.$.maximoAppHeader).find('.pageHeader').height();
		  $j(this.$.plansSelectorDiv).css('top', pageHeaderHeight+'px');
	  }
  	},
  	
  	/**
  	 * getExternal Value for a status
  	 */
  	_getExternalValueForWOstatus: function(internalValue){
  	  	var externalValue;
 		 if(this.domHost && this.domHost.domHost && this.domHost.domHost.woStatusSet){
 			this.domHost.domHost.woStatusSet.forEach(function(domainrecord){
 	   	    	if(domainrecord.defaults && domainrecord.maxvalue===internalValue){
 	   	    		externalValue = domainrecord.value;
 	   	    	}
 	 		});
 		 }
 		return externalValue;
  	},
  	/**
     * Return storeroom location description if available else storeroom location.
     */
    _getLocation : function(storeloc){
		return storeloc.description?storeloc.description:storeloc.location;
    },
  	
  	/**
	 * Reloads counter for task, material and tools except current selected tab which will fetch count on refresh record.
	 */
	refreshCounters: function() {
		var self = this;
		if(this.currentTab!=='section_tasks'){
			self.$.workordertasks.queryInfo=[];
			self.$.workordertasks.queryInfo[0]={'href':this.record.href+'/wo_tasks.mxapiwodetail'};
			this.$.workordertasks.$.mxajaxCollectionCount.url  = this.$.workordertasks.queryInfo[0].href;
			this.$.workordertasks.collectionUri = this.record.href+'/wo_tasks.mxapiwodetail';
			this.$.workordertasks.getCollectionCount().then(function(response){
		    	self.totalTaskCount = response.response.totalCount;
		    });
		}
		
		if(this.currentTab!=='section_materials'){ 
			this.$.wpmaterial.queryInfo=[];
			this.$.wpmaterial.queryInfo[0]={'href':this.record.href+'/wpmaterialtask'};
			this.$.wpmaterial.$.mxajaxCollectionCount.url  = this.$.wpmaterial.queryInfo[0].href;
			this.$.wpmaterial.collectionUri = this.record.href+'/wpmaterialtask';
			this.$.wpmaterial.getCollectionCount().then(function(response){
		    	self.totalMaterialCount = response.response.totalCount;
		    });
		}
		
		if(this.currentTab!=='section_tools'){
			this.$.wptools.queryInfo=[];
			this.$.wptools.queryInfo[0]={'href':this.record.href+'/showplantool'};
			this.$.wptools.$.mxajaxCollectionCount.url  = this.record.href+'/showplantool';
			this.$.wptools.collectionUri =	this.$.wptools.queryInfo[0].href;
			this.$.wptools.getCollectionCount().then(function(response){
		    	self.totalToolsCount = response.response.totalCount;
		    });
		}
		
	},
	/**
     * Handle the display of location description and storelocation.
     */
    _getStoreroomLocation : function(storeloc){
    	return storeloc.location.description?storeloc.location.description:storeloc.location.location;
    }
});
